#!/bin/bash
### SOURCE SAME BASIC VARS NEEDED
source /scripts/basic_vars

### SOURCE SAME BASIC FUNCTIONS NEEDED
source /scripts/basic_functions

### SOURCE CURL FUNCTION
source /root/admin-openrc.sh

if [ "$LOG_IN_FILE" -eq 1 ]; then
  enable_logging /var/log/homecalling
fi

apiport=30636

### GO GO GO
curl_retries=6
curl_attempt=1
curl_timeout=20
curl_sleep=10

### DOWNLOAD THE NEW CONFIG
function download_config () {
  SHATOKEN=$(cat /config/.serial-number)
  ZTPTOKEN=""
  if [ -f ${ztp_authfile} ]; then
    ZTPTOKEN=$(cat ${ztp_authfile})
  fi
  manager=$(sysrepocfg --datastore=running --format=xml --xpath="/ucpe:config/system:system/config-server/server" --export | grep "<server>" | cut -d '>' -f 2 | cut -d '<' -f1)
  if [ -n "${ZTPTOKEN}" ]; then
    http_code=$(curl -m $curl_timeout -H "Content-type: application/json" -H "Authentication:$SHATOKEN" -H "Token:$ZTPTOKEN"  -k -GET https://$manager:$apiport/config --write-out %{http_code} --silent --output /tmp/output.txt)
  else
    http_code=$(curl -m $curl_timeout -H "Content-type: application/json" -H "Authentication:$SHATOKEN" -k -GET https://$manager:$apiport/config --write-out %{http_code} --silent --output /tmp/output.txt)
  fi
}

function update_cfgs() {
  current_orchestrator=$1
  if [ -n "$current_orchestrator" ]; then
    sed -i "s/trapsess -v2c -c monitoring.*/trapsess -v2c -c monitoring $current_orchestrator:1062 -r 3 -t 10/g" /etc/snmp/snmpd.conf
    sed -i "s/snmptrap -v 2c -c monitoring -r 3 -t 7.*:1062\(.*\)$/snmptrap -v 2c -c monitoring -r 3 -t 7 $current_orchestrator:1062\1/g" /etc/bash.bashrc
    [[ -f /etc/freezer/env.conf ]] && sed -i "s#OS_AUTH_URL=https://.*#OS_AUTH_URL=https://$current_orchestrator:35357/v3#g" /etc/freezer/env.conf
  fi
}

function update_ztpstatus() {
  status=$1
  outf="$(mktemp "${TMPDIR:-/tmp}/XXXXXX")" || true
  /usr/bin/jq ".\"ucpe:status\".\"ztp-status\"=\"${status}\"" ${UCPE_STATUS_FILE} > $outf  || true
  mv $outf ${UCPE_STATUS_FILE} || true
  chmod 777 ${UCPE_STATUS_FILE} || true
}


### SKIP THIS IF WE ARE IN UPGRADE MODE
if [ "$force_config" -eq 0 ]; then
  curl_success=0
  update_ztpstatus "unknown"
  while [ "$curl_attempt" -le "$curl_retries" ]; do

    pretty_print "ATTEMPTING TO DOWNLOAD CONFIG #$curl_attempt"
    download_config

    code=$?
    sanity=""
    sanity=`grep ucpe:config /tmp/output.txt 2>/dev/null`
    ### IF CURL REQUEST WAS SUCCESSFUL
    if [ -n "$sanity" ] && [ "$code" -eq 0 ] && [ "$http_code" -ge 200 ] && [ "$http_code" -le 299 ]; then
      cat /tmp/output.txt | python3 -mjson.tool > $working_dir/$new_config_file 2>/dev/null
      /bin/rm /tmp/output.txt
      curl_success=1
      new_orchestrator=$(get_json_value //json/tve/ucpe:config/system:system/config-server $working_dir/$new_config_file | grep server | cut -d'"' -f4)
      cp $working_dir/$new_config_file $working_dir/tve_config_manager.json
      rm -f /run/ztp-failed-flag || true
      update_ztpstatus "completed"
      # python3 /scripts/apply_system_sysrepo.py -c $new_orchestrator
      exit 0
    fi

    #currently manager sends 500 for both bad serial and bad token...
    #if  [ "$http_code" -eq 500 ]; then
    update_ztpstatus "error-badser"
    #update_ztpstatusfi


    let curl_attempt+=1
    let curl_timeout+=2

    sleep $curl_sleep
  done

  if [ ${curl_success} -ne 1 ]; then
    #as of time of this update, old_orchestrator as a value is lost
    #new config download failed but we already updated /boot/config-server
    #so at least be consistent and make update to the end
    current_orchestrator=`echo $(sysrepocfg --datastore=running --format=json --xpath=\"/ucpe:config/system:system/config-server\" --export | grep \"server\": | cut -d'"' -f4)`
    if [ -n "$current_orchestrator" ]; then
      update_cfgs $current_orchestrator

      NEW_SVCS="freezer-scheduler.service"
      sed -i "/^SVC_LIST=/ s/$/$NEW_SVCS /" $svc_restart_file
    fi
    # Download failed !!!! This is fail case !
    exit 2
  fi
fi

### ADD A FAILSAFE WHEN OLD CONTROLLER IS NOT PRESENT
if [ ! -z "$new_orchestrator" ]; then
  update_cfgs $new_orchestrator

  NEW_SVCS="freezer-scheduler.service"
  sed -i "/^SVC_LIST=/ s/$/$NEW_SVCS /" $svc_restart_file
else
  pretty_print "NEW ORCHESTRATOR NOT SET! PLS CHECK THE CONFIG!"
  exit 1
fi

exit 0
