#!/bin/bash
# -*- coding: utf-8 -*-
# vim: tabstop=2 expandtab shiftwidth=2 softtabstop=2
# vim: set fileencoding=utf-8

### SOURCE SAME BASIC VARS NEEDED
source /scripts/basic_vars

### SOURCE SAME BASIC FUNCTIONS NEEDED
source /scripts/basic_functions

if [ "$LOG_IN_FILE" -eq 1 ]; then
  enable_logging /var/log/homecalling
fi

# Create capabilities dump if does not exist
if [ ! -f $working_dir/capabilities ] ; then
  get_device_capabilities.sh
fi

# Set all required variables
new_config=$1
old_config=$2
if_name=$3
new_state=$(get_json_value_by_key $new_config //config/networking/interfaces/ mode name=$if_name)
old_state=$(get_json_value_by_key $old_config //config/networking/interfaces/ mode name=$if_name)
sriov_config_file="/etc/neutron/plugins/ml2/sriov_agent.ini.agent"
ovs_config_file="/etc/neutron/plugins/ml2/openvswitch_agent.ini.agent"
ixgbe_conf="/etc/modprobe.d/ixgbe.conf"
nova_conf="/etc/nova/nova.conf.agent"
source $working_dir/capabilities

# Exit early if IOMMU is not supported
if [ "$IOMMU" == "no" ] ; then
  exit 0
fi
#
# Check if we need to adjust anything
if [ "$new_state" == "$old_state" ] ; then
  exit 0
fi
#
function set_num_vfs {
  vf_count=`cat /sys/class/net/${if_name}/device/sriov_totalvfs`
  echo ${vf_count} > /sys/class/net/${if_name}/device/sriov_numvfs
} # function set_num_vfs
#
function restart_services {
  # If the list of services is empty, append openstack-nova-compute there
  if grep -q "^SVC_LIST=\s\?$" $svc_restart_file ; then
    sed -i "s/^SVC_LIST=$/SVC_LIST=nova-compute neutron-sriov-agent/" $svc_restart_file
    exit 0
  fi

  svc_to_restart=`grep ^SVC_LIST=* $svc_restart_file`
  # If the list is not empty, append only if service are not on the list of services to restart
  if [[ ! $svc_to_restart = *"nova-compute"* ]] ; then
    sed -i "s/^SVC_LIST=.*/& nova-compute/" $svc_restart_file
  fi
  if [[ ! $svc_to_restart = *"neutron-sriov-agent"* ]] ; then
    sed -i "s/^SVC_LIST=.*/& neutron-sriov-agent/" $svc_restart_file
  fi
} # function restart_services
#
if ! grep -iq $if_name $sriov_config_file && [ "$new_state" == "sr-iov" ] ; then
  # Add mapping to sr-iov agent conf
  physnet=`grep -i "^bridge_mappings =" $ovs_config_file | grep -oP "physnet[0-9]:br-$if_name" | cut -f"1" -d":"`
  if grep -iq "^physical_device_mappings =\s\?$" $sriov_config_file ; then
    sed -i "/physical_device_mappings =/s/$/ $physnet\:$if_name/" $sriov_config_file
  else
    sed -i "/physical_device_mappings =/s/$/, $physnet\:$if_name/" $sriov_config_file
  fi

  # Remove interface from openvswitch agent configuration file
  sed -i -E "/^bridge_mappings(\s)?=/s/(\,)?(\s)?physnet[0-9]:br-$if_name//" $ovs_config_file

  # Add device to pci_whitelist in nova.conf.agent
  current_list=`grep -i "pci_passthrough_whitelist" $nova_conf | cut -f2 -d"="`
  pci_list=`python3 /scripts/pci_whitelist.py "add" "$if_name" "$physnet" "$current_list"`
  echo "/scripts/apply_config_sriov.sh: sed in /etc/nova/nova.conf.agent" | systemd-cat
  sed -i "s/^pci_passthrough_whitelist = .*/pci_passthrough_whitelist = $pci_list/" $nova_conf

  vf_count=`cat /sys/class/net/${if_name}/device/sriov_totalvfs`
  echo ${vf_count} > /sys/class/net/${if_name}/device/sriov_numvfs
  restart_services
elif grep -iq $if_name $sriov_config_file && [ "$new_state" != "sr-iov" ] ; then
  # Add mapping to ovs agent conf
  physnet=`grep -i "^physical_device_mappings =" $sriov_config_file | grep -oP "physnet[0-9]:$if_name" | cut -f"1" -d":"`
  sed -i "/^bridge_mappings =/s/$/, $physnet\:br-$if_name/" $ovs_config_file

  # Remove interface from sriov agent configuration file
  sed -i -E "/^physical_device_mappings(\s)?=/s/(\,)?(\s)?physnet[0-9]:$if_name//" $sriov_config_file

  # Remove device from pci_whitelist in nova.conf.agent
  current_list=`grep -i "pci_passthrough_whitelist" $nova_conf | cut -f2 -d"="`
  pci_list=`python3 /scripts/pci_whitelist.py "remove" "$if_name" "$physnet" "$current_list"`
  echo "/scripts/openvpn_up_handle.sh: sed in /etc/nova/nova.conf.agent" | systemd-cat
  sed -i "s/^pci_passthrough_whitelist = .*/pci_passthrough_whitelist = $pci_list/" $nova_conf

  echo 0 > /sys/class/net/${if_name}/device/sriov_numvfs
  restart_services
fi
#
exit 0
