#!/usr/bin/env bash
# -*- coding: utf-8 -*-
# coding: utf-8
# vim: tabstop=2 expandtab shiftwidth=2 softtabstop=2
# vim: set fileencoding=utf-8
source /scripts/basic_vars
set -o nounset
set -o pipefail
exec 1> >(exec logger -s -t $(basename $0)) 2>&1
doreboot=0
serial=$(cat /config/.serial-number)
hwtype=$(cat /config/.hwtype)
if [ -z "${serial}" -o -z "${hwtype}" ]; then
  logger -t apply_customization "Unknown serial --${serial}-- or hwtype --${hwtype}--!"
  exit 1
fi
# Try to mount USB, if present. Copy customziation files to $customization_dir ; unmount USB;
# Perform customizations (they might be put during installation already)
# don't forget to reboot.
#
for isCustUSB in $(ls /dev/disk/by-id); do
  mount -o ro /dev/disk/by-id/"${isCustUSB}" /mnt
  if [[ $? = 0 && -d /mnt/cert && -d /mnt/config && -d /mnt/filesystem ]]; then
    logger -p daemon.warn -t apply_customization "Found customization USB"
    cp -rf /mnt/cert/ ${customization_dir}
    cp -rf /mnt/config/ ${customization_dir}
    cp -rf /mnt/filesystem/ ${customization_dir}
    umount --force /mnt
    break
  fi
  umount --force /mnt
done
#
set -o errexit
set -o xtrace
#
cd /tmp
#
if [ -d ${customization_dir}/filesystem/${serial}/etc/udev/rules.d ]; then
  cp -f ${customization_dir}/filesystem//${serial}/etc/udev/rules.d/* /etc/udev/rules.d/
  doreboot=1
elif [ -d ${customization_dir}/filesystem/${hwtype}/etc/udev/rules.d ]; then
  cp -f ${customization_dir}/filesystem/${hwtype}/etc/udev/rules.d/* /etc/udev/rules.d/
  doreboot=1
elif [ -d ${customization_dir}/filesystem/ANY/etc/udev/rules.d ]; then
  cp -f ${customization_dir}/filesystem/ANY/etc/udev/rules.d/* /etc/udev/rules.d/
  doreboot=1
fi

#
isConfigServer="null"

if [ -f ${customization_dir}/config/${serial}/config.json.merge ]; then
  isConfigServer=$(/usr/bin/jq '.["ucpe:config"]["system:system"]["config-server"]' ${customization_dir}/config/${serial}/config.json.merge | tr -d '" \n[]{}' | cut -f2 -d':' || true)
  if [ "null" != "${isConfigServer}" ]; then
    /scripts/sysreposet --datastore=startup --delete "/ucpe:config/system:system/config-server"
  fi
  isRepoServer=$(/usr/bin/jq '.["ucpe:config"]["upgrade:upgrade"]["repo-server"]' ${customization_dir}/config/${serial}/config.json.merge | tr -d '" \n' || true)
  if [ "null" != "${isRepoServer}" ]; then
    /scripts/sysreposet --datastore=startup --delete "/ucpe:config/upgrade:upgrade/repo-server"
  fi
  netMode=$(/usr/bin/jq '.["ucpe:config"]["system:system"]["network-mode"]' ${customization_dir}/config/${serial}/config.json.merge | tr -d '" \n[]{}' | cut -f2 -d':' || true)
  if [ "vpp" == "${netMode}" ]; then
    /usr/bin/python3 /opt/bin/switch_ovs_vpp.py --enable vpp
  fi
  /scripts/sysreposet --datastore=startup --merge=${customization_dir}/config/${serial}/config.json.merge || true
  /usr/bin/sysrepocfg --datastore=startup --format=json --export=/config/ucpe_config.json -m ucpe || true
  doreboot=1
elif [ -f ${customization_dir}/config/${hwtype}/config.json.merge ]; then
  isConfigServer=$(/usr/bin/jq '.["ucpe:config"]["system:system"]["config-server"]' ${customization_dir}/config/${hwtype}/config.json.merge | tr -d '" \n[]{}' | cut -f2 -d':' || true)
  if [ "null" != "${isConfigServer}" ]; then
    /scripts/sysreposet --datastore=startup --delete "/ucpe:config/system:system/config-server"
  fi
  isRepoServer=$(/usr/bin/jq '.["ucpe:config"]["upgrade:upgrade"]["repo-server"]' ${customization_dir}/config/${hwtype}/config.json.merge | tr -d '" \n' || true)
  if [ "null" != "${isRepoServer}" ]; then
    /scripts/sysreposet --datastore=startup --delete "/ucpe:config/upgrade:upgrade/repo-server"
  fi
  netMode=$(/usr/bin/jq '.["ucpe:config"]["system:system"]["network-mode"]' ${customization_dir}/config/${hwtype}/config.json.merge | tr -d '" \n[]{}' | cut -f2 -d':' || true)
  if [ "vpp" == "${netMode}" ]; then
    /usr/bin/python3 /opt/bin/switch_ovs_vpp.py --enable vpp
  fi
  /scripts/sysreposet --datastore=startup --merge=${customization_dir}/config/${hwtype}/config.json.merge || true
  /usr/bin/sysrepocfg --datastore=startup --format=json --export=/config/ucpe_config.json -m ucpe || true
  doreboot=1
elif [ -f ${customization_dir}/config/ANY/config.json.merge ]; then
  isConfigServer=$(/usr/bin/jq '.["ucpe:config"]["system:system"]["config-server"]' ${customization_dir}/config/ANY/config.json.merge | tr -d '" \n[]{}' | cut -f2 -d':' || true)
  if [ "null" != "${isConfigServer}" ]; then
    /scripts/sysreposet --datastore=startup --delete "/ucpe:config/system:system/config-server"
  fi
  isRepoServer=$(/usr/bin/jq '.["ucpe:config"]["upgrade:upgrade"]["repo-server"]' ${customization_dir}/config/ANY/config.json.merge | tr -d '" \n' || true)
  if [ "null" != "${isRepoServer}" ]; then
    /scripts/sysreposet --datastore=startup --delete "/ucpe:config/upgrade:upgrade/repo-server"
  fi
  netMode=$(/usr/bin/jq '.["ucpe:config"]["system:system"]["network-mode"]' ${customization_dir}/config/ANY/config.json.merge | tr -d '" \n[]{}' | cut -f2 -d':' || true)
  if [ "vpp" == "${netMode}" ]; then
    /usr/bin/python3 /opt/bin/switch_ovs_vpp.py --enable vpp
  fi
  /scripts/sysreposet --datastore=startup --merge=${customization_dir}/config/ANY/config.json.merge || true
  /usr/bin/sysrepocfg --datastore=startup --format=json --export=/config/ucpe_config.json -m ucpe || true
  doreboot=1
fi

#nanopi does not have openstack subscriber running
if [ "nanopir2s" == "$(cat /config/.hwtype)" -o "nanopir4s" == "$(cat /config/.hwtype)" ]; then
  if [ "null" != "${isConfigServer}" ]; then
    sed -i "s/<Server \".*\" \"25826\"> \# LOGSTASH_SERVER_LINE/<Server \"${isConfigServer}\" \"25826\"> \# LOGSTASH_SERVER_LINE/g" /etc/collectd/collectd.conf
  fi
fi


version_str=$(cat /etc/nfvtime-os.release || true)
arm_ver=$(cat /config/.version | cut -d'b' -f2 | sed 's/^0*//' || true)
banner_path=""

#banner customization
if [ -f ${customization_dir}/config/${serial}/banner-txt ]; then
  banner_path=${customization_dir}/config/${serial}/banner-txt
elif [ -f ${customization_dir}/config/${hwtype}/banner-txt ]; then
  banner_path=${customization_dir}/config/${hwtype}/banner-txt
elif [ -f ${customization_dir}/config/ANY/banner-txt ]; then
  banner_path=${customization_dir}/config/ANY/banner-txt
fi
if [ -n "${banner_path}" ]; then
  cp -f ${banner_path} /config/motd || true
  cp -f /config/motd /usr/share/base-files/motd || true
  if [ -z "${arm_ver}" ]; then
    echo "Version ${version_str}" >> /usr/share/base-files/motd || true
  else
    echo "Version ${version_str}b${arm_ver}" >> /usr/share/base-files/motd || true
  fi
fi

restart_dashboard=""
banner_path=""
#web lct logo
if [ -f ${customization_dir}/config/${serial}/banner-img ]; then
  banner_path=${customization_dir}/config/${serial}/banner-img
elif [ -f ${customization_dir}/config/${hwtype}/banner-img ]; then
  banner_path=${customization_dir}/config/${hwtype}/banner-img
elif [ -f ${customization_dir}/config/ANY/banner-img ]; then
  banner_path=${customization_dir}/config/ANY/banner-img
fi

if [ -n "${banner_path}" ]; then
  cp -f ${banner_path} /config/banner-img || true
  cp -f /config/banner-img /var/www/ucpe-web-lct/assets/images/logo.png || true
  cp -f /config/banner-img /usr/share/openstack-dashboard/openstack_dashboard/static/dashboard/img/logo-splash.png || true
  cp -f /config/banner-img /usr/share/openstack-dashboard/openstack_dashboard/static/dashboard/img/logo.png || true
  restart_dashboard="yes"
fi


banner_watermark_path=""
#horizon watermark
if [ -f ${customization_dir}/config/${serial}/banner-watermark ]; then
  banner_watermark_path=${customization_dir}/config/${serial}/banner-watermark
elif [ -f ${customization_dir}/config/${hwtype}/banner-watermark ]; then
  banner_watermark_path=${customization_dir}/config/${hwtype}/banner-watermark
elif [ -f ${customization_dir}/config/ANY/banner-watermark ]; then
  banner_watermark_path=${customization_dir}/config/ANY/banner-watermark
fi

if [ -n "${banner_watermark_path}" ]; then
  cp -f ${banner_watermark_path} /config/banner-watermark || true
  cp -f /config/banner-watermark /usr/share/openstack-dashboard/openstack_dashboard/static/dashboard/img/telco-watermark.png || true
  restart_dashboard="yes"
fi


favicon_path=""
#horizon favicon
if [ -f ${customization_dir}/config/${serial}/favicon ]; then
  favicon_path=${customization_dir}/config/${serial}/favicon
elif [ -f ${customization_dir}/config/${hwtype}/favicon ]; then
  favicon_path=${customization_dir}/config/${hwtype}/favicon
elif [ -f ${customization_dir}/config/ANY/favicon ]; then
  favicon_path=${customization_dir}/config/ANY/favicon
fi

if [ -n "${favicon_path}" ]; then
  cp -f ${favicon_path} /config/favicon || true
  cp -f /config/favicon /usr/share/openstack-dashboard/openstack_dashboard/static/dashboard/img/favicon.ico || true
  cp -f /config/favicon /usr/share/openstack-dashboard/openstack_dashboard/static/dashboard/img/favicon.png || true
  cp -f /config/favicon /var/www/ucpe-web-lct/favicon.ico || true
  restart_dashboard="yes"
fi


productname_path=""
#web lct product name
if [ -f ${customization_dir}/config/${serial}/product-name ]; then
  productname_path=${customization_dir}/config/${serial}/product-name
elif [ -f ${customization_dir}/config/${hwtype}/product-name ]; then
  productname_path=${customization_dir}/config/${hwtype}/product-name
elif [ -f ${customization_dir}/config/ANY/product-name ]; then
  productname_path=${customization_dir}/config/ANY/product-name
fi

if [ -n "${productname_path}" ]; then
  cp -f ${productname_path} /config/product-name || true
  productname=`cat /config/product-name` || true
  cat /var/www/ucpe-web-lct/assets/config.json | /usr/bin/jq ".Brand_Name = \"${productname}\"" > /tmp/config.json || true
  mv /tmp/config.json /var/www/ucpe-web-lct/assets/config.json || true
fi



lct_css_path=""
#web lct css
if [ -f ${customization_dir}/config/${serial}/lct-css ]; then
  lct_css_path=${customization_dir}/config/${serial}/lct-css
elif [ -f ${customization_dir}/config/${hwtype}/lct-css ]; then
  lct_css_path=${customization_dir}/config/${hwtype}/lct-css
elif [ -f ${customization_dir}/config/ANY/lct-css ]; then
  lct_css_path=${customization_dir}/config/ANY/lct-css
fi

if [ -n "${lct_css_path}" ]; then
  cp -f ${lct_css_path} /config/lct-css || true
  existingfile=`find /var/www/ucpe-web-lct -name styles*.bundle.css | head -1`
  if [ -f "${existingfile}" ] ; then
    cp -f /config/lct-css ${existingfile} || true
  fi
fi



horizon_css_path=""
#horizon css
if [ -f ${customization_dir}/config/${serial}/horizon-css ]; then
  horizon_css_path=${customization_dir}/config/${serial}/horizon-css
elif [ -f ${customization_dir}/config/${hwtype}/horizon-css ]; then
  horizon_css_path=${customization_dir}/config/${hwtype}/horizon-css
elif [ -f ${customization_dir}/config/ANY/horizon-css ]; then
  horizon_css_path=${customization_dir}/config/ANY/horizon-css
fi

if [ -n "${horizon_css_path}" ]; then
  cp -f ${horizon_css_path} /config/horizon-css || true
  cp -f /config/horizon-css /usr/share/openstack-dashboard/openstack_dashboard/static/dashboard/scss/telco_horizon_theme.css || true
  restart_dashboard="yes"
fi

# Custom udev rules for LTE dongles
lte_udev_path=""
if [ -f ${customization_dir}/config/${serial}/lte-rules ]; then
  lte_udev_path=${customization_dir}/config/${serial}/lte-rules
elif [ -f ${customization_dir}/config/${hwtype}/lte-rules ]; then
  lte_udev_path=${customization_dir}/config/${hwtype}/lte-rules
elif [ -f ${customization_dir}/config/ANY/lte-rules ]; then
  lte_udev_path=${customization_dir}/config/ANY/lte-rules
fi

if [ -n "${lte_udev_path}" ]; then
  cp -f ${lte_udev_path} /etc/udev/rules.d/80-lte.rules || true
  doreboot=1
fi

#
if [ -n "$(ls ${customization_dir}/cert/${serial}/ipsec*.*)" ]; then
  cp -f ${customization_dir}/cert/${serial}/ipsec*.* /home/admin/user_files
elif [ -n "$(ls ${customization_dir}/cert/${hwtype}/ipsec*.*)" ]; then
  cp -f ${customization_dir}/cert/${hwtype}/ipsec*.* /home/admin/user_files
elif [ -n "$(ls ${customization_dir}/cert/ANY/ipsec*.*)" ]; then
  cp -f ${customization_dir}/cert/ANY/ipsec*.* /home/admin/user_files
fi
#

function find_customized_keys() {
  if [ -f ${customization_dir}/cert/${serial}/${1} ]; then
    echo ${customization_dir}/cert/${serial}/${1}
  elif [ -f ${customization_dir}/cert/${hwtype}/${1} ]; then
    echo ${customization_dir}/cert/${hwtype}/${1}
  elif [ -f ${customization_dir}/cert/ANY/${1} ]; then
    echo ${customization_dir}/cert/ANY/${1}
  fi
}
vpnca=$(find_customized_keys vpnca.crt)
vpncrt=$(find_customized_keys vpn.crt)
vpnkey=$(find_customized_keys vpn.key)
hbca=$(find_customized_keys hbca.crt)
hbcrt=$(find_customized_keys hb.crt)
hbkey=$(find_customized_keys hb.key)
webcrt=$(find_customized_keys web.crt)
webkey=$(find_customized_keys web.key)
#
if [ -z "${vpnca}" -o -z "${vpncrt}" -o -z "${vpnkey}" \
        -o -z "${hbca}" -o -z "${hbcrt}" -o -z "${hbkey}" \
        -o -z "${webcrt}" -o -z "${webkey}" ]; then
  logger -p daemon.warn -t apply_customization "Not full set of keys !"
else
  mtd=$(mktemp -d)
  pushd ${mtd}
  cp ${vpnca} ${vpncrt} ${vpnkey} \
     ${hbca} ${hbcrt} ${hbkey} ${webcrt} ${webkey} .
  tar cJf ucpe.txz *
  /usr/bin/python3 /scripts/cli/tvesh/apply_keys.py ucpe.txz
  popd
  rm -rf ${mtd}
  doreboot=1
fi
#

APPJSON_SERIAL="${customization_dir}/config/${serial}/app-template.json"
APPJSON_ANY="${customization_dir}/config/ANY/app-template.json"
APPJSON=""
if [ -f ${APPJSON_SERIAL} ]; then
  APPJSON=${APPJSON_SERIAL}
elif [ -f ${APPJSON_ANY} ]; then
  APPJSON=${APPJSON_ANY}
fi
if [ -f "${APPJSON}" ]; then
  APPNAME=$(/usr/bin/jq '.application.name' ${APPJSON} | tr -d '"' |tr '[:upper:]' '[:lower:]')
  USERDATAFILE=$(/usr/bin/jq '.application."user-data-file"' ${APPJSON} | tr -d '"')
  if [ -d /root/apps/${APPNAME} ]; then
    APPCONFIG=/root/apps/${APPNAME}/application.json
    CONFIGNAME=$(/usr/bin/jq '.config' ${APPCONFIG} | tr -d '"' |tr '[:upper:]' '[:lower:]')
    cp -f ${APPJSON} /root/apps/${APPNAME}/${CONFIGNAME}
    if [ -f ${customization_dir}/config/${serial}/${USERDATAFILE} ]; then
      cp -f ${customization_dir}/config/${serial}/${USERDATAFILE} /root/apps/${APPNAME}/sgwroot/core.cfg
    elif [ -f ${customization_dir}/config/ANY/${USERDATAFILE} ]; then
      cp -f ${customization_dir}/config/ANY/${USERDATAFILE} /root/apps/${APPNAME}/sgwroot/core.cfg
      touch /root/apps/${APPNAME}/custom_userdatafile
    fi

    ROUTING=$(/usr/bin/jq '.application.routing' ${APPJSON} | tr -d '"')
    if [ "true" = "${ROUTING}" ]; then
      sysctl -w net.ipv4.ip_forward=1 || true
      sed -i '/^net.ipv4.ip_forward/d' /etc/sysctl.conf
      sed -i '${x;/^$/{s//net.ipv4.ip_forward=1/;H};x}' /etc/sysctl.conf
    fi

    AUTOSTART=$(/usr/bin/jq '.application.autostart' ${APPJSON} | tr -d '"')
    if [ "true" = "${AUTOSTART}" ]; then
      systemctl enable ${APPNAME}.service 2>/dev/null || true
    fi

    doreboot=1
  fi
fi
#
touch ${customization_done_file}
if [ "1" == "${doreboot}" ]; then
  logger -p daemon.warn -t apply_customization "Requesting system reload" || true
  systemctl stop sshd 1> /dev/null
  systemctl stop ssh-root 1> /dev/null
  reboot
fi
#
exit 0
