#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import getopt
import sys

from sysrepo_dhcpserver import sysrepo_dhcpserver as api


def main(argv):
    drange = None
    dintf = None
    enabled = None
    gateway = None
    dns = None
    dlt = None
    mlt = None

    try:
        opts, args = getopt.getopt(argv, "r:i:e:g:d:l:m:")
    except getopt.GetoptError:
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-r':
            drange = arg
        elif opt == '-i':
            dintf = arg
        elif opt == '-e':
            enabled = arg
        elif opt == '-g':
            gateway = arg
        elif opt == '-d':
            dns = arg
        elif opt == '-l':
            dlt = arg
        elif opt == '-m':
            mlt = arg
        else:
            sys.exit(2)

    dhcp = api.SysrepoDhcpServer(app_name="set dhcpserver")

    if dintf is not None:
        dhcp.set_dhcp_int(dintf)

    if drange is not None:
        dhcp.set_dhcp_range(drange)

    if enabled is not None:
        if enabled == 'true' or enabled == '1':
            dhcp.set_state(True)
        elif enabled == 'false' or enabled == '0':
            dhcp.set_state(False)

    if gateway and gateway != "":
        dhcp.set_dhcp_router(gateway)
    else:
        dhcp.unset_dhcp_router()

    if dns and dns != "":
        dhcp.set_dhcp_domain_name_servers(dns)
    else:
        dhcp.unset_dhcp_domain_name_servers()

    if dlt and dlt != "":
        dhcp.set_dhcp_default_lease_time(int(dlt))
    else:
        dhcp.unset_dhcp_default_lease_time()

    if mlt and mlt != "":
        dhcp.set_dhcp_max_lease_time(int(mlt))
    else:
        dhcp.unset_dhcp_max_lease_time()

    dhcp.commit()

    sys.exit(0)


if __name__ == "__main__":
    main(sys.argv[1:])
