#!/usr/bin/env python3
import sys, getopt
import time
from sysrepo_systemresources import sysrepo_systemresources as api

def set_host_reserved_cpus(host_reserved_cpus=None):
    global system
    if host_reserved_cpus is not None:
        dpdk_reserved_cpus = system.get_dpdk_reserved_cpus()
        if not compare_args(dpdk_reserved_cpus.split(','), host_reserved_cpus.split(',')):
            system.set_host_reserved_cpus(host_reserved_cpus)
        else:
            sys.stdout.write("DPDK reserved CPUs and HOST reserved CPUs cannot have same CPU id.\n")
            sys.exit(0)
        
def set_host_reserved_memory(host_reserved_memory=None):
    global system
    if host_reserved_memory is not None:
        system.set_host_reserved_memory(host_reserved_memory)

def set_dpdk_reserved_cpus(dpdk_reserved_cpus=None):
    global system
    if dpdk_reserved_cpus is not None:
        host_reserved_cpus = system.get_host_reserved_cpus()
        if not compare_args(dpdk_reserved_cpus.split(','), host_reserved_cpus.split(',')):
            system.set_dpdk_reserved_cpus(dpdk_reserved_cpus)
        else:
            sys.stdout.write("DPDK reserved CPUs and HOST reserved CPUs cannot have same CPU id.\n")
            sys.exit(0)

def set_huge_page_size(huge_page_size=None):
    global system
    if huge_page_size is not None:
        system.set_huge_page_size(huge_page_size)
 
def set_huge_page_count(huge_page_count=None):
    global system
    if huge_page_count is not None:
        system.set_huge_page_count(huge_page_count)
 
def set_host_oversubscribe_enabled(enabled=None):
    global system
    if enabled is not None:
        if enabled == 'true' or enabled == '1':
            system.set_host_oversubscribe_enabled(True)
        elif enabled == 'false' or enabled == '0':
            system.set_host_oversubscribe_enabled(False)

def compare_args(list1, list2):
    a_set = set(list1)
    b_set = set(list2)
    if (a_set & b_set): 
        return True 
    else: 
        return False

def main(argv):
    hostcpu=None
    hostmem=None
    dpdkcpu=None
    hugesz=None
    hugecnt=None
    oversb=None
    dostartup=False
    doretry=False
    try:
        opts, args=getopt.getopt(argv,"trc:m:d:s:u:o:")
    except getopt.GetoptError:
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-c':
            hostcpu=arg
        elif opt == '-m':
            hostmem=arg
        elif opt == '-d':
            dpdkcpu=arg
        elif opt == '-s':
            hugesz=arg
        elif opt == '-u':
            hugecnt=arg
        elif opt == '-o':
            oversb=arg
        elif opt == '-t':
            dostartup=True
        elif opt == '-r':
            doretry=True
        else:
            sys.exit(2)

    global system
    system = api.SysrepoSystemResources(app_name="set systemresources")
    
    if dostartup:
        client.set_active_datastore("startup")
    set_host_reserved_cpus(hostcpu)
    set_host_reserved_memory(hostmem)
    set_dpdk_reserved_cpus(dpdkcpu)
    set_huge_page_size(hugesz)
    set_huge_page_count(hugecnt)
    set_host_oversubscribe_enabled(oversb)
    
    retrycnt = 1
    if doretry:
        retrycnt = 30
    for i in range(0,retrycnt):
        try:
            system.commit()
        except:
            if doretry:
                time.sleep(10)
                continue
        break
   
    #subprocess.run(['sudo', 'systemctl', 'stop', 'sshd'])
    #subprocess.run(['sudo', 'pkill', 'clish'])

    sys.exit(0)

if __name__ == "__main__":
    main(sys.argv[1:])
