#!/usr/bin/env python3

import sys, json, shutil

if len(sys.argv) < 2:
    raise Exception("Failed to apply config! Missing config file!")


f = open(sys.argv[1], 'r')
config=f.read()
if not config:
    shutil.copyfile(sys.argv[1], "/var/tmp/ucpe_config_stored.json")
    raise Exception("Failed to apply config file {}. Invalid config!".format(sys.argv[1]))


try:
    config = json.loads(config)
    if len(sys.argv) > 2:
        module_name = sys.argv[2]
    else:
        module_name = str(config["ucpe:config"].keys()[0])[:str(config["ucpe:config"].keys()[0]).find(":")]
except Exception as ex:
    raise Exception("Can't parse the module name from config file {}!".format(sys.argv[1]))

if module_name == "interfaces":
    from sysrepo_interfaces import sysrepo_interfaces3 as api
    client = api.SysrepoInterfaces(app_name="{} config".format(module_name))
elif module_name == "monitoring":
    from sysrepo_monitoring import sysrepo_monitoring as api
    client = api.SysrepoMonitoring(app_name="{} config".format(module_name))
elif module_name == "ntp":
    from sysrepo_ntp import sysrepo_ntp as api
    client = api.SysrepoNTP(app_name="{} config".format(module_name))
elif module_name == "kubernetes":
    from sysrepo_kubernetes import sysrepo_kubernetes as api
    client = api.SysrepoKubernetes(app_name="{} config".format(module_name))
elif module_name == "quagga":
    from sysrepo_quagga import sysrepo_quagga as api
    client = api.SysrepoQuagga(app_name="{} config".format(module_name))
elif module_name == "security":
    from sysrepo_security import sysrepo_security as api
    client = api.SysrepoSecurity(app_name="{} config".format(module_name))
    old_cfg = client.get_running_config(full_path=False)
    if 'authentication' not in config['ucpe:config']['security:security'] and 'authentication' in old_cfg:
        config['ucpe:config']['security:security']['authentication'] = old_cfg['authentication']
    if 'cli-whitelist' not in config['ucpe:config']['security:security'] and 'cli-whitelist' in old_cfg:
        config['ucpe:config']['security:security']['cli-whitelist'] = old_cfg['cli-whitelist']
elif module_name == "snmp-server":
    from sysrepo_snmp import sysrepo_snmp as api
    client = api.SysrepoSNMP(app_name="{} config".format(module_name))
elif module_name == "syslog":
    from sysrepo_syslog import sysrepo_syslog as api
    client = api.SysrepoSyslog(app_name="{} config".format(module_name))
elif module_name == "system":
    from sysrepo_system import sysrepo_system as api
    client = api.SysrepoSystem(app_name="{} config".format(module_name))
    old_cfg = client.get_running_config(full_path=False)
    if 'homecalling' not in config['ucpe:config']['system:system'] and 'homecalling' in old_cfg:
        config['ucpe:config']['system:system']['homecalling'] = old_cfg['homecalling']
    if 'config-server' not in config['ucpe:config']['system:system'] and 'config-server' in old_cfg:
        config['ucpe:config']['system:system']['config-server'] = old_cfg['config-server']
    if 'repo-server' not in config['ucpe:config']['system:system'] and 'repo-server' in old_cfg:
        config['ucpe:config']['system:system']['repo-server'] = old_cfg['repo-server']
    if 'hostname' not in config['ucpe:config']['system:system'] and 'hostname' in old_cfg:
        config['ucpe:config']['system:system']['hostname'] = old_cfg['hostname']
    if 'customer' not in config['ucpe:config']['system:system'] and 'customer' in old_cfg:
        config['ucpe:config']['system:system']['customer'] = old_cfg['customer']
    if 'service' not in config['ucpe:config']['system:system'] and 'service' in old_cfg:
        config['ucpe:config']['system:system']['service'] = old_cfg['service']
    if 'site' not in config['ucpe:config']['system:system'] and 'site' in old_cfg:
        config['ucpe:config']['system:system']['site'] = old_cfg['site']
    if 'operator' not in config['ucpe:config']['system:system'] and 'operator' in old_cfg:
        config['ucpe:config']['system:system']['operator'] = old_cfg['operator']
    if 'project' not in config['ucpe:config']['system:system'] and 'project' in old_cfg:
        config['ucpe:config']['system:system']['project'] = old_cfg['project']
elif module_name == "system-resources":
    from sysrepo_systemresources import sysrepo_systemresources as api
    client = api.SysrepoSystemResources(app_name="{} config".format(module_name))
elif module_name == "dhcpserver":
    from sysrepo_dhcpserver import sysrepo_dhcpserver as api
    client = api.SysrepoDhcpServer(app_name="{} config".format(module_name))
elif module_name == "dhcprelay":
    from sysrepo_dhcprelay import sysrepo_dhcprelay as api
    client = api.SysrepoDhcpRelay(app_name="{} config".format(module_name))
else:
    raise Exception("Unknown module name '{}'".format(module_name))

try:
    client.logger.info("Apply {} config: {}".format(module_name, client.print_json(config, return_data=True)))
    client.apply_json_config(config)
    client.commit()
    client.logger.info("Configuration applied successfully in sysrepo.")
except Exception as ex:
    client.logger.info("Error applying config in sysrepo!")
    client.logger.error(ex)
    sys.exit(1)

sys.exit(0)
