#!/usr/bin/env python3
import getopt
import json
import os
import sys

from sysrepo_system import sysrepo_system as api

UCPE_STATUS_FILE = "/config/_hb/ucpe_status.json"
system = None


def get_hw_type():
    if not os.path.isfile('/config/.hwtype'):
        return 'intel'

    with open('/config/.hwtype', 'r') as hw_type_file:
        return hw_type_file.readline().splitlines()[0]


hw_type = get_hw_type().lower()


def set_hostname(hostname=None):
    global system
    if hostname is not None:
        system.set_hostname(hostname)


def set_hc_timeout(timeout=None):
    global system
    if timeout is not None:
        system.set_homecalling_timeout(timeout)


def set_hc_enabled(enabled=None):
    global system
    if enabled is not None:
        if enabled == 'true' or enabled == '1':
            system.set_homecalling_enabled(True)
        elif enabled == 'false' or enabled == '0':
            system.set_homecalling_enabled(False)


def set_repository(repo=None):
    global system
    if repo is not None:
        system.set_repo_server(repo)


def set_config_server(config_server=None):
    global system
    if config_server:
        if os.path.exists(UCPE_STATUS_FILE):
            with open(UCPE_STATUS_FILE, 'r', encoding='utf-8') as f:
                data = json.load(f)

            if data:
                data['ucpe:status']['hb-status'] = 'not-contacted'
                with open(UCPE_STATUS_FILE, 'w', encoding='utf-8') as f:
                    print(json.dumps(data, indent=4, sort_keys=True), file=f)

        system.set_config_server(config_server)
        # nanoPI does not have subscriber to handle that
        if hw_type in ("nanopir2s", "NanoPI R2S", "nanopir4s", "NanoPI R4S"):
            system.logger.debug('large scale manager file detected')
            #os.system('sudo sed -i "s/tcp:.*/tcp:{}:30826/g" /lib/systemd/system/collectdToLogstash.service'.format(
            #    config_server))
            if os.path.exists('/etc/collectd/collectd.conf'):
                os.system('sudo sed -i "s/<Server \\".*\\" \\".*\\"> \\# LOGSTASH_SERVER_LINE/<Server \\"{}\\" \\"30826\\"> \\# LOGSTASH_SERVER_LINE/g" /etc/collectd/collectd.conf'. format(config_server))

            os.system('sudo systemctl --no-block restart collectd 2>/dev/null || true')

            #os.system('sudo systemctl daemon-reload 2>/dev/null || true')
            #os.system('sudo systemctl --no-block restart collectdToLogstash 2>/dev/null || true')


def set_network_mode(network_mode=None):
    global system
    if network_mode is not None:
        system.set_network_mode(network_mode)


def main(argv):
    hostname = None
    hc_timeout = None
    hc_enabled = None
    repo = None
    config_server = None
    network_mode = None

    try:
        opts, args = getopt.getopt(argv, "h:t:e:r:c:n:")
    except getopt.GetoptError:
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            hostname = arg
        elif opt == '-t':
            hc_timeout = arg
        elif opt == '-e':
            hc_enabled = arg
        elif opt == '-r':
            repo = arg
        elif opt == '-c':
            config_server = arg
        elif opt == '-n':
            network_mode = arg
        else:
            sys.exit(2)

    global system
    system = api.SysrepoSystem(app_name='apply_system_sysrepo')
    set_hostname(hostname)
    set_hc_timeout(hc_timeout)
    set_hc_enabled(hc_enabled)
    set_repository(repo)
    set_config_server(config_server)
    set_network_mode(network_mode)
    system.commit()

    sys.exit(0)


if __name__ == "__main__":
    main(sys.argv[1:])
