#!/usr/bin/env bash
exec 1> >(systemd-cat -p info -t "azure-healthcheck") 2>&1

function get_ip_config_tool {
  cat << EOF > /tmp/pyscript.py
#!/usr/bin/env python3
import json
import sys
import logging
from systemd.journal import JournalHandler

log = logging.getLogger(__name__)
log_ch = JournalHandler(SYSLOG_IDENTIFIER='azure-healthcheck-pyscript')
log.addHandler(log_ch)
log.setLevel(logging.DEBUG)
log.debug('dbg:')

a=json.load(sys.stdin)
for x in a:
    if x["name"] == "\${floatip}":
        log.debug('dbg: in floatip')
        log.debug('dbg: x start:')
        log.debug(x)
        log.debug('dbg: x end:')
        if "applicationSecurityGroups" in x and x["applicationSecurityGroups"] is None:
            print("{}\n{}\n{}\n{}\n{}".format(x["privateIPAddress"],x["privateIPAddressVersion"],x["publicIpAddress"]["id"],x["subnet"]["id"],""))
        else:
            print("{}\n{}\n{}\n{}\n{}".format(x["privateIPAddress"],x["privateIPAddressVersion"],x["publicIpAddress"]["id"],x["subnet"]["id"]," ".join(y["id"] for y in x["applicationSecurityGroups"])))

        break
    else:
        continue

EOF

  chmod 755 /tmp/pyscript.py
}

if [ -f "/config/azure.json" ]; then
  # This will return a lot of info in JSON format about the local VM instance
  arr1=( $(curl -H Metadata:true --noproxy "*" http://169.254.169.254/metadata/instance/compute?api-version=2020-09-01 | python3 -c 'import json,sys;x=json.load(sys.stdin);print("{}\n{}\n{}\n{}".format(x["name"],x["location"],x["resourceGroupName"],x["subscriptionId"]))' 2>/dev/null) )
  vm=`echo ${arr1[0]}`
  location=`echo ${arr1[1]}`
  resource=`echo ${arr1[2]}`
  subscription=`echo ${arr1[3]}`
  floatip=`cat /config/azure.json | python3 -c 'import json,sys;print( json.load(sys.stdin)["azure"]["floating_ip"] )' 2>/dev/null`

  while [ true ]; do
    # check if Master has the floating IP every 3 minutes
    sleep 180
    az login --identity
    if [ $? != 0 ]; then
      echo "Unsuccessful authentication atempt to Azure AD!"
    else
      # check for the floating address is on local nic
      nic=`az vm nic list -g $resource --vm-name $vm --subscription "$subscription" | python3 -c 'import json,sys;print(json.load(sys.stdin)[0]["id"])' 2>/dev/null | xargs basename 2>/dev/null`
      get_ip_config_tool
      arr1=( $(az network nic ip-config list -g $resource --nic-name $nic --subscription "$subscription" | /tmp/pyscript.py 2>/dev/null) )
      rm /tmp/pyscript.py
      prvtipaddr=`echo ${arr1[0]}`
      prvtipver=`echo ${arr1[1]}`
      pubip=`echo ${arr1[2]}`
      if [ -z "${prvtipaddr}" ] || [ -z "${prvtipver}" ] || [ -z "${pubip}" ]; then
        if [ -z "$(pgrep keepalived_azure.sh)" ]; then
          echo "Current HA role is MASTER, but floating IP is missing!"
          echo "Retrieving floating IP..."
          mktemp /config/_azure/azure_handle.XXXXXX
        fi
      fi

      az logout
    fi
  done
fi

