import requests
import json
import os
from vm_state_handler import main as run_second_script
import logging
import time

cbox_file = "/config/k8s-single-machine"
central_controller = "/config/central-box"
FLAG_FILES_DIR = "/config/vnf/"
MAX_RETRIES = 10
WAIT_SECONDS = 60

logging.basicConfig(
    level=logging.INFO, 
    format='%(asctime)s [%(levelname)s] - %(message)s',
    handlers=[
        logging.StreamHandler()
    ]
)

def fetch_token():
    auth_data = {
        "user": "admin",
        "password": "Nfv@admin21"
    }
    headers = {
        "Content-type": "application/json"
    }
    url = "https://127.0.0.1:17374/v3/auth/tokens"
    response = requests.get(url, json=auth_data, headers=headers, verify=False)
    response.raise_for_status()  # raise exception if status code is not 200
    return response.text.strip()

def fetch_k8s_token(token):
    headers = {"X-Auth-Token": token}
    response = requests.get('https://127.0.0.1:7373/v1/kubernetes/k8s_token', headers=headers, verify=False)
    response.raise_for_status()  # raise exception if status code is not 200
    return response.text.strip()

def is_flag_file_exists(vm_key):
    flag_file_path = f"{FLAG_FILES_DIR}run_{vm_key}"
    return os.path.isfile(flag_file_path)

def create_flag_file(vm_key):
    namespace, vm_name = vm_key
    flag_file_path = os.path.join('/config/vnf/', f'run_{namespace}_{vm_name}')
    open(flag_file_path, 'w').close()  # Create a flag file

def delete_flag_file(vm_key):
    namespace, vm_name = vm_key
    flag_file_path = os.path.join('/config/vnf/', f'run_{namespace}_{vm_name}')
    if os.path.exists(flag_file_path):
        os.remove(flag_file_path)  # Delete flag file if it exists

def get_flag_file_path(vm_key):
    namespace, vm_name = vm_key
    return os.path.join('/config/vnf/', f'run_{namespace}_{vm_name}')

def flag_file_exists(vm_key):
    return os.path.exists(get_flag_file_path(vm_key))

def dict_compare(d1, d2, excluded_keys):
    for k in set(d1.keys()).union(d2.keys()):
        if k not in excluded_keys:
            if d1.get(k) != d2.get(k):
                return False
    return True

def watch_vm_creation(k8s_token):
    watch_url = 'https://127.0.0.1:6443/apis/kubevirt.io/v1/virtualmachines?watch=true'
    watch_headers = {'Authorization': f'Bearer {k8s_token}'}
    r = requests.get(watch_url, headers=watch_headers, stream=True, verify=False)
    r.raise_for_status()  # raise exception if status code is not 200

    for line in r.iter_lines():
        if line:
            event = json.loads(line)
            if 'object' in event and 'status' in event['object']:
                vm = event['object']
                vm_name = vm['metadata']['name']
                namespace = vm['metadata']['namespace']
                vm_key = (namespace, vm_name)  # unique key for this VM

                if event['type'] == 'DELETED':
                    delete_flag_file(vm_key)

                elif (event['type'] in ['ADDED', 'MODIFIED'] and 
                    event['object']['status'].get('printableStatus') == 'Running' and 
                    str(event['object']['status'].get('ready')) == "True"):
    
                    stateChangeRequests = event['object']['status'].get('stateChangeRequests', [])
                    if any(req.get('action') in ['Stop', 'Start'] for req in stateChangeRequests):
                        continue

                    flag_file_path = get_flag_file_path(vm_key)
                    if not flag_file_exists(vm_key):
                        try:
                            create_flag_file(vm_key)
                            run_second_script(namespace, vm_name)
                        except Exception as e:
                            logging.error(f"An error occurred while running the second script: {e}")
                            if flag_file_exists(vm_key):
                                delete_flag_file(vm_key)
                        if not 'vnf-onboot-script-once' in vm['metadata'].get('annotations', {}):
                            delete_flag_file(vm_key)

def main():
    if not os.path.exists(cbox_file) and not os.path.exists(central_controller):
        logging.info("Only available for CBOX or Central Controler")
        exit(0)

    retry_count = 0

    while retry_count < MAX_RETRIES:
        try:
            token = fetch_token()
            k8s_token = fetch_k8s_token(token)
            watch_vm_creation(k8s_token)
            break  # If the commands are successful, break the loop
        except Exception as e:
            retry_count += 1
            if retry_count < MAX_RETRIES:
                logging.info(f"Retrying in {retry_count * 10} seconds... An error occurred: {e}.")
                time.sleep(retry_count * 10)  # Wait for 10, 20, 30, 40 seconds etc. before retrying
            else:
                logging.exception(f"An error occurred: {e}. Exiting after {max_retries} attempts.")

if __name__ == '__main__':
    main()
