#! /usr/bin/env python3
from __future__ import print_function
import sys,os
import subprocess
import time

class Image(object):
  def __init__(self, index, uid, buildId, isCurrent=False):
    self.index = index
    self.uid   = uid
    self.buildId = buildId
    self.isCurrent = isCurrent

  @classmethod
  def fromString(cls, str):
    pass
    #return cls()

  def __str__(self):
    return \
    "%s\n%s\n%s\n%s\n"%(str(self.index), self.uid, self.buildId, self.isCurrent)

  def prettyString(self):
    return \
    "Unique ID: %s\nBuild ID:%s\n%s"%(self.uid, self.buildId, ("<CURRENT IMAGE>\n" if self.isCurrent else ""))

  @staticmethod
  def doModemPriImagesMatch(modemImg, priImg):
    if modemImg.buildId.endswith("?"):
      return priImg.buildId.startswith(modemImg.buildId[:-1])
    else:
      return priImg.buildId.startswith(modemImg.buildId)

#

class ModemImage(Image):
  def matchesPriImage(self, priImg):
    return super(ModemImage, ModemImage).doModemPriImagesMatch(self, priImg)
  def prettyString(self):
    return \
    "modem%d\nUnique ID: %s\nBuild ID:%s\n%s"%(self.index, self.uid, self.buildId, ("<CURRENT IMAGE>\n" if self.isCurrent else ""))

class PriImage(Image):
  def matchesModemImage(self, modemImg):
    return super(PriImage, PriImage).doModemPriImagesMatch(modemImg, self)
  def prettyString(self):
    return \
    "pri%d\nUnique ID: %s\nBuild ID:%s\n%s"%(self.index, self.uid, self.buildId, ("<CURRENT IMAGE>\n" if self.isCurrent else ""))

#

def tryCommand(cmd, errmsg):
  for i in range(5):
    try:
      out = subprocess.check_output(cmd, stderr = subprocess.STDOUT).decode()
      return True, out
    except subprocess.CalledProcessError as e:
      if i == 4:
        print(errmsg)
        print(e.output)
        return False, e.output
      else:
        time.sleep(0.5)
  return False, ""

#

def tryAddImage(index, uid, bid, current, imgType, lModemImages, lPriImages):
  if index != -1 and uid != "" and bid != "" and imgType != "":
    if imgType == "modem":
      lModemImages.append( ModemImage(index, uid, bid, current) )
      return True
    if imgType == "pri":
      lPriImages.append( PriImage(index, uid, bid, current) )
      return True
  return False

def getImageLists(modem):
  lModemImages = []
  lPriImages = []
  index = -1
  uid = ""
  bid = ""
  current = False
  imgType = ""
  isModemCurrentSet = 0
  isPriCurrentSet = 0

  success, out = tryCommand(['/usr/bin/qmicli','-d', modem, '--dms-list-stored-images'], "Error: Can't get image list for modem " + modem)
  if not success:
    sys.exit(1)

  for line in out.splitlines():
    # skip meaningless lines
    if line.isspace() or \
       "Maximum: '" in line or\
       line == "":
      continue

    if "[0] Type:    'modem'" in line:
      imgType = "modem"
      continue
    elif "[1] Type:    'pri'" in line:
      if tryAddImage(index, uid, bid, current, imgType, lModemImages, lPriImages):
        index = -1
        uid = ""
        bid = ""
        current = False

      imgType = "pri"
      #print ("set imgType to pri")
      continue

    if "[modem" in line or "[pri" in line:
      if tryAddImage(index, uid, bid, current, imgType, lModemImages, lPriImages):
        index = -1
        uid = ""
        bid = ""
        current = False

      i = line.find("[modem") + 6
      if i == 5:
        i = line.find("[pri") + 4
        if i == 3:
          print("Error: Can't parse image index!")
          sys.exit(1)

      digits = 1
      while line[i + digits] != ']':
        digits += 1
      index = int(line[i:i+digits])
      #print ("set index to " + str(index))
      continue

    if "Unique ID" in line:
      start = line.find("'")
      end = line[start+1:].find("'")

      if start == -1 or end == -1:
        print("Error: Can't parse image UID")
        sys.exit(1)

      uid = line[start+1:][:end]
      #print ("set uid to " + uid)
      continue

    if "Build ID" in line:
      start = line.find("'")
      end = line[start+1:].find("'")

      if start == -1 or end == -1:
        print("Error: Can't parse image UID")
        sys.exit(1)

      bid = line[start+1:][:end]
      #print ("set bid to " + bid)
      continue

    if ">>>>>>>>>> [CURRENT] <<<<<<<<<<" in line:
      if imgType == "modem":
        current = True
        isModemCurrentSet = 1
      
      if tryAddImage(index, uid, bid, current, imgType, lModemImages, lPriImages):
        index = -1
        uid = ""
        bid = ""
        current = False

      if imgType == "pri":
        current = True
        isPriCurrentSet = 1
  # for end
  tryAddImage(index, uid, bid, current, imgType, lModemImages, lPriImages)

  # if the last image shown is current, it is not shown (qmicli bug)
  if not isPriCurrentSet:
    lPriImages[-1].isCurrent = True
  if not isModemCurrentSet:
    lModemImages[-1].isCurrent = True

  return (lModemImages, lPriImages)

#

def tryChangeImages(modemImage, priImage):
  if not tryCommand(['/usr/bin/qmicli','-d', modem, "--dms-select-stored-image=modem%s,pri%s"%(modemImage.index,priImage.index)], "Error: Can't configure images!")[0]:
    return False

  print("Configured successfully! Resetting modem...")

  if not tryCommand(['/usr/bin/qmicli','-d', modem, "--dms-set-operating-mode=offline"], "Error: Can't reset modem!")[0] or \
          not tryCommand(['/usr/bin/qmicli','-d', modem, "--dms-set-operating-mode=reset"], "Error: Can't reset modem!")[0]:
    return False

  return True

#

def menu(lModemImages, lPriImages):
  extraMessage = ""

  while(1):
    i = 1
    print() #os.system('clear')

    # 1. Modem
    print("Choose Modem Image:")
    for m in lModemImages:
      print( "%d. %s"%( i, m.prettyString() ) )
      i += 1
    print("0. Abort\n")

    if extraMessage:
      print(extraMessage)
      extraMessage = ""

    iModem = raw_input("Choice: ")

    try:
      iModem = int(iModem) -1
    except ValueError as e:
      extraMessage = "Please, choose an image from 1 to %d\n"%len(lModemImages)
      continue
    if iModem == -1:
      sys.exit(0)
    if iModem < 0 or iModem > len(lModemImages) - 1:
      extraMessage = "Please, choose an image from 1 to %d\n"%len(lModemImages)
      continue

    lValidPris = []
    for pri in lPriImages:
      if lModemImages[iModem].matchesPriImage(pri):
        lValidPris.append(pri)
    if len(lValidPris) == 0:
      extraMessage = "No matching PRI images found for modem image%d!\n"%iModem+1
      continue

    # 2. PRI
    while(1):
      i = 1
      print() #os.system('clear')

      print("Choose PRI image:")
      for p in lValidPris:
        print( "%d. %s"%( i, p.prettyString() ) )
        i += 1
      print("0. Abort\n")

      if extraMessage:
        print(extraMessage)
        extraMessage = ""

      iPri = raw_input("Choice: ")

      try:
        iPri = int(iPri) -1
      except ValueError as e:
        extraMessage = "Choose an image from 1 to %d\n"%len(lValidPris)
        continue
      if iPri == -1:
        sys.exit(0)
      if iPri < 0 or iPri > len(lValidPris) - 1:
        extraMessage = "Choose an image from 1 to %d\n"%len(lValidPris)
        continue
      break # pri choice

    if lModemImages[iModem].isCurrent and lValidPris[iPri].isCurrent:
      cont = raw_input("The chosen images are currently in use. Force configuration?[y/n] ")
      if cont != "y":
        sys.exit(0)

    if tryChangeImages(lModemImages[iModem], lValidPris[iPri]):
      print("Configuration Applied!")
      print("Reboot is required to obtain a connection.")
    else:
      print("Configuration NOT Applied!")
      sys.exit(1)
    break # modem choice

#

net_cfg_dir="/etc/network/interfaces.d"

SCRIPTS_DIR='/scripts'

apn_dbase='/scripts/apn/apns-conf.xml'
apn_mcc_file='/scripts/apn/apncountrycode.txt'
modem_default='/dev/cdc-wdm0'
modem_tty_default='/dev/ttyUSB2'

#

if __name__ == "__main__":
  modem = modem_default

  if not os.path.exists(modem):
    print("Modem %s Not Found!"%modem)
    sys.exit(1)

  lModemImages, lPriImages = getImageLists(modem)
  
  if not lModemImages:
    print("ERROR: No Modem Images Found!")
    sys.exit(1)
  if not lPriImages:
    print("ERROR: No PRI Images Found!")
    sys.exit(1)

  print("Warning - changing Modem FW will temporarily stop its connection.")

  menu(lModemImages, lPriImages)
  sys.exit(0)
