#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import getopt
from sysrepo_ntp import sysrepo_ntp as api
import subprocess

def set_ntp(server, zone):
    ntp = api.SysrepoNTP(app_name="cli_ntp")

    if server is not None:
        for srv in ntp.get_ntp_servers():
            ntp.unset_ntp_server(srv)

        ntp.set_ntp_server(server)

    zones=subprocess.check_output(['timedatectl', 'list-timezones']).decode("utf-8")
    all_zones=zones.split("\n")    
    if zone is not None:
        if zone in all_zones:
            ntp.set_timezone(zone)
        else:
            cmd='echo "{}" > /tmp/zones'.format(zones)
            subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)

            print("An error has occurred during validation of time zone.")
            #print("Please use one of the zones in /tmp/zones")

    try:
        ntp.commit()
    except Exception as e:
        print("{}".format(e))
        sys.exit(1)

def main(argv):
    zone=None
    server=None

    try:
        opts, args=getopt.getopt(argv,"s:z:")
    except getopt.GetoptError:
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-s':
            server=arg
        elif opt == '-z':
            zone=arg
        else:
            sys.exit(2)

    set_ntp(server, zone)

    sys.exit(0)

if __name__ == "__main__":
    main(sys.argv[1:])
