#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import getopt
from sysrepo_dhcprelay import sysrepo_dhcprelay as api


def apply_dhcp_relay_sysrepo( dhcp_server, enable=None, if_names=None , link_if=None):
    dhcprelay = api.SysrepoDhcpRelay(app_name="set dhcp relay")

    if enable is not None:
        dhcprelay.set_state(enable)
    if dhcp_server is not None:
        dhcprelay.set_relay_server(dhcp_server)
    else:
        dhcprelay.unset_relay_server()
    if if_names is not None:
        dhcprelay.set_relay_int(if_names)
    else:
        dhcprelay.unset_relay_int()
    if link_if is not None:
        dhcprelay.set_relay_link_int(link_if)
    else:
        dhcprelay.unset_relay_link_int()

    dhcprelay.commit()


def print_help( name ):
    print("Params:")
    print(" -e --enable <1,0> \t Enable/Disable DHCP relay")
    print(" -r --relay-if <,NAME> \t List of ETH interfaces")
    print(" -s --server <,IPv4 prefix> \t IPv4 address of DHCP server")
    print(" -l --link <NAME> \t Link interface")
    print(" -h \t This help")

def main( name, argv ):
    enable = None
    if_names = None
    dhcp_server = None
    link_if = None

    try:
        opts, args = getopt.getopt(argv,"e:r:s:l:h",["enable=", "relay_if=", "server=", "link="])
    except getopt.GetoptError:
        print_help( name )
        sys.exit(2)

    for opt, arg in opts:
        if opt == '-h':
            print_help( name )
            sys.exit(0)
        elif opt in ("-e", "--enable"):
            if arg:
                if '1' == arg:
                    enable=True
                else:
                    enable=False
        elif opt in ("-r", "--relay_if"):
            if arg:
                #if_names = arg.split( ',' )
                if_names = arg
        elif opt in ("-s", "--server"):
            if arg:
                #dhcp_server = arg.split( ',' )
                dhcp_server = arg
        elif opt in ("-l", "--link"):
            if arg:
                link_if = arg

    if enable == True:
        mgmt_ifs = ['data', 'mgmt']
        not_ifs = ['tun', 'vti0', 'nat', 'lte']

        if not dhcp_server:
            print("%% DHCP Server must be provided !!!")
            return

        if if_names:
            if any(x in if_names for x in not_ifs):
                print("%% Only data or mgmt interfaces are supported !!!")
                return
            if not any(x in if_names for x in mgmt_ifs):
                print("%% Only data or mgmt interfaces are supported !!!")
                return

    apply_dhcp_relay_sysrepo( dhcp_server, enable=enable, if_names=if_names, link_if=link_if )

if __name__ == '__main__':
    main(sys.argv[0], sys.argv[1:])

