#!/bin/bash
# -*- coding: utf-8 -*-
source /scripts/ipv6_functions

TVE_IP=${*: -3:1}
_prepareForIPv6 ${TVE_IP} TVE_IP
SCRIPTS_PATH=${*: -2:1}

empty_string='""'

USER=$(whoami)
GROUP=$(groups $USER | cut -d' ' -f3)

source ${SCRIPTS_PATH}/common-commands.sh
source /scripts/basic_vars
source /scripts/basic_functions

function set_dhcp_server() {
  if [ "${GROUP}" != "admin" ] && [ "${GROUP}" != "operator" ]; then
    echo "You are not authorized to change Ethernet NIC configuration"
    exit 1
  fi

  IFS='#' tokens=( $1 )
  local new_enable=${tokens[0]}
  local new_addr_range=${tokens[1]}
  local new_bind_int=${tokens[2]}
  local new_router=${tokens[3]}
  local new_dns=${tokens[4]}
  local new_dlt=${tokens[5]}
  local new_mlt=${tokens[6]}

  if [ ! -z $new_enable ] && [ "$new_enable" == "0" ] ; then
    /scripts/apply_dhcp_server_sysrepo.py -e "$new_enable"
    exit 0
  fi

  if [ -z $new_addr_range ] || [ -z $new_bind_int ] ; then
    echo "Syntax error: The command is not completed"
    exit 0
  fi

  local f1=$(mktemp)
  $(sysrepocfg --datastore=running --format=json --xpath="/ucpe:config/interfaces:interface[name='${new_bind_int}']//*" --export> ${f1})
  grep -i ${new_bind_int} ${f1} 1> /dev/null
  if [ 1 == $? ]; then
    echo "Interface ${new_bind_int} doesn't exits on this system !"
    rm -f ${f1}
    exit 1
  fi
  rm -f ${f1}

  interface_ip6=$(sysrepocfg --datastore=running --format=json --xpath="/ucpe:config/interfaces:interface[name='${new_bind_int}']/ipv6-prefix" --export | grep ipv6-prefix | cut -d' ' -f10 )
  if [ -z "$interface_ip6" ]; then
    interface_ip4=$(sysrepocfg --datastore=running --format=json --xpath="/ucpe:config/interfaces:interface[name='${new_bind_int}']/ipv4-prefix" --export | grep ipv4-prefix | cut -d' ' -f10 )
    if [ -z "$interface_ip4" ]; then
      echo "Interface $new_bind_int does not have a valid ip address !"
      exit 1
    else
      interface_ip=${interface_ip4}
    fi
  else
    interface_ip=${interface_ip6}
  fi

  if [[ "$new_addr_range" == *":"* ]]; then
    ip_range_ver=6
  else
    ip_range_ver=4
  fi

  if [[ "$interface_ip" == *":"* ]]; then
    ip_int_ver=6
  else
    ip_int_ver=4
  fi

  if [ $ip_range_ver != $ip_int_ver ]; then
    echo "It is not allowed to configure dhcp6 server without valid IPv6 address on $new_bind_int interface !"
    exit 1
  fi

  enable_func="bool"
  if [ ! ${new_enable} ]; then
    new_enable=${empty_string}
    enable_func="null"
  fi

  addr_range_func="none"
  if [ ! ${new_addr_range} ]; then
    new_addr_range=${empty_string}
    addr_range_func="null"
  fi

  bind_int_func="none"
  if [ ! ${new_bind_int} ]; then
    new_bind_int=${empty_string}
    bind_int_func="null"
  fi

  if [ ! -z $new_enable ] && [ ! -z $new_addr_range ] && [ ! -z $new_bind_int ] ; then
    /scripts/apply_dhcp_server_sysrepo.py -e "$new_enable" -r "$new_addr_range" -i "$new_bind_int" -g "$new_router" -d "$new_dns" -l "$new_dlt" -m "$new_mlt"
  fi

  exit 0
}

set_dhcp_server $1

exit 0
