#!/bin/bash
source /scripts/ipv6_functions

TVE_IP=${*: -3:1}
_prepareForIPv6 ${TVE_IP} TVE_IP
SCRIPTS_PATH=${*: -2:1}
PPKLISH_PID=${!#}

source /scripts/basic_vars

SERIAL=$(sudo cat /config/.serial-number)
#echo SERIAL
### LOCAL VARS
ftp_cmd_file=".ftp_import_config"
ftp_cmd_log_file=".ftp_import_config.log"
export_config_file="ucpe_config_export_${SERIAL}.json"
function export_via_ftp(){

    cat > $working_dir/$ftp_cmd_file << EOF
quote USER $1
quote PASS $2
verbose
binary
cd $4
put $5 $6
bye
EOF

    ### PUT CONFIG
    ftp -n $3 < $working_dir/$ftp_cmd_file > $working_dir/$ftp_cmd_log_file
}

function export_via_sftp(){

    export SSHPASS=$2
    sshpass -e sftp -oBatchMode=no -oStrictHostKeyChecking=no -oCheckHostIP=no -b - $1@$3 <<!
cd $4
put $5
bye
!
}

### GO GO

#Get config from sysrepo
sysrepocfg -f json --xpath="/ucpe:config" --export=${working_dir}/${export_config_file}1
sysrepocfg -f json --xpath="/ietf-netconf-server:netconf-server" -X | jq "del(.[][][][]?.ssh.\"ssh-server-parameters\")" > ${working_dir}/${export_config_file}2
if [ ! -f  $working_dir/${export_config_file}1 ] || [ ! -s  $working_dir/${export_config_file}1 ]; then
  echo "Export configuration failed !"
  rm -rf ${working_dir}/${export_config_file}1 ${working_dir}/${export_config_file}2
  exit 1
fi
cat ${working_dir}/${export_config_file}1 ${working_dir}/${export_config_file}2 | jq -s '.[0] * .[1]' > ${working_dir}/${export_config_file}
rm -rf ${working_dir}/${export_config_file}1 ${working_dir}/${export_config_file}2

IFS='#' tokens=( $1 )

protocol=${tokens[0]}
directory=${tokens[1]}
ipaddr=${tokens[2]}
username=${tokens[3]}
# pass=${tokens[4]}
pass="${tokens[4]//\\/}" # NFV-14324 --> remove backlash from password string
path=$working_dir/$export_config_file
file=$(basename $path)

empty_string='""'
if [ -z $pass ]; then
    pass=$empty_string
fi

if [ "$protocol" == "ftp" ]; then
    export_via_ftp $username $pass $ipaddr $directory $path $file
fi

if [ "$protocol" == "sftp" ]; then
    if [[ "$ipaddr" == *":"* ]]; then
        export_via_sftp $username $pass [$ipaddr] $directory $path
    else
        export_via_sftp $username $pass $ipaddr $directory $path
    fi
fi

if [ -f $working_dir/$export_config_file ]; then
  rm $working_dir/$export_config_file
fi
