#!/bin/bash
source /scripts/ipv6_functions

TVE_IP=${*: -3:1}
_prepareForIPv6 ${TVE_IP} TVE_IP
SCRIPTS_PATH=${*: -2:1}
PPKLISH_PID=${!#}

source /scripts/basic_vars

### LOCAL VARS
ftp_cmd_file=".ftp_import_config"
ftp_cmd_log_file=".ftp_import_config.log"

function apply_ucpe {
  cat >> /tmp/fix_import_ucpe.py << EOF
#!/usr/bin/env python3
import json

cfg_file = open('$1', 'r')
config = json.loads(cfg_file.read())
cfg_file.close()
if 'ucpe:config' not in config:
    exit(0)
data = {'ucpe:config':{}}
data['ucpe:config']=config.pop('ucpe:config')

data=json.dumps(data,indent=4,sort_keys=True)
cfg_file = open('/tmp/ucpe.json', 'w+')
cfg_file.write(data)
cfg_file.close()
EOF
  sudo chmod +x /tmp/fix_import_ucpe.py
  sudo /tmp/fix_import_ucpe.py
  sudo rm -rf /tmp/fix_import_ucpe.py

  if [ -f /tmp/ucpe.json ]; then
    sudo sysrepocfg --datastore=startup --format=json --import=/tmp/ucpe.json -m ucpe
    if [ $? -eq 0 ]; then
      sudo touch /tmp/ucpe_ok
    fi
    sudo rm -rf /tmp/ucpe.json
  fi
} # function netconf_config

function apply_netconf {
  cat >> /tmp/fix_import_netconf.py << EOF
#!/usr/bin/env python3
import json

cfg_file = open('$1', 'r')
config = json.loads(cfg_file.read())
cfg_file.close()
if 'ietf-netconf-server:netconf-server' not in config:
    exit(0)
data = {'ietf-netconf-server:netconf-server':{}}
data['ietf-netconf-server:netconf-server']=config.pop('ietf-netconf-server:netconf-server')

if 'listen' in data['ietf-netconf-server:netconf-server'] and \
   'endpoint' in data['ietf-netconf-server:netconf-server']['listen']:
    for i in data['ietf-netconf-server:netconf-server']['listen']['endpoint']:
        if i['ssh']:
            i['ssh'].update({'ssh-server-parameters':{'server-identity':{'host-key':[{'name':'default-key','public-key':{'keystore-reference':'genkey'}}]},'client-authentication':{'supported-authentication-methods':{'publickey':[None],'passsword':[None]}}}})

data=json.dumps(data,indent=4,sort_keys=True)
cfg_file = open('/tmp/netconf.json', 'w+')
cfg_file.write(data)
cfg_file.close()
EOF
  sudo chmod +x /tmp/fix_import_netconf.py
  sudo /tmp/fix_import_netconf.py
  sudo rm -rf /tmp/fix_import_netconf.py

  if [ -f /tmp/netconf.json ]; then
    sudo sysrepocfg --datastore=startup --format=json --import=/tmp/netconf.json -m ietf-netconf-server
    if [ $? -eq 0 ]; then
      sudo touch /tmp/netconf_ok
    fi
    sudo rm -rf /tmp/netconf.json
  fi
}

function apply_config () {
	apply_ucpe $1
	apply_netconf $1
    if [[ -f /tmp/ucpe_ok && -f /tmp/netconf_ok ]]; then
        sudo rm -rf /tmp/ucpe_ok /tmp/netconf_ok
        sudo reboot -f
    else
        sudo rm -rf /tmp/ucpe_ok /tmp/netconf_ok
    fi
}

function download_via_ftp () {

    cat > $working_dir/$ftp_cmd_file << EOF
quote USER $1
quote PASS $2
verbose
binary
cd $4
get $5
bye
EOF

    ### GET THE NEW CONFIG
    ftp -n $3 < $working_dir/$ftp_cmd_file > $working_dir/$ftp_cmd_log_file
    ###APPLY NEW CONFIG FILE
    if [ $? -eq 0 ]; then 
        apply_config ${5}
    fi
}

function download_via_sftp () {

    export SSHPASS=$2
    sshpass -e sftp -oBatchMode=no -oStrictHostKeyChecking=no -oCheckHostIP=no -b - $1@$3 <<!
cd $4
get $5
bye
!
    ###APPLY NEW CONFIG FILE
    if [ $? -eq 0 ]; then 
        apply_config ${5}
    fi 
}

### GO GO
IFS='#' tokens=( $1 )

protocol=${tokens[0]}
path=${tokens[1]}
ipaddr=${tokens[2]}
username=${tokens[3]}
pass=${tokens[4]}
directory=${path%/*}
filename=$(basename $path)

empty_string='""'
if [ -z $pass ]; then
    pass=$empty_string
fi

if [ "$protocol" == "ftp" ]; then
    download_via_ftp $username $pass $ipaddr $directory $filename
fi

if [ "$protocol" == "sftp" ]; then
    if [[ "$ipaddr" == *":"* ]]; then
        download_via_sftp $username $pass [$ipaddr] $directory $filename
    else
        download_via_sftp $username $pass $ipaddr $directory $filename
    fi
fi
