#!/bin/bash
# -*- coding: utf-8 -*-
# coding: utf-8
# vim: tabstop=2 expandtab shiftwidth=2 softtabstop=2
# vim: set fileencoding=utf-8
set -o nounset
set -o pipefail
set -o errexit
#set -o xtrace
#
#exec 1>> >(systemd-cat -p info -t license_handle) 2>&1
#
LICENSE_PACK_NAME="ucpe_license.lic"
CPU_CAPACITY_FILE=/config/license/cpu
DOWNLOADED_LICENSE=/home/admin/user_files/${LICENSE_PACK_NAME}
VERIFIED_LICENSE=/config/license/${LICENSE_PACK_NAME}
K8S_ENABLED_FILE=/config/k8s-enabled
K8S_DISABLED_FILE=/config/k8s-disabled
NATIVE_ENABLED_FILE=/config/native-enabled
NATIVE_DISABLED_FILE=/config/native-disabled
CPU_PRESENT=/sys/devices/system/cpu/present
K8S_SINGLE_MACHINE=/config/k8s-single-machine

action=""
serial=""
cpu=""
features=""
k8s_enabled_by_license=0
native_app_enabled_by_license=0

args=$#
action=$1
  case $action in
    "renew" )
        serial=$2
        ;;
    "register"|"register_v2"|"upgrade" )
        serial=$2
        cpu=$3
        features=$4
        ;;
    "unregister" )
        ;;
  esac

#echo $action $serial $cpu $features

if [[ "$features" == *"k8s"* ]]; then
    k8s_enabled_by_license=1
fi

if [[ "$features" == *"native_app"* ]]; then
    native_app_enabled_by_license=1
fi

#
[ ! -d /config/license ] && sudo mkdir /config/license; sudo chmod 777 /config/license || true
#

function k8s_enable_function() {
  sudo systemctl --no-block enable --now kubelet.service 2>/dev/null || true
  sudo systemctl --no-block enable --now containerd.service 2>/dev/null || true
  sudo systemctl --no-block enable --now nfvtime-kubernetes-images.service 2>/dev/null || true
  sudo systemctl --no-block enable --now nfvtime-kubernetes-init.service 2>/dev/null || true
  sudo systemctl --no-block enable --now sr-kubernetes.service 2>/dev/null || true
  if [ -f ${K8S_SINGLE_MACHINE} ]; then
     sudo /bin/rm -f ${K8S_SINGLE_MACHINE} >/dev/null || true
     sudo /usr/bin/touch ${K8S_SINGLE_MACHINE} || true
  fi
  #reload
}

function k8s_disable_function() { 
  sudo systemctl --no-block disable kubelet.service 2>/dev/null || true
  sudo systemctl --no-block disable containerd.service 2>/dev/null || true
  sudo systemctl --no-block disable nfvtime-kubernetes-images.service 2>/dev/null || true
  sudo systemctl --no-block disable nfvtime-kubernetes-init.service 2>/dev/null || true
  sudo systemctl --no-block disable kube-proxy.service 2>/dev/null || true  
  sudo python3 /scripts/sr-kubernetes-commands.py k8sdisabled
  sudo systemctl --no-block disable sr-kubernetes.service 2>/dev/null || true
  #reload
}


function native_enable_function() {
  return
}

function native_disable_function() {
  sudo bash -c 'cd /root/apps/ && for dirname in `ls */ -d | cut -f1 -d'/'`; do sudo systemctl disable $dirname.service; sudo systemctl stop $dirname.service; done' 1>/dev/null 2>/dev/null || true
  return
}

function download_license() {
  if [ ! -f ${DOWNLOADED_LICENSE} ]; then
    echo "Missing file ${DOWNLOADED_LICENSE}. Download again !"
    exit 0
  fi
  sudo chmod 777 ${DOWNLOADED_LICENSE}

  sudo rm -f ${VERIFIED_LICENSE}
  sudo rm -f ${CPU_CAPACITY_FILE}
  sudo mv -f ${DOWNLOADED_LICENSE} ${VERIFIED_LICENSE}
  sudo touch ${CPU_CAPACITY_FILE}
  sudo chmod 666 ${CPU_CAPACITY_FILE} || true
  echo "$cpu" | sudo tee  ${CPU_CAPACITY_FILE} > /dev/null || true
  
  return
} # function download_license
#
function unregister_license() {
  sudo /bin/rm -f ${DOWNLOADED_LICENSE} 1>/dev/null 2>/dev/null
  sudo /bin/rm -f /config/license/serial 1>/dev/null 2>/dev/null
  sudo /bin/rm -f ${VERIFIED_LICENSE} 1>/dev/null 2>/dev/null
  sudo /bin/rm -f ${CPU_CAPACITY_FILE} 1>/dev/null 2>/dev/null
  sudo /usr/bin/python3 /scripts/clear_ucpe_config.py
  if [ -f /opt/bin/collectd_license_unregister.py ]; then
    sudo /usr/bin/python3 /opt/bin/collectd_license_unregister.py 1>/dev/null 2>/dev/null || true
  fi
  
  cpu_present=$(cat ${CPU_PRESENT} | cut -d'-' -f2)  
  #Disable all except first 4 cpus (0-3)  
  sudo touch /config/_cpu/offline/${cpu_present} 1>/dev/null 2>/dev/null || true
      
#  for (( cpu_i=2; cpu_i<=$cpu_present; cpu_i++ )); do
#    sudo echo "0">/sys/devices/system/cpu/cpu${cpu_i}/online
#  done

  # (vstojkov) NFV-18455 Create in tmp unregister_license and check for this file in cluster_reset function.
  # If file is available, don't remove disks from /var/lib/kubestore for next licensing (this is user files
  # and isn't good user experience to delete them).
  sudo touch /tmp/unregister_license
  k8s_disable_function
  native_disable_function
   
  echo "License unregistration successful!"
  sudo touch /run/.reboot_in_progress
  logger -p daemon.warn "Requesting system reload [system license unregister]" || true
  sleep 5s
  sudo mkdir -p /config/_reboot/tve/now
  
  if [ -f ${K8S_SINGLE_MACHINE} ]; then
     sudo /bin/rm -f ${K8S_SINGLE_MACHINE} 1>/dev/null 2>/dev/null || true
     sudo /usr/bin/touch ${K8S_SINGLE_MACHINE} 1>/dev/null 2>/dev/null || true
  fi
  
  return 0
} # function unregister_license
#
function register_license() {
  download_license
  if [ ! -f ${VERIFIED_LICENSE} ]; then
    echo "No license file found! Please download license file."
    exit 1
  fi

  #res=`/usr/bin/python3 /scripts/verify_signature.py ${serial}`
  #if [[ "$res" == *"Valid serial number"* ]]; then
    sudo touch /config/license/serial
    sudo chmod 666 /config/license/serial
    echo "$serial" | sudo tee /config/license/serial> /dev/null || true

    # if license enables k8s
    if [ "$k8s_enabled_by_license" == "1" ]; then
    
      # if k8s_worker is enabled in configuration
      if [ -f ${K8S_ENABLED_FILE} ]; then
         k8s_enable_function
                  
      # if k8s_worker is disabled in configuration
      #elif [ -f ${K8S_DISABLED_FILE} ]; then
          # nothing to do
      fi
    
    # if license disables k8s
    else

      # if k8s_worker is enabled in configuration
      if [ -f ${K8S_ENABLED_FILE} ]; then
         k8s_disable_function
                  
      # if k8s_worker is disabled in configuration
      #elif [ -f ${K8S_DISABLED_FILE} ]; then
          # nothing to do
      fi       
       
    fi    

    # if license enables native applications
    if [ "$native_app_enabled_by_license" == "1" ]; then
    
      # if native app is enabled in configuration
      if [ -f ${NATIVE_ENABLED_FILE} ]; then
         native_enable_function
                  
      # if native app is disabled in configuration
      #elif [ -f ${NATIVE_DISABLED_FILE} ]; then
          # nothing to do
      fi
    
    # if license disables native applications
    else

      # if native app is enabled in configuration
      if [ -f ${NATIVE_ENABLED_FILE} ]; then
         native_disable_function
                  
      # if native app is disabled in configuration
      #elif [ -f ${NATIVE_DISABLED_FILE} ]; then
          # nothing to do
      fi       
       
    fi

  #Enable cpus according to license
  cpu_present=$(cat ${CPU_PRESENT} | cut -d'-' -f2)
  cpu=$(cat ${CPU_CAPACITY_FILE} | xargs || true) 
  for (( cpu_i=0; cpu_i<=$cpu_present; cpu_i++ )); do
        echo '1' | sudo tee /sys/devices/system/cpu/cpu${cpu_i}/online> /dev/null || true
  done
  if [[ "$cpu" != *"unlimited"* ]]; then
    #Enable all cpus according to license cpu value
    cpus=$((cpu - 1))
    for (( cpu_i=$cpu; cpu_i<=$cpu_present; cpu_i++ )); do
        echo "set cpu $cpu_i offline"  || true
        echo '0' | sudo tee /sys/devices/system/cpu/cpu${cpu_i}/online> /dev/null || true
    done
  fi
  
  sudo touch /run/.reboot_in_progress
  logger -p daemon.warn "Requesting system reload [system license register]" || true
  sleep 5s
  sudo mkdir -p /config/_reboot/tve/now

  return
} # function register_license
#
function renew_license() {
  download_license
  if [ ! -f ${VERIFIED_LICENSE} ]; then
    echo "No license file found! Please download license file."
    exit 1
  fi

  # Replace new license serial; no change in enabled functionality
  sudo touch /config/license/serial
  sudo chmod 644 /config/license/serial
  echo "$serial" | sudo tee /config/license/serial> /dev/null || true
  return
}
#
function show_license() {
  if [ -f /config/license/serial ]; then
    echo "License $(cat /config/license/serial) has been registered on device."
  else
    echo "No license has been registered on device!"
  fi
  return
} # function show_license
#
if [ "${action}" == "register" ] || [ "${action}" == "register_v2" ]; then
  register_license
elif [ "${action}" == "unregister" ]; then
  unregister_license
elif [ "${action}" == "show" ]; then
  show_license
elif [ "${action}" == "download" ]; then
  download_license
elif [ "${action}" == "renew" ]; then
  renew_license
elif [ "${action}" = "upgrade" ]; then
  register_license
fi
#
exit 0
