#!/bin/bash
# -*- coding: utf-8 -*-
# vim: tabstop=2 expandtab shiftwidth=2 softtabstop=2
# vim: set fileencoding=utf-8
source /scripts/ipv6_functions

TVE_IP=${*: -3:1}
_prepareForIPv6 ${TVE_IP} TVE_IP
SCRIPTS_PATH=${*: -2:1}

empty_string='""'
vnf_management_if="mgmt0"

source /scripts/token-env.sh

command_header="curl -s -H \"X-Auth-Token:$TOKEN\" -H \"Content-type:application/json\" "
command_path="https://${TVE_IP}:7373"
ssl_insecure=$(cat /etc/tveapi/tveapi.conf | egrep -v "(^#.*)" | grep ssl_no_validate | cut -f2 -d'=' | tr -d ' ')
if [ -z  ${ssl_insecure} ]; then
  ssl_insecure=False
elif [ "true" == "${ssl_insecure}" ] || [ "True" == "${ssl_insecure}"  ]; then
  command_header+="--insecure"
fi


source ${SCRIPTS_PATH}/common-commands.sh
source /scripts/basic_vars
source /scripts/basic_functions

cdr2mask ()
{
  if [ -n "$1" ] ; then
    # Number of args to shift, 255..255, first non-255 byte, zeroes
    set -- $(( 5 - ($1 / 8) )) 255 255 255 255 $(( (255 << (8 - ($1 % 8))) & 255 )) 0 0 0
    [ $1 -gt 1 ] && shift $1 || shift
    echo ${1-0}.${2-0}.${3-0}.${4-0}
  else
    echo \"\"
  fi
}

function get_interface_ip(){
  local interface=$1
  if [ ! -z $interface ] ; then
    local interface_proto=$(sysrepocfg --datastore=running --format=json --xpath="/ucpe:config/interfaces:interface[name='$interface']/bootproto" --export | grep bootproto | cut -d' ' -f10)
    local interface_vlan=$(sysrepocfg --datastore=running --format=json --xpath="/ucpe:config/interfaces:interface[name='$interface']/vlan" --export | grep vlan | cut -d' ' -f10)
    if [ -n "$interface_proto" ] && [ "none" = "$interface_proto" ]; then
      interface_ip=$(sysrepocfg --datastore=running --format=json --xpath="/ucpe:config/interfaces:interface[name='$interface']/ipv4-prefix" --export | grep ipv4-prefix | cut -d' ' -f10 | cut -d'/' -f1)
    else
      if [ -n "$interface_vlan" ]; then
        interface=$interface.$interface_vlan
      fi
      if [ -d "/sys/class/net/br-$interface" ]; then
        interface=br-$interface
      fi
      if [ -d "/sys/class/net/$interface" ]; then
        interface_ip=$(/sbin/ifconfig $interface |  grep "inet " | awk '{print $2}')
      fi
    fi
  fi
}

function set_network_eth() {
  if [ "${GROUP}" != "admin" ] && [ "${GROUP}" != "operator" ]; then
    echo "You are not authorized to change Ethernet NIC configuration"
    exit 1
  fi
  IFS='#' tokens=( $1 )
  local name=${tokens[0]}
  local new_bootproto=${tokens[1]}
  local new_ip=${tokens[2]}
  local new_mode=${tokens[3]}
  local new_management=${tokens[4]}
  local new_vlan=${tokens[5]}
  local new_gateway=${tokens[6]}
  local new_dns=${tokens[7]}
  local new_ipv6=${tokens[8]}
  local new_metric=${tokens[9]}
  local new_trace=${tokens[10]}
  local new_vfs=${tokens[11]}
  local new_linkstateprop=${tokens[12]}
  local new_ippass=${tokens[13]}
  local new_dns6=${tokens[14]}
  local new_gateway6=${tokens[15]}

  vlan_func="int"
  if [ ! ${new_vlan} ]; then
    new_vlan=${empty_string}
    vlan_func="null"
  fi

  bootproto_func="none"
  if [ ! ${new_bootproto} ]; then
    new_bootproto=${empty_string}
    bootproto_func="null"
  fi
  ip_func="none"
  if [ ! ${new_ip} ]; then
    new_ip=${empty_string}
    ip_func="null"
  fi
  mode_func="none"
  if [ ! ${new_mode} ]; then
    new_mode=${empty_string}
    mode_func="null"
  fi
  mgmt_func="bool"
  if [ ! ${new_management} ]; then
    new_management=${empty_string}
    mgmt_func="null"
  fi
  dns_func="none"
  if [ ! ${new_dns} ]; then
    new_dns=${empty_string}
    dns_func="null"
  fi
  gate_func="none"
  if [ ! ${new_gateway} ]; then
    new_gateway=${empty_string}
    gate_func="null"
  fi
  ipv6_func="none"
  if [ ! ${new_ipv6} ]; then
    new_ipv6=${empty_string}
    ipv6_func="null"
  fi
  metric_func="int"
  if [ ! ${new_metric} ]; then
    new_metric=${empty_string}
    metric_func="null"
  fi
  trace_func="none"
  if [ ! ${new_trace} ]; then
    new_trace=${empty_string}
    trace_func="null"
  fi
  vfs_func="int"
  if [ ! ${new_vfs} ]; then
    new_vfs=${empty_string}
    vfs_func="null"
  fi
  linkstateprop_func="bool"
  if [ ! ${new_linkstateprop} ]; then
    new_linkstateprop=${empty_string}
    linkstateprop_func="null"
  fi
  if [ ! ${new_ippass} ]; then
    new_ippass=${empty_string}
  fi
  dns6_func="none"
  if [ ! ${new_dns6} ]; then
    new_dns6=${empty_string}
    dns6_func="null"
  fi
  gate6_func="none"
  if [ ! ${new_gateway6} ]; then
    new_gateway6=${empty_string}
    gate6_func="null"
  fi


  if [ "$name" == "$vnf_management_if" ]; then

    if [ "dhcp" == ${new_bootproto} ]; then
      echo "Command failed! You should not configure dhcp bootproto for this interface!"
      exit 0
    fi

    #new_bootproto=${empty_string}
    new_vlan=${empty_string}
    new_mode=${empty_string}
    new_management=${empty_string}
    new_gateway=${empty_string}
    new_dns=${empty_string}
    new_metric=${empty_string}
    new_trace=${empty_string}
    new_ippass=${new_ippass}
    new_dns6=${empty_string}
    new_gateway6=${empty_string}
  fi


  SAVEIFS=$IFS
  IFS='/' tokens=( $new_ip )
  IFS=$SAVEIFS

  network_mask=$(cdr2mask ${tokens[1]})

  ${SCRIPTS_PATH}/apply_if_sysrepo.py set_eth_int $name $new_bootproto $new_ip $new_mode $new_management $new_vlan $new_gateway $new_dns $new_ipv6 $new_metric $new_trace $new_linkstateprop ${new_ippass} ${new_dns6} ${new_gateway6}
}

function set_network_lte() {
  if [ "${GROUP}" != "admin" ] && [ "${GROUP}" != "operator" ]; then
    echo "You are not authorized to change LTE interface configuration"
    exit 1
  fi
  IFS='#' tokens=( $1 )
  local name=${tokens[0]}
  local new_enable=${tokens[1]}
  local new_apn=${tokens[2]}
  local new_user=${tokens[3]}
  local new_pass=${tokens[4]}
  local new_management=${tokens[5]}
  local new_metric=${tokens[6]}
  local new_trace=${tokens[7]}
  local new_ippass=${tokens[8]}

  enable_func="bool"
  if [ ! ${new_enable} ]; then
    new_enable=${empty_string}
    enable_func="null"
  fi
  management_func="bool"
  if [ ! ${new_management} ]; then
    new_management=${empty_string}
    management_func="null"
  fi
  apn_func="none"
  if [ ! ${new_apn} ]; then
    new_apn=${empty_string}
    apn_func="null"
  fi
  user_func="none"
  if [ ! ${new_user} ]; then
    new_user=${empty_string}
    user_func="null"
  fi
  pass_func="none"
  if [ ! ${new_pass} ]; then
    new_pass=${empty_string}
    pass_func="null"
  fi
  metric_func="int"
  if [ ! ${new_metric} ]; then
    new_metric=${empty_string}
    metric_func="null"
  fi
  trace_func="none"
  if [ ! ${new_trace} ]; then
    new_trace=${empty_string}
    trace_func="null"
  fi
  if [ ! ${new_ippass} ]; then
    new_ippass=${empty_string}
  fi

  #klish_json_set_array "$command_header" "${command_path}/v1/config/networking/interfaces" "name=$name" ${SCRIPTS_PATH} \
  #                     "enable" "$new_enable" "${enable_func}" \
  #                     "apn" $new_apn "${apn_func}" \
  #                     "user" $new_user "${user_func}" \
  #                     "pass" $new_pass "${pass_func}" \
  #                     "management" $new_management "${management_func}" \
  #                     "metric" $new_metric "${metric_func}" \
  #                     "trace-host" "$new_trace" "${trace_func}"
  ${SCRIPTS_PATH}/apply_if_sysrepo.py set_lte_int $name $new_enable $new_apn $new_user $new_pass $new_management $new_metric $new_trace ${new_ippass}
}

function set_network_vpn() {
  if [ "${GROUP}" != "admin" ] && [ "${GROUP}" != "operator" ]; then
    echo "You are not authorized to change VPN interface configuration"
    exit 1
  fi
  IFS='#' tokens=( $1 )
  local name=${tokens[0]}
  local new_enable=${tokens[1]}
  local new_management=${tokens[2]}
  local new_type=${tokens[3]}
  local new_ovpn_endpoint=${tokens[4]}
  local new_ovpn_port=${tokens[5]}

  enable_func="bool"
  if [ ! ${new_enable} ]; then
    new_enable=${empty_string}
    enable_func="null"
  fi
  management_func="bool"
  if [ ! ${new_management} ]; then
    new_management=${empty_string}
    management_func="null"
  fi

  type_func="none"
  if [ ! ${new_type} ]; then
    new_type=${empty_string}
    type_func="null"
  fi

  ovpn_endpoint_func="none"
  if [ ! ${new_ovpn_endpoint} ]; then
    new_ovpn_endpoint=${empty_string}
    ovpn_endpoint_func="null"
  fi

  ovpn_port_func="int"
  if [ ! ${new_ovpn_port} ]; then
    new_ovpn_port=${empty_string}
    ovpn_port_func="null"
  fi

  #${SCRIPTS_PATH}/vpn_if_json_set.py set_ovpn "$TOKEN" $ssl_insecure $TVE_IP $name $new_enable $new_management $new_type $new_ovpn_endpoint $new_ovpn_port
  ${SCRIPTS_PATH}/apply_if_sysrepo.py set_vpn_int $name $new_enable $new_management $new_type $new_ovpn_endpoint $new_ovpn_port

}

function set_network_ipsec() {
  if [ "${GROUP}" != "admin" ] && [ "${GROUP}" != "operator" ]; then
    echo "You are not authorized to change IPsec interface configuration"
    exit 1
  fi
  IFS='#' tokens=( $1 )
  local name=${tokens[0]}
  local new_enable=${tokens[1]}
  local new_management=${tokens[2]}
  local new_remote_gateway=${tokens[3]}
  local new_local_id=${tokens[4]}
  local new_remote_id=${tokens[5]}
  local new_ike_version=${tokens[6]}
  local new_ike_mode=${tokens[7]}
  local new_protocol_mode=${tokens[8]}
  local new_ike_phase_1_rounds=${tokens[9]}
  local new_ike_phase_1_id=${tokens[10]}
  local new_ike_phase_1_encryption=${tokens[11]}
  local new_ike_phase_1_authentication=${tokens[12]}
  local new_ike_phase_1_dh_group=${tokens[13]}
  local new_ike_phase_2_local_ts=${tokens[14]}
  local new_ike_phase_2_remote_ts=${tokens[15]}
  local new_ike_phase_2_id=${tokens[16]}
  local new_ike_phase_2_encryption=${tokens[17]}
  local new_ike_phase_2_authentication=${tokens[18]}
  local new_ike_phase_2_dh_group=${tokens[19]}

  if [ ! ${new_enable} ]; then
    new_enable=${empty_string}
  fi

  if [ ! ${new_management} ]; then
    new_management=${empty_string}
  fi

  if [ ! ${new_remote_gateway} ]; then
    new_remote_gateway=${empty_string}
  fi

  if [ ! ${new_local_id} ]; then
    new_local_id=${empty_string}
  fi

  if [ ! ${new_remote_id} ]; then
    new_remote_id=${empty_string}
  fi

  if [ ! ${new_ike_version} ]; then
    new_ike_version=${empty_string}
  fi

  if [ ! ${new_ike_mode} ]; then
    new_ike_mode=${empty_string}
  fi

  if [ ! ${new_protocol_mode} ]; then
    new_protocol_mode=${empty_string}
  fi

  if [ ! ${new_ike_phase_1_rounds} ]; then
    new_ike_phase_1_rounds=${empty_string}
  fi

  if [ ! ${new_ike_phase_1_id} ]; then
    new_ike_phase_1_id=${empty_string}
  fi

  if [ ! ${new_ike_phase_1_encryption} ]; then
    new_ike_phase_1_encryption=${empty_string}
  fi

  if [ ! ${new_ike_phase_1_authentication} ]; then
    new_ike_phase_1_authentication=${empty_string}
  fi

  if [ ! ${new_ike_phase_1_dh_group} ]; then
    new_ike_phase_1_dh_group=${empty_string}
  fi

  if [ ! ${new_ike_phase_2_local_ts} ]; then
    new_ike_phase_2_local_ts=${empty_string}
  fi

  if [ ! ${new_ike_phase_2_remote_ts} ]; then
    new_ike_phase_2_remote_ts=${empty_string}
  fi

  if [ ! ${new_ike_phase_2_id} ]; then
    new_ike_phase_2_id=${empty_string}
  fi

  if [ ! ${new_ike_phase_2_encryption} ]; then
    new_ike_phase_2_encryption=${empty_string}
  fi

  if [ ! ${new_ike_phase_2_authentication} ]; then
    new_ike_phase_2_authentication=${empty_string}
  fi

  if [ ! ${new_ike_phase_2_dh_group} ]; then
    new_ike_phase_2_dh_group=${empty_string}
  fi

  ${SCRIPTS_PATH}/apply_if_sysrepo.py set_ipsec_int $name $new_enable $new_management $new_remote_gateway \
                 $new_local_id $new_remote_id $new_ike_version $new_ike_mode $new_protocol_mode \
                 $new_ike_phase_1_rounds $new_ike_phase_1_id \
                 $new_ike_phase_1_encryption $new_ike_phase_1_authentication $new_ike_phase_1_dh_group \
                 $new_ike_phase_2_local_ts $new_ike_phase_2_remote_ts $new_ike_phase_2_id \
                 $new_ike_phase_2_encryption $new_ike_phase_2_authentication $new_ike_phase_2_dh_group

}

function set_hostname() {
  IFS='#' tokens=( $1 )
  local new_hostname=${tokens[0]}

  if [ ! -z $new_hostname ] ; then
    vms_count=`ls /run/ | grep "\.vnic" | wc -l`
    if (( "$vms_count" < 1)); then
      /scripts/apply_system_sysrepo.py -h $new_hostname
      if [ "active" = "$(systemctl is-active kubelet.service)" ]; then
        if [ -f "/etc/kubernetes/pki/ca.crt" -a ! -f "/config/k8s-disabled" ]; then
          sudo systemctl restart nfvtime-kubernetes-init.service
        fi
      fi
    else
      echo "Cannot change hostname while VMs are running."
    fi
  fi
}

case "$1" in
  set_network_eth)
    set_network_eth $2
    ;;
  set_network_lte)
    set_network_lte $2
    ;;
  set_network_vpn)
    set_network_vpn $2
    ;;
  set_network_ipsec)
    set_network_ipsec $2
    ;;
  set_hostname)
    set_hostname $2
    ;;
  *)
    echo "Unknown operation"
    exit 1
    ;;
esac

exit 0
