#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <dlfcn.h>
#include <string.h>

/* From quagga-0.99.24.1/lib/command.h - Start */
/* Structure of command element. */
struct cmd_element
{
  const char *string; /* Command specification by string. */
  int (*func) (struct cmd_element *, char * vty, int, const char *[]); // not quite
  const char *doc; /* Documentation of this command. */
  int daemon; /* Daemon to which this command belong. */
  int tokens; /* Vector of cmd_tokens */
  u_char attr; /* Command attributes */
};
/* There are some command levels which called from command node. */
enum node_type
{
  AUTH_NODE, /* Authentication mode of vty interface. */
  RESTRICTED_NODE, /* Restricted view mode */
  VIEW_NODE, /* View node. Default mode of vty interface. */
  AUTH_ENABLE_NODE, /* Authentication mode for change enable. */
  ENABLE_NODE, /* Enable node. */
  CONFIG_NODE, /* Config node. Default mode of config file. */
  SERVICE_NODE, /* Service node. */
  DEBUG_NODE, /* Debug node. */
  AAA_NODE, /* AAA node. */
  KEYCHAIN_NODE, /* Key-chain node. */
  KEYCHAIN_KEY_NODE, /* Key-chain key node. */
  INTERFACE_NODE, /* Interface mode node. */
  ZEBRA_NODE, /* zebra connection node. */
  TABLE_NODE, /* rtm_table selection node. */
  RIP_NODE, /* RIP protocol mode node. */
  RIPNG_NODE, /* RIPng protocol mode node. */
  BABEL_NODE, /* Babel protocol mode node. */
  BGP_NODE, /* BGP protocol mode which includes BGP4+ */
  BGP_VPNV4_NODE, /* BGP MPLS-VPN PE exchange. */
  BGP_IPV4_NODE, /* BGP IPv4 unicast address family. */
  BGP_IPV4M_NODE, /* BGP IPv4 multicast address family. */
  BGP_IPV6_NODE, /* BGP IPv6 address family */
  BGP_IPV6M_NODE, /* BGP IPv6 multicast address family. */
  OSPF_NODE, /* OSPF protocol mode */
  OSPF6_NODE, /* OSPF protocol for IPv6 mode */
  ISIS_NODE, /* ISIS protocol mode */
  PIM_NODE, /* PIM protocol mode */
  MASC_NODE, /* MASC for multicast. */
  IRDP_NODE, /* ICMP Router Discovery Protocol mode. */
  IP_NODE, /* Static ip route node. */
  ACCESS_NODE, /* Access list node. */
  PREFIX_NODE, /* Prefix list node. */
  ACCESS_IPV6_NODE, /* Access list node. */
  PREFIX_IPV6_NODE, /* Prefix list node. */
  AS_LIST_NODE, /* AS list node. */
  COMMUNITY_LIST_NODE, /* Community list node. */
  RMAP_NODE, /* Route map node. */
  SMUX_NODE, /* SNMP configuration node. */
  DUMP_NODE, /* Packet dump node. */
  FORWARDING_NODE, /* IP forwarding node. */
  PROTOCOL_NODE, /* protocol filtering node */
  VTY_NODE /* Vty node. */
};

/* From quagga-0.99.24.1/lib/command.h - End */

typedef void (*original_install_element_f_type)(enum node_type ntype, struct cmd_element *cmd);

void install_element (enum node_type ntype, struct cmd_element *cmd )
{
  if ( VTY_NODE >= ntype && AUTH_NODE <= ntype &&
       NULL != cmd && NULL != cmd->string ) {
    FILE *forbiddenCommandFile = fopen ("/scripts/cli/tvesh/quaggaForbiddenCommands.txt", "r");
    if ( NULL != forbiddenCommandFile ) {
      char *line = NULL, *sline = NULL;
      size_t len = 0;
      ssize_t read = 0;
      char *command = NULL, *replace = NULL, *replace_end = NULL;
      while ( -1 != (read = getline(&line, &len, forbiddenCommandFile))) {
        sline = NULL;
        replace = NULL;
        replace_end = NULL;
        command = strtok_r ( line, "#", &sline);
        if ( NULL == command ) {
          //printf ("command or node is null\n");
          continue;
        }
        replace = strtok_r ( NULL, "#", &sline);
        replace_end = strtok_r ( NULL, "#", &sline);
        //printf ( "command is #%s#%lu# \n", command, strlen(command) );
        //printf ( "cmd->string is #%s#%lu# \n", cmd->string, strlen(cmd->string) );
        if ( 0 == strncmp ( cmd->string, command, strlen ( command ) ) ) {
          //match found
          if(NULL != replace && NULL != replace_end)
          {
           //printf ( "\n\nCommand found %s, Replace with %s \n\n", command, replace);
              cmd->string = strdup(replace);
              break;
          }
          else
          {
            //printf ( "\n\nCommand found %s, Skip \n\n", command);
            free(line);
            fclose(forbiddenCommandFile);
          }
          return;
        }
      }
      free(line);
      fclose(forbiddenCommandFile);
    }
  }
  original_install_element_f_type original_install_element = (original_install_element_f_type) dlsym ( RTLD_NEXT, "install_element" );
  if ( NULL == original_install_element ) {
    fputs (dlerror(), stderr);
    exit ( EXIT_FAILURE );
  }
  return original_install_element ( ntype, cmd );
}
