#!/bin/bash
TVE_IP=$1
SCRIPTS_HOME="/scripts/cli/tvesh"
SESSION_PID=$2

function restoreHP {
  hpcount=`python3 /scripts/get_json_value //huge-page-count /config/ucpe_config.json`
  hpsize=`python3 /scripts/get_json_value //huge-page-size /config/ucpe_config.json`

  if [ -n "$hpcount" -a -n "$hpsize" ]; then
    efibootmgr 2>/dev/null
    if [ "$?" -eq 0 ]; then
      BOOTL_PATH="/boot/efi/EFI/refind/refind.conf"
    else
      BOOTL_PATH="/etc/default/grub"
    fi
    FSTAB_PATH="/etc/fstab"

    sed -i "s/ hugepages=\([0-9]\+\)/\ hugepages=${hpcount}/" $BOOTL_PATH
    sed -i "/.*hugetlbfs.*/c\\hugetlbfs       /dev/hugepages  hugetlbfs       rw,pagesize=${hpsize}MB,mode=1770        0 0" $FSTAB_PATH
    sed -i "s/default_hugepagesz=[^[:space:]]\+ hugepagesz=[^[:space:]]\+/default_hugepagesz=${hpsize}M hugepagesz=${hpsize}M/" $BOOTL_PATH
  fi
}

if [ -e "/config/_sysrepo" ]; then
  if [ -e "/config/ucpe_config.json" ]; then
    cat /config/ucpe_config.json | grep network-mode | grep -q ovs && sudo /usr/bin/python3 /opt/bin/switch_ovs_vpp.py --to ovs
    sudo systemctl stop sr-*
    echo {} | /usr/bin/sysrepocfg --datastore=startup --format=json -I 1>/dev/null 2>&1
    echo {} | /usr/bin/sysrepocfg --format=json -I 1>/dev/null 2>&1
    /usr/bin/sysrepocfg --datastore=startup --format=json --import=/config/ucpe_config.json -m ucpe 1>/dev/null 2>&1
    /usr/bin/sysrepocfg --datastore=startup --format=xml --import=/config/modules/ietf-keystore.xml -m ietf-keystore 1>/dev/null 2>&1
    /usr/bin/sysrepocfg --datastore=startup --format=xml --import=/config/modules/ietf-netconf-server.xml -m ietf-netconf-server 1>/dev/null 2>&1
    /usr/bin/sysrepocfg -C startup
    restoreHP
  else
    echo "Backup file does not exist."
    exit 1
  fi
  sudo systemctl stop incron.service

  if [ -e "/etc/frr/frr.conf" ]; then
    sudo rm -f /etc/frr/frr.conf
  fi

  if [ ! -f "/config/central-box" ]; then
    sudo /scripts/cli/tvesh/license_handle.sh unregister
    sudo /bin/rm -f /config/license/* >/dev/null || true
  fi

  [ -d /root/apps ] && for dirname in `sudo ls /root/apps/*/ -d | cut -f4 -d'/'`; do systemctl disable $dirname.service; done
  sudo systemctl start incron.service

  sudo cp /root/admin-openrc.sh /var/opt/
  sudo cp /root/admin-openrc.env /var/opt/
  sudo sed -i 's/\/root\/admin-openrc.env/\/var\/opt\/admin-openrc.env/' /var/opt/admin-openrc.sh
  source /var/opt/admin-openrc.sh

  if [ -e "/etc/hosts.backup" ]; then
    sudo cp -f /etc/hosts.backup /etc/hosts
  fi
else
  if [ -e "/config/tve_config.json.backup" -a -e "/etc/hosts.backup" ]; then
    sudo rm -f /config/tve_config.json
    sudo rm -f /etc/hosts
  else
    echo "Backup file does not exist."
    exit 1
  fi
  sudo cp /config/tve_config.json.backup /config/tve_config.json
  sudo cp /etc/hosts.backup /etc/hosts
  sudo chmod 777 /config/tve_config.json /etc/hosts
  sudo echo 1 > /config/force_config
fi

if [ ! -f /etc/apache2/conf-enabled/splash.* ]; then
  sudo a2enconf splash > /dev/null 2>&1
fi
if [ -e "/config/added_users" ]; then
  exec ${SCRIPTS_HOME}/security-commands.sh delete_all_users ${TVE_IP} ${SCRIPTS_HOME} ${SESSION_PID}
fi

sudo sed -i "s/^bind-address.*$/bind-address\ =\ 127.0.0.1/" /etc/mysql/mariadb.conf.d/50-server.cnf || true

logger -p daemon.warn "Requesting system reload [system restore configuration command]" || true
if [ -f "/config/central-box" -o -f "/config/k8s-single-machine" ]; then
  rm -rf /root/.kube
  truncate -s 0 /kubernetes/calico-ready
  sudo kubeadm reset --cert-dir="/etc/kubernetes/pki" --ignore-preflight-errors=all --force > /dev/null 2>&1
  sudo rm -rf /etc/cni/ /etc/kubernetes/ /root/.kube/ /home/tveapi/.kube/ /var/lib/kubestore/* /var/lib/etcd/* > /dev/null 2>&1
  sudo rm -f /config/k8s-single-machine
  sudo systemctl enable nfvtime-kubernetes-init.service 1>/dev/null 2>&1
  sudo docker container stop registry 1>/dev/null 2>&1
  sudo docker container rm -v registry 1>/dev/null 2>&1
  sudo rm -rf /var/lib/docker-registry/
  if [ -f "/etc/systemd/system/kubelet.service.d/10-kubeadm.conf" ]; then
    flags=$(grep -m1 "Environment" /etc/systemd/system/kubelet.service.d/10-kubeadm.conf | sed 's/.$//' | sed -n 's/Environment="KUBELET_KUBECONFIG_ARGS//p')
    flags_without_ip=$(echo $flags | sed "s/ --node-ip=.*$//g")
    sed -i "s|$flags|$flags_without_ip|g" /etc/systemd/system/kubelet.service.d/10-kubeadm.conf
  fi
else
  sudo /scripts/sr-kubernetes-commands.py reset
fi
rm -rf /config/_reboot/tve/*
sudo mkdir -p /config/_reboot/tve/now
echo "Success"
exit 0
