#!/bin/bash
#
# show LTE info

modem_default='/dev/cdc-wdm0'
qmi_default='/dev/cdc-wdm0'
mbim_default='/dev/cdc-wdm_pcie'
#set -o errexit

### SOURCE SAME BASIC VARS NEEDED
source /scripts/basic_vars

### SOURCE SAME BASIC FUNCTIONS NEEDED
source /scripts/basic_functions

IFACE=$1
OPTION=$2
SCRIPTS_PATH=$3
DETAILS=$4
EXTERNAL_LTE=0

port_separator="-------------------------------------------------------------------------------"

function is_network_connected {
  retcode=$(qmicli -d ${MODEM} --wds-get-packet-service-status | grep disconnected)
  if [[ -z "$retcode" ]]; then
    return 1
  fi
  return 0
}

function show_network_settings {
  if [[ "$EXTERNAL_LTE" -eq "0" ]]; then
    if [[ "${MODMODE}" == "2" ]]; then
       mbimcli -d ${MODEM}  -p  --no-close --query-ip-configuration  2>/dev/null | grep -v TRID |  grep -v  "Warning" | grep -v /"dev/"
    else
       is_network_connected
       if [[ $? == 1 ]]; then
          qmicli -d ${MODEM} --wds-get-current-settings 2>/dev/null
        else
          echo "Interface is disconnected."
       fi
    fi
  fi
}

function show_signal_strength {
  if [[ "$EXTERNAL_LTE" -eq "0" ]]; then
    if [[ "${MODMODE}" == "2" ]]; then
       mbimcli -d ${MODEM}  -p  --no-close --query-signal-state  2>/dev/null | grep -v TRID |  grep -v  "Warning" | grep -v /"dev/"
    else
       rsrp=$(qmicli -d ${MODEM} --nas-get-signal-info 2>/dev/null | grep RSRP | head -1 |  cut -d '-' -f2 | cut -d ' ' -f1)

       asu=$((rsrp+144))

       if [[ ${DETAILS} != "details" ]]; then
          echo "RSRP: -$rsrp dBm"
          echo "ASU:  $asu"
       else
          qmicli -d ${MODEM} --nas-get-signal-strength
          echo "ASU:    $asu"
       fi
    fi
 fi
}

function show_module_info {
  if [[ "$EXTERNAL_LTE" -eq "0" ]]; then
    if [[ "${MODMODE}" == "2" ]]; then 
       mbimcli -d ${MODEM}  -p  --no-close --query-device-caps 2>/dev/null | grep -v TRID | grep -v "/dev/" | grep -v  "Warning"
    else 
       qmicli -d ${MODEM} --dms-get-manufacturer 2>/dev/null  | grep Manufacturer: | awk '{$1=$1};1'
       qmicli -d ${MODEM} --dms-get-model 2>/dev/null         | grep Model:        | awk '{$1=$1};1'
       qmicli -d ${MODEM} --dms-get-ids   2>/dev/null         | grep MEI           | awk '{$1=$1};1'
       qmicli -d ${MODEM} --dms-uim-get-iccid 2>/dev/null     | grep ICCID:        | awk '{$1=$1};1'
   fi
  else
    echo "External LTE modem"
    module_info=$(udevadm info /sys/class/net/${IFACE})
    vendor=$(echo "${module_info}" | grep ID_VENDOR_FROM_DATABASE= | cut -d'=' -f2)
    model=$(echo "${module_info}" | grep ID_MODEL_FROM_DATABASE= | cut -d'=' -f2)
    usb_driver=$(echo "${module_info}" | grep ID_USB_DRIVER= | cut -d'=' -f2)
    usb_class=$(echo "${module_info}" | grep ID_USB_CLASS_FROM_DATABASE= | cut -d'=' -f2)
    vendor_id=$(echo "${module_info}" | grep ID_VENDOR_ID= | cut -d'=' -f2)
    model_id=$(echo "${module_info}" | grep ID_MODEL_ID= | cut -d'=' -f2)
    echo "Vendor: ${vendor}"
    echo "Model: ${model}"
    echo "Driver: ${usb_driver}"
    echo "Class: ${usb_class}"
    echo "Vendor ID: ${vendor_id}"
    echo "Model ID: ${model_id}"
  fi
}

function show_firmware {
  if [[ "$EXTERNAL_LTE" -eq "0" ]]; then
    if [[ "${MODMODE}" == "2" ]]; then
       mbimcli -d ${MODEM}  -p  --no-close --ms-query-firmware-id  2>/dev/null | grep -v TRID |  grep -v  "Warning" | grep "Firmware" | cut -f2-10 -d' '
    else
       echo $(qmicli -d ${MODEM} --dms-get-revision 2>/dev/null | grep Revision:)
    fi
  fi
}

function show_sim_status {
  if [[ "$EXTERNAL_LTE" -eq "0" ]]; then
    if [[ "${MODMODE}" == "2" ]]; then
       echo "Not supported."
    else
       qmicli -d ${MODEM} --uim-get-card-status 2>/dev/null
    fi
  fi
}

function show_network_scan {
  if [[ "$EXTERNAL_LTE" -eq "0" ]]; then
    if [[ "${MODMODE}" == "2" ]]; then
       /usr/bin/timeout 60s mbimcli -d ${MODEM}  -p  --no-close --query-visible-providers  2>/dev/null | grep -v TRID |  grep -v  "Warning" | grep -v /"dev/"
    else
       if [ ! -f /run/ltescan.txt ]; then
           echo "Networks information is not available."; exit 0;
       fi
       if [ $(cat /run/ltescan.txt | wc -l) -le 1 ]; then
          /usr/bin/timeout 60s /usr/bin/qmicli -d ${modem_default} --nas-network-scan  | python3 /scripts/ltescan.py
       else
          cat /run/ltescan.txt | python3 /scripts/ltescan.py
       fi
     fi
  fi
}

function show_network_details {
  if [[ "$EXTERNAL_LTE" -eq "0" ]]; then
    if [[ "${MODMODE}" == "2" ]]; then
       echo "         Home provider:"
       mbimcli -d ${MODEM}  -p  --no-close --query-home-provider  2>/dev/null | grep -v TRID |  grep -v  "Warning" | grep -v /"dev/"  
    else
       qmicli -d ${MODEM} --nas-get-home-network 2>/dev/null
    fi
  fi
}


function show_identifiers {
  if [[ "$EXTERNAL_LTE" -eq "0" ]]; then
    if [[ "${MODMODE}" == "2" ]]; then
       mbimcli -d ${MODEM}  -p  --no-close --query-subscriber-ready-status  2>/dev/null | grep -v TRID |  grep -v  "Warning" | grep -v /"dev/"
    else
       qmicli -d ${MODEM} --dms-get-ids 2>/dev/null        | grep IMEI   | awk '{$1=$1};1'
       qmicli -d ${MODEM} --dms-uim-get-iccid 2>/dev/null  | grep ICCID: | awk '{$1=$1};1'
       qmicli -d ${MODEM} --dms-uim-get-imsi 2>/dev/null   | grep IMSI:  | awk '{$1=$1};1'
    fi
  fi
}

function show_firmware_preference {
  if [[ "$EXTERNAL_LTE" -eq "0" ]]; then
    if [[ "${MODMODE}" == "2" ]]; then
       echo "Not supported."
    else
       qmicli -d ${MODEM} --dms-get-firmware-preference 2>/dev/null | grep -v "firmware preference successfully retrieved"
    fi
  fi
}

function show_firmware_images {
  if [[ "$EXTERNAL_LTE" -eq "0" ]]; then
    if [[ "${MODMODE}" == "2" ]]; then
       echo "Not supported."
    else
       	qmicli -d ${MODEM} --dms-list-stored-images 2>/dev/null | grep -v "Device list of stored images retrieved"
    fi
  fi 
}

function show_port {
  if [[ "$EXTERNAL_LTE" -eq "0" ]]; then
    if [[ "${MODMODE}" == "2" ]]; then
       echo "Not supported."
fi
fi

  echo -e "$(readlink /sys/class/net/${IFACE}/device | cut -d'/' -f4)"
  lsusb -t
  echo
  
  oldIFS="$IFS"
  IFS="#" udevarr=(`udevadm info --attribute-walk --path=/sys/class/net/${IFACE} -x | egrep "looking at|busnum|devpath|SUBSYSTEMS|removable|idProduct|idVendor|manufacturer|product" | sed 's/.*looking at.*/#/' | tr "\n" ";" | tr " " "+"`)
  echo "Udev information:"
  for val in "${udevarr[@]}"
  do
      if [[ $val == *"removable}==\"removable"* ]] && [[ $val == *"SUBSYSTEMS==\"usb"* ]]; then
        echo $port_separator
        echo $val |  tr ";" "\n" | tr "+" " "  | grep -v "removable" | grep -v "SUBSYSTEM"
      fi
  done
  IFS=$oldIFS
  echo
}

function show_troubleshoot_info {
  if [[ "$EXTERNAL_LTE" -eq "1" ]]; then
    echo "${port_separator}"
    echo "Troubleshooting information for ${IFACE}"
    echo "External LTE modem - limited info supported"
    echo "${port_separator}"
    echo -e "\nModule info:"
    show_module_info
    echo -e "\nPort:"
    show_port
    return
  fi
  echo "${port_separator}"
  echo "Troubleshooting information for ${MODEM}"
  echo "${port_separator}"
  echo -e "\nNetwork settings:"
  show_network_settings
  echo -e "\nSignal strength:"
  show_signal_strength
  echo -e "\nModule info:"
  show_module_info
  echo -e "\nFirmware:"
  show_firmware
  echo -e "\nSIM status:"
  show_sim_status
  echo -e "\nNetwork details:"
  show_network_details
  echo -e "\nIdentifiers:"
  show_identifiers
  echo -e "\nFirmware Preference:"
  show_firmware_preference
  echo -e "\nFirmware Images:"
  show_firmware_images
}

# # # GO

MODEM=$modem_default
MODMODE=`/scripts/lte-type.py ${IFACE}`

if [[ "${MODMODE}" == "2" ]]; then
#mbim mode
MODEM=$mbim_default
modem_default=$mbim_default

elif [[ "${MODMODE}" == "1" ]]; then
#qmi mode
MODEM=$qmi_default
modem_default=$qmi_default

elif [[ "${MODMODE}" == "3" ]]; then
#dongle
  # Check if buildin or external LTE
  if [[ "${IFACE}" == "lte0" ]]; then
    echo "Built-in LTE Modem not found."
    exit 1
  fi

else
#unknown
 echo "LTE Modem not found."
  exit 1

fi

if [[ "${IFACE}" != "lte0" ]]; then
  if [[ -d /sys/class/net/${IFACE} ]]; then
    ALLOWED_OPTIONS=("port module-info troubleshoot-info")
    if [[ ! " ${ALLOWED_OPTIONS[*]} " =~ " ${OPTION} " ]]; then
      echo "Operation not supported for external LTE dongles."
      echo "Supported operations: module-info, port"
      exit 1
    fi
    EXTERNAL_LTE=1
  else
    echo "External modem ${IFACE} not found."
    exit 1
  fi
fi

case "$OPTION" in
  network-settings)
    show_network_settings
    ;;
  signal-strength)
    show_signal_strength
    ;;
  module-info)
    show_module_info
    ;;
  firmware)
    show_firmware
    ;;
  sim-status)
    show_sim_status
    ;;
  network-details)
    show_network_details
    ;;
  network-scan)
    show_network_scan
    ;;
  identifiers)
    show_identifiers
    ;;
  troubleshoot-info)
    show_troubleshoot_info
    ;;
  firmware-preference)
    show_firmware_preference
    ;;
  firmware-images)
    show_firmware_images
    ;;
  port)
    show_port
    ;;
  *)
    echo "Unknown operation __$1__ __$2__ __$3__ __$4__"
    exit 1
    ;;
esac

exit 0
