#!/bin/bash
source /scripts/basic_vars
source /scripts/basic_functions
set -o errexit
json_file=/config/tve_config.json
isvpp=0
SCRIPTS_PATH=$2

port_seprator="-------------------------------------------------------------------------------"
declare -A speed_array=( [0]='unknown' [10]='10Mb/s' [100]='100Mb/s' [1000]='1Gb/s' [10000]='10Gb/s' [25000]='25Gb/s' [40000]='40Gb/s' [100000]='100Gb/s' [200000]='200Gb/s' [400000]='400Gb/s' [10000000]='10Mb/s' [100000000]='100Mb/s' [1000000000]='1Gb/s' [10000000000]='10Gb/s' [25000000000]='25Gb/s' [40000000000]='40Gb/s' [100000000000]='100Gb/s' [200000000000]='200Gb/s' [400000000000]='400Gb/s' )
declare -A stat

function get_statistics() {
  local interface=$1

  # Filter "ip_vti*" from "vti*" interfaces
  if [[ $interface == vti* ]]; then
    valid_interface=($(cat /proc/net/dev | grep -E [^_]$interface | awk '{print $1}' | sed 's/://'))
  else
    valid_interface=($(cat /proc/net/dev | egrep "^[\t ]*${interface}"  | awk '{print $1}' | sed 's/://'))
  fi

  if [ ! -d "/sys/class/net/$interface" ]; then
    if [ -d "/sys/class/net/br-$interface" ]; then
     if [[ $isvpp -ne 1 ]]; then
        interface=br-$interface
     fi
    fi
  fi

  # Check for DPDK interface
  if [ -e "/var/run/openvswitch/$interface.snoop" ]; then
    valid_interface=$interface
  else
    if [[ "$valid_interface" != "$interface" && $isvpp == 1 ]]; then
        isvalid="$(sudo /usr/bin/vppctl show hardware-interface $interface |  grep 'unknown input')" || true
        if [ -n "${isvalid}" ]; then
            echo "Invalid interface" $1
            exit 1
        else
            valid_interface=$interface
        fi
    fi
  fi


  if [[ ! -z "$valid_interface" ]]; then
    if [[ $valid_interface == tun0 ]] || [[ $valid_interface == lte* ]] || [[ $valid_interface == nat* ]] || [[ $valid_interface == inband* ]] || [[ $valid_interface == vti* ]] || [[ $valid_interface == wifi* ]] || [[ $valid_interface == mgmt* ]] || [[ $valid_interface == lo* ]]; then

      # Get admin state
      if [ -n "$(/sbin/ifconfig $interface | grep UP)" ]; then
        status[0]="up"
      else
        status[0]="down"
      fi

      # Get link state
      if [ "$(cat /sys/class/net/$interface/carrier)" == "1" ]; then
        status[1]="up"
      else
        status[1]="down"
      fi

      # Get address
      status[2]="$(cat /sys/class/net/$interface/address)"

      cd /sys/class/net/$interface/statistics/
      res=( * )
      for i in ${!res[@]}; do
        key="${res[$i]}"
        var="$(cat "$key")"
        stat[$key]="$var"
      done
    else
      if [[ $isvpp == 1 ]]; then
       if  [[ $interface == data* || $interface == mgmt* || $interface == lte* || $interface == wifi* ]]; then
        if [ -f "/sys/class/net/${interface}/operstate" ]; then
            if_host_name=$(echo "host-$interface")
        else
            if_host_name=$interface
        fi

        if  [[ "$interface" == "mgmt0" ]]; then
            if_host_name="tap300"
        elif [[ "$interface" == "lte"* ]]; then
            if_host_name="tap100"
        elif [[ $interface == nat* ]]; then
            if_host_name=${interface/nat/tap20}
        elif [[ $interface == wifi0 ]]; then
            if_host_name="tap400"
        fi

        isvalid="$(sudo /usr/bin/vppctl show hardware-interface $if_host_name |  grep 'unknown input')" || true
        if [ -n "${isvalid}" ]; then
          echo "Invalid interface" $1
          exit 1
        fi
        status[0]="up"
        status[1]="$(sudo /usr/bin/vppctl show hardware-interface $if_host_name | grep ${if_host_name} |  tr -s ' ' | cut -f3 -d ' ')"
        status[2]="$(sudo /usr/bin/vppctl show hardware-interface $if_host_name | grep Ethernet |  tr -s ' ' | cut -f4 -d ' ')"


        if [ -d "/sys/class/net/$interface/statistics/" ]; then
        #virtio mode , vpp
            cd /sys/class/net/$interface/statistics/
            res=( * )
            for i in ${!res[@]}; do
              key="${res[$i]}"
              var="$(cat "$key")"
              stat[$key]="$var"
            done
        else
            #dpdk , vpp
            statsall=$(sudo /usr/bin/vppctl show hardware-interface {$interface} > /tmp/statsall)
            stat[rx_packets]=$(cat /tmp/statsall | grep "rx good packets" | egrep -o "[0-9]+" || true)
            stat[tx_packets]=$(cat /tmp/statsall | grep "tx good packets" | egrep -o "[0-9]+" || true)
            stat[rx_bytes]=$(cat /tmp/statsall | grep "rx good bytes" | egrep -o "[0-9]+" || true)
            stat[tx_bytes]=$(cat /tmp/statsall | grep "tx good bytes" | egrep -o "[0-9]+" || true)
        fi



#        echo "------- ${stat[rx_packets]}"
      fi
      else
        if [[ "$interface" == *"br-"* ]]; then
          interface=$(echo $interface | cut -d "-" -f 2)
        fi
        status=($(sudo ovs-vsctl find Interface name=$interface | awk '/mac_in_use/ {print $3} /admin_state/ {print $3} /link_state/ {print $3}'))
        if [ -z "${status}" ]; then
          echo "Invalid interface" $1
          exit 1
        fi
        res=($(sudo ovs-vsctl get interface $interface statistics))
        if [ -z "${res}" ]; then
          echo "Invalid interface" $1
          exit 1
        fi
        for i in ${!res[@]}; do
	  res[$i]="$(echo ${res[$i]})" #| sed 's/[{}"]//g')"
	  var="$(echo ${res[$i]} | cut -f1 -d"=")"
          [ -n "$var" ] && stat[$var]="$(echo ${res[$i]} | cut -f2 -d"=" | sed 's/,//')"
        done
      fi
    fi
  else
    echo "Invalid interface" $1
    exit 1
  fi

}
#
function show_statistics() {
  local interface=$1
  local csv=""
  if [[ $2 == "csv" ]]; then
    csv=$1
     echo "NAME,MAC,ADMSTATE,LINKSTATE,RxBytes,TxBytes,RxPkt,TxPkt,RxDrop,TxDrop,RxErr,TxErr,RxFrmErr,Oversize,CrcErr,Collision"
  fi
  get_statistics $interface


  # In case that the field is empty, do not left it empty, because wrong statistic will be shown
  if [ -z "${status[2]}" ]; then
    if [  -z $csv ]; then
        printf "\nInterface %s(no MAC) admin state: %s, link state: %s\n" $1 ${status[0]} ${status[1]}
    else
        printf "\n%s,,%s,%s," $interface ${status[0]} ${status[1]}
    fi
  else
     if [ $isvpp == 1 ]; then
         if [  -z $csv ]; then
            echo "Interface: $1, MAC: ${status[2]}"
            echo "admin state: ${status[0]}, link state: ${status[1]}"
         else
           printf "%s," $interface
           mac=${status[2]}
           echo -n ${mac%$'\r'}
           printf ",%s," ${status[0]}
           printf "%s," ${status[1]}
         fi
     else
         if [  -z $csv ]; then
            printf "\nInterface %s(%s) admin state: %s, link state: %s\n" $1 ${status[2]//\"/} ${status[0]} ${status[1]}
         else
           printf "\n%s,%s,%s,%s," $1 ${status[2]//\"/} ${status[0]} ${status[1]}
         fi
     fi
  fi


  if [  -z $csv ]; then
      printf "\t\t\t\t\t\t\t  rx\t\t  tx\n"
      echo ${port_seprator}
      if test ${stat[rx_1_to_64_packets]+exist}; then
        printf "Packets 1-64\t\t\t\t\t%*s\t%*s\n" 12 "${stat[rx_1_to_64_packets]}" 12 "-"
      fi
      if test ${stat[rx_65_to_127_packets]+exist}; then
        printf "Packets 65-127\t\t\t\t\t%*s\t%*s\n" 12 "${stat[rx_65_to_127_packets]}" 12 "-"
      fi
      if test ${stat[rx_128_to_255_packets]+exist}; then
        printf "Packets 128-255\t\t\t\t\t%*s\t%*s\n" 12 "${stat[rx_128_to_255_packets]}" 12 "-"
      fi
      if test ${stat[rx_256_to_511_packets]+exist}; then
        printf "Packets 256-511\t\t\t\t\t%*s\t%*s\n" 12 "${stat[rx_256_to_511_packets]}" 12 "-"
      fi
      if test ${stat[rx_512_to_1023_packets]+exist}; then
        printf "Packets 512-1023\t\t\t\t%*s\t%*s\n" 12 "${stat[rx_512_to_1023_packets]}" 12 "-"
      fi
      if test ${stat[rx_1024_to_1522_packets]+exist}; then
        printf "Packets 1024-1518\t\t\t\t%*s\t%*s\n" 12 "${stat[rx_1024_to_1522_packets]}" 12 "-"
      fi
      if test ${stat[rx_1523_to_max_packets]+exist}; then
        printf "Packets 1523-max\t\t\t\t%*s\t%*s\n" 12 "${stat[rx_1523_to_max_packets]}" 12 "-"
      fi
      if test ${stat[rx_packets]+exist}; then
        printf "Packets\t\t\t\t\t\t%*s" 12 "${stat[rx_packets]}"
      else
        printf "Packets\t\t\t\t\t\t%*s" 12 "-"
      fi
      if test ${stat[tx_packets]+exist}; then
        printf "\t%*s\t\n" 12 "${stat[tx_packets]}"
      else
        printf "\t%*s\t\n" 12 "-"
      fi
      if test ${stat[rx_bytes]+exist}; then
        printf "Bytes\t\t\t\t\t\t%*s" 12 "${stat[rx_bytes]}"
      else
        printf "Bytes\t\t\t\t\t\t%*s" 12 "-"
      fi
      if test ${stat[tx_bytes]+exist}; then
        printf "\t%*s\t\n" 12 "${stat[tx_bytes]}"
      else
        printf "\t%*s\t\n" 12 "-"
      fi
      if test ${stat[rx_dropped]+exist}; then
        printf "Drop\t\t\t\t\t\t%*s" 12 "${stat[rx_dropped]}"
      else
        printf "Drop\t\t\t\t\t\t%*s" 12 "-"
      fi
      if test ${stat[tx_dropped]+exist}; then
        printf "\t%*s\t\n" 12 "${stat[tx_dropped]}"
      else
        printf "\t%*s\t\n" 12 "-"
      fi
      if test ${stat[rx_errors]+exist}; then
        printf "Errors\t\t\t\t\t\t%*s" 12 "${stat[rx_errors]}"
      else
        printf "Errors\t\t\t\t\t\t%*s" 12 "-"
      fi
      if test ${stat[tx_errors]+exist}; then
        printf "\t%*s\t\n" 12 "${stat[tx_errors]}"
      else
        printf "\t%*s\t\n" 12 "-"
      fi
      if test ${stat[rx_frame_err]+exist}; then
        printf "Frame errors\t\t\t\t\t%*s" 12 "${stat[rx_frame_err]}"
      elif test ${stat[rx_frame_errors]+exist}; then
        printf "Frame errors\t\t\t\t\t%*s" 12 "${stat[rx_frame_errors]}"
      else
        printf "Frame errors\t\t\t\t\t%*s" 12 "-"
      fi
      printf "\t%*s\t\n" 12 "-"
      if test ${stat[rx_over_err]+exist}; then
        printf "Oversize\t\t\t\t\t%*s" 12 "${stat[rx_over_err]}"
      elif test ${stat[rx_over_errors]+exist}; then
        printf "Oversize\t\t\t\t\t%*s" 12 "${stat[rx_over_errors]}"
      else
        printf "Oversize\t\t\t\t\t%*s" 12 "-"
      fi
      printf "\t%*s\t\n" 12 "-"
      if test ${stat[rx_crc_err]+exist}; then
        printf "CRC errors\t\t\t\t\t%*s" 12 "${stat[rx_crc_err]}"
      elif test ${stat[rx_crc_errors]+exist}; then
        printf "CRC errors\t\t\t\t\t%*s" 12 "${stat[rx_crc_errors]}"
      else
        printf "CRC errors\t\t\t\t\t%*s" 12 "-"
      fi
      printf "\t%*s\t\n" 12 "-"
      printf "Collisions\t\t\t\t\t%*s" 12 "-"
      if test ${stat[collisions]+exist}; then
        printf "\t%*s\t\n" 12 "${stat[collisions]}"
      else
        printf "\t%*s\t\n" 12 "-"
      fi
    else
      if test ${stat[rx_bytes]+exist}; then
        printf "%s" "${stat[rx_bytes]}"
      fi
      printf ","

      if test ${stat[tx_bytes]+exist}; then
        printf "%s" "${stat[tx_bytes]}"
      fi
      printf ","

      if test ${stat[rx_packets]+exist}; then
        printf "%s" "${stat[rx_packets]}"
      fi
      printf ","

      if test ${stat[tx_packets]+exist}; then
        printf "%s" "${stat[tx_packets]}"
      fi
      printf ","

      if test ${stat[rx_dropped]+exist}; then
        printf "%s" "${stat[rx_dropped]}"
      fi
      printf ","

      if test ${stat[tx_dropped]+exist}; then
        printf "%s" "${stat[tx_dropped]}"
      fi
      printf ","


      if test ${stat[rx_errors]+exist}; then
        printf "%s" "${stat[rx_errors]}"
      fi
      printf ","

      if test ${stat[tx_errors]+exist}; then
        printf "%s" "${stat[tx_errors]}"
      fi
      printf ","


      if test ${stat[rx_frame_err]+exist}; then
        printf "%s" "${stat[rx_frame_err]}"
      elif test ${stat[rx_frame_errors]+exist}; then
        printf "%s" "${stat[rx_frame_errors]}"
      fi
      printf ","


      if test ${stat[rx_over_errors]+exist}; then
        printf "%s" "${stat[rx_over_errors]}"
      fi
      printf ","


      if test ${stat[rx_crc_errors]+exist}; then
        printf "%s" "${stat[rx_crc_errors]}"
      fi
      printf ","

      if test ${stat[collisions]+exist}; then
        printf "%s" "${stat[collisions]}"
      fi
      printf "\n"
    fi
}
#
function show_virt_int() {
  local csv=""
  if [[ $1 == "csv" ]]; then
    csv=$1
  fi
  cd /sys/class/net
  if [  -z $csv ]; then
  	echo ${port_seprator}
  	echo "TAP interfaces:"
  fi
  for tap_name in $(ls -d tap* 2> /dev/null); do
    local tapX_number=$(ls -ld ${tap_name} | wc -l)
    local tapX_macs=($(cat ${tap_name}/address))
    local tapX_operstate=($(cat ${tap_name}/carrier))
    #
    local tapX_RX1=($(cat ${tap_name}/statistics/rx_packets))
    local tapX_TX1=($(cat ${tap_name}/statistics/tx_packets))
    local tapX_RX1_dropped=($(cat ${tap_name}/statistics/rx_dropped))
    local tapX_TX1_dropped=($(cat ${tap_name}/statistics/tx_dropped))
    local tapX_RX1_errors=($(cat ${tap_name}/statistics/rx_errors))
    local tapX_TX1_errors=($(cat ${tap_name}/statistics/tx_errors))
    local INTERVAL="1"
    sleep ${INTERVAL}
    local tapX_RX2=($(cat ${tap_name}/statistics/rx_packets))
    local tapX_TX2=($(cat ${tap_name}/statistics/tx_packets))
    local tapX_RX2_dropped=($(cat ${tap_name}/statistics/rx_dropped))
    local tapX_TX2_dropped=($(cat ${tap_name}/statistics/tx_dropped))
    local tapX_RX2_errors=($(cat ${tap_name}/statistics/rx_errors))
    local tapX_TX2_errors=($(cat ${tap_name}/statistics/tx_errors))
    local tapX_name=($(echo "${tap_name}"))
    local tapX_ip=($(/sbin/ifconfig ${tap_name} | awk '/inet / {print $2}'))
    local tapX_netmask=($(/sbin/ifconfig ${tap_name} | awk '/netmask / {print $4}'))
    #
    if [ ! -z $csv ]; then
      display_me_in_csv ${tapX_number} "${tapX_name[*]}" "${tapX_macs[*]}" "${tapX_operstate[*]}" \
                        "${tapX_RX1[*]}" "${tapX_RX1_dropped[*]}" "${tapX_RX1_errors[*]}" \
                        "${tapX_TX1[*]}" "${tapX_TX1_dropped[*]}" "${tapX_TX1_errors[*]}" \
                        "${tapX_RX2[*]}" "${tapX_RX2_dropped[*]}" "${tapX_RX2_errors[*]}" \
                        "${tapX_TX2[*]}" "${tapX_TX2_dropped[*]}" "${tapX_TX2_errors[*]}" \
                        tapX_ip tapX_netmask
    else
      echo ${port_seprator}
      display_me ${tapX_number} "${tapX_name[*]}" "${tapX_macs[*]}" "${tapX_operstate[*]}" \
                 "${tapX_RX1[*]}" "${tapX_RX1_dropped[*]}" "${tapX_RX1_errors[*]}" \
                 "${tapX_TX1[*]}" "${tapX_TX1_dropped[*]}" "${tapX_TX1_errors[*]}" \
                 "${tapX_RX2[*]}" "${tapX_RX2_dropped[*]}" "${tapX_RX2_errors[*]}" \
                 "${tapX_TX2[*]}" "${tapX_TX2_dropped[*]}" "${tapX_TX2_errors[*]}" \
                 tapX_ip tapX_netmask
    fi
  done
  cd /run


  if [  -z $csv ]; then
    if [ "nanopir2s" != "$(cat /config/.hwtype)" -a "nanopir4s" != "$(cat /config/.hwtype)" ]; then
        echo ${port_seprator}
        echo "VHU interfaces:"
    fi
  fi
  vhuX_number=0
  for vuhX_filename in $(ls -d *.vnic 2>/dev/null); do
    vhuX_number=$((vhuX_number+1))
    local vhuX_name=$(cat ${vuhX_filename} | grep dev | tr -d "\"" | sed 's/\dev=//g' | tr -d '[:space:]')
    if [[ "$vhuX_name" == *"vhu"* ]]; then
      local vhuX_macs=($(sudo ovs-vsctl --if-exists get Interface ${vhuX_name} external_ids:attached-mac))
      local vhuX_operstate=($(sudo ovs-vsctl --if-exists get Interface ${vhuX_name} link_state))
      local vhuX_RX1=($(sudo ovs-vsctl --if-exists get Interface ${vhuX_name} statistics:rx_packets))
      local vhuX_TX1=($(sudo ovs-vsctl --if-exists get Interface ${vhuX_name} statistics:tx_packets))
      local vhuX_RX1_dropped=($(sudo ovs-vsctl --if-exists get Interface ${vhuX_name} statistics:rx_dropped))
      local vhuX_TX1_dropped=($(sudo ovs-vsctl --if-exists get Interface ${vhuX_name} statistics:tx_dropped))
      local vhuX_RX1_errors=($(sudo ovs-vsctl --if-exists get Interface ${vhuX_name} statistics:rx_errors))
      local vhuX_TX1_errors=($(sudo ovs-vsctl --if-exists get Interface ${vhuX_name} statistics:tx_errors))
      vhuX_TX1_errors=${vhuX_TX1_errors:=0}
      local INTERVAL="1"
      sleep ${INTERVAL}
      local vhuX_RX2=($(sudo ovs-vsctl --if-exists get Interface ${vhuX_name} statistics:rx_packets))
      local vhuX_TX2=($(sudo ovs-vsctl --if-exists get Interface ${vhuX_name} statistics:tx_packets))
      local vhuX_RX2_dropped=($(sudo ovs-vsctl --if-exists get Interface ${vhuX_name} statistics:rx_dropped))
      local vhuX_TX2_dropped=($(sudo ovs-vsctl --if-exists get Interface ${vhuX_name} statistics:tx_dropped))
      local vhuX_RX2_errors=($(sudo ovs-vsctl --if-exists get Interface ${vhuX_name} statistics:rx_errors))
      local vhuX_TX2_errors=($(sudo ovs-vsctl --if-exists get Interface ${vhuX_name} statistics:tx_errors))
      vhuX_TX2_errors=${vhuX_TX2_errors:=0}
      #
      if [ ! -z $csv ]; then
        display_me_in_csv ${vhuX_number} "${vhuX_name[*]}" "${vhuX_macs[*]}" "${vhuX_operstate[*]}" \
                          "${vhuX_RX1[*]}" "${vhuX_RX1_dropped[*]}" "${vhuX_RX1_errors[*]}" \
                          "${vhuX_TX1[*]}" "${vhuX_TX1_dropped[*]}" "${vhuX_TX1_errors[*]}" \
                          "${vhuX_RX2[*]}" "${vhuX_RX2_dropped[*]}" "${vhuX_RX2_errors[*]}" \
                          "${vhuX_TX2[*]}" "${vhuX_TX2_dropped[*]}" "${vhuX_TX2_errors[*]}"
      else
        echo ${port_seprator}
        display_me "1" "${vhuX_name[*]}" "${vhuX_macs[*]}" "${vhuX_operstate[*]}" \
                  "${vhuX_RX1[*]}" "${vhuX_RX1_dropped[*]}" "${vhuX_RX1_errors[*]}" \
                  "${vhuX_TX1[*]}" "${vhuX_TX1_dropped[*]}" "${vhuX_TX1_errors[*]}" \
                  "${vhuX_RX2[*]}" "${vhuX_RX2_dropped[*]}" "${vhuX_RX2_errors[*]}" \
                  "${vhuX_TX2[*]}" "${vhuX_TX2_dropped[*]}" "${vhuX_TX2_errors[*]}"
      fi
    fi
  done
}
#
function display_me() {
  number=${1}
  declare -a name=(${2})
  declare -a macs=(${3})
  declare -a state=(${4})
  declare -a rx_packets1=(${5})
  declare -a rx_dropped1=(${6})
  declare -a rx_errors1=(${7})
  declare -a tx_packets1=(${8})
  declare -a tx_dropped1=(${9})
  declare -a tx_errors1=(${10})
  declare -a rx_packets2=(${11})
  declare -a rx_dropped2=(${12})
  declare -a rx_errors2=(${13})
  declare -a tx_packets2=(${14})
  declare -a tx_dropped2=(${15})
  declare -a tx_errors2=(${16})
  if [ 18 -eq $# ]; then
    declare -n ip=${17}
    declare -n netmask=${18}
  fi
  declare -a port=(${19})
  for i in $(seq 0 $(expr ${number} - 1)); do
    if [ -n "${name[$i]+x}" ]; then
      echo -e "Name:\t\t\t${name[$i]}"
    fi
    if [ -n "${macs[$i]+x}" ]; then
      echo -e "MAC:\t\t\t${macs[$i]}"
    fi
    if [ -n "${port[$i]+x}" ]; then
      echo -e "PORT:\t\t\t${port[$i]}"
    fi
    if [ -n "${state[$i]+x}" ] && [ "${state[$i]}" == "1" ]; then
      echo -e "State:\t\t\tup"
    elif [ -n "${state[$i]+x}" ] && [ "${state[$i]}" == "0" ]; then
      echo -e "State:\t\t\tdown"
    else
      if [[ ${name[$i]} == lte* ]]; then
        echo -e "State:\t\t\tdown"
      else
        echo -e "State:\t\t\t${state[$i]}"
      fi
    fi
    if [[ ${name[$i]} == lte* || ${name[$i]} == tun* || ${name[$i]} == vti*  || ${name[$i]} == mgmt* || ${name[$i]} == ppp* ]] || [[ $isvpp == 1 && ${name[$i]} == nat* || ${name[$i]} == lo* || ${name[$i]} == wg* ]]; then
      declare speed=($(cat /sys/class/net/${name[$i]}/speed 2>/dev/null))
      declare duplex=($(cat /sys/class/net/${name[$i]}/duplex 2>/dev/null))
      declare count_div=1
    elif [[ $isvpp == 1 && ${name[$i]} == data* ]]; then
          declare duplex="unknown"
      declare count_div=7
      if [ -f "/sys/class/net/${name[$i]}/operstate" ]; then
        data_host_name=${name[$i]}
        declare speed=($(cat /sys/class/net/${name[$i]}/speed 2>/dev/null))
        declare duplex=($(cat /sys/class/net/${name[$i]}/duplex 2>/dev/null))
      else
        data_host_name=${name[$i]}
        declare str_speed=$(sudo /usr/bin/vppctl show hardware-interface ${name[$i]} | grep speed |  cut -f2 -d':' |cut -f2-3 -d' '  | tr -d '\r' )
        declare duplex=$(sudo /usr/bin/vppctl show hardware-interface ${name[$i]} | grep -i duplex | egrep "[a-zA-Z]+ duplex" -o | cut -f1 -d' ')
        str_speed=${str_speed// Mbps/}
	str_speed=${str_speed// Gbps/000000000}
	speed=$((str_speed +0))
      fi

      if [ "$str_speed" == "unknown" ]; then
        speed=0
      fi
      #declare duplex=($(cat /sys/class/net/${name[$i]}/duplex 2>/dev/null))
    else
      if  [[ ${name[$i]} == lte* || ${name[$i]} == wifi* ]] ; then
            declare speed="unknown"
            declare duplex="unknown"
      else
        declare speed=($(sudo ovs-vsctl list interface ${name[$i]} | grep link_speed | cut -f2 -d':'))
        declare duplex=($(sudo ovs-vsctl list interface ${name[$i]} | grep duplex | cut -f2 -d':'))
      fi
      declare count_div=5
    fi
    if [ -n "${speed+x}" ] && [ "${speed}" != "[]" ] && [ -n "${state[$i]+x}" ] && [ "${state[$i]}" == "1" -o "up" == "${state[$i]}" ]; then
      echo -e "Speed:\t\t\t${speed_array[$speed]}"
    else
      echo -e "Speed:\t\t\tunknown"
    fi
    if [ -n "${duplex+x}" ] && [ "${duplex}" != "[]" ] && [ -n "${state[$i]+x}" ] && [ "${state[$i]}" == "1" -o "up" == "${state[$i]}" ]; then
      echo -e "Duplex:\t\t\t${duplex}"
    else
      echo -e "Duplex:\t\t\tunknown"
    fi
    if [ ! -z "${ip[$i]}" ]; then
      echo -e "IP:\t\t\t${ip[$i]}"
    fi
    if [ ! -z "${netmask[$i]}" ]; then
      echo -e "NETMASK:\t\t${netmask[$i]}"
    fi
    if [ -n "${rx_packets1[$i]+x}" ] && [ -n "${rx_packets2[$i]+x}" ]; then
      echo -e "RX Packets Rate:\t$(( (${rx_packets2[$i]} - ${rx_packets1[$i]}) / ${count_div} ))"
    else
      echo -e "RX Packets Rate:\t0"
    fi
    if [ -n "${rx_dropped1[$i]+x}" ] && [ -n "${rx_dropped2[$i]+x}" ]; then
      echo -e "RX Dropped Rate:\t$(( (${rx_dropped2[$i]} - ${rx_dropped1[$i]}) / ${count_div} ))"
    else
      echo -e "RX Dropped Rate:\t0"
    fi
    if [ -n "${rx_errors2[$i]+x}" ] && [ -n "${rx_errors1[$i]+x}" ]; then
      echo -e "RX Errors Rate:\t\t$(( (${rx_errors2[$i]} - ${rx_errors1[$i]}) / ${count_div} ))"
    else
      echo -e "RX Errors Rate:\t\t0"
    fi
    if [ -n "${tx_packets1[$i]+x}" ] && [ -n "${tx_packets2[$i]+x}" ]; then
      echo -e "TX Packets Rate:\t$(( (${tx_packets2[$i]} - ${tx_packets1[$i]}) / ${count_div} ))"
    else
      echo -e "TX Packets Rate:\t0"
    fi
    if [ -n "${tx_dropped1[$i]+x}" ] && [ -n "${tx_dropped2[$i]+x}" ]; then
      echo -e "TX Dropped Rate:\t$(( (${tx_dropped2[$i]} - ${tx_dropped1[$i]}) / ${count_div} ))"
    else
      echo -e "TX Dropped Rate:\t0"
    fi
    if [ -n "${tx_errors2[$i]+x}" ] && [ -n "${tx_errors1[$i]+x}" ]; then
      echo -e "TX Errors Rate:\t\t$(( (${tx_errors2[$i]} - ${tx_errors1[$i]}) / ${count_div} ))"
    else
      echo -e "TX Errors Rate:\t\t0"
    fi
    echo ${port_seprator}
  done
}
#
function display_me_in_csv() {
  number=${1}
  declare -a name=(${2})
  declare -a macs=(${3})
  declare -a state=(${4})
  declare -a rx_packets1=(${5})
  declare -a rx_dropped1=(${6})
  declare -a rx_errors1=(${7})
  declare -a tx_packets1=(${8})
  declare -a tx_dropped1=(${9})
  declare -a tx_errors1=(${10})
  declare -a rx_packets2=(${11})
  declare -a rx_dropped2=(${12})
  declare -a rx_errors2=(${13})
  declare -a tx_packets2=(${14})
  declare -a tx_dropped2=(${15})
  declare -a tx_errors2=(${16})
  if [ 18 -eq $# ]; then
    declare -n ip=${17}
    declare -n netmask=${18}
  fi
  for i in $(seq 0 $(expr ${number} - 1)); do
    if [ -n "${name[$i]+x}" ]; then
      echo -e -n "${name[$i]},"
    else
      continue
    fi
    if [ -n "${macs[$i]+x}" ]; then
      echo -e -n "${macs[$i]},"
    else
      echo -e -n "none,"
    fi
    if [ -n "${state[$i]+x}" ] && [ "${state[$i]}" == "1" ]; then
      echo -e -n "up,"
    elif [ -n "${state[$i]+x}" ] && [ "${state[$i]}" == "0" ]; then
      echo -e -n "down,"
    else
      echo -e -n "${state[$i]},"
    fi
    if [[ ${name[$i]} == lte* || ${name[$i]} == tun* || ${name[$i]} == vti* || ${name[$i]} == mgmt* || ${name[$i]} == lo* || ${name[$i]} == wg* || ${name[$i]} == ppp* ]] || [[ $isvpp == 1 && ${name[$i]} == nat* ]]; then
      declare speed=($(cat /sys/class/net/${name[$i]}/speed 2>/dev/null))
      declare duplex=($(cat /sys/class/net/${name[$i]}/duplex 2>/dev/null))
      declare count_div=1
    elif [[ $isvpp == 1 && ${name[$i]} == data* ]]; then
      if [[ -f "/sys/class/net/${name[$i]}/operstate" ]]; then
        data_host_name=$(echo "host-${name[$i]}")
      else
        data_host_name=${name[$i]}
      fi
      declare speed_str=$(sudo ovs-vsctl list interface $data_host_name | grep speed |  tr -s ' ' | cut -f4 -d' ')
      declare speed_print=$(sudo ovs-vsctl list interface $data_host_name | grep speed |  tr -s '' | cut -f2 -d':' | tr -d '\r' | tr -d ' ')
      declare duplex_str=$(sudo ovs-vsctl list interface $data_host_name | grep duplex |  tr -s ' '| cut -f4 -d' ')

      if [ "$speed_str" == "[]" ] || [ "$speed_str" == "unknown" ]; then
        speed=0
      else
	speed=$speed_str
      fi
      if [ "$duplex_str" == "unknown" ]; then
	declare duplex="unknown"
      else
	declare duplex=$duplex_str
      fi
      #declare duplex=($(cat /sys/class/net/${name[$i]}/duplex 2>/dev/null))
      declare count_div=7
    else
      if  [[ ${name[$i]} == lte* || ${name[$i]} == wifi* || ${name[$i]} == Virtual* ]] ; then
            declare speed="unknown"
            declare duplex="unknown"
      else
            declare speed=($(sudo ovs-vsctl list interface ${name[$i]} | grep link_speed | cut -f2 -d':'))
            declare duplex=($(sudo ovs-vsctl list interface ${name[$i]} | grep duplex | cut -f2 -d':'))
      fi
      declare count_div=5
    fi
    if [ -n "${speed+x}" ] && [ "${speed}" != "[]" ] && [ -n "${state[$i]+x}" ] && [ "${state[$i]}" == "1" -o "up" == "${state[$i]}" ]; then
      echo -e -n "${speed_array[$speed]},"
    else
      echo -e -n "none,"
    fi
    if [ -n "${duplex+x}" ] && [ "${duplex}" != "[]" ] && [ -n "${state[$i]+x}" ] && [ "${state[$i]}" == "1" -o "up" == "${state[$i]}" ]; then
      echo -e -n "${duplex},"
    else
      echo -e -n "none,"
    fi
    if [ ! -z "${ip[$i]}" ]; then
      echo -e -n "${ip[$i]},"
    else
      echo -e -n "none,"
    fi
    if [ ! -z "${netmask[$i]}" ]; then
      echo -e -n "${netmask[$i]},"
    else
      echo -e -n "none,"
    fi
    if [ -n "${rx_packets1[$i]+x}" ] && [ -n "${rx_packets2[$i]+x}" ]; then
      echo -e -n "$(( (${rx_packets2[$i]} - ${rx_packets1[$i]}) / ${count_div} )),"
    else
      echo -e -n "0,"
    fi
    if [ -n "${rx_dropped1[$i]+x}" ] && [ -n "${rx_dropped2[$i]+x}" ]; then
      echo -e -n "$(( (${rx_dropped2[$i]} - ${rx_dropped1[$i]}) / ${count_div} )),"
    else
      echo -e -n "0,"
    fi
    if [ -n "${rx_errors2[$i]+x}" ] && [ -n "${rx_errors1[$i]+x}" ]; then
      echo -e -n "$(( (${rx_errors2[$i]} - ${rx_errors1[$i]}) / ${count_div} )),"
    else
      echo -e -n "0,"
    fi
    if [ -n "${tx_packets1[$i]+x}" ] && [ -n "${tx_packets2[$i]+x}" ]; then
      echo -e -n "$(( (${tx_packets2[$i]} - ${tx_packets1[$i]}) / ${count_div} )),"
    else
      echo -e -n "0,"
    fi
    if [ -n "${tx_dropped1[$i]+x}" ] && [ -n "${tx_dropped2[$i]+x}" ]; then
      echo -e -n "$(( (${tx_dropped2[$i]} - ${tx_dropped1[$i]}) / ${count_div} )),"
    else
      echo -e -n "0,"
    fi
    if [ -n "${tx_errors2[$i]+x}" ] && [ -n "${tx_errors1[$i]+x}" ]; then
      echo -e -n "$(( (${tx_errors2[$i]} - ${tx_errors1[$i]}) / ${count_div} ))"
    else
      echo -e -n "0"
    fi
    echo ""
  done
}
#
function display_me_in_csv_vpp() {
  number=${1}
  declare -a name=(${2})
  declare -a macs=(${3})
  declare -a state=(${4})
  declare -a rx_packets1=(${5})
  declare -a rx_dropped1=(${6})
  declare -a rx_errors1=(${7})
  declare -a tx_packets1=(${8})
  declare -a tx_dropped1=(${9})
  declare -a tx_errors1=(${10})
  declare -a rx_packets2=(${11})
  declare -a rx_dropped2=(${12})
  declare -a rx_errors2=(${13})
  declare -a tx_packets2=(${14})
  declare -a tx_dropped2=(${15})
  declare -a tx_errors2=(${16})
  if [ 18 -eq $# ]; then
    declare -n ip=${17}
    declare -n netmask=${18}
  fi
  declare -a port=(${19})


  for i in $(seq 0 $(expr ${number} - 1)); do

    if [[ "${name[$i]}" == "" ]]; then
       continue
    fi
    if [ -n "${name[$i]+x}" ]; then
      #echo -e "Name:\t\t\t${name[$i]}"
      echo -e -n "${name[$i]},"
    fi

    if [ -n "${macs[$i]+x}" ]; then
      echo -e -n "${macs[$i]},"
    else
      echo -e -n "none,"
    fi

    if [ -n "${state[$i]+x}" ] && [ "${state[$i]}" == "1" ]; then
      echo -e -n "up,"
    elif [ -n "${state[$i]+x}" ] && [ "${state[$i]}" == "0" ]; then
      echo -e -n "down,"
    else
      echo -e -n "${state[$i]},"
    fi

    if [[ ${name[$i]} == nat* ]]; then
      declare speed=($(cat /sys/class/net/${name[$i]}/speed 2>/dev/null))
      declare duplex=($(cat /sys/class/net/${name[$i]}/duplex 2>/dev/null))
      declare count_div=1
    else
      if [[ ${name[$i]} == data* ]]; then
        data_host_name=$(echo "host-${name[$i]}")
      else [[ ${name[$i]} == VirtualEthernet* ]]
        data_host_name=${name[$i]}
      fi


      if [ -f "/sys/class/net/${name[$i]}/operstate" ]; then
      	data_host_name=${name[$i]}
        declare speed_str=($(cat /sys/class/net/${name[$i]}/speed 2>/dev/null))
        declare duplex_str=($(cat /sys/class/net/${name[$i]}/duplex 2>/dev/null))
        speed_print=${speed_str// Mbps/}
        speed_print=${speed_str// Gbps/000000000}
        speed=${speed_array[$speed_print]}
                #speed=${speed_str}
                #echo $speed_str
      else
                #echo 'tuk vliza'
      	data_host_name=${name[$i]}
                #declare speed_str=$(sudo /usr/bin/vppctl show hardware-interface ${name[$i]} | grep speed |  cut -f2 -d':' |cut -f2-3 -d' '  | tr -d '\r' )
        declare speed_str=$(sudo /usr/bin/vppctl show hardware-interface ${name[$i]} | grep speed |  tr -s '' | cut -f2 -d':' | tr -d '\r' | tr -d ' ')
        declare duplex_str=$(sudo /usr/bin/vppctl show hardware-interface ${name[$i]} | grep -i duplex | egrep "[a-zA-Z]+ duplex" -o | cut -f1 -d' ')
        speed_print=${speed_str// Mbps/}
        speed_print=${speed_str// Gbps/000000000}
        speed=${speed_print}
      fi

      #declare speed_str=$(sudo /usr/bin/vppctl show hardware-interface $data_host_name | grep speed |  tr -s ' ' | cut -f4 -d' ')
      #declare speed_print=$(sudo /usr/bin/vppctl show hardware-interface $data_host_name | grep speed |  tr -s '' | cut -f2 -d':' | tr -d '\r' | tr -d ' ')
      #declare duplex_str=$(sudo /usr/bin/vppctl show hardware-interface $data_host_name | grep duplex |  tr -s ' '| cut -f4 -d' ')
      if [ "$speed_str" == "unknown" ] || [ "$speed_str" == "-1" ]; then
        speed=0
      else
	      speed=$speed
	#speed=$speed_print
      fi
      if [ "$duplex_str" == "unknown" ] || [ "$duplex_str" == "-1" ]; then
        declare duplex="unknown"
      else
        declare duplex=$duplex_str
      fi
      declare count_div=7
    fi

    if [ -n "${speed+x}" ] && [ "${speed}" != "[]" ] && [ -n "${state[$i]+x}" ] && [ "${state[$i]}" == "1" -o "up" == "${state[$i]}" ]; then
      echo -e -n "$speed,"
    else
      echo -e -n "none,"
    fi
    if [ -n "${duplex+x}" ] && [ "${duplex}" != "[]" ] && [ -n "${state[$i]+x}" ] && [ "${state[$i]}" == "1" -o "up" == "${state[$i]}" ]; then
      echo -e -n "${duplex},"
    else
      echo -e -n "none,"
    fi
    if [ ! -z "${ip[$i]}" ]; then
      echo -e -n "${ip[$i]},"
    else
      echo -e -n "none,"
    fi
    if [ ! -z "${netmask[$i]}" ]; then
      echo -e -n "${netmask[$i]},"
    else
      echo -e -n "none,"
    fi
    if [ -n "${rx_packets1[$i]+x}" ] && [ -n "${rx_packets2[$i]+x}" ]; then
      echo -e -n "$(( (${rx_packets2[$i]} - ${rx_packets1[$i]}) / ${count_div} )),"
    else
      echo -e -n "0,"
    fi
    if [ -n "${rx_dropped1[$i]+x}" ] && [ -n "${rx_dropped2[$i]+x}" ]; then
      echo -e -n "$(( (${rx_dropped2[$i]} - ${rx_dropped1[$i]}) / ${count_div} )),"
    else
      echo -e -n "0,"
    fi
    if [ -n "${rx_errors2[$i]+x}" ] && [ -n "${rx_errors1[$i]+x}" ]; then
      echo -e -n "$(( (${rx_errors2[$i]} - ${rx_errors1[$i]}) / ${count_div} )),"
    else
      echo -e -n "0,"
    fi
    if [ -n "${tx_packets1[$i]+x}" ] && [ -n "${tx_packets2[$i]+x}" ]; then
      echo -e -n "$(( (${tx_packets2[$i]} - ${tx_packets1[$i]}) / ${count_div} )),"
    else
      echo -e -n "0,"
    fi
    if [ -n "${tx_dropped1[$i]+x}" ] && [ -n "${tx_dropped2[$i]+x}" ]; then
      echo -e -n "$(( (${tx_dropped2[$i]} - ${tx_dropped1[$i]}) / ${count_div} )),"
    else
      echo -e -n "0,"
    fi
    if [ -n "${tx_errors2[$i]+x}" ] && [ -n "${tx_errors1[$i]+x}" ]; then
      echo -e -n "$(( (${tx_errors2[$i]} - ${tx_errors1[$i]}) / ${count_div} ))"
    else
      echo -e -n "0"
    fi
    echo ""
  done
}
#
function display_me_vpp() {
  number=${1}
  declare -a name=(${2})
  declare -a macs=(${3})
  declare -a state=(${4})
  declare -a rx_packets1=(${5})
  declare -a rx_bytes1=(${6})
  declare -a tx_packets1=(${7})
  declare -a tx_bytes1=(${8})
  declare -a rx_packets2=(${9})
  declare -a rx_bytes2=(${10})
  declare -a tx_packets2=(${11})
  declare -a tx_bytes2=(${12})

  for i in $(seq 0 $(expr ${number} - 1)); do
    if [[ "${name[$i]}" == "" ]]; then
       continue
    fi
    if [ -n "${name[$i]+x}" ]; then
      echo -e "Name:\t\t\t${name[$i]}"
    fi
    if [ -n "${macs[$i]+x}" ]; then
      echo -e "MAC:\t\t\t${macs[$i]}"
    fi
    if [ -n "${state[$i]+x}" ] && [ "${state[$i]}" == "1" ]; then
      echo -e "State:\t\t\tup"
    elif [ -n "${state[$i]+x}" ] && [ "${state[$i]}" == "0" ]; then
      echo -e "State:\t\t\tdown"
    else
      if [[ ${name[$i]} == lte* ]]; then
        echo -e "State:\t\t\tdown"
      else
        echo -e "State:\t\t\t${state[$i]}"
      fi
    fi
    if [[ ${name[$i]} == nat* ]]; then
      declare speed=($(cat /sys/class/net/${name[$i]}/speed 2>/dev/null))
      declare duplex=($(cat /sys/class/net/${name[$i]}/duplex 2>/dev/null))
      declare count_div=1
    else
      if [[ ${name[$i]} == data* ]]; then
        if [ -f "/sys/class/net/${name[$i]}/operstate" ]; then
            data_host_name=$(echo "host-${name[$i]}")
        else
            data_host_name=${name[$i]}
        fi
      else [[ ${name[$i]} == VirtualEthernet* ]]
        data_host_name=${name[$i]}
      fi

      declare speed_str=$(sudo /usr/bin/vppctl show hardware-interface ${data_host_name} | grep speed | tr -s '' | cut -f2 -d':' | tr -d '\r' | tr -d ' ')
      speed_print=${speed_str// Mbps/}
      speed_print=${speed_str// Gbps/000000000}
      speed=${speed_print}

      declare duplex_str=$(sudo /usr/bin/vppctl show hardware-interface ${name[$i]} | grep -i duplex | egrep "[a-zA-Z]+ duplex" -o | cut -f1 -d' ')

      if [ "$speed_str" == "unknown" ] || [ "$speed_str" == "-1" ]; then
        speed="unknown"
      else
        speed=$speed
      fi

      if [ "$duplex_str" == "unknown" ] || [ "$duplex_str" == "-1" ] || [ "$duplex_str" == "None" ]; then
        declare duplex="unknown"
      else
        declare duplex=$duplex_str
      fi

      if [ -z "$duplex" ]; then
        declare duplex="unknown"
      fi

      #declare duplex=($(cat /sys/class/net/${name[$i]}/duplex 2>/dev/null))
      declare count_div=7
    fi
    if [ -n "${speed+x}" ] && [ "${speed}" != "[]" ] && [ -n "${state[$i]+x}" ] && [ "${state[$i]}" == "1" -o "up" == "${state[$i]}" ]; then
      echo -e "Speed:\t\t\t${speed}"
    else
      echo -e "Speed:\t\t\tunknown"
    fi
    if [ -n "${duplex+x}" ] && [ "${duplex}" != "[]" ] && [ -n "${state[$i]+x}" ] && [ "${state[$i]}" == "1" -o "up" == "${state[$i]}" ]; then
      echo -e "Duplex:\t\t\t${duplex}"
    else
      echo -e "Duplex:\t\t\tunknown"
    fi
    if [ -n "${rx_packets1[$i]+x}" ] && [ -n "${rx_packets2[$i]+x}" ]; then
      echo -e "RX Packets Rate:\t$(( (${rx_packets2[$i]} - ${rx_packets1[$i]}) / ${count_div} ))"
    else
      echo -e "RX Packets Rate:\t0"
    fi
    if [ -n "${rx_bytes1[$i]+x}" ] && [ -n "${rx_bytes2[$i]+x}" ]; then
      echo -e "RX Octets Rate:\t\t$(( (${rx_bytes2[$i]} - ${rx_bytes1[$i]}) / ${count_div} ))"
    else
      echo -e "RX Octets Rate:\t\t0"
    fi
    if [ -n "${tx_packets1[$i]+x}" ] && [ -n "${tx_packets2[$i]+x}" ]; then
      echo -e "TX Packets Rate:\t$(( (${tx_packets2[$i]} - ${tx_packets1[$i]}) / ${count_div} ))"
    else
      echo -e "TX Packets Rate:\t0"
    fi
    if [ -n "${tx_bytes1[$i]+x}" ] && [ -n "${tx_bytes2[$i]+x}" ]; then
      echo -e "TX Octets Rate:\t\t$(( (${tx_bytes2[$i]} - ${tx_bytes1[$i]}) / ${count_div} ))"
    else
      echo -e "TX Octets Rate:\t\t0"
    fi
    echo ${port_seprator}
  done
}


function show_virtual_interfaces_vpp() {
  # show virtual interfaces (vhost and tap)

  local csv=""
  if [[ $1 == "csv" ]]; then
    csv=$1
  fi

  declare -a interface_names
  declare -a interface_macs
  declare -a interface_operstates
  declare -a interface_RXs
  declare -a interface_RX_bytes
  declare -a interface_TXs
  declare -a interface_TX_bytes

  local vhost_list="$(sudo /usr/bin/vppctl show interface | grep VirtualEthernet | awk '{print $1}')"
  local tap_list="$(sudo /usr/bin/vppctl show tap | awk '/^Interface:/ {interface=$2} /name "eth/ {print interface}')"

  if [ -z "$csv" ]; then
      echo $port_seprator
      echo "Virtual interfaces:"
      echo $port_seprator
  fi


  # Check if vhost_list is empty
  if [ ! -z "$vhost_list" ]; then
    # Split the variable into an array
    readarray -t vhost_array <<< "$vhost_list"

    # Collect initial statistics for vhost interfaces
    for vhost_iterator in "${vhost_array[@]}"; do
      vhost_name="$vhost_iterator"  # Assign the iterator value to vhost_name
      interface_names+=( "$vhost_name" )
      interface_macs+=( "$(sudo /usr/bin/vppctl show hardware-interface "$vhost_name" | grep "Ethernet addr" |  tr -s ' ' | cut -f4 -d ' ' | cut -c 1-17)" )
      interface_operstates+=( "$(sudo /usr/bin/vppctl show interface "$vhost_name" | awk -v name="$vhost_name" '$1 == name {print $3}')" )
      interface_RXs+=( "$(sudo /usr/bin/vppctl show interface "$vhost_name" | grep "rx packets" | tr -s ' ' | cut -f7 -d' ')" )
      interface_RX_bytes+=( "$(sudo /usr/bin/vppctl show interface "$vhost_name" | grep "rx bytes" | tr -s ' ' | cut -f4 -d' ')" )
      interface_TXs+=( "$(sudo /usr/bin/vppctl show interface "$vhost_name" | grep "tx packets" | tr -s ' ' | cut -f4 -d' ')" )
      interface_TX_bytes+=( "$(sudo /usr/bin/vppctl show interface "$vhost_name" | grep "tx bytes" | tr -s ' ' | cut -f4 -d' ')" )
    done
  fi

  # Check if tap_list is empty
  if [ ! -z "$tap_list" ]; then
    # Split the variable into an array
    readarray -t tap_array <<< "$tap_list"

    # Collect initial statistics for tap interfaces
    for tap_iterator in "${tap_array[@]}"; do
      tap_name="$tap_iterator"  # Assign the iterator value to tap_name
      interface_names+=( "$tap_name" )
      interface_macs+=( "$(sudo /usr/bin/vppctl show hardware-interface "$tap_name" | grep "Ethernet addr" |  tr -s ' ' | cut -f4 -d ' ' | cut -c 1-17)" )
      interface_operstates+=( "$(sudo /usr/bin/vppctl show interface "$tap_name" | awk -v name="$tap_name" '$1 == name {print $3}')" )
      interface_RXs+=( "$(sudo /usr/bin/vppctl show interface "$tap_name" | grep "rx packets" | tr -s ' ' | cut -f7 -d' ')" )
      interface_RX_bytes+=( "$(sudo /usr/bin/vppctl show interface "$tap_name" | grep "rx bytes" | tr -s ' ' | cut -f4 -d' ')" )
      interface_TXs+=( "$(sudo /usr/bin/vppctl show interface "$tap_name" | grep "tx packets" | tr -s ' ' | cut -f4 -d' ')" )
      interface_TX_bytes+=( "$(sudo /usr/bin/vppctl show interface "$tap_name" | grep "tx bytes" | tr -s ' ' | cut -f4 -d' ')" )
    done
  fi

  # Sleep for the specified interval
  sleep 7

  # Calculate rates and print information for each interface
  for ((i=0; i<${#interface_names[@]}; i++)); do
    interface_name="${interface_names[$i]}"
    interface_mac="${interface_macs[$i]}"
    interface_operstate="${interface_operstates[$i]}"
    interface_RX="${interface_RXs[$i]}"
    interface_RX_bytes="${interface_RX_bytes[$i]}"
    interface_TX="${interface_TXs[$i]}"
    interface_TX_bytes="${interface_TX_bytes[$i]}"

    interface_RX2=$(sudo /usr/bin/vppctl show interface "$interface_name" | grep "rx packets" | tr -s ' ' | cut -f7 -d' ')
    interface_RX2_bytes=$(sudo /usr/bin/vppctl show interface "$interface_name" | grep "rx bytes" | tr -s ' ' | cut -f4 -d' ')
    interface_TX2=$(sudo /usr/bin/vppctl show interface "$interface_name" | grep "tx packets" | tr -s ' ' | cut -f4 -d' ')
    interface_TX2_bytes=$(sudo /usr/bin/vppctl show interface "$interface_name" | grep "tx bytes" | tr -s ' ' | cut -f4 -d' ')

    interface_RX_rate=$(awk -v rx2="$interface_RX2" -v rx1="$interface_RX" -v interval="7" 'BEGIN { printf "%d", int(rx2 - rx1) / interval }')
    interface_TX_rate=$(awk -v tx2="$interface_TX2" -v tx1="$interface_TX" -v interval="7" 'BEGIN { printf "%d", int(tx2 - tx1) / interval }')
    interface_RX2_bytes_rate=$(awk -v rx2b="$interface_RX2_bytes" -v rx1b="$interface_RX_bytes" -v interval="7" 'BEGIN { printf "%d", int(rx2b - rx1b) / interval }')
    interface_TX2_bytes_rate=$(awk -v tx2b="$interface_TX2_bytes" -v tx1b="$interface_TX_bytes" -v interval="7" 'BEGIN { printf "%d", int(tx2b - tx1b) / interval }')

    if [ ! -z $csv ]; then
      csv_string=$"$interface_name,$interface_mac,$interface_operstate,$interface_RX_rate,$interface_RX2_bytes_rate,$interface_TX_rate,$interface_TX2_bytes_rate"
      echo -e $csv_string
    else

      printf "Name:\t\t\t%s\n" "${interface_name}"
      printf "MAC:\t\t\t%s\n" "${interface_mac}"
      printf "State:\t\t\t%s\n" "${interface_operstate}"
      printf "RX Packets Rate:\t%s\n" "${interface_RX_rate}"
      printf "RX Octets Rate:\t\t%s\n" "${interface_RX2_bytes_rate}"
      printf "TX Packets Rate:\t%s\n" "${interface_TX_rate}"
      printf "TX Octets Rate:\t\t%s\n" "${interface_TX2_bytes_rate}"
      echo ${port_seprator}
    fi
  done
}


#
function show_physical_int_vpp() {
  # host-data* interfaces
  local csv=""
  if [[ $1 == "csv" ]]; then
    csv=$1
  fi
  local dataX_number=$(sudo /usr/bin/vppctl show interface | egrep "(data[0-9]+)" -o | sort -u | wc -l)
  declare -a dataX_name
  declare -a dataX_host_name
  declare -a dataX_macs
  declare -a dataX_operstate
  declare -a dataX_RX1
  declare -a dataX_TX1
  declare -a dataX_RX1_dropped
  declare -a dataX_TX1_dropped
  declare -a dataX_RX1_errors
  declare -a dataX_TX1_errors
  declare -a dataX_RX2
  declare -a dataX_TX2
  declare -a dataX_RX2_dropped
  declare -a dataX_TX2_dropped
  declare -a dataX_RX2_errors
  declare -a dataX_TX2_errors
  for data in $(seq 0 $(expr ${dataX_number} - 1)); do
    dataX_name[$data]=$(echo "data$data")

    if [ -d "/sys/class/net/data$data" ]; then
        #virtio mode , vpp
        dataX_host_name[$data]=$(echo "data$data")
        dataX_macs[$data]=$(cat /sys/class/net/data$data/address)
        dataX_operstate[$data]=$(cat /sys/class/net/data$data/operstate)
        dataX_RX1[$data]=$(cat /sys/class/net/data$data/statistics/rx_packets)
        dataX_TX1[$data]=$(cat /sys/class/net/data$data/statistics/tx_packets)
        dataX_RX1_dropped[$data]=$(cat /sys/class/net/data$data/statistics/rx_dropped)
        dataX_TX1_dropped[$data]=$(cat /sys/class/net/data$data/statistics/tx_dropped)
        dataX_RX1_errors[$data]=$(cat /sys/class/net/data$data/statistics/rx_errors)
        dataX_TX1_errors[$data]=$(cat /sys/class/net/data$data/statistics/tx_errors)

    else
        #dpdk , vpp
        dataX_host_name[$data]=$(echo "data$data")
        dataX_macs[$data]=$(sudo /usr/bin/vppctl show hardware-interface ${dataX_host_name[$data]} | grep Ethernet |  tr -s ' ' | cut -f4 -d ' ' | cut -c 1-17)
        dataX_operstate[$data]=$(sudo /usr/bin/vppctl show hardware-interface ${dataX_host_name[$data]} | grep ${dataX_host_name[$data]} |  tr -s ' ' | cut -f3 -d ' ')
        statsall=$(sudo /usr/bin/vppctl show hardware-interface ${dataX_host_name[$data]} > /tmp/statsall || true)
        #get_statistics ${dataX_name[$data]}
	#dataX_RX1[$data]='0'
        dataX_RX1[$data]=$(cat /tmp/statsall | grep "rx good packets" | egrep -o "[0-9]+" || echo "0")
	#echo "data$data"
	#echo $(cat /tmp/statsall | grep "rx good packets" | egrep -o "[0-9]+" || echo "0")
	#dataX_TX1[$data]='0'
        dataX_TX1[$data]=$(cat /tmp/statsall | grep "tx good packets" | egrep -o "[0-9]+" || echo "0")
        dataX_RX1_dropped[$data]="0"
        dataX_TX1_dropped[$data]="0"
        dataX_RX1_errors[$data]="0"   # workaround-for-detail-cmd-crashing-vpp $(cat /tmp/statsall | grep "rx errors" | egrep -o "[0-9]+")
        dataX_TX1_errors[$data]="0"  # workaround-for-detail-cmd-crashing-vpp $(cat /tmp/statsall | grep "tx errors" | egrep -o "[0-9]+")
    fi

  done
  # tap* interfaces
  local br_dataX_number=$dataX_number
  declare -a br_dataX_name
  declare -a br_dataX_ip
  declare -a br_dataX_netmask
  declare -a br_dataX_macs
  declare -a br_dataX_operstate
  for data in $(seq 0 $(expr ${br_dataX_number} - 1)); do
    br_dataX_name[$data]=$(echo "br-data$data")
    br_dataX_tap_name[$data]=$(echo "tap$data")
    br_dataX_ip[$data]=$(/sbin/ifconfig br-data$data | awk '/inet / {print $2}')
    br_dataX_netmask[$data]=$(/sbin/ifconfig br-data$data | awk '/netmask / {print $4}')
    br_dataX_macs[$data]=$(sudo /usr/bin/vppctl show hardware-interface ${br_dataX_tap_name[$data]} | grep Ethernet |  tr -s ' ' | cut -f4 -d ' ')
    br_dataX_operstate[$data]=$(sudo /usr/bin/vppctl show hardware-interface ${br_dataX_tap_name[$data]} | grep ${br_dataX_tap_name[$data]} |  tr -s ' ' | cut -f3 -d ' ')
    ###get_statistics ${br_dataX_name[$data]}
  done
  #
  local INTERVAL="7"
  sleep ${INTERVAL}
  for data in $(seq 0 $(expr ${dataX_number} - 1)); do

  if [ -d "/sys/class/net/data$data" ]; then
        #virtio mode , vpp
        dataX_RX2[$data]=$(cat /sys/class/net/data$data/statistics/rx_packets)
        dataX_TX2[$data]=$(cat /sys/class/net/data$data/statistics/tx_packets)
        dataX_RX2_dropped[$data]=$(cat /sys/class/net/data$data/statistics/rx_dropped)
        dataX_TX2_dropped[$data]=$(cat /sys/class/net/data$data/statistics/tx_dropped)
        dataX_RX2_errors[$data]=$(cat /sys/class/net/data$data/statistics/rx_errors)
        dataX_TX2_errors[$data]=$(cat /sys/class/net/data$data/statistics/tx_errors)

    else
        #dpdk , vpp
        statsall=$(sudo /usr/bin/vppctl show hardware-interface data$data  > /tmp/statsall || true)
        #get_statistics ${dataX_name[$data]}
	#dataX_RX2[$data]="0"
        dataX_RX2[$data]=$(cat /tmp/statsall | grep "rx good packets" | egrep -o "[0-9]+" || echo "0")
        #dataX_TX2[$data]="0"
        dataX_TX2[$data]=$(cat /tmp/statsall | grep "tx good packets" | egrep -o "[0-9]+" || echo "0")
        dataX_RX2_dropped[$data]="0"
        dataX_TX2_dropped[$data]="0"
        dataX_RX2_errors[$data]="0" # workaround-for-detail-cmd-crashing-vpp $(cat /tmp/statsall | grep "rx errors" | egrep -o "[0-9]+")
        dataX_TX2_errors[$data]="0" # workaround-for-detail-cmd-crashing-vpp $(cat /tmp/statsall | grep "tx errors" | egrep -o "[0-9]+")
    fi

  done
  # br-data* interfaces
  ########for data in $(seq 0 $(expr ${br_dataX_number} - 1)); do
    ###########get_statistics ${br_dataX_name[$data]}
  ############done
  #
  if [ ! -z $csv ]; then
    display_me_in_csv_vpp ${dataX_number} \
                      "${dataX_name[*]}" "${dataX_macs[*]}" "${dataX_operstate[*]}" \
                      "${dataX_RX1[*]}" "${dataX_RX1_dropped[*]}" "${dataX_RX1_errors[*]}" \
                      "${dataX_TX1[*]}" "${dataX_TX1_dropped[*]}" "${dataX_TX1_errors[*]}" \
                      "${dataX_RX2[*]}" "${dataX_RX2_dropped[*]}" "${dataX_RX2_errors[*]}" \
                      "${dataX_TX2[*]}" "${dataX_TX2_dropped[*]}" "${dataX_TX2_errors[*]}" \
                      br_dataX_ip br_dataX_netmask
  else
    echo ${port_seprator}
    echo "Physical interfaces:"
    echo ${port_seprator}

    display_me_vpp ${dataX_number} \
               "${dataX_name[*]}" "${dataX_macs[*]}" "${dataX_operstate[*]}" \
               "${dataX_RX1[*]}" "${dataX_RX1_dropped[*]}" \
               "${dataX_TX1[*]}" "${dataX_TX1_dropped[*]}" \
               "${dataX_RX2[*]}" "${dataX_RX2_dropped[*]}" \
               "${dataX_TX2[*]}" "${dataX_TX2_dropped[*]}" \
               br_dataX_ip br_dataX_netmask
  fi
}

function show_physical_int() {
  # data* interfaces
  local csv=""
  if [[ $1 == "csv" ]]; then
    csv=$1
  fi
  local dataX_number=$(sudo ovs-vsctl list interface | grep data | grep -v "br-data" | grep -v "error" | wc -l)
  declare -a dataX_name
  declare -a dataX_macs
  declare -a dataX_operstate
  declare -a dataX_RX1
  declare -a dataX_TX1
  declare -a dataX_RX1_dropped
  declare -a dataX_TX1_dropped
  declare -a dataX_RX1_errors
  declare -a dataX_TX1_errors
  declare -a dataX_RX2
  declare -a dataX_TX2
  declare -a dataX_RX2_dropped
  declare -a dataX_TX2_dropped
  declare -a dataX_RX2_errors
  declare -a dataX_TX2_errors
  for data in $(seq 0 $(expr ${dataX_number} - 1)); do
    dataX_name[$data]=$(echo "data$data")
    dataX_macs[$data]=$(sudo ovs-vsctl list interface ${dataX_name[$data]} | grep mac_in_use | cut -f2 -d '"' | tr -d '\r' | tr -d ' ')
    dataX_operstate[$data]=$(sudo ovs-vsctl list interface ${dataX_name[$data]} | grep link_state | cut -f2 -d':' | tr -d '\r' | tr -d ' ')
    #|  tr -s '' | cut -f2 -d':' | tr -d '\r' | tr -d ' '
    #echo ${dataX_macs[$data]}
    if [ ${dataX_operstate[$data]} != "down" ] && [ ${dataX_operstate[$data]} != "up" ] ; then
    dataX_operstate[$data]="unknown"
    #echo $dataX_macs[$data]
    fi
    if [ ${dataX_macs[$data]} == "mac_in_use:[]" ] ; then
    #echo ${dataX_macs[$data]}
    dataX_macs[$data]="unknown"
    fi
    get_statistics ${dataX_name[$data]}
    dataX_RX1[$data]=${stat[rx_packets]}
    #set -x
    dataX_TX1[$data]=${stat[tx_packets]} | sed 's/[^0-9]*//g'
    dataX_RX1_dropped[$data]=${stat[rx_dropped]}
    dataX_TX1_dropped[$data]=${stat[tx_dropped]}
    dataX_RX1_errors[$data]=${stat[rx_errors]}
    dataX_TX1_errors[$data]=${stat[tx_errors]}
  done
  # br-data* interfaces
  local br_dataX_number=$(sudo ovs-vsctl list interface | grep ' "br-data' | wc -l)
  declare -a br_dataX_name
  declare -a br_dataX_ip
  declare -a br_dataX_netmask
  declare -a br_dataX_macs
  declare -a br_dataX_operstate
  for data in $(seq 0 $(expr ${br_dataX_number} - 1)); do
    br_dataX_name[$data]=$(echo "br-data$data")
    br_dataX_ip[$data]=$(/sbin/ifconfig br-data$data | awk '/inet / {print $2}')
    br_dataX_netmask[$data]=$(/sbin/ifconfig br-data$data | awk '/netmask / {print $4}')
    br_dataX_macs[$data]=$(sudo ovs-vsctl list interface ${br_dataX_name[$data]} | grep mac_in_use | cut -f2 -d '"')
    br_dataX_operstate[$data]=$(sudo ovs-vsctl list interface ${br_dataX_name[$data]} | grep link_state | cut -f2 -d':')
    get_statistics ${br_dataX_name[$data]}
  done
  #
  local INTERVAL="7"
  sleep ${INTERVAL}
  # data* interfaces
  for data in $(seq 0 $(expr ${dataX_number} - 1)); do
    get_statistics ${dataX_name[$data]}
    dataX_RX2[$data]=${stat[rx_packets]}
    dataX_TX2[$data]=${stat[tx_packets]}
    dataX_RX2_dropped[$data]=${stat[rx_dropped]}
    dataX_TX2_dropped[$data]=${stat[tx_dropped]}
    dataX_RX2_errors[$data]=${stat[rx_errors]}
    dataX_TX2_errors[$data]=${stat[tx_errors]}
  done
  # br-data* interfaces
  for data in $(seq 0 $(expr ${br_dataX_number} - 1)); do
    get_statistics ${br_dataX_name[$data]}
  done
  #
  if [ ! -z $csv ]; then
    display_me_in_csv ${dataX_number} \
                      "${dataX_name[*]}" "${dataX_macs[*]}" "${dataX_operstate[*]}" \
                      "${dataX_RX1[*]}" "${dataX_RX1_dropped[*]}" "${dataX_RX1_errors[*]}" \
                      "${dataX_TX1[*]}" "${dataX_TX1_dropped[*]}" "${dataX_TX1_errors[*]}" \
                      "${dataX_RX2[*]}" "${dataX_RX2_dropped[*]}" "${dataX_RX2_errors[*]}" \
                      "${dataX_TX2[*]}" "${dataX_TX2_dropped[*]}" "${dataX_TX2_errors[*]}" \
                      br_dataX_ip br_dataX_netmask
  else
    echo ${port_seprator}
    echo "Physical interfaces:"
    echo ${port_seprator}
    #
    display_me ${dataX_number} \
               "${dataX_name[*]}" "${dataX_macs[*]}" "${dataX_operstate[*]}" \
               "${dataX_RX1[*]}" "${dataX_RX1_dropped[*]}" "${dataX_RX1_errors[*]}" \
               "${dataX_TX1[*]}" "${dataX_TX1_dropped[*]}" "${dataX_TX1_errors[*]}" \
               "${dataX_RX2[*]}" "${dataX_RX2_dropped[*]}" "${dataX_RX2_errors[*]}" \
               "${dataX_TX2[*]}" "${dataX_TX2_dropped[*]}" "${dataX_TX2_errors[*]}" \
               br_dataX_ip br_dataX_netmask
  fi
}
#
function show_nat_int() {
  local csv=""
  if [[ $1 == "csv" ]]; then
    csv=$1
  fi
  local natX_number=$(ls -ld /sys/class/net/nat* | wc -l)
  local natX_macs=($(cat /sys/class/net/nat*/address))
  local natX_operstate=($(cat /sys/class/net/nat*/operstate))
  #
  local natX_RX1=($(cat /sys/class/net/nat*/statistics/rx_packets))
  local natX_TX1=($(cat /sys/class/net/nat*/statistics/tx_packets))
  local natX_RX1_dropped=($(cat /sys/class/net/nat*/statistics/rx_dropped))
  local natX_TX1_dropped=($(cat /sys/class/net/nat*/statistics/tx_dropped))
  local natX_RX1_errors=($(cat /sys/class/net/nat*/statistics/rx_errors))
  local natX_TX1_errors=($(cat /sys/class/net/nat*/statistics/tx_errors))
  local INTERVAL="1"
  sleep ${INTERVAL}
  local natX_RX2=($(cat /sys/class/net/nat*/statistics/rx_packets))
  local natX_TX2=($(cat /sys/class/net/nat*/statistics/tx_packets))
  local natX_RX2_dropped=($(cat /sys/class/net/nat*/statistics/rx_dropped))
  local natX_TX2_dropped=($(cat /sys/class/net/nat*/statistics/tx_dropped))
  local natX_RX2_errors=($(cat /sys/class/net/nat*/statistics/rx_errors))
  local natX_TX2_errors=($(cat /sys/class/net/nat*/statistics/tx_errors))
  #
  declare -a natX_name
  declare -a natX_ip
  declare -a natX_netmask
  for i in $(seq 0 $(expr ${natX_number} - 1)); do
    natX_name[$i]=$(echo "nat$i")
    natX_ip[$i]=$(/sbin/ifconfig nat$i | awk '/inet / {print $2}')
    natX_netmask[$i]=$(/sbin/ifconfig nat$i | awk '/netmask / {print $4}')
    if [ "${natX_operstate[$i]}" == "unknown" -a -n "$(ip address show up | grep -i nat$i)" ]; then
      natX_operstate[$i]="up"
    fi
  done
  #
  if [ ! -z $csv ]; then
    display_me_in_csv ${natX_number} "${natX_name[*]}" "${natX_macs[*]}" "${natX_operstate[*]}" \
                      "${natX_RX1[*]}" "${natX_RX1_dropped[*]}" "${natX_RX1_errors[*]}" \
                      "${natX_TX1[*]}" "${natX_TX1_dropped[*]}" "${natX_TX1_errors[*]}" \
                      "${natX_RX2[*]}" "${natX_RX2_dropped[*]}" "${natX_RX2_errors[*]}" \
                      "${natX_TX2[*]}" "${natX_TX2_dropped[*]}" "${natX_TX2_errors[*]}" \
                      natX_ip natX_netmask
  else
    echo ${port_seprator}
    echo "NAT interfaces:"
    echo ${port_seprator}
    #
    display_me ${natX_number} "${natX_name[*]}" "${natX_macs[*]}" "${natX_operstate[*]}" \
               "${natX_RX1[*]}" "${natX_RX1_dropped[*]}" "${natX_RX1_errors[*]}" \
               "${natX_TX1[*]}" "${natX_TX1_dropped[*]}" "${natX_TX1_errors[*]}" \
               "${natX_RX2[*]}" "${natX_RX2_dropped[*]}" "${natX_RX2_errors[*]}" \
               "${natX_TX2[*]}" "${natX_TX2_dropped[*]}" "${natX_TX2_errors[*]}" \
               natX_ip natX_netmask
  fi
}

function show_mgmt_int() {
  local csv=""
  if [[ $1 == "csv" ]]; then
    csv=$1
  fi
  local mgmtX_number=$(ls -ld /sys/class/net/mgmt* | wc -l)
  local mgmtX_macs=($(cat /sys/class/net/mgmt*/address))
  local mgmtX_operstate=($(cat /sys/class/net/mgmt*/operstate))
  #
  local mgmtX_RX1=($(cat /sys/class/net/mgmt*/statistics/rx_packets))
  local mgmtX_TX1=($(cat /sys/class/net/mgmt*/statistics/tx_packets))
  local mgmtX_RX1_dropped=($(cat /sys/class/net/mgmt*/statistics/rx_dropped))
  local mgmtX_TX1_dropped=($(cat /sys/class/net/mgmt*/statistics/tx_dropped))
  local mgmtX_RX1_errors=($(cat /sys/class/net/mgmt*/statistics/rx_errors))
  local mgmtX_TX1_errors=($(cat /sys/class/net/mgmt*/statistics/tx_errors))
  local INTERVAL="1"
  sleep ${INTERVAL}
  local mgmtX_RX2=($(cat /sys/class/net/mgmt*/statistics/rx_packets))
  local mgmtX_TX2=($(cat /sys/class/net/mgmt*/statistics/tx_packets))
  local mgmtX_RX2_dropped=($(cat /sys/class/net/mgmt*/statistics/rx_dropped))
  local mgmtX_TX2_dropped=($(cat /sys/class/net/mgmt*/statistics/tx_dropped))
  local mgmtX_RX2_errors=($(cat /sys/class/net/mgmt*/statistics/rx_errors))
  local mgmtX_TX2_errors=($(cat /sys/class/net/mgmt*/statistics/tx_errors))
  #
  declare -a mgmtX_name
  declare -a mgmtX_ip
  declare -a mgmtX_netmask
  for i in $(seq 0 $(expr ${mgmtX_number} - 1)); do
    mgmtX_name[$i]=$(echo "mgmt$i")
    mgmtX_ip[$i]=$(/sbin/ifconfig mgmt$i | awk '/inet / {print $2}')
    mgmtX_netmask[$i]=$(/sbin/ifconfig mgmt$i | awk '/netmask / {print $4}')
    if [ "${mgmtX_operstate[$i]}" == "unknown" -a -n "$(ip address show up | grep -i mgmt$i)" ]; then
      mgmtX_operstate[$i]="up"
    fi
  done
  #
  if [ ! -z $csv ]; then
    display_me_in_csv ${mgmtX_number} "${mgmtX_name[*]}" "${mgmtX_macs[*]}" "${mgmtX_operstate[*]}" \
                      "${mgmtX_RX1[*]}" "${mgmtX_RX1_dropped[*]}" "${mgmtX_RX1_errors[*]}" \
                      "${mgmtX_TX1[*]}" "${mgmtX_TX1_dropped[*]}" "${mgmtX_TX1_errors[*]}" \
                      "${mgmtX_RX2[*]}" "${mgmtX_RX2_dropped[*]}" "${mgmtX_RX2_errors[*]}" \
                      "${mgmtX_TX2[*]}" "${mgmtX_TX2_dropped[*]}" "${mgmtX_TX2_errors[*]}" \
                      mgmtX_ip mgmtX_netmask
  else
    #
    display_me ${mgmtX_number} "${mgmtX_name[*]}" "${mgmtX_macs[*]}" "${mgmtX_operstate[*]}" \
               "${mgmtX_RX1[*]}" "${mgmtX_RX1_dropped[*]}" "${mgmtX_RX1_errors[*]}" \
               "${mgmtX_TX1[*]}" "${mgmtX_TX1_dropped[*]}" "${mgmtX_TX1_errors[*]}" \
               "${mgmtX_RX2[*]}" "${mgmtX_RX2_dropped[*]}" "${mgmtX_RX2_errors[*]}" \
               "${mgmtX_TX2[*]}" "${mgmtX_TX2_dropped[*]}" "${mgmtX_TX2_errors[*]}" \
               mgmtX_ip mgmtX_netmask
  fi
}

function show_lo_int() {
  local csv=""
  if [[ $1 == "csv" ]]; then
    csv=$1
  fi

  if [ -z $csv ]; then
    echo ${port_seprator}
    echo "Loopback interfaces:"
    echo ${port_seprator}
  fi


  for lo_dir in /sys/class/net/lo*; do
    if [ -d ${lo_dir}/statistics ]; then
      local loX_name=$(echo ${lo_dir} | rev | cut -d'/' -f1 | rev)
      local loX_ip=$(/sbin/ifconfig ${loX_name} | awk '/inet / {print $2}')
      local loX_netmask=$(/sbin/ifconfig ${loX_name} | awk '/netmask / {print $4}')

      local loX_number=1 #$(echo ${loX_name} | tr -dc '0-9')
      local loX_macs=($(cat ${lo_dir}/address))
      local loX_operstate=($(cat ${lo_dir}/operstate))
      #
      local loX_RX1=($(cat ${lo_dir}/statistics/rx_packets))
      local loX_TX1=($(cat ${lo_dir}/statistics/tx_packets))
      local loX_RX1_dropped=($(cat ${lo_dir}/statistics/rx_dropped))
      local loX_TX1_dropped=($(cat ${lo_dir}/statistics/tx_dropped))
      local loX_RX1_errors=($(cat ${lo_dir}/statistics/rx_errors))
      local loX_TX1_errors=($(cat ${lo_dir}/statistics/tx_errors))
      local INTERVAL="1"
      sleep ${INTERVAL}
      local loX_RX2=($(cat ${lo_dir}/statistics/rx_packets))
      local loX_TX2=($(cat ${lo_dir}/statistics/tx_packets))
      local loX_RX2_dropped=($(cat ${lo_dir}/statistics/rx_dropped))
      local loX_TX2_dropped=($(cat ${lo_dir}/statistics/tx_dropped))
      local loX_RX2_errors=($(cat ${lo_dir}/statistics/rx_errors))
      local loX_TX2_errors=($(cat ${lo_dir}/statistics/tx_errors))
      loX_ip=$(/sbin/ifconfig lo | awk '/inet / {print $2}')
      loX_netmask=$(/sbin/ifconfig lo | awk '/netmask / {print $4}')
      #
      #
      if [ ! -z $csv ]; then
        display_me_in_csv "${loX_number[*]}" "${loX_name[*]}" "${loX_macs[*]}" "${loX_operstate[*]}" \
                          "${loX_RX1[*]}" "${loX_RX1_dropped[*]}" "${loX_RX1_errors[*]}" \
                          "${loX_TX1[*]}" "${loX_TX1_dropped[*]}" "${loX_TX1_errors[*]}" \
                          "${loX_RX2[*]}" "${loX_RX2_dropped[*]}" "${loX_RX2_errors[*]}" \
                          "${loX_TX2[*]}" "${loX_TX2_dropped[*]}" "${loX_TX2_errors[*]}" \
                          loX_ip loX_netmask "${loX_port}"
      else
        #
        display_me "${loX_number[*]}" "${loX_name[*]}" "${loX_macs[*]}" "${loX_operstate[*]}" \
                  "${loX_RX1[*]}" "${loX_RX1_dropped[*]}" "${loX_RX1_errors[*]}" \
                  "${loX_TX1[*]}" "${loX_TX1_dropped[*]}" "${loX_TX1_errors[*]}" \
                  "${loX_RX2[*]}" "${loX_RX2_dropped[*]}" "${loX_RX2_errors[*]}" \
                  "${loX_TX2[*]}" "${loX_TX2_dropped[*]}" "${loX_TX2_errors[*]}" \
                  loX_ip loX_netmask "${loX_port}"
      fi
    fi
  done
}
#


function show_wg_int() {
  local csv=""
  if [[ $1 == "csv" ]]; then
    csv=$1
  fi
  if [ -z $csv ]; then
    # echo ${port_seprator}
    echo "Wireguard interfaces:"
    echo ${port_seprator}
  fi


  for wg_dir in /sys/class/net/wg*; do
    if [ -d ${wg_dir}/statistics ]; then
      local wgX_name=$(echo ${wg_dir} | rev | cut -d'/' -f1 | rev)
      local wgX_ip=$(/sbin/ifconfig ${wgX_name} | awk '/inet / {print $2}')
      local wgX_netmask=$(/sbin/ifconfig ${wgX_name} | awk '/netmask / {print $4}')

      local wgX_number=1 #$(echo ${wgX_name} | tr -dc '0-9')
      local wgX_macs=($(cat ${wg_dir}/address))
      local wgX_operstate=($(cat ${wg_dir}/operstate))
      #
      local wgX_RX1=($(cat ${wg_dir}/statistics/rx_packets))
      local wgX_TX1=($(cat ${wg_dir}/statistics/tx_packets))
      local wgX_RX1_dropped=($(cat ${wg_dir}/statistics/rx_dropped))
      local wgX_TX1_dropped=($(cat ${wg_dir}/statistics/tx_dropped))
      local wgX_RX1_errors=($(cat ${wg_dir}/statistics/rx_errors))
      local wgX_TX1_errors=($(cat ${wg_dir}/statistics/tx_errors))
      local INTERVAL="1"
      sleep ${INTERVAL}
      local wgX_RX2=($(cat ${wg_dir}/statistics/rx_packets))
      local wgX_TX2=($(cat ${wg_dir}/statistics/tx_packets))
      local wgX_RX2_dropped=($(cat ${wg_dir}/statistics/rx_dropped))
      local wgX_TX2_dropped=($(cat ${wg_dir}/statistics/tx_dropped))
      local wgX_RX2_errors=($(cat ${wg_dir}/statistics/rx_errors))
      local wgX_TX2_errors=($(cat ${wg_dir}/statistics/tx_errors))
      wgX_ip=$(/sbin/ifconfig lo | awk '/inet / {print $2}')
      wgX_netmask=$(/sbin/ifconfig lo | awk '/netmask / {print $4}')
      #
      #
      if [ ! -z $csv ]; then
        display_me_in_csv "${wgX_number[*]}" "${wgX_name[*]}" "${wgX_macs[*]}" "${wgX_operstate[*]}" \
                          "${wgX_RX1[*]}" "${wgX_RX1_dropped[*]}" "${wgX_RX1_errors[*]}" \
                          "${wgX_TX1[*]}" "${wgX_TX1_dropped[*]}" "${wgX_TX1_errors[*]}" \
                          "${wgX_RX2[*]}" "${wgX_RX2_dropped[*]}" "${wgX_RX2_errors[*]}" \
                          "${wgX_TX2[*]}" "${wgX_TX2_dropped[*]}" "${wgX_TX2_errors[*]}" \
                          wgX_ip wgX_netmask "${wgX_port}"
      else
        #
        display_me "${wgX_number[*]}" "${wgX_name[*]}" "${wgX_macs[*]}" "${wgX_operstate[*]}" \
                  "${wgX_RX1[*]}" "${wgX_RX1_dropped[*]}" "${wgX_RX1_errors[*]}" \
                  "${wgX_TX1[*]}" "${wgX_TX1_dropped[*]}" "${wgX_TX1_errors[*]}" \
                  "${wgX_RX2[*]}" "${wgX_RX2_dropped[*]}" "${wgX_RX2_errors[*]}" \
                  "${wgX_TX2[*]}" "${wgX_TX2_dropped[*]}" "${wgX_TX2_errors[*]}" \
                  wgX_ip wgX_netmask "${wgX_port}"
      fi
    fi
  done
}





function show_lte_int() {
  local csv=""
  if [[ $1 == "csv" ]]; then
    csv=$1
  fi
  for lte_dir in /sys/class/net/lte*; do
    if [ -d ${lte_dir}/statistics ]; then
      local lteX_name=$(echo ${lte_dir} | rev | cut -d'/' -f1 | rev)
      local lteX_ip=$(/sbin/ifconfig ${lteX_name} | awk '/inet / {print $2}')
      local lteX_netmask=$(/sbin/ifconfig ${lteX_name} | awk '/netmask / {print $4}')

      local lteX_number=1 #$(echo ${lteX_name} | tr -dc '0-9')
      local lteX_macs=($(cat ${lte_dir}/address))
      local lteX_port=($(ls -l ${lte_dir}/device | cut -d '>' -f2 | cut -d'/' -f4))
      local lteX_operstate=($(cat ${lte_dir}/carrier 2>/dev/null)) || true
      #
      local lteX_RX1=($(cat ${lte_dir}/statistics/rx_packets))
      local lteX_TX1=($(cat ${lte_dir}/statistics/tx_packets))
      local lteX_RX1_dropped=($(cat ${lte_dir}/statistics/rx_dropped))
      local lteX_TX1_dropped=($(cat ${lte_dir}/statistics/tx_dropped))
      local lteX_RX1_errors=($(cat ${lte_dir}/statistics/rx_errors))
      local lteX_TX1_errors=($(cat ${lte_dir}/statistics/tx_errors))
      local INTERVAL="1"
      sleep ${INTERVAL}
      local lteX_RX2=($(cat ${lte_dir}/statistics/rx_packets))
      local lteX_TX2=($(cat ${lte_dir}/statistics/tx_packets))
      local lteX_RX2_dropped=($(cat ${lte_dir}/statistics/rx_dropped))
      local lteX_TX2_dropped=($(cat ${lte_dir}/statistics/tx_dropped))
      local lteX_RX2_errors=($(cat ${lte_dir}/statistics/rx_errors))
      local lteX_TX2_errors=($(cat ${lte_dir}/statistics/tx_errors))
      #
      #
      if [ ! -z $csv ]; then
        display_me_in_csv "${lteX_number[*]}" "${lteX_name[*]}" "${lteX_macs[*]}" "${lteX_operstate[*]}" \
                          "${lteX_RX1[*]}" "${lteX_RX1_dropped[*]}" "${lteX_RX1_errors[*]}" \
                          "${lteX_TX1[*]}" "${lteX_TX1_dropped[*]}" "${lteX_TX1_errors[*]}" \
                          "${lteX_RX2[*]}" "${lteX_RX2_dropped[*]}" "${lteX_RX2_errors[*]}" \
                          "${lteX_TX2[*]}" "${lteX_TX2_dropped[*]}" "${lteX_TX2_errors[*]}" \
                          lteX_ip lteX_netmask "${lteX_port}"
      else
        echo ${port_seprator}
        echo "LTE interface: ${lteX_name}"
        echo ${port_seprator}
        #
        display_me "${lteX_number[*]}" "${lteX_name[*]}" "${lteX_macs[*]}" "${lteX_operstate[*]}" \
                  "${lteX_RX1[*]}" "${lteX_RX1_dropped[*]}" "${lteX_RX1_errors[*]}" \
                  "${lteX_TX1[*]}" "${lteX_TX1_dropped[*]}" "${lteX_TX1_errors[*]}" \
                  "${lteX_RX2[*]}" "${lteX_RX2_dropped[*]}" "${lteX_RX2_errors[*]}" \
                  "${lteX_TX2[*]}" "${lteX_TX2_dropped[*]}" "${lteX_TX2_errors[*]}" \
                  lteX_ip lteX_netmask "${lteX_port}"
      fi
    fi
  done
}
#
function show_tunnel_int() {
  local tun=""
  local csv=""
  if [[ $1 == "tun" ]] || [[ $1 == "vti" ]]; then
    tun=$1
  fi
  if [[ $2 == "csv" ]]; then
    csv=$2
  fi
  if [ -d /sys/devices/virtual/net/$tun*/statistics ]; then
    local tunX_number=$(ls -ld /sys/class/net/$tun* | wc -l)
    local tunX_macs=($(cat /sys/class/net/$tun*/address))
    local tunX_operstate=($(cat /sys/class/net/$tun*/carrier))
    #
    local tunX_RX1=($(cat /sys/class/net/$tun*/statistics/rx_packets))
    local tunX_TX1=($(cat /sys/class/net/$tun*/statistics/tx_packets))
    local tunX_RX1_dropped=($(cat /sys/class/net/$tun*/statistics/rx_dropped))
    local tunX_TX1_dropped=($(cat /sys/class/net/$tun*/statistics/tx_dropped))
    local tunX_RX1_errors=($(cat /sys/class/net/$tun*/statistics/rx_errors))
    local tunX_TX1_errors=($(cat /sys/class/net/$tun*/statistics/tx_errors))
    local INTERVAL="1"
    sleep ${INTERVAL}
    local tunX_RX2=($(cat /sys/class/net/$tun*/statistics/rx_packets))
    local tunX_TX2=($(cat /sys/class/net/$tun*/statistics/tx_packets))
    local tunX_RX2_dropped=($(cat /sys/class/net/$tun*/statistics/rx_dropped))
    local tunX_TX2_dropped=($(cat /sys/class/net/$tun*/statistics/tx_dropped))
    local tunX_RX2_errors=($(cat /sys/class/net/$tun*/statistics/rx_errors))
    local tunX_TX2_errors=($(cat /sys/class/net/$tun*/statistics/tx_errors))
    #
    declare -a tunX_name
    declare -a tunX_ip
    declare -a tunX_netmask
    for i in $(seq 0 $(expr ${tunX_number} - 1)); do
      tunX_name[$i]=$(echo "$tun$i")
      tunX_ip[$i]=$(/sbin/ifconfig $tun$i | awk '/inet / {print $2}')
      tunX_netmask[$i]=$(/sbin/ifconfig $tun$i | awk '/netmask / {print $4}')
    done
    #
    if [ ! -z $csv ]; then
      display_me_in_csv ${tunX_number} "${tunX_name[*]}" "${tunX_macs[*]}" "${tunX_operstate[*]}" \
                        "${tunX_RX1[*]}" "${tunX_RX1_dropped[*]}" "${tunX_RX1_errors[*]}" \
                        "${tunX_TX1[*]}" "${tunX_TX1_dropped[*]}" "${tunX_TX1_errors[*]}" \
                        "${tunX_RX2[*]}" "${tunX_RX2_dropped[*]}" "${tunX_RX2_errors[*]}" \
                        "${tunX_TX2[*]}" "${tunX_TX2_dropped[*]}" "${tunX_TX2_errors[*]}" \
                        tunX_ip tunX_netmask
    else
      echo ${port_seprator}
      if [[ $tun == "tun" ]]; then
        echo "TUN interfaces:"
      else
        echo "VTI interfaces:"
      fi
      echo ${port_seprator}
      #
      display_me ${tunX_number} "${tunX_name[*]}" "${tunX_macs[*]}" "${tunX_operstate[*]}" \
                 "${tunX_RX1[*]}" "${tunX_RX1_dropped[*]}" "${tunX_RX1_errors[*]}" \
                 "${tunX_TX1[*]}" "${tunX_TX1_dropped[*]}" "${tunX_TX1_errors[*]}" \
                 "${tunX_RX2[*]}" "${tunX_RX2_dropped[*]}" "${tunX_RX2_errors[*]}" \
                 "${tunX_TX2[*]}" "${tunX_TX2_dropped[*]}" "${tunX_TX2_errors[*]}" \
                 tunX_ip tunX_netmask
    fi
  fi
}
#
function show_wifi_int() {
  local csv=""
  if [[ $1 == "csv" ]]; then
    csv=$1
  fi
  if [ -d /sys/class/net/wifi0/statistics ]; then
    local wifi_number=1
    local wifi_macs=($(cat /sys/class/net/wifi0/address))
    local wifi_operstate=($(cat /sys/class/net/wifi0/carrier 2>/dev/null)) || true
    #
    local wifi_RX1=($(cat /sys/class/net/wifi0/statistics/rx_packets))
    local wifi_TX1=($(cat /sys/class/net/wifi0/statistics/tx_packets))
    local wifi_RX1_dropped=($(cat /sys/class/net/wifi0/statistics/rx_dropped))
    local wifi_TX1_dropped=($(cat /sys/class/net/wifi0/statistics/tx_dropped))
    local wifi_RX1_errors=($(cat /sys/class/net/wifi0/statistics/rx_errors))
    local wifi_TX1_errors=($(cat /sys/class/net/wifi0/statistics/tx_errors))
    local INTERVAL="1"
    sleep ${INTERVAL}
    local wifi_RX2=($(cat /sys/class/net/wifi0/statistics/rx_packets))
    local wifi_TX2=($(cat /sys/class/net/wifi0/statistics/tx_packets))
    local wifi_RX2_dropped=($(cat /sys/class/net/wifi0/statistics/rx_dropped))
    local wifi_TX2_dropped=($(cat /sys/class/net/wifi0/statistics/tx_dropped))
    local wifi_RX2_errors=($(cat /sys/class/net/wifi0/statistics/rx_errors))
    local wifi_TX2_errors=($(cat /sys/class/net/wifi0/statistics/tx_errors))
    #
    declare -a wifi_name
    declare -a wifi_ip
    declare -a wifi_netmask
    wifi_name[0]=$(echo "wifi0")
    wifi_ip[0]=$(/sbin/ifconfig wifi0 | awk '/inet / {print $2}')
    wifi_netmask[0]=$(/sbin/ifconfig wifi0 | awk '/netmask / {print $4}')
    #
    if [ ! -z $csv ]; then
      display_me_in_csv ${wifi_number} "${wifi_name[*]}" "${wifi_macs[*]}" "${wifi_operstate[*]}" \
                        "${wifi_RX1[*]}" "${wifi_RX1_dropped[*]}" "${wifi_RX1_errors[*]}" \
                        "${wifi_TX1[*]}" "${wifi_TX1_dropped[*]}" "${wifi_TX1_errors[*]}" \
                        "${wifi_RX2[*]}" "${wifi_RX2_dropped[*]}" "${wifi_RX2_errors[*]}" \
                        "${wifi_TX2[*]}" "${wifi_TX2_dropped[*]}" "${wifi_TX2_errors[*]}" \
                        wifi_ip wifi_netmask
    else
      echo ${port_seprator}
      echo "WiFi interface:"
      echo ${port_seprator}
      #
      display_me ${wifi_number} "${wifi_name[*]}" "${wifi_macs[*]}" "${wifi_operstate[*]}" \
                 "${wifi_RX1[*]}" "${wifi_RX1_dropped[*]}" "${wifi_RX1_errors[*]}" \
                 "${wifi_TX1[*]}" "${wifi_TX1_dropped[*]}" "${wifi_TX1_errors[*]}" \
                 "${wifi_RX2[*]}" "${wifi_RX2_dropped[*]}" "${wifi_RX2_errors[*]}" \
                 "${wifi_TX2[*]}" "${wifi_TX2_dropped[*]}" "${wifi_TX2_errors[*]}" \
                 wifi_ip wifi_netmask
    fi
  fi
}

function show_pppoe_int() {
  local csv=""
  if [[ $1 == "csv" ]]; then
    csv=$1
  fi
  if [ -z $csv ]; then
    # echo ${port_seprator}
    echo "PPPoE interfaces:"
    echo ${port_seprator}
  fi

  for ppp_dir in /sys/class/net/ppp*; do
    if [ -d ${ppp_dir}/statistics ]; then
      local pppX_name=$(echo ${ppp_dir} | rev | cut -d'/' -f1 | rev)
      local pppX_ip=$(/sbin/ifconfig ${pppX_name} | awk '/inet / {print $2}')
      local pppX_netmask=$(/sbin/ifconfig ${pppX_name} | awk '/netmask / {print $4}')

      local pppX_number=1 #$(echo ${pppX_name} | tr -dc '0-9')
      local pppX_macs=($(cat ${ppp_dir}/address))
      local pppX_operstate=($(cat ${ppp_dir}/operstate))
      #
      local pppX_RX1=($(cat ${ppp_dir}/statistics/rx_packets))
      local pppX_TX1=($(cat ${ppp_dir}/statistics/tx_packets))
      local pppX_RX1_dropped=($(cat ${ppp_dir}/statistics/rx_dropped))
      local pppX_TX1_dropped=($(cat ${ppp_dir}/statistics/tx_dropped))
      local pppX_RX1_errors=($(cat ${ppp_dir}/statistics/rx_errors))
      local pppX_TX1_errors=($(cat ${ppp_dir}/statistics/tx_errors))
      local INTERVAL="1"
      sleep ${INTERVAL}
      local pppX_RX2=($(cat ${ppp_dir}/statistics/rx_packets))
      local pppX_TX2=($(cat ${ppp_dir}/statistics/tx_packets))
      local pppX_RX2_dropped=($(cat ${ppp_dir}/statistics/rx_dropped))
      local pppX_TX2_dropped=($(cat ${ppp_dir}/statistics/tx_dropped))
      local pppX_RX2_errors=($(cat ${ppp_dir}/statistics/rx_errors))
      local pppX_TX2_errors=($(cat ${ppp_dir}/statistics/tx_errors))
      pppX_ip=$(/sbin/ifconfig lo | awk '/inet / {print $2}')
      pppX_netmask=$(/sbin/ifconfig lo | awk '/netmask / {print $4}')
      #
      #
      if [ ! -z $csv ]; then
        display_me_in_csv "${pppX_number[*]}" "${pppX_name[*]}" "${pppX_macs[*]}" "${pppX_operstate[*]}" \
                          "${pppX_RX1[*]}" "${pppX_RX1_dropped[*]}" "${pppX_RX1_errors[*]}" \
                          "${pppX_TX1[*]}" "${pppX_TX1_dropped[*]}" "${pppX_TX1_errors[*]}" \
                          "${pppX_RX2[*]}" "${pppX_RX2_dropped[*]}" "${pppX_RX2_errors[*]}" \
                          "${pppX_TX2[*]}" "${pppX_TX2_dropped[*]}" "${pppX_TX2_errors[*]}" \
                          pppX_ip pppX_netmask "${pppX_port}"
      else
        #
        display_me "${pppX_number[*]}" "${pppX_name[*]}" "${pppX_macs[*]}" "${pppX_operstate[*]}" \
                  "${pppX_RX1[*]}" "${pppX_RX1_dropped[*]}" "${pppX_RX1_errors[*]}" \
                  "${pppX_TX1[*]}" "${pppX_TX1_dropped[*]}" "${pppX_TX1_errors[*]}" \
                  "${pppX_RX2[*]}" "${pppX_RX2_dropped[*]}" "${pppX_RX2_errors[*]}" \
                  "${pppX_TX2[*]}" "${pppX_TX2_dropped[*]}" "${pppX_TX2_errors[*]}" \
                  pppX_ip pppX_netmask "${pppX_port}"
      fi
    fi
  done
}


#
function show_interfaces() {
  if [ $isvpp == 1 ]; then
    show_virtual_interfaces_vpp $1
    show_physical_int_vpp $1
  else
    show_virt_int $1
    show_physical_int $1
  fi
  show_pppoe_int $1
  show_nat_int $1
  show_tunnel_int "tun" $1
  show_tunnel_int "vti" $1
  show_lte_int $1
  show_wifi_int $1
  show_mgmt_int $1
  show_lo_int $1
  show_wg_int $1

}
#
#if [[ $2 == "csv" ]]; then
#  echo "NAME,MAC,STATE,SPEED,DUPLEX,IP,NETMASK,RXPkt,RXDrop,RXErr,TXPkt,TXDrop,TXerr"
#fi
#

if [ -f '/usr/bin/vppctl' ]; then
  if [ -n "$(sudo /usr/bin/vppctl 2> /dev/null show cpu | grep Model)" ]; then
     isvpp=1
  fi
fi

case "$1" in
  interfaces)
    show_interfaces $2
    ;;
  virtual)
    if [ $isvpp == 1 ]; then
      show_virtual_interfaces_vpp $2
    else
      show_virt_int $2
    fi
    ;;
  physical)
    if [ $isvpp == 1 ]; then
      show_physical_int_vpp $2
    else
      show_physical_int $2
    fi
    ;;
  nat)
    show_nat_int $2
    ;;
  mgmt)
    show_mgmt_int $2
    ;;
  loopback)
    show_lo_int $2
    ;;
  wireguard)
    show_wg_int $2
    ;;
  lte)
    show_lte_int $2
    ;;
  vpn)
    show_tunnel_int "tun" $2
    show_tunnel_int "vti" $2
    ;;
  statistics)
    show_statistics $2 $3
    ;;
  wifi)
    show_wifi_int $2
    ;;
  pppoe)
    show_pppoe_int $2
    ;;
  *)
    echo "Unknown operation __$1__ __$2__"
    exit 1
    ;;
esac

exit 0
