#!/usr/bin/env python3

import vppapi_if.vppapi_if as vppif
from sysrepo_interfaces import sysrepo_interfaces3 as sr
import sys
import syslog
import subprocess
import shlex

PPP_OFFSET = 700
INDENT = 4 * ' '
VPP_SH_IFACE_COMMAND = 'sudo /usr/bin/vppctl show interface '
vpp_if = vppif.VppApiInterface('pppoe-commands')


def get_pppox_ifidx(name):
    tap_id = int(name.replace('ppp', ''))
    host_if_name = 'tap{}'.format(PPP_OFFSET + tap_id)
    host_ifidx = vpp_if.get_ifidx_by_name(host_if_name)
    try:	
        return vpp_if.get_pppox_conn_by_host_ifidx(host_ifidx).get('pppox_sw_if_index')
    except (AttributeError, OSError) as e:
        return None

def get_statistics(name):
    statistics_copy = []
    padding = 0
    pppox_ifidx = get_pppox_ifidx(name)

    if pppox_ifidx is None:
        return

    pppox = vpp_if.get_if_by_ifidx(pppox_ifidx).get('ifname')
    statistics = subprocess.check_output(shlex.split(VPP_SH_IFACE_COMMAND + pppox)).decode('utf-8').splitlines()

    first_line = statistics.pop(0).split()
    # cut unnecessary items
    del first_line[1]
    del first_line[3]

    first_line_len = len(first_line)
    statistics_len = len(statistics)

    if statistics_len > 0:

        statistics_copy = statistics[0].split()
        del statistics_copy[1]
        statistics_copy = ' '.join(statistics_copy)
        statistics[0] = statistics_copy
        statistics_copy = []
        statistics_len = len(statistics)

        for i in range(statistics_len):
            statistics[i] = statistics[i].replace('tx bytes', 'tx_bytes')
            statistics[i] = statistics[i].replace('tx packets', 'tx_packets')
            statistics[i] = statistics[i].replace(pppox, name)
            statistics_line = statistics[i].split()
            less_colls_lines = first_line_len - len(statistics_line) if first_line_len - len(statistics_line) else 0
            for y in range(less_colls_lines):
                padding += len(first_line[y])
                if y == less_colls_lines - 1:
                    y += 1
                    padding += first_line[y].count(' ') + less_colls_lines
            #add padding to lines like 'tx bytes', i.e. lines starting from the forth collumn
            first_item = 0
            statistics_line[first_item] = ''.join((padding*' ', statistics_line[first_item]))
            padding = 0
            first_item += 1
            for word in range(less_colls_lines + 1, first_line_len):
                offset = (len(first_line[word - 1]) - first_line[word - 1].count(' ')) - (
                            len(statistics_line[first_item - 1]) - statistics_line[first_item - 1].count(' '))
                offset += first_line[word].count(' ')
                if offset > 0:
                    statistics_line[first_item] = ''.join((offset*' ', statistics_line[first_item]))
                else:
                    for up_rows in range(i - 1, -1, -1):
                        print("IN")
                        statistics_copy[up_rows][word] = ''.join((-offset * ' ', statistics_copy[up_rows][word]))
                    first_line[word] = ''.join((-offset*' ', first_line[word]))
                first_item += 1
            statistics_copy.append(statistics_line)

    for word in first_line:
        print(word, end=' ')
    print('\n', end='')
    for line in statistics_copy:
        for word in line:
            word = word.replace('tx_bytes', 'tx bytes')
            word = word.replace('tx_packets', 'tx packets')
            word = word.replace('/0/0/0', 6*' ')
            print(word, end=' ')
        print('\n', end='')

def get_ppp_err(pppoe_client, pppox_session):
    err_msg = ''
    # if pppoe_client['state'] == 'PPPOE_CLIENT_DISCOVERY' or pppoe_client['state'] == 'PPPOE_CLIENT_REQUEST':
    if pppoe_client['err_code'] == 0 and pppox_session['err_code'] == 0:
            err_msg = pppox_session['err_reason']

    elif pppoe_client['err_code'] != 0:
        err_msg = pppoe_client['err_reason']
        if pppox_session['err_code'] != 0:
            err_msg += ('\n' + INDENT + len('state ' )*' ' + pppox_session['err_reason'])
    else:
        if pppox_session['err_code'] != 0:
            err_msg = pppox_session['err_reason']

    return err_msg

def get_status(name):
    err_str = ''
    sri = sr.SysrepoInterfaces(app_name='show-pppoe')
    ether_iface = sri.get_interface_param(name, 'ethernet-interface')
    pppox_ifidx = get_pppox_ifidx(name)

    if pppox_ifidx is None:
        return

    pppoe_client = vpp_if.get_pppoe_client_by_pppox_ifidx(pppox_ifidx)
    pppox_session = vpp_if.get_pppox_conn_by_pppox_ifidx(pppox_ifidx)
    err_str = get_ppp_err(pppoe_client, pppox_session)

    print("Interface {}".format(name))
    print(INDENT + 'state ' + pppoe_client['state'])
    if err_str != '':
        print(INDENT + len('state ' )*' ' + err_str)
    print(INDENT + 'session-id ' + str(pppoe_client['session_id']))
    print(INDENT + 'ethernet-interface ' + ether_iface)
    print(INDENT + 'ac-mac-address ' + pppoe_client['ac_mac_addr'])
    print(INDENT + 'ip4-address ' + pppox_session['local_address'])

if __name__ == "__main__":
    argc = len(sys.argv)

    if 2 > argc:
        syslog.syslog(syslog.LOG_ERR, "No enough arguments. Exit!")
        sys.exit(1)
    if sys.argv[2] == "statistics":
        get_statistics(sys.argv[1])
        sys.exit(0)
    elif sys.argv[2] == "status":
        get_status(sys.argv[1])

sys.exit(0)
