#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import syslog
import subprocess


def get_ipsvpn_if_status():
    try:
        status = subprocess.check_output("sudo ipsec statusall", shell=True)
        status = status.decode("utf-8")
        data = str(status).split('\n')
        imp_data = ""
        readImportant = False
        for i in range(0, len(data)):
            if 'Connections:' in data[i]:
                readImportant = True
            if readImportant:
                imp_data += data[i] + '\n'
        return str(imp_data)
    except:
        return ""


def transform_proposals(proposals):
    DH_GROUP = dict(
        [('modp768',1 ), ('modp1024', 2), ('modp1536', 5), ('modp2048', 14), ('modp3072', 15), ('modp4096', 16),
         ('modp6144', 17),
         ('modp8192', 18), ('modp1024s160', 22), ('modp2048s224', 23), ('modp2048s256', 24), ('ecp192', 25),
         ('ecp224', 26),
         ('ecp256', 19), ('ecp384', 20), ('ecp521', 21), ('ecp224bp', 27), ('ecp256bp', 28), ('ecp384bp', 29),
         ('ecp512bp', 30), ('curve25519', 31)])
    SA_ENCR_ALG = dict([('DES_CBC', 'des'), ('3DES_CBC', '3des'), ('AES_CBC_128', 'aes128'), ('AES_CBC_192', 'aes192'), ('AES_CBC_256', 'aes256')])
    SA_INTEGR_ALG = dict([('HMAC_MD5_96', 'md5'), ('HMAC_SHA1_96', 'sha1'), ('HMAC_SHA2_256_128', 'sha256'),
                          ('HMAC_SHA2_384_192', 'sha384'), ('HMAC_SHA2_512_256', 'sha512')])

    params = str(proposals).split('/')
    if len(params) > 1:
        if len(params) > 3:
            for i in range(0, len(params)):
                if 'PRF' in params[i]:
                    del params[i]
                    break

        params[0] = SA_ENCR_ALG.get(params[0])
        params[1] = SA_INTEGR_ALG.get(params[1])
        params[2] = DH_GROUP.get(params[2].split('_')[0].lower() + params[2].split('_')[1].lower())
        return params[0] + "/" + params[1] + " DH group: " + str(params[2])
    else:
        return ""


def status_ipsecvpn(ifname):
    # data = dict()
    result = ""
    data = get_ipsvpn_if_status()
    if data is "":
        print('No active security associations for interface \'{}\''.format(ifname))
        sys.exit(0)

    params = data.split('\n')
    for i in range (0, len(params)):
        if 'Security Associations (' in params[i]:
            if 'none' in params[i+1]:
                print('No active security associations for interface \'{}\''.format(ifname))
                sys.exit(0)

    all_sa_list_of_dicts = []
    all_sa_list_of_lists = []
    sa_list = []
    sa_meet_flag = False
    for i in range (0, len(params)):
        if sa_meet_flag:
            sa_list.append(params[i])
            if "===" in params[i]:
                all_sa_list_of_lists.append(sa_list.copy())
                sa_list.clear()

        if 'Security Associations (' in params[i]:
            sa_meet_flag = True

    sa_dict = {}

    for i in range (0, len(all_sa_list_of_lists)):
        # do stuf
        sa_dict['Connection'] = None
        sa_dict['Local'] = None
        sa_dict['Local_id'] = None
        sa_dict['Remote'] = None
        sa_dict['Remote_id'] = None
        sa_dict['Virt_local'] = None
        sa_dict['Virt_remote'] = None
        sa_dict['State'] = None
        sa_dict['IKEv'] = None
        sa_dict['IKE_proposal_ph1'] = None
        sa_dict['Child'] = None
        sa_dict['Type'] = None
        sa_dict['Protocol'] = None
        sa_dict['IKE_proposal_ph2'] = None

        sa_dict['Pkt-in'] = None
        sa_dict['Pkt-out'] = None
        sa_dict['bytes-in'] = None
        sa_dict['bytes-out'] = None

        for k in range(0, len(all_sa_list_of_lists[i])):
            line = all_sa_list_of_lists[i][k]
            if "ESTABLISHED" in line and "ago," in line:
                sa_dict['Local'] = line.split(',')[1].split('[')[0].strip()
                sa_dict['Local_id'] = line.split('...')[0].split('[')[2].strip(']')
                sa_dict['Remote'] = line.split('...')[1].split('[')[0].strip('[')
                sa_dict['Remote_id'] = line.split('...')[1].split('[')[1].strip(']')
                sa_dict['Connection'] = line.split(':')[0].strip().strip(':')
                sa_dict['Remote_id']
                sa_dict['State'] = line.split(':')[1].split(',')[0]
            if "IKEv" in line and "rekeying in" in line:
                sa_dict['IKEv'] = line.split('IKEv')[1].split(' ')[0]
                sa_dict['State'] = sa_dict['State'] + ", rekeying in" + line.split('rekeying in')[1]
            if "IKE proposal: " in line:
                sa_dict['IKE_proposal_ph1'] = transform_proposals(line.split('IKE proposal: ')[1])
            if "TUNNEL, reqid" in line:
                sa_dict['Child'] = line.split(':')[0].strip()
                sa_dict['Type'] = line.split(':')[1].split('reqid')[0].strip().strip(',')
                if "expires in" in line.split(',')[3]:
                    sa_dict['Protocol'] = ""
                else:
                    sa_dict['Protocol'] = line.split(',')[3].split(' ')[1]
            if "bytes_i" in line and "bytes_o" in line:
                sa_dict['IKE_proposal_ph2'] = transform_proposals(line.split(':')[1].split(',')[0].strip())
                sa_dict['Pkt-out'] = 0
                sa_dict['Pkt-in'] = 0

                sa_dict['bytes-in'] = line.split('bytes_i')[0].split(',')[1].strip()
                sa_dict['bytes-out'] = line.split("bytes_o")[0].strip().split(" ")[-1]

                temp = line.split('bytes_o')[1]
                if "pkts" in temp:
                    sa_dict['Pkt-out'] = temp.split('pkts')[0].strip().strip('(')
                
                temp = line.split('bytes_i')[1]
                if "pkts" in temp:
                    sa_dict['Pkt-in'] = temp.split('pkts')[0].strip().strip('(')

            if "===" in line and sa_dict['Child'] in line:
                elem_num = len(line.split(':')) - 1
                sa_dict['Virt_local'] = line.split(':')[elem_num].split('===')[0].strip()
                sa_dict['Virt_remote'] = line.split(':')[elem_num].split('===')[1].strip()

        all_sa_list_of_dicts.append(sa_dict.copy())

    for j in range(0, len(all_sa_list_of_dicts)):
        if all_sa_list_of_dicts[j]['Connection'] is not None:
            result += "\tConnection: {}\n ".format(all_sa_list_of_dicts[j]['Connection'])
        if all_sa_list_of_dicts[j]['Local'] is not None and all_sa_list_of_dicts[j]['Local_id'] is not None:
            result += "\tLocal: {}, ID: {}\n".format(all_sa_list_of_dicts[j]['Local'], all_sa_list_of_dicts[j]['Local_id'])
        if all_sa_list_of_dicts[j]['Remote'] is not None and all_sa_list_of_dicts[j]['Remote_id'] is not None:
            result += "\tRemote: {}, ID: {}\n".format(all_sa_list_of_dicts[j]['Remote'], all_sa_list_of_dicts[j]['Remote_id'])
        if all_sa_list_of_dicts[j]['State'] is not None:
            result += "\tState: {} \n".format(all_sa_list_of_dicts[j]['State'])
        if all_sa_list_of_dicts[j]['IKEv'] is not None and  all_sa_list_of_dicts[j]['IKE_proposal_ph1'] is not None:
            result += "\tIKEv{}, IKE proposal: {}\n".format(all_sa_list_of_dicts[j]['IKEv'], all_sa_list_of_dicts[j]['IKE_proposal_ph1'])
        if all_sa_list_of_dicts[j]['Child'] is not None and all_sa_list_of_dicts[j]['Type'] is not None and all_sa_list_of_dicts[j]['Protocol'] is not None:
            result += "\tChild: {}, {}, protocol:{}\n".format(all_sa_list_of_dicts[j]['Child'], all_sa_list_of_dicts[j]['Type'], all_sa_list_of_dicts[j]['Protocol'])
        if all_sa_list_of_dicts[j]['Virt_local'] is not None and all_sa_list_of_dicts[j]['Virt_remote'] is not None:
            result += "\t\t[\'{}\'] === [\'{}\'] \n".format(all_sa_list_of_dicts[j]['Virt_local'], all_sa_list_of_dicts[j]['Virt_remote'])
        if all_sa_list_of_dicts[j]['IKE_proposal_ph2'] is not None:
            result += "\t\tIKE proposal {}\n".format(all_sa_list_of_dicts[j]['IKE_proposal_ph2'])
        if all_sa_list_of_dicts[j]['Pkt-in'] is not None and all_sa_list_of_dicts[j]['bytes-in'] is not None:
            result += "\t\tPkt-in: {},\tbytes-in: {}\n".format(all_sa_list_of_dicts[j]['Pkt-in'], all_sa_list_of_dicts[j]['bytes-in'])
        if all_sa_list_of_dicts[j]['Pkt-out'] is not None and all_sa_list_of_dicts[j]['bytes-out'] is not None:
            result += "\t\tPkt-out: {},\tbytes-out: {}\n".format(all_sa_list_of_dicts[j]['Pkt-out'], all_sa_list_of_dicts[j]['bytes-out'])

        print(result.strip('\n'))
        result = ""


if __name__ == "__main__":
    argc = len(sys.argv)
    if 2 > argc:
        syslog.syslog(syslog.LOG_ERR, "No enough arguments. Exit!")
        sys.exit(1)

    if sys.argv[1] == "ipsec":
        if argc == 3:
            status_ipsecvpn(sys.argv[2])
        else:
            syslog.syslog(syslog.LOG_ERR, "Missing interface name! Exit!")
            sys.exit(0)
    else:
        syslog.syslog(syslog.LOG_ERR, "Unknown argument \"%s\"" % sys.argv[1])
        sys.exit(1)

    sys.exit(0)


