#!/usr/bin/env python3
import sys
import syslog
import subprocess
import json

def converthtz(val):
    KHz = 1000
    MHz = 1000000
    GHz = 1000000000

    if val < KHz:
        return '{0}'.format(val,'Hz')
    elif KHz <= val < MHz:
        return '{0}KHz'.format(val/KHz)
    elif MHz <= val:
        return '{0}MHz'.format(val/MHz)


def convertbytes(B):
    KB = 1000
    MB = 1000000
    GB = 1000000000
    TB = 1000000000000

    if B < KB:
        return '{0} {1}'.format(B,'Bytes' if 0 == B > 1 else 'Byte')
    elif KB <= B < MB:
        return '{0}K'.format(B/KB)
    elif MB <= B < GB:
        return '{0}M'.format(B/MB)
    elif GB <= B < TB:
        return '{0}G'.format(B/GB)
    elif TB <= B:
        return '{0}T'.format(B/TB)


def convertfbytes(B):
  B = float(B)
  KB = float(1024)
  MB = float(KB ** 2) # 1,048,576
  GB = float(KB ** 3) # 1,073,741,824
  TB = float(KB ** 4) # 1,099,511,627,776

  if B < KB:
    return '{0} {1}'.format(B,'Bytes' if 0 == B > 1 else 'Byte')
  elif KB <= B < MB:
    return '{0:g}KiB'.format(B/KB)
  elif MB <= B < GB:
    return '{0:g}MiB'.format(B/MB)
  elif GB <= B < TB:
    return '{0:g}GiB'.format(B/GB)
  elif TB <= B:
    return '{0:g}TiB'.format(B/TB)


def show_hw_details():
  command = 'sudo /usr/bin/lshw -json -quiet'
  res = subprocess.check_output ( command , shell=True)
  result = res.decode('utf8')
  cfg = json.loads(result)
  print ("Hardware information")
  print (" class: " + cfg['id'])
  print (" description: " + cfg['description'])
  print (" product: " + cfg['product'])
  if 'vendor' in cfg:
      print (" vendor: " + cfg['vendor'])
  if 'version' in cfg:
    print (" version: " + cfg['version'])
  if 'serial' in cfg:
    print (" serial: " + cfg['serial'])
  print (" width: " + str(cfg['width']) + " bits")

  print (" capabilities: " + ", ".join(cfg['capabilities']))
  for childs in cfg['children']:
    print (" *-" + childs['id'])
    if 'description' in childs:
        print ("  description: " + childs['description'])
    if ('product' in childs):
      print ("  product: " + childs['product'])
    if ('vendor' in childs):
      print ("  vendor: " + childs['vendor'])
      print ("  physical id: " + childs['physid'])
    if ('logicalname' in childs):
      print ("  logical name: " + str(childs['logicalname']))
    if ('version' in childs):
      print ("  version: " + childs['version'])
    if 'serial' in childs:
      print ("  serial: " + childs['serial'])
    if ('size' in childs):
      print ("  size: " + convertbytes(childs['size']) + str(childs['units']))
    if ('slot' in childs):
      print ("  slot: " + childs['slot'])
      if ('capacity' in childs):
        if (childs['units'] == 'mWh'):
          print ("  capacity: " + str(childs['capacity']) + childs['units'])
        else:
          print ("  capacity: " + convertbytes(childs['capacity']) + childs['units'])
    if ('capabilities' in childs):
      print ("  capabilities: " + ', '.join(childs['capabilities']))

    config = "  configuration: "
    if ('configuration' in childs):
      if ('autonegotiation' in childs['configuration']):
        config = config + "autonegotiation=" + childs['configuration']['autonegotiation'] + " "
      if ('broadcast' in childs['configuration']):
        config = config + "broadcast=" + childs['configuration']['broadcast'] + " "
      if ('driver' in childs['configuration']):
        config = config + "driver=" + childs['configuration']['driver'] + " "
      if ('driverversion' in childs['configuration']):
        config = config + "driverversion=" + childs['configuration']['driverversion'] + " "
      if ('duplex' in childs['configuration']):
        config = config + "duplex=" + childs['configuration']['duplex'] + " "
      if ('link' in childs['configuration']):
        config = config + "link=" + childs['configuration']['link'] + " "
      if 'multicast' in childs['configuration']:
        config = config + "multicast=" + childs['configuration']['multicast'] + " "
      if ('port' in childs['configuration']):
        config = config + "port=" + childs['configuration']['port'] + " "
      if ('speed' in childs['configuration']):
        config = config + "speed=" + childs['configuration']['speed'] + " "
      print (config)

    if ('children' in childs):
      for child1 in childs['children']:
        print ("   *-" + child1['id'])
        print ("      class: " + child1['class'])
        if ('description' in child1):
          print ("      description: " + child1['description'])
        if ('product' in child1):
          print ("      product: " + child1['product'])
        if ('vendor' in child1):
          print ("      vendor: " + child1['vendor'])
        print ("      physical id: " + child1['physid'])
        if ('version' in child1):
          print ("      version: " + child1['version'])
          if ('date' in child1):
            print ("      date: " + child1['date'])
        if ('slot' in child1):
          print ("      slot: " + child1['slot'])
        if ('size' in child1):
          if (child1['id'] == "firmware" or child1['id'] == "memory" or 'cache' in child1['id']):
            print ("      size: " + convertfbytes(child1['size']))
          else:
            print ("      size: " + converthtz(child1['size']))
        if ('capacity' in child1):
          if (child1['id'] == "firmware" or 'cache' in child1['id']):
            print ("      capacity: " + convertbytes(child1['capacity']))
          else:
            print ("      capacity: " + converthtz(child1['capacity']))
        if ('width' in child1):
          print ("      width: " + str(child1['width']) + " bits")
        if ('clock' in child1):
          print ("      clock: " + converthtz(child1['clock']))
        if ('logicalname' in child1):
          print ("      logical name: " + child1['logicalname'])
        if ('capabilities' in child1):
          print  ("      capabilities: " + ', '.join(child1['capabilities']))

        config = "      configuration: "
        if ('configuration' in child1):
          if ('cores' in child1['configuration']):
            config = config + "cores=" +child1['configuration']['cores'] + " "
          if ('enabledcores' in child1['configuration']):
            config = config + "enabledcores=" +child1['configuration']['enabledcores'] + " "
          if ('threads' in child1['configuration']):
            config = config + "threads=" +child1['configuration']['threads'] + " "
          print (config)

        if ('children' in child1):
          for child2 in child1['children']:
            print ("       *-" + child2['id'])
            if ('description' in child2):
              print ("         description: " + child2['description'])
            if ('product' in child2):
              print ("         product: " + child2['product'])
            if ('vendor' in child2):
              print ("         vendor: " + child2['vendor'])
            print ("         physical id: " + child2['physid'])
            if ('serial' in child2):
              print ("         serial: " + child2['serial'])
            if ('slot' in child2):
              print ("         slot: " + child2['slot'])
            if ('size' in child2):
              print ("         size: " + convertfbytes(child2['size']))
            if ('capacity' in child2):
              print ("         capacity: " + convertbytes(child2['capacity']) + child2['units'])
            if ('width' in child2):
              print ("         width: " + str(child2['width']) + " bits")
            if ('clock' in child2):
              print ("         clock: " + converthtz(child2['clock']))
            if ('capabilities' in child2):
              print ("         capabilities: " + ', '.join(child2['capabilities']))

            config = "         configuration: "
            if ('configuration' in child2):
              if ('autonegotiation' in child2['configuration']):
                config = config + "autonegotiation=" + child2['configuration']['autonegotiation'] + " "
              if ('broadcast' in child2['configuration']):
                config = config + "broadcast=" + child2['configuration']['broadcast'] + " "
              if ('driver' in child2['configuration']):
                config = config + "driver=" + child2['configuration']['driver'] + " "
              if ('driverversion' in child2['configuration']):
                config = config + "driverversion=" + child2['configuration']['driverversion'] + " "
              if ('firmware' in child2['configuration']):
                config = config + "firmware=" + child2['configuration']['firmware'] + " "
              if ('latency' in child2['configuration']):
                config = config + "latency=" + child2['configuration']['latency'] + " "
              if ('link' in child2['configuration']):
                config = config + "link=" + child2['configuration']['link'] + " "
              if ('multicast' in child2['configuration']):
                config = config + "multicast=" + child2['configuration']['multicast'] + " "
              if ('port' in child2['configuration']):
                config = config + "port=" + child2['configuration']['port'] + " "
              if ('level' in child2['configuration']):
                config = config + "level=" + child2['configuration']['level'] + " "
              print (config)

            if ('children' in child2):
              for child3 in child2['children']:
                print ("        *-" + child3['id'])
                if ('description' in child3):
                  print ("          description: " + child3['description'])
                if ('product' in child3):
                  print ("          product: " + child3['product'])
                if ('vendor' in child3):
                  print ("          vendor: " + child3['vendor'])
                print ("          physical id: " + child3['physid'])
                if ('logicalname' in child3):
                  if isinstance(child3['logicalname'], list):
                    res = ""', '.join(child3['logicalname'])
                    print ("          logical name: " + res)
                  else:
                    print ("          logical name: " + str(child3['logicalname']))
                if ('version' in child3):
                  print ("          version: " + child3['version'])
                if ('serial' in child3):
                  print ("          serial: " + child3['serial'])
                if ('slot' in child3):
                  print ("          slot: " + child3['slot'])
                if ('size' in child3):
                  print ("          size: " + convertbytes(child3['size']))
                if ('capacity' in child3):
                  print ("          capacity: " + convertbytes(child3['capacity']))
                if ('width' in child3):
                  print ("          width: " + str(child3['width']) + " bits")
                if ('clock' in child3):
                  print ("          clock: " + converthtz(child3['clock']))
                if ('capabilities' in child3):
                  print  ("          capabilities: " + ', '.join(child3['capabilities']))

                config = "          configuration: "
                if ('configuration' in child3):
                  if ('autonegotiation' in child3['configuration']):
                    config = config + "autonegotiation=" + child3['configuration']['autonegotiation'] + " "
                  if ('broadcast' in child3['configuration']):
                    config = config + "broadcast=" + child3['configuration']['broadcast'] + " "
                  if ('driver' in child3['configuration']):
                    config = config + "driver=" + child3['configuration']['driver'] + " "
                  if ('slots' in child3['configuration']):
                    config = config + "slots=" + child3['configuration']['slots'] + " "
                  if ('speed' in child3['configuration']):
                    config = config + "speed=" + child3['configuration']['speed'] + " "
                  if ('driverversion' in child3['configuration']):
                    config = config + "driverversion=" + child3['configuration']['driverversion'] + " "
                  if ('firmware' in child3['configuration']):
                    config = config + "firmware=" + child3['configuration']['firmware'] + " "
                  if ('latency' in child3['configuration']):
                    config = config + "latency=" + child3['configuration']['latency'] + " "
                  if ('link' in child3['configuration']):
                    config = config + "link=" + child3['configuration']['link'] + " "
                  if ('multicast' in child3['configuration']):
                    config = config + "multicast=" + child3['configuration']['multicast'] + " "
                  if ('port' in child3['configuration']):
                    config = config + "port=" + child3['configuration']['port'] + " "
                  if ('created' in child3['configuration']):
                    config = config + "created=" + child3['configuration']['created'] + " "
                  if ('filesystem' in child3['configuration']):
                    config = config + "filesystem=" + child3['configuration']['filesystem'] + " "
                  if ('lastmountpoint' in child3['configuration']):
                    config = config + "lastmountpoint=" + child3['configuration']['lastmountpoint'] + " "
                  if ('modified' in child3['configuration']):
                    config = config + "modified=" + child3['configuration']['modified'] + " "
                  if ('mount.fstype' in child3['configuration']):
                    config = config + "mount.fstype=" + child3['configuration']['mount.fstype'] + " "
                  if ('mount.options' in child3['configuration']):
                    config = config + "mount.options=" + child3['configuration']['mount.options'] + " "
                  if ('mounted' in child3['configuration']):
                    config = config + "mounted=" + child3['configuration']['mounted'] + " "
                  if ('state' in child3['configuration']):
                    config = config + "state=" + child3['configuration']['state'] + " "
                  print (config)

                  if ('children' in child3):
                    for child4 in child3['children']:
                      print ("         *-" + child4['id'])
                      if ('description' in child4):
                        print ("           description: " + child4['description'])
                      if ('vendor' in child4):
                        print ("           vendor: " + child4['vendor'])
                      print ("           physical id: " + child4['physid'])
                      if ('version' in child4):
                        print ("           version: " + child4['version'])
                      if ('capabilities' in child4):
                        print ("           capabilities: " + ', '.join(child4['capabilities']))

                      config = "           configuration: "
                      if ('configuration' in child4):
                        if ('driver' in child4['configuration']):
                          config = config + "driver=" + child4['configuration']['driver'] + " "
                        if ('slots' in child4['configuration']):
                          config = config + "slots=" + child4['configuration']['slots'] + " "
                        if ('speed' in child4['configuration']):
                          config = config + "speed=" + child4['configuration']['speed'] + " "
                        print (config)


def show_device_details(details):
    command = 'sudo /usr/bin/lshw -json -quiet'
    configuration = str()
    try:
        configuration = subprocess.check_output ( command , shell=True)
    except subprocess.CalledProcessError as e:
        print ( "Command failed with code %d" % (e.returncode) )
    except OSError as e:
        print ( "System error %s" % (e.strerror) )

    result = configuration.decode('UTF-8').replace ('\n','')
    cfg = json.loads(result)

    command = 'sudo /bin/lsblk -OJb'
    configuration = str()
    try:
        configuration = subprocess.check_output ( command , shell=True)
    except subprocess.CalledProcessError as e:
        print ( "Command failed with code %d" % (e.returncode) )
    except OSError as e:
        print ( "System error %s" % (e.strerror) )

    result = configuration.decode('UTF-8').replace ('\n','')
    cfg_bak = json.loads(result)

    get_serial_number_command = 'sudo cat /config/.serial-number'
    serial_number_string = str()
    try:
        serial_number_string = subprocess.check_output ( get_serial_number_command , shell=True)
    except subprocess.CalledProcessError as e:
        print ( "Command failed with code %d" % (e.returncode) )
    except OSError as e:
        print ( "System error %s" % (e.strerror) )

    hw_type = 'Unknown'
    try:
        hw_type_string = subprocess.check_output ( "sudo cat /config/.hwtype" , shell=True)
        hw_type = hw_type_string.decode('UTF-8').replace ('\n','')
    except:
        pass

    serial_number = serial_number_string.decode('UTF-8').replace ('\n','')
    i = 1

    print('-' * 50)
    first_dirty_string="(To be filled by O.E.M.)"
    product_dirty_string = cfg['product']
    product_string = product_dirty_string.replace ( first_dirty_string, "" )
    product_string = product_string.replace("LS1046A RDB Board", "HawkEye 6010")
    arm_cpu_flag = False
    if 'vendor' in cfg:
      print(" Device type\t\t: " + cfg['vendor'] + " " + product_string)
    else:
      print(" Device type\t\t: " + product_string)
    print(" Serial number\t\t: " + serial_number)
    print(" Hardware type\t\t: " + hw_type)
    if details == 1:
        for childs in cfg['children']:
            if ('children' in childs):
                for child1 in childs['children']:
                    if (child1['id'] == "cpu") or ('cpu:' in child1['id'] and not 'disabled' in child1):
                        if 'product' in child1:
                            if child1['product'] == "cpu" and product_string == "HawkEye 6010":
                                if not arm_cpu_flag:
                                    print('-' * 50)
                                    print(" CPU Type\t\t: LS1046A/1.6GHz/4 cores/ARM-A72")
                                    arm_cpu_flag = True
                                else:
                                    continue
                            elif (child1['product'] in ['cpu', 'cpu-pw20'] and
                                    product_string == "NSA3640 based NXP2088"):
                                if not arm_cpu_flag:
                                    print('-' * 50)
                                    print(" CPU Type\t\t: LS2088A/1.8GHz/8 cores/ARM-A72")
                                    arm_cpu_flag = True
                                else:
                                    continue
                            else:
                                print('-' * 50)
                                print(" CPU Type\t\t: " + child1['product'])
                            if ('configuration' in child1):
                                if 'threads' in child1['configuration']:
                                    print(" CPU Cores\t\t: " + child1['configuration']['cores'] + "C / " + child1['configuration']['threads'] +"T") 
                                else:
                                    print(" CPU Cores\t\t: " + child1['configuration']['cores'] + "C")

                    if (child1['id'] == "memory"):
                        print('-' * 50)
                        print (" RAM Size\t\t: " + convertfbytes(child1['size']))
                        if 'children' in child1:
                            for child2 in child1['children']:
                                #if ('description' in child1):
                                print (" RAM Type\t\t: " + child2['id'] + " " + child2['description'])
                                #If we want the RAM in single line
                                #if (child1['children']):
                                #print" TEST test\t\t: " + child1['children'][0]['description']

                    if (child1['id'] == "scsi"):
                        for child2 in child1['children']:
                            print('-' * 50)
                            print (" Disk #" + str(i) + " Type \t\t: "  + child2['description'])
                            if 'size' in child2.keys():
                                print (" Disk #" + str(i) + " Size \t\t: "  + convertbytes(child2['size'])+"B"+ " ("+convertfbytes(child2['size'])+")")

                            print (" Disk #" + str(i) + " Model \t\t: " + child2['product'])
                            i = i + 1

                    if (child1['id'] == "pci"):
                        for child2 in child1['children']:
                            if (child2['id'] == "scsi"):
                                for child3 in child2['children']:
                                    print('-' * 50)
                                    print (" Disk #" + str(i) + " Type \t\t: "  + child3['description'])
                                    if 'size' in child3.keys():
                                        print (" Disk #" + str(i) + " Size \t\t: "  + convertbytes(child3['size'])+"B" + " ("+convertfbytes(child3['size'])+")")

                                    if 'product' in child3.keys():
                                        print (" Disk #" + str(i) + " Model \t\t: " + child3['product'])

                                    i = i + 1

                    if i == 1 and arm_cpu_flag:
                        for child3 in cfg_bak['blockdevices']:
                            if (child3['hotplug'] == "0"):
                                if 'size' in child3.keys() and child3['size'] is not None and 'model' in child3.keys() and child3['model'] is not None:
                                    print('-' * 50)
                                    print (" Disk #" + str(i) + " Name \t\t: "  + child3['name'])
                                    print (" Disk #" + str(i) + " Size \t\t: "  + convertbytes(int(child3['size']))+"B" + " ("+convertfbytes(int(child3['size']))+")")
                                    print (" Disk #" + str(i) + " Model \t\t: " + child3['model'])

                                i = i + 1

if __name__ == "__main__":
    argc = len( sys.argv )
    if 1 > argc:
        syslog.syslog( syslog.LOG_ERR, "No enough arguments. Exit!" )
        sys.exit( 1 )

    if sys.argv[1] == "hardware":
        show_hw_details()
        sys.exit( 0 )
    elif sys.argv[1] == "device":
        if argc == 3:
            show_device_details(1)
        else:
            show_device_details(0)
            sys.exit( 0 )
    else:
        syslog.syslog( syslog.LOG_ERR, "Unknown argument \"%s\"" % sys.argv[1] )
        sys.exit( 1 )

sys.exit( 0 )
