#!/bin/bash
set -e # EXIT IMMEDIATELY UPON ERROR!
source /scripts/basic_vars
source /scripts/basic_functions

SCRIPTS_PATH=$2

function show_status() {
  j=0
  loads=($(mpstat -P ALL 1 1 | awk '/Average:/ {print 100-$NF}'))
  for i in "${loads[@]}"; do
    :
    if [[ $j == 0 ]]; then
      j=1
      continue
    fi
    if [[ $j == 1 ]]; then
      printf " CPU average load: %s\n" "$i""%"
      j=2
      echo " CPU usage per core:"
      continue
    fi
    printf ' Cpu%d: %s\n' $((j-2)) "$i""%"
    j=$((j+1))
  done

  HPSIZE=`cat /proc/meminfo  | grep Hugepagesize | awk {'print $2'}`
  HPSIZE=$((HPSIZE/1024))
  HPTOTAL=`cat /proc/meminfo  | grep HugePages_Total | awk {'print $2'}`
  HPFREE=`cat /proc/meminfo  | grep HugePages_Free | awk {'print $2'}`
  HP_MEMFREE=$((HPFREE * ${HPSIZE}))
  HP_MEMTOTAL=$((HPTOTAL * ${HPSIZE}))
  free -m | awk -v memused="$HP_MEMFREE" 'NR==2{printf " Total RAM usage: %s/%sMB (%.2f%%)\n", $3-memused,$2,($3-memused)*100/$2 }'
    echo "$HPTOTAL $HPFREE $HP_MEMTOTAL $HP_MEMFREE" | awk '{printf " HugePages usage: %s/%s (%s/%sMB)\n", $1-$2, $1, $3-$4, $3 }'

  echo "$HPSIZE" | awk {'printf " HugePages size: %sMB\n", $1'}
  df -h | awk '$NF=="/"{printf " Disk Usage: %s/%s (%s)\n", $3,$2,$5}'
  date=($(date))
  echo " Current time: $(date)"
}

function show_services() {
  if [[ $1 == "no-paginate" ]]; then
    systemctl list-units -t service --no-pager | awk '{if (system("echo " $1 " | grep -f /config/crit_services 1>/dev/null") == 0) print $0 " Critical: Yes"; else if ($1 ~ ".service") print $0 " Critical: No"; else print $0}'
  elif [[ $1 == "csv" ]]; then
    systemctl list-units -t service --no-legend --no-pager | tr -s ' ' | awk '{gsub(",","");ORS=""; if ($1 == "●" || $1 == "*") {print $2 "," $3 "," $4 "," $5 ","; for(i=6; i<=NF; i++) print $i" "; print ","; if (system("echo " $2 " | grep -f /config/crit_services 1>/dev/null") == 0) {print "Yes";} else {print "No";}} else {print $1 "," $2 "," $3 "," $4 ","; for(i=5; i<=NF; i++) print $i" "; print ","; if (system("echo " $1 " | grep -f /config/crit_services 1>/dev/null") == 0) {print "Yes";} else {print "No";}} print "\n"}' | sed -e 's/ *,/,/g'
  elif [[ $2 == "csv" ]]; then
    systemctl list-units -t service --no-legend --no-pager | tr -s ' ' | awk '$4 ~ /'$1'/{gsub(",","");ORS=""; if (($1 == "●" || $1 == "*") || system("echo " $2 " | grep /config/crit_services 1>/dev/null") == 0) {print $2 "," $3 "," $4 "," $5 ","; for(i=6;i<=NF;i++) {print $i" "}; print ","; if (($1 == "●" || $1 == "*") || system("echo " $2 " | grep /config/crit_services 1>/dev/null") == 0) {print "Yes";} else {print "No";}} else {print $1 "," $2 "," $3 "," $4 ","; for(i=5;i<=NF;i++) {print $i" "}; print ","; if (system("echo " $1 " | grep -f /config/crit_services 1>/dev/null") == 0) {print "Yes";} else {print "No";}} print "\n"}' | sed -e 's/ *,/,/g'
   elif [ ! -z $1 ]; then
    systemctl list-units -t service --no-pager | awk 'NR==1 || $4 ~ /'$1'/ {if (system("echo " $1 " | grep -f /config/crit_services 1>/dev/null") == 0) print $0 " Critical: Yes"; else if ($1 ~ ".service") print $0 " Critical: No"; else print $0}'
  else
    systemctl list-units -t service --no-pager | awk '{if ($1 ~ /[*●]/) {if (system("echo " $2 " | grep -f /config/crit_services 1>/dev/null") == 0) {print " " substr($0,  2) " Critical: Yes";} else if ($1 ~ ".service") {print " "  substr($0,  2) " Critical: No";} else {print $0;}} else {if (system("echo " $1 " | grep -f /config/crit_services 1>/dev/null") == 0) {print $0 " Critical: Yes";} else if ($1 ~ ".service") {print $0 " Critical: No";} else {print $0;}}}'
  fi
}

function show_hw() {
  ${SCRIPTS_PATH}/show-system.py hardware
}

function show_device() {
  ${SCRIPTS_PATH}/show-system.py device $1
}

function show_sensors() {
  sensorFile=$(mktemp)
  if [ -f ${sensorFile} ]; then
    sudo sensors 2>&1 > ${sensorFile}
    if [ "" == "$(grep 'Core 14' ${sensorFile})" ]; then
      sudo sensors 2>&1 | {
        while read line
        do
        if [ "No sensors found!" == "$line" ]; then
          echo "$line"
          exit 1
        fi
        echo "$line"
        done
      }
    else
      awk '
BEGIN {}
/Core 0:/ {print}
/Core 2:/ {print "Core 1:        " $3 "  " $4 " " $5 " " $6 " " $7 " " $8 " " $9}
/Core 4:/ {print "Core 2:        " $3 "  " $4 " " $5 " " $6 " " $7 " " $8 " " $9}
/Core 6:/ {print "Core 3:        " $3 "  " $4 " " $5 " " $6 " " $7 " " $8 " " $9}
/Core 8:/ {print "Core 4:        " $3 "  " $4 " " $5 " " $6 " " $7 " " $8 " " $9}
/Core 10:/ {print "Core 5:        " $3 "  " $4 " " $5 " " $6 " " $7 " " $8 " " $9}
/Core 12:/ {print "Core 6:        " $3 "  " $4 " " $5 " " $6 " " $7 " " $8 " " $9}
/Core 14:/ {print "Core 7:        " $3 "  " $4 " " $5 " " $6 " " $7 " " $8 " " $9}
/No sensors found!/ {print}
!/Core/ {print}
END {} ' ${sensorFile}
    fi

    rm -f ${sensorFile}

  fi
}

function show_top() {
  if [[ $1 == "dynamic" ]]; then
    top
  else
    top -bn2 | awk '/^top -/ { p=!p } { if (!p) print }'
  fi
}


function show_ztp() {
  if [ -f /config/_hb/last_known_err ]; then
    cat /config/_hb/last_known_err
    echo ""
  else
    echo "No error."
  fi
}

case "$1" in
  status)
    show_status
    ;;
  services)
    show_services $3 $4
    ;;
  hardware)
    show_hw
    ;;
  device)
    show_device $3
    ;;
  sensors)
    show_sensors
    ;;
  top)
    show_top $3
    ;;
  last-known-ztp-error)
    show_ztp
    ;;
  *)
    echo "Unknown operation __$1__ __$2__"
    exit 1
    ;;
esac

exit 0
