#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import subprocess
import sys
import syslog
import vppapi_if.vppapi_if as vppif


INDENT = 2 * ' '

def get_network_mode():

	net_mode = subprocess.check_output(['sysrepocfg', '--datastore=startup', '--format=json', '--xpath=/ucpe:config//system:system//network-mode', '--export']).decode("utf-8")
	if net_mode.find('"network-mode": "vpp"') > -1:
		return "vpp"

	return "ovs"

def show_vsw_ports_vpp(bd_tag):

    vpp_if = vppif.VppApiInterface('vswitch-commands')
    bd = vpp_if.get_bd_by_tag(tag=bd_tag)
    if not bd:
        return
    print(INDENT + 'ports:')
    bd_ifs = list(map(lambda x: x['sw_if_index'], bd['sw_if_details']))
    for idx in bd_ifs:
        ifname = vpp_if.get_ifname_by_ifidx(idx)
        if 'data' in ifname:
            print(3*INDENT + str(ifname).replace('host-', ''))

    del(vpp_if)

def show_vsw_ports(bd_tag):
    if 'ovs' == get_network_mode():
        sys.exit(0)
    else:
        show_vsw_ports_vpp(bd_tag)

def show_vsw_all(bd_tag):
    print("Vswitch {}".format(bd_tag))
    show_vsw_ports(bd_tag)

if __name__ == "__main__":
    argc = len(sys.argv)

    if 2 > argc:
        syslog.syslog(syslog.LOG_ERR, "No enough arguments. Exit!")
        sys.exit(1)

    if argc == 2:
        show_vsw_all(sys.argv[1])
    else:
        if sys.argv[2] == "ports":
            print("Vswitch {}".format(sys.argv[1]))
            show_vsw_ports(sys.argv[1])
        else:
            syslog.syslog(syslog.LOG_ERR, "Unknown argument \"%s\"" % sys.argv[2])
            sys.exit(1)

    sys.exit(0)

