#!/bin/bash
# -*- coding: utf-8 -*-
# vim: tabstop=2 expandtab shiftwidth=2 softtabstop=2
# vim: set fileencoding=utf-8
source /scripts/ipv6_functions

TVE_IP=${*: -3:1}
_prepareForIPv6 ${TVE_IP} TVE_IP
SCRIPTS_PATH=${*: -2:1}

USER=$(whoami)
GROUP=$(groups $USER | cut -d' ' -f3)
        
source ${SCRIPTS_PATH}/common-commands.sh

function set_enable() {
  IFS='#' tokens=( $1 )
  local new_enable=${tokens[0]}
  local new_bind_addr=${tokens[1]}
  local new_port=${tokens[2]}
  local new_auth_trap=${tokens[3]}
  local new_system_name=${tokens[4]}
  local new_system_loc=${tokens[5]}
  local new_system_cont=${tokens[6]}
  local new_system_desc=${tokens[7]}

  empty_string='""'
  if [ ! ${new_system_name} ]; then
    new_system_name=${empty_string}
  fi
  if [ ! ${new_system_loc} ]; then
    new_system_loc=${empty_string}
  fi
  if [ ! ${new_system_cont} ]; then
    new_system_cont=${empty_string}
  fi
  if [ ! ${new_system_desc} ]; then
    new_system_desc=${empty_string}
  fi

  ${SCRIPTS_PATH}/snmp_json_set.py set_enable $new_enable $new_bind_addr $new_port $new_auth_trap $new_system_name $new_system_loc $new_system_cont $new_system_desc
}

function set_view() {
  IFS='#' tokens=( $1 )
  local curr_id=${tokens[0]}
  local new_view_name=${tokens[1]}
  local new_type=${tokens[2]}
  local new_oid=${tokens[3]}

  empty_string='""'
  if [ ! ${new_view_name} ]; then
    new_view_name=${empty_string}
  fi

  ${SCRIPTS_PATH}/snmp_json_set.py set_view $curr_id $new_view_name $new_type $new_oid
}

function set_user() {
  IFS='#' tokens=( $1 )
  local curr_id=${tokens[0]}
  local new_user_name=${tokens[1]}
  local new_message_model=${tokens[2]}
  local new_security_level=${tokens[3]}
  local new_auth_proto=${tokens[4]}
  local new_auth_proto_pass=${tokens[5]}
  local new_priv_proto=${tokens[6]}
  local new_priv_proto_pass=${tokens[7]}
  local new_remote_engine_id=${tokens[8]}
  local new_read_view=${tokens[9]}
  local new_write_view=${tokens[10]}
  local new_notify_view=${tokens[11]}

  empty_string='""'
  if [ ! ${new_user_name} ]; then
    new_user_name=${empty_string}
  fi  
  if [ ! ${new_auth_proto_pass} ]; then
    new_auth_proto_pass=${empty_string}
  fi
  if [ ! ${new_priv_proto_pass} ]; then
    new_priv_proto_pass=${empty_string}
  fi
  if [ ! ${new_remote_engine_id} ]; then
    new_remote_engine_id=${empty_string}
  fi
  if [ ! ${new_read_view} ]; then
    new_read_view=${empty_string}
  fi
  if [ ! ${new_write_view} ]; then
    new_write_view=${empty_string}
  fi
  if [ ! ${new_notify_view} ]; then
    new_notify_view=${empty_string}
  fi

  ${SCRIPTS_PATH}/snmp_json_set.py set_user $curr_id $new_user_name $new_message_model $new_security_level $new_auth_proto $new_auth_proto_pass $new_priv_proto $new_priv_proto_pass $new_remote_engine_id $new_read_view $new_write_view $new_notify_view
}

function set_targets() {
  IFS='#' tokens=( $1 )
  local curr_id=${tokens[0]}
  local new_enabled=${tokens[1]}
  local new_target_name=${tokens[2]}
  local new_sec_name=${tokens[3]}
  local new_type=${tokens[4]}
  local new_auth_priv=${tokens[5]}
  local new_ip_address=${tokens[6]}
  local new_port=${tokens[7]}
  local new_timeout=${tokens[8]}
  local new_retry_count=${tokens[9]}
  local new_notif_type=${tokens[10]}

  empty_string='""'
  if [ ! ${new_target_name} ]; then
    new_target_name=${empty_string}
  fi
  if [ ! ${new_sec_name} ]; then
    new_sec_name=${empty_string}
  fi
  if [ ! ${new_ip_address} ]; then
    new_ip_address=${empty_string}
  fi

  ${SCRIPTS_PATH}/snmp_json_set.py set_targets $curr_id $new_enabled $new_target_name $new_sec_name $new_type $new_auth_priv $new_ip_address $new_port $new_timeout $new_retry_count $new_notif_type
}

case "$1" in
  set_enable)
    set_enable $2
    ;;
  set_view)
    set_view $2
    ;;
  set_user)
    set_user $2
    ;;
  set_targets)
    set_targets $2
    ;;
  *)
    echo "Unknown operation __$1__ __$2__"
    exit 1
    ;;
esac

exit 0

