#!/bin/bash
# -*- coding: utf-8 -*-
source /scripts/ipv6_functions

TVE_IP=${*: -3:1}
_prepareForIPv6 ${TVE_IP} TVE_IP
SCRIPTS_PATH=${*: -2:1}
PPKLISH_PID=${!#}

source /scripts/token-env.sh

command_header="curl -s -H \"X-Auth-Token:$TOKEN\" -H \"Content-type:application/json\" "
command_path="https://${TVE_IP}:7373/v1"
command_beauty="| python3 -mjson.tool | json2yaml"
ssl_insecure=$(cat /etc/tveapi/tveapi.conf | egrep -v "(^#.*)" | grep ssl_no_validate | cut -f2 -d'=' | tr -d ' ')
if [ -z  ${ssl_insecure} ]; then
  ssl_insecure=False
elif [ "true" == "${ssl_insecure}" ] || [ "True" == "${ssl_insecure}"  ]; then
  command_header+="--insecure"
fi

source ${SCRIPTS_PATH}/common-commands.sh

function show_config_all() {
  local command="${command_header} ${command_path}/config/all -X GET ${command_beauty}"
  eval "${command}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/config/all | python3 -mjson.tool
}
function show_config_image_server() {
  local command="${command_header} ${command_path}/config/image-server -X GET ${command_beauty}"
  eval "${command}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/config/image-server | python3 -mjson.tool
}
function show_config_homecalling() {
  local command="${command_header} ${command_path}/config/settings -X GET ${command_beauty}"
  eval "${command}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/config/settings | python3 -mjson.tool
}
function show_config_logging() {
  local command="${command_header} ${command_path}/config/logging -X GET ${command_beauty}"
  eval "${command}"
}
function show_config_security_auth() {
  local command="${command_header} ${command_path}/config/security/authentication -X GET ${command_beauty}"
  eval "${command}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/config/security | python3 -mjson.tool
}
function show_config_security_whitelist() {
  local command="${command_header} ${command_path}/config/security/cli-whitelist -X GET ${command_beauty}"
  eval "${command}"
}
function show_config_hostname() {
  local command="${command_header} ${command_path}/config/hostname -X GET ${command_beauty}"
  eval "${command}"
}
function show_config_server() {
  local command="${command_header} ${command_path}/config/config-server -X GET ${command_beauty}"
  eval "${command}"
}
function show_config_networking() {
  local command="${command_header} ${command_path}/config/networking -X GET ${command_beauty}"
  eval "${command}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/config/networking | python3 -mjson.tool
}
function show_config_networking_interfaces() {
  local command="${command_header} ${command_path}/config/networking/interfaces -X GET ${command_beauty}"
  eval "${command}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/config/networking/interfaces | python3 -mjson.tool
}

function show_config_interface() {
  IFS=':' tokens=( $1 )
  local new_interface=${tokens[0]}
  local f1=`mktemp`
  local get_interface_names="${command_header} ${command_path}/config/networking/interfaces -X GET | /usr/bin/jq '.[] | .name' > ${f1}"
  eval "${get_interface_names}"
  grep -i ${new_interface} ${f1}
  if [ 1 == $? ]; then
    echo "Interface ${new_interface} doesn't exits on this system !"
    rm -f ${f1}
    exit 1
  fi
  rm -f ${f1}
  local command="${command_header} ${command_path}/config/networking/interfaces/?name=${new_interface} -X GET ${command_beauty}"
  eval "${command}"
}

function show_config_quagga() {
  local command="${command_header} ${command_path}/config/networking/quagga -X GET ${command_beauty}"
  eval "${command}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/config/networking/quagga | python3 -mjson.tool
}

function show_config_ntp() {
  local command="${command_header} ${command_path}/config/ntp -X GET ${command_beauty}"
  eval "${command}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/config/ntp | python3 -mjson.tool
}

function show_config_networking_routing_bgp() {
  local command="${command_header} ${command_path}/config/routing/bgp -X GET"
  eval "${command}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/config/routing/bgp | python3 -mjson.tool
}
function show_config_networking_routing_static() {
  local command="${command_header} ${command_path}/config/routing/static -X GET"
  eval "${command}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/config/routing/static | python3 -mjson.tool
}
function show_config_scripts() {
  local command="${command_header} ${command_path}/config/scripts-server -X GET ${command_beauty}"
  eval "${command}"
}

function show_status_system_resources() {
  /usr/bin/sysrepocfg --datastore=running --format=json -X --xpath="/ucpe:config/system-resources:system-resources" | /usr/bin/json2yaml
}

function show_status_all() {
  #echo "/v1/status/image-server"
  local command2="${command_header} ${command_path}/status/image-server -X GET"
  eval "${command2}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/status/image-server
  #echo "/v1/status/settings"
  local command3="${command_header} ${command_path}/status/settings -X GET"
  eval "${command3}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/status/settings
  # TBD-status-cbox-start
  # echo
  # echo "/v1/status/cbox"
  # local command4="${command_header} ${command_path}/status/cbox -X GET"
  # eval "${command4}"
  # TBD-status-cbox-end
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/status/cbox
  # TBD-logging-start
  # echo
  # echo "/v1/status/logging"
  # local command5="${command_header} ${command_path}/status/logging -X GET"
  # eval "${command5}"
  # TBD-logging-end
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/status/logging
  #echo "/v1/status/networking/routes"
  local command6="${command_header} ${command_path}/status/networking/routes -X GET"
  eval "${command6}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/status/networking
  #echo "/v1/status/ntp"
  local command7="${command_header} ${command_path}/status/ntp -X GET"
  eval "${command7}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/status/ntp
  # TBD-routing-bgp-start
  # echo
  # echo "/v1/status/routing/bgp"
  # local command9="${command_header} ${command_path}/status/routing/bgp -X GET"
  # eval "${command9}"
  # TBD-routing-bgp-end
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/status/routing/bgp
  #echo "/v1/status/routing/static"
  local command10="${command_header} ${command_path}/status/routing/static -X GET"
  eval "${command10}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/status/routing/static

}
function show_status_image_server() {
  local command="${command_header} ${command_path}/status/image-server -X GET -w '\n'"
  eval "${command}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/status/image-server
}
function show_status_settings() {
  local command="${command_header} ${command_path}/status/settings -X GET"
  eval "${command}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/status/settings
}
function show_status_cbox() {
  local command="${command_header} ${command_path}/status/cbox -X GET"
  eval "${command}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/status/cbox
}
function show_status_logging() {
  local command="${command_header} ${command_path}/status/logging -X GET"
  eval "${command}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/status/logging
}
function show_status_networking_routes() {
  local command="${command_header} ${command_path}/status/networking/routes -X GET"
  eval "${command}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/status/networking
}
function show_status_interfaces() {
  local command="${command_header} ${command_path}/status/networking/interfaces -X GET"
  eval "${command}"
}
function show_status_ntp() {
  local command="timedatectl"
  eval "${command}"
}
function show_status_routing_bgp() {
  local command="${command_header} ${command_path}/status/routing/bgp -X GET"
  eval "${command}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/status/routing/bgp
}
function show_status_routing_static() {
  local command="${command_header} ${command_path}/status/routing/static -X GET"
  eval "${command}"
  #  curl -s -H "X-Auth-Token:$TOKEN" -H "Content-type: application/json" -X GET http://${TVE_IP}:7373/v1/status/routing/static
}

function show_ns_status() {
  local command="${command_header} ${command_path}/status/ucpe?status=nsStatus -X GET"
  eval "${command}"
}

case "$1" in
  config_all)
    show_config_all
    ;;
  config_image-server)
    show_config_image_server
    ;;
  config_homecalling)
    show_config_homecalling
    ;;
  config_logging)
    show_config_logging
    ;;
  config_security_auth)
    show_config_security_auth
    ;;
  config_security_whitelist)
    show_config_security_whitelist
    ;;
  config_hostname)
    show_config_hostname
    ;;
  config_server)
    show_config_server
    ;;
  config_networking)
    show_config_networking
    ;;
  config_interfaces)
    show_config_networking_interfaces
    ;;
  config_interface)
    show_config_interface $2
    ;;
  config_quagga)
    show_config_quagga
    ;;
  config_ntp)
    show_config_ntp
    ;;
  config_static)
    show_config_networking_routing_static
    ;;
  config_scripts)
    show_config_scripts
    ;;
  status_image-server)
    show_status_image_server
    ;;
  status_homecalling)
    show_status_settings
    ;;
  status_cbox)
    show_status_cbox
    ;;
  status_logging)
    show_status_logging
    ;;
  status_interfaces)
    show_status_interfaces
    ;;
  status_ntp)
    show_status_ntp
    ;;
  status_bgp)
    show_status_routing_bgp
    ;;
  status_static)
    show_status_routing_static
    ;;
  status_system_resources)
    show_status_system_resources
    ;;
  status_ns)
    show_ns_status
    ;;
  *)
    echo "Unknown operation __$1__ __$2__"
    exit 1
    ;;
esac

exit 0
