#!/usr/bin/env python3
# This version of the show-status-vpn.py script is to be used when this sysrepo bug is fixed:
# https://github.com/sysrepo/sysrepo/issues/1591
import sys
import syslog
from sysrepo_interfaces import sysrepo_interfaces3 as api

def get_ipsvpn_if_status(ifname):
  i = api.SysrepoInterfaces(app_name="ipsecvpn status")
  return i.get_if_status(ifname)

def status_ipsecvpn(ifname):
  data = dict()
  xpath = "/ucpe:status/interfaces:interface[name='{}']/" .format(ifname)
  if ifname is None:
    sys.exit( 1 )
  data = get_ipsvpn_if_status(ifname)
  if not data:
    print('No active security associations for interface \'{}\''.format(ifname))
    sys.exit( 0 )
  
  i = api.SysrepoInterfaces(app_name="ipsecvpn status")
  print ("Interface {ifname} Security Associations: \n"\
      "\tConnection: {conname}, \n" \
      "\tLocal: {local}, ID: {localid} \n" \
      "\tRemote: {remote}, ID: {remoteid} \n" \
      "\tState: {state}, established {established}s ago, rekeying in {rekey}s \n" \
      "\tIKEv{version}, IKE proposal: {ph1_encr}{ph1_encr_ksize}/{ph1_integ} DH group: {ph1_dh}" \
      .format (ifname=ifname,
          conname=data.get(xpath + 'conname'),
          local=data.get(xpath + 'local'), localid=data.get(xpath + 'local-id'),
          remote=data.get(xpath + 'remote'), remoteid=data.get(xpath + 'remote-id'),
          state=data.get(xpath + 'state'), established=data.get(xpath+'established'),
          rekey=data.get(xpath+'rekey-time'), version=data.get(xpath + 'version'), 
          ph1_dh=data.get(xpath + 'dh-group'), ph1_integ=data.get(xpath + 'integ-alg'),
          ph1_encr=data.get(xpath + 'encr-alg'), ph1_encr_ksize=data.get(xpath + 'encr-ksize')
          ))

  # Get the names of all child connections
  chnames_tmp = []
  chname_start_key = "[chname='"
  chname_end_key = "']"
  for data_indx in data.items():
    chname_str_idx = data_indx[0].find(chname_start_key)
    chname_end_idx = data_indx[0].rfind(chname_end_key)

    if  chname_str_idx > 0 and chname_end_idx > chname_str_idx:
      chname = data_indx[0][chname_str_idx:chname_end_idx]
      if chname != "":
        chnames_tmp.append(chname[len(chname_start_key):])

  chnames = []
  if (len(chnames_tmp) > 0):
    chnames_uniq = sorted(set(chnames_tmp))
    chnames = list(chnames_uniq)

  # Iterrate all child connections
  if(len(chnames) > 0):
      for chname in chnames:
          xpath_child = "{}child[chname='{}']/".format(xpath, chname)
          if data.get(xpath_child +'state'):
              print ("\tChild: {chname}, {chstate}, {mode}, protocol:{proto} \n" \
              "\t\t{localts} === {remotets} \n" \
              "\t\tIKE proposal: {ph2_encr}{ph2_encr_ksize}/{ph2_integ} DH group: {ph2_dh} \n" \
              "\t\tPkt-in: {pktin},\tbytes-in: {bytesin} \n" \
              "\t\tPkt-out: {pktout},\tbytes-out: {bytesout}" \
              .format (ifname=ifname,
                  chname=data.get(xpath_child +'chname'),
                  chstate=data.get(xpath_child +'state'),
                  mode=data.get(xpath_child +'mode'), proto=data.get(xpath_child +'proto'),
                  localts=data.get(xpath_child +'local-ts'), remotets=data.get(xpath_child +'remote-ts'),
                  pktin=data.get(xpath_child +'pkt-in'), pktout=data.get(xpath_child +'pkt-out'),
                  bytesin=data.get(xpath_child +'bytes-in'), bytesout=data.get(xpath_child +'bytes-out'),
                  ph2_integ=data.get(xpath_child + 'integ-alg'), ph2_encr=data.get(xpath_child + 'encr-alg'),
                  ph2_dh=data.get(xpath_child + 'dh-group'), ph2_encr_ksize=data.get(xpath_child + 'encr-ksize')
                  ))
  else:
    print ("Child: not established")


if __name__ == "__main__":
  argc = len( sys.argv )
  if 2 > argc:
    syslog.syslog( syslog.LOG_ERR, "No enough arguments. Exit!" )
    sys.exit( 1 )

  if sys.argv[1] == "ipsec":
    if argc == 3:
      status_ipsecvpn(sys.argv[2])
    else:
      syslog.syslog( syslog.LOG_ERR, "Missing interface name! Exit!" )
      sys.exit( 0 )
  else:
    syslog.syslog( syslog.LOG_ERR, "Unknown argument \"%s\"" % sys.argv[1] )
    sys.exit( 1 )

sys.exit( 0 )
