#!/bin/bash

# set -o errexit
# set -o xtrace
# set -o nounset
# set -o pipefail
# -tttt timestamp on each dump line
# -n Don't convert host addresses to names
# -N Don't print domain name qualification of host names
# -K Don't attempt to verify IP, TCP, or UDP checksums
# -s snaplen for decoding in Wireshark
# -v slightly more verbose output
# -l Make stdout line buffered. Useful if you want to see the data while capturing

interface=""
timeinseconds=10
packet_expression=""
filename=""
while getopts i:t:f:p: option 2>/dev/null
do
	case "${option}"
	in
		i) interface=${OPTARG};;
		t) timeinseconds=${OPTARG};;
		p) packet_expression=${OPTARG};;
		f) filename=${OPTARG};;
	esac
done

if [[ ! -d /home/admin/user_files/ ]]; then
	/usr/bin/sudo /bin/mkdir /home/admin/user_files/
	/usr/bin/sudo /bin/chmod 755 /home/admin/user_files/
fi

if [ 10 -gt ${timeinseconds} ]; then
	/bin/echo "minimum duration is 10 seconds"
	exit 0
fi

cd /home/admin/user_files/
if [ -n "${filename}" ] && [ "." != "$(dirname ${filename})" ]; then
	/bin/echo "Use filename without directory path"
	exit 0
fi

ifmode=$(sysrepocfg --datastore=running --format=xml --xpath="/ucpe:config/interfaces:interface[name='${interface}']/mode" --export | grep "<mode>" | cut -d '>' -f 2 | cut -d '<' -f1)
if [ "lte0" == "${interface}" ] || [ "lte1" == "${interface}" ] || [ "lte2" == "${interface}" ] || [ "tun0" == "${interface}" ] || [ "vti0" == "${interface}" ] || [[ -f "/sys/class/net/${interface}/operstate" && "dpdk" != "${ifmode}" ]]; then
	if [ -n "${filename}" ]; then
		/usr/bin/sudo /usr/bin/timeout --signal=SIGINT ${timeinseconds}s /usr/bin/tcpdump -i ${interface} -tttt -v -K -n -N -s 65535 "${packet_expression}" -w /home/admin/user_files/${filename}
	else
		/usr/bin/sudo /usr/bin/timeout --signal=SIGINT ${timeinseconds}s /usr/bin/tcpdump -i ${interface} -tttt -v -K -n -N -s 65535 "${packet_expression}"
	fi
else   #OVS or VPP
	if [ $(systemctl is-enabled vpp-config) == "enabled" ]; then
		#vpp
		if [ -z "${filename}" ]; then
			/bin/echo "Must specify filename. Capture on this interface without filename is not supported."
			exit 0
		fi
		sudo touch /tmp/${filename}
		sudo /usr/bin/vppctl pcap trace off > /dev/null  || true
		sudo /usr/bin/vppctl pcap trace rx tx file ${filename} intfc ${interface} max 999999999 > /dev/null  || true
		for i in $(seq 1 ${timeinseconds}); do sleep 1s; sudo /usr/bin/vppctl pcap trace status | head -1 | tr '\n' -d ; done
		sudo /usr/bin/vppctl pcap trace off > /dev/null  || true
		sudo mv /tmp/${filename} /home/admin/user_files/${filename} || true
		echo "  done"
	else
		if [ -n "${filename}" ]; then
			/usr/bin/sudo /usr/bin/timeout --signal=SIGINT ${timeinseconds}s /usr/bin/ovs-tcpdump -i ${interface} -tttt -v -K -n -N -s 65535 "${packet_expression}" -w /home/admin/user_files/${filename}
		else
			/usr/bin/sudo /usr/bin/timeout --signal=SIGINT ${timeinseconds}s /usr/bin/ovs-tcpdump -i ${interface} -tttt -v -K -n -N -s 65535 "${packet_expression}"
		fi
	fi
fi
