#!/bin/bash
# -*- coding: utf-8 -*-
PPKLISH_PID=$1
TVE_IP=$2

# echo "ppid is ${PPKLISH_PID}"
TOKEN_FILE=/tmp/token.${PPKLISH_PID}
if [[ ! -w "${TOKEN_FILE}" ]]; then
  echo "${TOKEN_FILE} doesn't exists or no permissions !!!"
  echo
  echo `ls -la ${TOKEN_FILE}`
  echo
  exit 0
fi

source /scripts/ipv6_functions

cbox_type=$(sysrepocfg -f json -g /ucpe:config/openstack:openstack/cbox/enabled 2>/dev/null | grep enable | cut -d' ' -f10)
controller=$(sysrepocfg -f json -g /ucpe:config/openstack:openstack/controller 2>/dev/null | grep controller | cut -d' ' -f8 | tr -d "\"")
_prepareForIPv6 $controller controller
command_header="curl -si --connect-timeout 5 "
ssl_insecure=$(cat /etc/tveapi/tveapi.conf | egrep -v "(^#.*)" | grep ssl_no_validate | cut -f2 -d'=' | tr -d ' ')
if [[ -z  ${ssl_insecure} ]]; then
  ssl_insecure=False
elif [ "true" == "${ssl_insecure}" ] || [ "True" == "${ssl_insecure}"  ]; then
  command_header+="--insecure"
fi



function get_token_public () {
  local f=`mktemp`
  local user=`awk -F"=" '/\yusername\y/ {print $2}' ${TOKEN_FILE}`
  local pass=`awk -F"=" '/\ypassword\y/ {print $2}' ${TOKEN_FILE}`
  local group=`awk -F"=" '/\ygroup\y/ {print $2}' ${TOKEN_FILE}`
  if [ -z ${user} -o -z ${pass} -o -z ${group} ]; then
    echo "No user or pass or group: ${user} ${pass} ${group}"
    exit 1
  fi
  # echo "user is $user pass $pass group $group"

  local get_public_token_command=`$command_header -d '
 {
  "auth": {
   "identity": {
    "methods": [
     "password"
    ],
    "password": {
     "user": {
      "domain": {
       "name": "default"
      },
      "name": "'$user'",
      "password": "'$pass'"
     }
    }
   },
   "scope": {
    "project": {
     "domain": {
      "name": "default"
     },
     "name": "admin"
    }
   }
  }
 }
 ' -H "Content-type: application/json" https://localhost:35357/v3/auth/tokens -o $f`
  eval "$get_public_token_command";

  local TOKEN=`awk '/X-Subject-Token/ {print $2}' $f | tr -d '\r'`
  if [[ -z ${TOKEN} ]]; then
    echo "Can't get token !!!"
    exit 1
  fi
  
  local EXPIRE=`sed -e '1,10d' < $f  | python3 -mjson.tool | awk '/\yexpires_at\y/ {print $2}' | tr -d '\r' | tr -d '"' | tr -d ',' | awk -F. '{print $1}'`
  if [[ -z ${EXPIRE} ]]; then
    echo "Can't get expiration date"
    exit 1
  fi

  controller_token=`awk -F"=" '/\ycontroller_token\y/ {print $0}' ${TOKEN_FILE}`
  controller_expired_at=`awk -F"=" '/\ycontroller_expires_at\y/ {print $0}' ${TOKEN_FILE}`

  echo "session_pid=${PPKLISH_PID}" > ${TOKEN_FILE}
  echo "username=${user}" >> ${TOKEN_FILE}
  echo "password=${pass}" >> ${TOKEN_FILE}
  echo "group=${group}" >> ${TOKEN_FILE}
  echo "token=${TOKEN}" >> ${TOKEN_FILE}
  echo "expires_at=${EXPIRE}" >> ${TOKEN_FILE}
  echo "${controller_token}" >> ${TOKEN_FILE}
  echo "${controller_expired_at}" >> ${TOKEN_FILE}
  rm $f
}


function get_token () {
  local f=`mktemp`
  local user=`awk -F"=" '/\yusername\y/ {print $2}' ${TOKEN_FILE}`
  local pass=`awk -F"=" '/\ypassword\y/ {print $2}' ${TOKEN_FILE}`
  local group=`awk -F"=" '/\ygroup\y/ {print $2}' ${TOKEN_FILE}`
  if [ -z ${user} -o -z ${pass} -o -z ${group} ]; then
    echo "No user or pass or group: ${user} ${pass} ${group}"
    exit 1
  fi
  # echo "user is $user pass $pass group $group"

  local get_token_command=`$command_header -u $user:$pass -d '
  {
   "auth": {
    "identity": {
     "methods": [
      "pam"
     ],
     "pam": {
      "identity_provider": "pam-idp",
      "protocol":"pam"
      }
    },
    "scope": {
     "project": {
      "domain": {
       "name": "default"
      },
     "name":"admin"
     }
    }
   }
  }
  ' -H "Content-type: application/json" https://localhost:5000/v3/auth/tokens -o $f`
  # echo "$get_token_command"
  eval "$get_token_command"

  local TOKEN=`awk '/X-Subject-Token/ {print $2}' $f | tr -d '\r'`
  if [[ -z ${TOKEN} ]]; then
    if [[ -f "/config/no-openstack" ]]; then
        get_token_public
        return
    else
        echo "Can't get token !!!"
        exit 1
    fi
  fi
  local EXPIRE=`sed -e '1,10d' < $f  | python3 -mjson.tool | awk '/\yexpires_at\y/ {print $2}' | tr -d '\r' | tr -d '"' | tr -d ',' | awk -F. '{print $1}'`
  if [[ -z ${EXPIRE} ]]; then
    echo "Can't get expiration date"
    exit 1
  fi

  controller_token=`awk -F"=" '/\ycontroller_token\y/ {print $0}' ${TOKEN_FILE}`
  controller_expired_at=`awk -F"=" '/\ycontroller_expires_at\y/ {print $0}' ${TOKEN_FILE}`

  echo "session_pid=${PPKLISH_PID}" > ${TOKEN_FILE}
  echo "username=${user}" >> ${TOKEN_FILE}
  echo "password=${pass}" >> ${TOKEN_FILE}
  echo "group=${group}" >> ${TOKEN_FILE}
  echo "token=${TOKEN}" >> ${TOKEN_FILE}
  echo "expires_at=${EXPIRE}" >> ${TOKEN_FILE}
  echo "${controller_token}" >> ${TOKEN_FILE}
  echo "${controller_expired_at}" >> ${TOKEN_FILE}
  rm $f
}

function get_controller_token () {
  local f=`mktemp`
  local user=`awk -F"=" '/\yusername\y/ {print $2}' ${TOKEN_FILE}`
  local pass=`awk -F"=" '/\ypassword\y/ {print $2}' ${TOKEN_FILE}`
  local group=`awk -F"=" '/\ygroup\y/ {print $2}' ${TOKEN_FILE}`
  if [ -z ${user} -o -z ${pass} -o -z ${group} ]; then
    echo "No user or pass or group: ${user} ${pass} ${group}"
    exit 1
  fi
  # echo "user is $user pass $pass group $group"

  local get_remote_token_command=`$command_header -d '
 {
  "auth": {
   "identity": {
    "methods": [
     "password"
    ],
    "password": {
     "user": {
      "domain": {
       "name": "default"
      },
      "name": "'$user'",
      "password": "'$pass'"
     }
    }
   },
   "scope": {
    "project": {
     "domain": {
      "name": "default"
     },
     "name": "admin"
    }
   }
  }
 }
 ' -H "Content-type: application/json" https://${controller}:35357/v3/auth/tokens -o $f`
  eval "$get_remote_token_command";

  local CONTROLLER_TOKEN=`awk '/X-Subject-Token/ {print $2}' $f | tr -d '\r'`
  if [[ -z ${CONTROLLER_TOKEN} ]]; then
    echo "Can't get token !!!"
    exit 1
  fi
  local CONTROLLER_EXPIRE=`sed -e '1,10d' < $f  | python3 -mjson.tool | awk '/\yexpires_at\y/ {print $2}' | tr -d '\r' | tr -d '"' | tr -d ',' | awk -F. '{print $1}'`
  if [[ -z ${CONTROLLER_EXPIRE} ]]; then
    echo "Can't get expiration date"
    exit 1
  fi

  local_token=`awk -F"=" '/\ytoken\y/ {print $0}' ${TOKEN_FILE}`
  local_expired_at=`awk -F"=" '/\yexpires_at\y/ {print $0}' ${TOKEN_FILE}`

  echo "session_pid=${PPKLISH_PID}" > ${TOKEN_FILE}
  echo "username=${user}" >> ${TOKEN_FILE}
  echo "password=${pass}" >> ${TOKEN_FILE}
  echo "group=${group}" >> ${TOKEN_FILE}
  echo "${local_token}" >> ${TOKEN_FILE}
  echo "${local_expired_at}" >> ${TOKEN_FILE}
  echo "controller_token=${CONTROLLER_TOKEN}" >> ${TOKEN_FILE}
  echo "controller_expires_at=${CONTROLLER_EXPIRE}" >> ${TOKEN_FILE}
  rm $f
}

function main () {
  if [ -w ${TOKEN_FILE} -a -s ${TOKEN_FILE} ]; then
    if [ "$TVE_IP" != "localhost" ] && [ "${TVE_IP}" != "127.0.0.1" ] && [ "${TVE_IP}" != "[::1]" ] && [ "$cbox_type" == "false" ]; then
      if [[ -z `awk -F"=" '/\ycontroller_expires_at\y/ {print $2}' ${TOKEN_FILE}` ]]; then
        # Try to get new token
        get_controller_token
      fi
      local TOKEN_DATE=`awk -F"=" '/\ycontroller_expires_at\y/ {print $2}' ${TOKEN_FILE}`
      # echo $TOKEN_DATE
      if [[ -z ${TOKEN_DATE} ]]; then
        echo "${TOKEN_FILE} missing controller_expires_at !"
        exit 1
      fi
    else
      local TOKEN_DATE=`awk -F"=" '/\yexpires_at\y/ {print $2}' ${TOKEN_FILE}`
      # echo $TOKEN_DATE
      if [[ -z ${TOKEN_DATE} ]]; then
        echo "${TOKEN_FILE} missing expire_at !"
        exit 1
      fi
    fi
    local CURRENT_DATE=`date -u -Is | awk -F+ '{print $1}'`
    # echo $CURRENT_DATE $TOKEN_DATE
    local EPOCH_CURRENT=`date +"%s" -d$CURRENT_DATE`
    local EPOCH_TOKEN=`date +"%s" -d$TOKEN_DATE`
    # echo $EPOCH_CURRENT $EPOCH_TOKEN
    if [ ${EPOCH_TOKEN} -le ${EPOCH_CURRENT} ]; then
      # echo "Must renew token !!!"
      if [ "$TVE_IP" != "localhost" ] && [ "${TVE_IP}" != "127.0.0.1" ] && [ "${TVE_IP}" != "[::1]" ] && [ "$cbox_type" == "false" ]; then
        get_controller_token
      else
        get_token
      fi
    fi
  else
    if [ "$TVE_IP" != "localhost" ] && [ "${TVE_IP}" != "127.0.0.1" ] && [ "${TVE_IP}" != "[::1]" ] && [ "$cbox_type" == "false" ]; then
      get_controller_token
    else
      get_token
    fi
  fi
}

main

exit 0
