#!/bin/bash
# -*- coding: utf-8 -*-
# vim: tabstop=2 expandtab shiftwidth=2 softtabstop=2
# vim: set fileencoding=utf-8
source /scripts/ipv6_functions

#source basic functions and variables
source /scripts/basic_functions
source /scripts/basic_vars

if [ -f /config/.version ] ; then
    ARM_VERSION_STR=`cat /config/.version`
else
    ARM_VERSION_STR=''
fi
VERSION_STR="`dpkg -s nfvtimeos | grep -Po "(?<=^Version: )([=><:+0-9~.a-zA-Z\-]*)$"` $ARM_VERSION_STR"
CHANGELOG_FILE=/etc/nfvtime-os.changelog

USER=$(whoami)
PASS=$(sudo cat /root/admin-openrc.env | grep OS_PASS | cut -d'=' -f2)
command_header="curl -s --connect-timeout 5 --insecure"
TOKEN="${command_header} -d '{\"user\":\"${USER}\", \"password\":\"${PASS}\"}' -H Content-type: application/json -X GET https://localhost:7374/v3/auth/tokens"

if [ -z "$TOKEN" ];
  then
  echo "No token !"
  exit 1
fi

command_header="curl -s -H \"X-Auth-Token:$TOKEN\" -H \"Content-type:application/json\" "
command_path="https://${TVE_IP}:7373/v1"
command_beauty="| python3 -mjson.tool | json2yaml"
ssl_insecure=$(cat /etc/tveapi/tveapi.conf | egrep -v "(^#.*)" | grep ssl_no_validate | cut -f2 -d'=' | tr -d ' ')
if [ -z  ${ssl_insecure} ]; then
  ssl_insecure=False
elif [ "true" == "${ssl_insecure}" ] || [ "True" == "${ssl_insecure}"  ]; then
  command_header+="--insecure"
fi

function get_local_token() {
  local output=$(mktemp)

  #echo $output
  local get_local_token_command="${command_header} -d '{\"user\":\"${USER}\", \"password\":\"${PASS}\"}' -H Content-type: application/json -X GET https://localhost:7374/v3/auth/tokens"
  eval "$get_local_token_command" > "$output"
  LOCAL_TOKEN=$(cat "$output")
  echo $LOCAL_TOKEN
    #LOCAL_TOKEN=$(awk '/X-Subject-Token/ {print $2}' $output | tr -d '\r')
    #if [ -n "${LOCAL_TOKEN}" ]; then
    #  LOCAL_EXPIRE=$(sed -e '1,10d' < $output | python3 -mjson.tool | awk '/\yexpires_at\y/ {print $2}' | tr -d '\r' | tr -d '"' | tr -d ',' | awk -F. '{print $1}')
    #fi
  rm -f ${output}
}

function show_version() {

  local f1=`mktemp`
  local command="${command_header} ${command_path}/config/all -X GET -o ${f1}"
  eval "${command}"
  echo "----------------------------------------------------------------------"
  cat /usr/share/base-files/motd
  echo "----------------------------------------------------------------------"
  echo " Up Time          : "`uptime -p`
  echo "----------------------------------------------------------------------"
  rm -f ${f1}
}

function show_nfvtimeos_version() {
if [ -f /config/motd ]; then
  echo " Version: ${VERSION_STR}"
else
  echo " Telco Systems Edgility-OS version: ${VERSION_STR}"
fi
  echo "----------------------------------------------------------------------"
  echo " Up Time          : "`uptime -p`
  echo "----------------------------------------------------------------------"
}

function show_nfvtimeos_changelog() {
  if [ ! -r ${CHANGELOG_FILE} ]; then
    exit 0
  fi

  cat ${CHANGELOG_FILE}
  
}

case "$1" in
  show_version)
    show_version
    ;;
  show_nfvtimeos_version)
    show_nfvtimeos_version
    ;;
  show_nfvtimeos_changelog)
    show_nfvtimeos_changelog
    ;;
  *)
    echo "Unknown operation __$1__ __$2__"
    exit 1
    ;;
esac

exit 0
