#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import getopt
import subprocess
import datetime
import logging
from sysrepo_interfaces import sysrepo_interfaces3


def connected_ap():
    result = subprocess.run(['/sbin/iw', 'dev', 'wifi0', 'link'],
                            stdout=subprocess.PIPE, encoding='UTF-8')
    if result.returncode == 0:
        for line in result.stdout.split('\n'):
            print('{}'.format(line))
            if not line:
                break


def card_information():
    result = subprocess.run(['/usr/bin/lspci'],
                            stdout=subprocess.PIPE, encoding='UTF-8')
    if result.returncode == 0:
        for line in result.stdout.split('\n'):
            if 'wireless' in line.lower():
                lcpci_numer = line[0:7]
                card_result = subprocess.run(['/usr/bin/lspci', '-vv', '-s', lcpci_numer],
                                             stdout=subprocess.PIPE, encoding='UTF-8')
                if card_result.returncode == 0:
                    print('{}'.format(card_result.stdout))


def firmware():
    result = subprocess.run(['/sbin/iw', 'phy'],
                            stdout=subprocess.PIPE, encoding='UTF-8')
    if result.returncode == 0:
        print('{}'.format(result.stdout))


def device_info():
    result = subprocess.run(['/usr/bin/lspci'],
                            stdout=subprocess.PIPE, encoding='UTF-8')
    if result.returncode == 0:
        for line in result.stdout.split('\n'):
            if 'wireless' in line.lower():
                print('{}'.format(line))


def connected_clients(details=False):
    interface_wifi = sysrepo_interfaces3.SysrepoInterfaces(app_name="wifi_cli",log_level=logging.ERROR)
    wifi_type = interface_wifi.get_interface_param('wifi0', "wifi-mode")
    if wifi_type == 'client':
        wifi_type = None
        interface_wifi = None
        return

    result = subprocess.run(['/sbin/iw', 'dev', 'wifi0', 'station', 'dump'],
                            stdout=subprocess.PIPE, encoding='UTF-8')
    if result.returncode == 0:
        if not details:
            print('-------------------------------------------------------------------------------------------------------------')
            print('| Stations  MAC     | IP address      | auth | Tx/Rx bitrate MBit/s  | Signal strength dBm | Associated time  ')
            print('-------------------------------------------------------------------------------------------------------------')

        for stations in result.stdout.split('Station '):
            station_mac = None
            human_time = None
            tx_rate = None
            rx_rate = None
            authenticated = None
            signal = None
            for line in stations.split('\n'):
                if 'on wifi0' in line and not details:
                    station_mac = '{}'.format(line[:17])
                    #print('{}'.format(station_mac))
                elif 'rx bitrate' in line and not details:
                    rx_rate_str = '{}'.format(line[13:-6])
                    rx_mbits = line.find('MBit/s')
                    if rx_mbits > 1:
                        rx_rate_str = '{}'.format(line[13:rx_mbits])
                        rx_rate = float(rx_rate_str)

                elif 'tx bitrate' in line and not details:
                    tx_rate_str = '{}'.format(line[13:-6])
                    tx_mbits = line.find('MBit/s')
                    if tx_mbits > 1:
                        tx_rate_str = '{}'.format(line[13:tx_mbits])
                        tx_rate = float(tx_rate_str)

                    #print('--{}--'.format(tx_rate))
                elif 'authenticated' in line and not details:
                    authenticated = '{}'.format(line[16:])
                    #print('--{}--'.format(authenticated))
                elif 'signal: ' in line and not details:
                    # signal = '{}'.format(line[11:-4])
                    first_signal_digit = line.split()
                    signal_str = '{}'.format(first_signal_digit[1])
                    signal = int(signal_str)
                    #print('--{}--'.format(signal))
                elif 'connected time' in line and not details:
                    station_time = int(line[16:].replace('seconds', ''))
                    # 86400seconds = 24H:00M:00S
                    station_time_days = int(station_time / 86400)
                    station_time_no_days = int(station_time % 86400)
                    human_time = '{} days {}'.format(
                        station_time_days,
                        datetime.timedelta(seconds=int(station_time_no_days)))
                    #print('--{}--'.format(human_time))
                elif details:
                    if 'authenticated' not in line \
                       and 'WMM/WME' not in line \
                       and 'preamble' not in line \
                       and 'MFP' not in line \
                       and 'TDLS peer' not in line \
                       and 'DTIM period' not in line \
                       and 'beacon interval' not in line \
                       and 'authenticated' not in line \
                       and 'short slot time' not in line:
                        print('{}'.format(line))

            station_ip = 'N/A'
            ip_addresses = subprocess.run(['/usr/sbin/arp', '-ne', '--device', 'br-wifi0'],
                                          stdout=subprocess.PIPE, encoding='UTF-8')
            if ip_addresses.returncode == 0 and station_mac:
                station_mac_position = ip_addresses.stdout.find(station_mac)
                if station_mac_position >= 0:
                    newline_before_ip = ip_addresses.stdout.rfind('\n', 0, station_mac_position)
                    station_ip = ip_addresses.stdout[newline_before_ip:station_mac_position].split(' ')[0].strip()
                    # station_ip="255.255.255.255"
                    # station_ip='N/A'
            if not details and station_mac and human_time \
               and tx_rate and rx_rate and authenticated:
                if signal:
                    print('| {0} | {1:15s} | {2}  |     {3:5.1f} / {4:5.1f}     |         {5:3d}         | {6}    '.
                        format(station_mac, station_ip, authenticated,
                                tx_rate, rx_rate, signal, human_time))
                else:
                    print('| {0} | {1:15s} | {2}  |     {3:5.1f} / {4:5.1f}     |         N/A         | {5}    '.
                        format(station_mac, station_ip, authenticated, tx_rate, rx_rate, human_time))
                print('-------------------------------------------------------------------------------------------------------------')

    wifi_type = None
    interface_wifi = None


def network_details():
    interface_wifi = sysrepo_interfaces3.SysrepoInterfaces(app_name="wifi_cli")
    wifi_type = interface_wifi.get_interface_param('wifi0', "wifi-mode")
    if wifi_type == "AP":
        print("This command is applicable only in WiFi client mode.")

    scan_result = subprocess.run(['/sbin/iwlist', 'wifi0', 'scan'],
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.DEVNULL, encoding='UTF-8')
    if scan_result.returncode == 0:
        available_ap = scan_result.stdout.split('Cell ')
        interfaces = sysrepo_interfaces3.SysrepoInterfaces(app_name="wifi_network_details")
        connected_ssid = interfaces.get_interface_param('wifi0', "ssid")
        for cell in available_ap:
            ssid = cell.find('ESSID:"{}"'.format(connected_ssid))
            if ssid > -1:
                for line in cell.split('\n'):
                    if not 'unknown' in line.lower():
                        print('{}'.format(line.replace('                    ', '')))

                break

        connected_ssid = None
        interfaces = None


def scan():
    scan_result = None
    retry = 5
    while retry > 0:
        scan_result = subprocess.run(['/sbin/iwlist', 'wifi0', 'scan'],
                                     stdout=subprocess.PIPE,
                                     encoding='UTF-8')
        if scan_result.returncode == 0:
            if 'resource busy' in scan_result.stdout:
                retry = retry - 1
                continue
            else:
                break
        else:
            return

    print('------------------------------------------------------------------------------------------')
    if scan_result and scan_result.returncode == 0:
        available_ap = scan_result.stdout.split('Cell ')
        for cell in available_ap:
            ssid = cell.find('ESSID:')
            ssid_quotes_start = cell.find('"', ssid)
            ssid_quotes_end = cell.find('"', ssid_quotes_start + 1)
            quality = cell.find('Quality=')
            quality_end = cell.find('\n', quality + 1)
            address = cell.find('Address: ')
            address_end = cell.find('\n', address + 1)
            if ssid > -1 and ssid_quotes_start > -1 and ssid_quotes_end > -1 \
               and quality > -1 and quality_end > -1 \
               and address > -1 and address_end > -1:
                print('{0:28.30s}\t{2}\t{1}'.
                      format(cell[ssid_quotes_start + 1:ssid_quotes_end],
                             cell[address + 9:address_end],
                             cell[quality:quality_end]))
                print('------------------------------------------------------------------------------------------')

def just_all():
    network_details()
    firmware()
    device_info()
    card_information()
    scan()
    interface_wifi = sysrepo_interfaces3.SysrepoInterfaces(app_name="wifi_cli")
    wifi_type = interface_wifi.get_interface_param('wifi0', "wifi-mode")
    if wifi_type == 'client':
        connected_clients()
    else:
        connected_ap()


def main(argv):
    try:
        opts, args = getopt.getopt(argv, "c:s:d:")
    except getopt.GetoptError:
        sys.exit(2)

    details = ''
    for opt, arg in opts:
        if opt == '-c':
            command = arg
        elif opt in "-s":
            subcommand = arg.replace(':', '')
        elif opt in "-d":
            details = arg.replace(':', '')

    if command == 'show':
        if subcommand == 'scan':
            scan()
        elif subcommand == 'network-details':
            network_details()
        elif subcommand == 'firmware':
            firmware()
        elif subcommand == 'device-info':
            device_info()
        elif subcommand == 'card-information':
            card_information()
        elif subcommand == 'connected-ap':
            connected_ap()
        elif subcommand == 'connected-clients':
            if details == 'details':
                connected_clients(details=True)
            else:
                connected_clients(details=False)

        else:
            just_all()

    sys.exit(0)


if __name__ == "__main__":
    main(sys.argv[1:])
