#!/bin/bash
# -*- mode: sh -*-
# -*- coding: utf-8 -*-
# vim: tabstop=2 expandtab shiftwidth=2 softtabstop=2
# vim: set fileencoding=utf-8
exec 1> >(exec logger -s -t $(basename $0)) 2>&1
set -o xtrace
set -o nounset
set -o pipefail
set -o errexit
#
declare -a nic_pci_ids
declare -a dataif_pci_id
declare -a nic_names
def_mode=virtio
ucpe_cfg_file=/config/ucpe_config.json
#

udevrulesfile=/etc/udev/rules.d/70-persistent-net.rules
udevtempfile=/var/tmp/70-persistent-net.rules
#

function detect_nics {
  echo "### NICs ###"
  
  current_nics_count=$(cat ${udevrulesfile} | grep NAME=\"data | wc -l) || true
  if [ ${current_nics_count} -eq 0 ]; then
    #fallback, if we fail to get interface count from udev rules
    current_nics_count=$(/usr/bin/sysrepocfg --datastore=startup --format=json --xpath="/ucpe:config/interfaces:interface/*" --export | grep "name\":" | grep data | wc -l) || true
  fi
  
  echo "" > $udevtempfile

  igbparam=""
  sed -i "s/^bridge_mappings.*/bridge_mappings = physnet99:br-chain, physnet98:nat0, physnet97:nat1, physnet96:nat2, physnet95:nat3, physnet94:nat4/" /etc/neutron/plugins/ml2/openvswitch_agent.ini.agent || true


  nic_pci_ids=($(lspci | grep Ethernet | grep -v Wireless | cut -d' ' -f1))
  
  local i=0
  while [ -n "${nic_pci_ids[${i}]+x}" ]; do
    echo " Original PCI ID: ${nic_pci_ids[$i]}"
    dataif_pci_id[$i]="0000:${nic_pci_ids[$i]}"
    dataif_name[$i]="data$i"
    # nic_names[$i]=$(grep "${nic_pci_ids[$i]}" ${visible_network_devices} | cut -d' ' -f10)

    echo " Reworked PCI ID: ${dataif_pci_id[$i]}"
    # echo " Original IF Name: ${nic_names[$i]}"
    echo " Reworked IF Name: ${dataif_name[$i]}"

    i=$((i+1))
  done

  if [ -z "${dataif_pci_id[0]+x}" ]; then
    echo "ERROR: No NICs detected!"
  fi

  nic_count=$i
  echo ""
  echo "$nic_count NICs detected."
  echo "${dataif_pci_id}"
  echo "${nic_pci_ids}"
  echo "${dataif_name}"
  
  interfaceId=0
  for (( i=${nic_count}; i>0; i-- )); do
    if [ -n "${dataif_name[$i]+x}" ]; then
      #arrays are 0 based
      sed -i "/^bridge_mappings/ s/$/,physnet${interfaceId}:br-${dataif_name[$i]}/" -i /etc/neutron/plugins/ml2/openvswitch_agent.ini.agent
      interfaceId=`expr $i + 1`
      cat >> ${udevtempfile} << EOF
SUBSYSTEM=="net", ACTION=="add", DRIVERS=="?*", ATTR{dev_id}=="0x0", ATTR{type}=="1", KERNELS=="${dataif_pci_id[$i]}", NAME="${dataif_name[$i]}"

EOF


      if [ -e "/sys/bus/pci/drivers/igb/${dataif_pci_id[$i]}" ]; then
         if [ -z "${igbparam}" ]; then
            igbparam="0"
         else
            igbparam="${igbparam},0"
         fi
      fi
      
    fi
    
    #if there is igb driver found, init its sr-iov config
    if [ -n "${igbparam}" ]; then
      echo "options igb max_vfs=${igbparam}" > /etc/modprobe.d/igb.conf
    fi
  done
  
  if [ -n "${dataif_name[0]+x}" ]; then
    #arrays are 0 based
    cat >> ${udevtempfile} << EOF
SUBSYSTEM=="net", ACTION=="add", DRIVERS=="?*", ATTR{dev_id}=="0x0", ATTR{type}=="1", KERNELS=="${dataif_pci_id[0]}", NAME="${dataif_name[0]}"

EOF
  fi

  new_nics_count=$(cat ${udevtempfile} | grep NAME=\"data | wc -l)

  if [ ${new_nics_count} -gt ${current_nics_count} ]; then
     #we have to add nics to the config
      /usr/bin/sysrepocfg --verbosity=4 --datastore=startup --format=json --export=/tmp/editconfig.json -m ucpe || true
      last_nic_id=$(expr $new_nics_count - 1)
      for i in $(seq $current_nics_count $last_nic_id) ; do 
      
        cat /tmp/editconfig.json | /usr/bin/jq ". \"ucpe:config\".\"interfaces:interface\" += [{\"name\": \"data${i}\",\"bootproto\": \"none\",\"ip-passthrough\": false,\"link-state-propagation\": true,\"management\": true,\"metric\": 100,\"mode\": \"virtio\",\"type\": \"ethernet\",\"vswitch\": \"br-data${i}\"}]" > /tmp/editconfig2.json
        mv -f /tmp/editconfig2.json /tmp/editconfig.json
         
      done
       /usr/bin/sysrepocfg --verbosity=4 --datastore=startup --format=json --import=/tmp/editconfig.json  -m ucpe || true
      
  fi
  
  if [ ${new_nics_count} -lt ${current_nics_count} ]; then
     #we have to remove nics from the config
     /usr/bin/sysrepocfg --verbosity=4 --datastore=startup --format=json --export=/tmp/editconfig.json  -m ucpe || true
     last_nic_id=$(expr $current_nics_count - 1)

     for i in $(seq $new_nics_count $last_nic_id) ; do 
          #we have to delete data$i
          all_ifs=`cat /tmp/editconfig.json | /usr/bin/jq " .\"ucpe:config\".\"interfaces:interface\"" | grep \"name\": | wc -l`
          all_ifs_end=$(expr $all_ifs - 1)
          for jsoni in $(seq 0 $all_ifs_end) ; do 
               match=`cat /tmp/editconfig.json | /usr/bin/jq " .\"ucpe:config\".\"interfaces:interface\" [${jsoni}]" | grep name | grep \"data${i}\"` || true
               if [ -n "${match}" ] ; then 
                 #we have found an interface to delete
                 cat /tmp/editconfig.json | /usr/bin/jq " del (.\"ucpe:config\".\"interfaces:interface\" [${jsoni}])  "  >  /tmp/editconfig2.json
                 mv -f /tmp/editconfig2.json /tmp/editconfig.json
                 break
               fi
          done
     done
     /usr/bin/sysrepocfg --verbosity=4 --datastore=startup --format=json --import=/tmp/editconfig.json -m ucpe || true
  fi
  
  rm /etc/udev/rules.d/90-wifi.rules || true
  if [ -n "$(lspci | grep "Network controller: Qualcomm Atheros QCNFA765")" ]; then
  wifi_pci_id=($(lspci | grep "Network controller: Qualcomm Atheros QCNFA765" | head -n 1 | cut -d' ' -f1))
  cat > /etc/udev/rules.d/90-wifi.rules << EOF
SUBSYSTEM=="net", ACTION=="add", DRIVERS=="?*", ATTR{dev_id}=="0x0", ATTR{type}=="1", KERNELS=="0000:${wifi_pci_id}", NAME="wifi0"

EOF

  else
  
#wifi dongles
cat > /etc/udev/rules.d/90-wifi.rules << EOF
SUBSYSTEM=="net", SUBSYSTEMS=="usb", ACTION=="add", DRIVERS=="mt76x0u", NAME="wifi0"
SUBSYSTEM=="net", SUBSYSTEMS=="usb", ACTION=="add", DRIVERS=="mt76x2u", NAME="wifi0"
SUBSYSTEM=="net", SUBSYSTEMS=="usb", ACTION=="add", DRIVERS=="88XXau", NAME="wifi0"

EOF

#workaround for mt76x02u_mcu_wait_resp failed with -108 
echo "options mt76_usb disable_usb_sg=1" > /etc/modprobe.d/mt76_usb.conf || true

  
  fi
  
  mv -f ${udevtempfile} ${udevrulesfile}

  echo "NICs Done"

} #function detect_nics
#
detect_nics
exit 0
