#!/bin/bash
source /scripts/basic_vars || true
source /scripts/basic_functions || true
RESOLVCONF="/etc/resolv.conf"
NUM_NAT_IFS=5
vpp_dhcp_ippass_script=/opt/bin/vpp_ippass_dhcp.py
if [ "$LOG_IN_FILE" -eq 1 ]; then
  enable_logging /var/log/homecalling
fi
echo "DHCP event on interface $1, system is $system_status, reason $2, new $3, old $4" | systemd-cat -p info -t "dhclient-event-process"
interface=$1
reason=$2
function remove_srvs_resolv_conf() {
  if [ -z "${1}" ] ; then
    return
  fi
  rscf="$(mktemp "${TMPDIR:-/tmp}/XXXXXX")"
  if [ -z "${rscf}" ] ; then
    return
  fi
  truncate -s 0 ${rscf}
  b_skip=0
  while read -r line; do
    if [ "0" == "$b_skip" ] ; then
      if [[ $line = *" ${1} :"* ]] ; then
        b_skip=1
      else
        echo "${line}" >> "${rscf}"
      fi
    else
      b_skip=0
    fi
  done < "${RESOLVCONF}"
  newres=$(cat "${rscf}");
  eval $(echo "${newres}" > "${RESOLVCONF}"; echo "status=$?")
  if [ $status -eq 0 ]; then
    echo "updated ${RESOLVCONF} with ${newres}" | systemd-cat -p info -t "dhclient-event-process"
    [[ -e /var/run/nscd/socket ]] && /usr/sbin/nscd -i hosts; # invalidate cache
  fi
  rm -f "${rscf}"
} # function remove_srvs_resolv_conf
function update_k8s_bind_address() {
    if [ -f $K8S_DISABLED_FILE -a ! -f /config/central-box ]; then
      return 0
    fi

    old_ip=$4
    new_ip=$3
    reason=$2
    if [ -z "$new_ip" ] ; then
      if [[ $reason != *6 ]]; then
        new_ip=$(ifconfig $1 | grep "inet " | awk '{print $2}')
      else
        new_ip=$(ifconfig $1 | grep "inet6 " | grep -v -e "fe80:" | awk '{print $2}')
      fi
    fi

    bind_interface=$(sysrepocfg --datastore=running --format=xml --xpath="/ucpe:config/kubernetes:kubernetes/bind-interface" --export | grep bind-interface | cut -d '>' -f 2 | cut -d '<' -f1)
    # Only data interfaces are suitible for being bind interfaces for now
    grep -q "data" <<< $bind_interface && bind_interface="br-$bind_interface" && [[ $bind_interface == $1 ]] && /scripts/handle_k8s_ip_change.sh $new_ip $reason || true

} # function update_k8s_bind_address
function update_bind_address() {
  new_ip=$2
  old_ip=$3
  if [ -z "$new_ip" ] ; then
    if [[ $reason != *6 ]]; then
      new_ip=$(ifconfig $1 | grep "inet " | awk '{print $2}')
    else
      new_ip=$(ifconfig $1 | grep "inet6 " | grep -v -e "fe80:" | awk '{print $2}')
    fi
  fi

  vlan_if=$(echo $1 | cut -d. -f1)

  # New address will be assigned without sysrepo knowledge, so inform current config subscriber to can manage that situation
  if [ "$cbox_state" == "true" ] && [ -n "$bind_interface" ]; then
    if [ "$bind_interface" == "$1" ] || [ "br-$bind_interface" == "$1" ]; then
      echo -e "$1\n$new_ip" > /tmp/newcbox_ip
    elif [ "$bind_interface" == "$vlan_if" ]; then
      echo -e "$bind_interface\n$new_ip" > /tmp/newcbox_ip
    fi
    rm -rf /config/_sendconfig/config/*
    mkdir -p /config/_sendconfig/config/config
  fi
CHANGE_IP=0
  if [ -n "$bind_interface" ] ; then
    if [ "$bind_interface" == "$1" ] || [ "br-$bind_interface" == "$1" ] || [ "$bind_interface" == "$vlan_if" ]; then
      #if [ -z "$old_ip" ] ; then
      #old_ip=$(cat /etc/hosts | grep "controller" | awk '{print $1}')
      #fi
      if [[ "$reason" == "BOUND"  || "$reason" == "BOUND6" ]]; then
        CHANGE_IP=1
      fi
      if [[ "$reason" == "REBOOT" || "$reason" == "REBOOT6" ]]; then
        CHANGE_IP=1
      fi
      if [[ "$reason" == "RENEW" || "$reason" == "RENEW6"  ]]; then
        CHANGE_IP=1
      fi
    fi
  fi
  if [ "$CHANGE_IP" -eq 1 ]; then
    /scripts/set_openstack_address.sh $new_ip || true
  fi
  touch $hb_force_file
} # function update_bind_address
function update_NAT_table() {
  if=$1
  # Advance only if working with bridge interface
  if [[ "$if" == "br-data"* || "$if" == "br-lte"* || "$if" == "br-wifi"* ]]; then
    # ----------
    # Go through all NATs and check if the current interface is found as wan-interface in a NAT
    # If found, check that NAT's routing table and refresh it (if the current interface is missing)
    # ----------
    phys_if=$(echo $if | cut -d '-' -f2)
    gw="$(/sbin/ip route list exact default | grep $if | cut -d ' ' -f3)"
    metric="$(/sbin/ip route list exact 0.0.0.0/0 | grep $if | cut -d ' ' -f7)"
    if [[ -z "$metric" ]]; then metric="none"; fi
    gw6="$(/sbin/ip -6 route list exact default | grep $if | cut -d ' ' -f3)"
    metric6="$(/sbin/ip -6 route list exact default | grep $if | cut -d ' ' -f9)"
    if [[ -z "$metric6" ]]; then metric6="none"; fi

    for (( nat_id=0; nat_id<NUM_NAT_IFS; nat_id++ )); do
      nat_enabled="$(sysrepocfg --datastore=running --format=json --xpath="/ucpe:config/interfaces:interface[name='nat$nat_id']//*" --export | grep enable | tr -d ' ' | cut -d ':' -f 2 | cut -d ','  -f 1)"
      nat_active="$(ip link | grep nat0 | cut -d" " -f9)"

      if [[ "$nat_enabled" == "true" && "$nat_active" != "DOWN" ]]; then
        is_wan_interface="$(sysrepocfg --datastore=running --format=json --xpath="/ucpe:config/interfaces:interface[name='nat$nat_id']/wan-interface//*" --export | grep $phys_if)"
        if [[ "$is_wan_interface" != "" ]]; then
          table_id="$(expr $nat_id + 1)" # table_index is nat_index+1
          route_check=$(ip route show table $table_id | grep $if | grep default)
          route_check_6=$(ip -6 route show table $table_id | grep $if | grep default)

          # Updating NAT table if route is missing
          if [[ "$route_check" == "" && -n "$gw" && -n "$metric" ]]; then
            python3 /usr/lib/python3/dist-packages/tveif/if_nat.py $if $nat_id $gw $metric
          fi

          # Updating NAT table if route is missing (IPv6)
          if [[ "$route_check_6" == "" && -n "$gw6" && -n "$metric6" ]]; then
            python3 /usr/lib/python3/dist-packages/tveif/if_nat.py $if $nat_id $gw6 $metric6
          fi
        fi
      fi
    done
  fi
} # funciton update_NAT_table
#
# main
if [ -f $reboot_in_progress ] ; then
  echo "Reboot is in progress, exiting" | systemd-cat -p info -t "dhclient-event-process"
  exit
fi
case $2 in
  "REBOOT"|"BOUND"|"RENEW"|"REBIND"|"REBOOT6"|"BOUND6"|"RENEW6"|"REBIND6")
    update_bind_address ${1} ${3} ${4}
    update_NAT_table $1
    if [ "$2" != "REBIND" ]; then
        update_k8s_bind_address ${1} ${2} ${3} ${4}
    fi
    if [ -f $vpp_dhcp_ippass_script ]; then
      if [ ! -z "${4}" ] ; then
        echo "going to del ippass ipv4 addr: ${4}"
        $vpp_dhcp_ippass_script ippass_permit_host --ifname ${1} --ip-addr ${4} --delete
      fi
      if [ ! -z "${3}" ] ; then
        echo "going to add ippass ipv4 addr: ${3}"
        $vpp_dhcp_ippass_script ippass_permit_host --ifname ${1} --ip-addr ${3}
      fi
    fi
    ;;
  "RELEASE")
    remove_srvs_resolv_conf ${1}
    echo "release"
    echo ${1} ${4}
    if [ ! -z "${4}" ] ; then
      echo "going to del ippass addr"
      $vpp_dhcp_ippass_script ippass_permit_host --ifname ${1} --ip-addr ${4} --delete
    fi
    ;;
  "EXPIRE"|"FAIL"|"STOP"|"TIMEOUT"|"EXPIRE6"|"STOP6")
    if [[ "$interface" == "lte*" ]]; then
      # start modem again in case the vendor disconnected us.
      #it did not called it now , so commenting  /scripts/setup_lte_qmi_dev.py || remove_srvs_resolv_conf ${1}
      :
    else
      remove_srvs_resolv_conf ${1}
    fi
    echo "fail"
    echo ${1} ${3} ${4}
    if [ ! -z "${4}" ] ; then
      echo "going to del ippass addr"
      $vpp_dhcp_ippass_script ippass_permit_host --ifname ${1} --ip-addr ${4} --delete
    fi
    ;;
  *)
    echo "Unknown reason $2" | systemd-cat -p info -t "dhclient-event-process"
    exit 0
    ;;
esac
exit 0
