#!/usr/bin/env bash
exec 1> >(exec logger -s -t $(basename $0)) 2>&1
source /scripts/basic_vars
source /scripts/basic_functions
set -o xtrace
set -o nounset
set -o pipefail

if [[ -c /dev/cdc-wdm_pcie ]] ; then
  # mbim modem found
  /sbin/ip link set mbim_mhi0 down || true
  /sbin/ip link set mbim_mhi0 name lte0 || true
  /sbin/ip link set lte0 up || true
  /bin/rm -f /tmp/mbim-network-state-cdc-wdm_pcie || true
fi

# Resize the virtual disk if there is a difference in the partition relative to the disk (ONLY FOR VMs)
isHypervisor=$(cat /proc/cpuinfo | grep flags | grep hypervisor || true)
if [[ $isHypervisor ]]; then
  partition=`mount -v | fgrep 'on / ' | awk '{print $1}' | cut -f3 -d "/"`
  disk=`echo $partition | tr -d [:digit:]`
  partition_num=$(cat /sys/block/${disk}/${partition}/partition)
  disk_size=`lsblk -l | grep -w $disk | awk '{print $4+0}'`
  part_size=`lsblk -l | grep -w $partition | awk '{print $4+0}'`
  if [ $disk_size -gt $part_size ]; then
    growpart /dev/$disk $partition
    resize2fs /dev/${partition} || true
  fi
else
  if [ -n "$(dpkg --list | grep -i walinuxagent || true)" ]; then
    DEBIAN_FRONTEND=noninteractive apt-get -y -qq --no-install-recommends --no-install-suggests --allow-unauthenticated purge walinuxagent cloud-init amazon-ssm-agent cloud-guest-utils python3-jinja2 || true
  fi
fi


touch /run/manager-contacted || true

UCPE_CONFIG='/config/ucpe_config.json'
#
# workaround to disable kernel lockdown in securedboot
sysctl -w kernel.sysrq=1 || true
echo x >/proc/sysrq-trigger || true
sysctl -w kernel.sysrq=176 || true
if [ "nanopir2s" != "$(cat /config/.hwtype)" -a "nanopir4s" != "$(cat /config/.hwtype)" ]; then
  # enable ipv4 forwarding
  sysctl -w net.ipv4.ip_forward=1 || true
  # during upgrade from older releases, there is leftover 'net.ipv4.ip_forward=0' at the end of the file
  # and with the previous handling of uncommenting '#net.ipv4.ip_forward=1' line and rolling back to the original version,
  # resulted in duplicate lines and error in /scripts/ovs-config.py
  sed -i '/^net.ipv4.ip_forward/d' /etc/sysctl.conf || true
  sed -i '${x;/^$/{s//net.ipv4.ip_forward=1/;H};x}' /etc/sysctl.conf || true
  sed -i '/^net.ipv6.conf..*.disable_ipv6/d' /etc/sysctl.conf || true
else
  if [ ! -d /root/apps/clavister* ] && [ ! -d /root/apps/Clavister* ] && [ ! -d /root/apps/slmprobe ]; then
    # only if no applications which control forwaring
    sysctl -w net.ipv4.ip_forward=1 || true
    sed -i '/^net.ipv4.ip_forward/d' /etc/sysctl.conf || true
    sed -i '${x;/^$/{s//net.ipv4.ip_forward=1/;H};x}' /etc/sysctl.conf || true
  fi
fi

if [ "nanopir2s" = "$(cat /config/.hwtype)" -o "nanopir4s" = "$(cat /config/.hwtype)" ]; then
  if [ ! -f /config/.serial-number ]; then
    echo "$(shuf -i 100000-1000000 -n 1 | tr -d ' \n')" > /config/.serial-number || true
  fi
  newserial="dmesg | grep -i 'friendlyelec-board board' | cut -d ':' -f3 | tr -d ' \n'"
  newserial=""
  if [ -f /sys/firmware/devicetree/base/serial-number ]; then
    newserial="cat /sys/firmware/devicetree/base/serial-number | tr -d ' \n'"
  fi
  if [ -n "$(eval ${newserial})" ] && [ "$(eval ${newserial})" != "$(cat /config/.serial-number)" ]; then
    echo "$(eval ${newserial})" > /config/.serial-number || true
    newhostname="$(cat /config/.hwtype | tr -d ' \n')$(cat /config/.serial-number | tr -d ' \n')"
    /usr/bin/sysrepocfg --verbosity=4 --datastore=startup --format=json --export=/tmp/oldhostname.json || true
    has_hostname_config=$(/usr/bin/jq ".[]" /tmp/oldhostname.json)
    first_first=0
    if [ -z "${has_hostname_config}" ]; then
      cat ${UCPE_CONFIG} > /tmp/oldhostname.json
      first_first=1
    fi
    oldhostname="$(grep hostname /tmp/oldhostname.json | cut -d ':' -f2 | tr -d '" ,\n')"
    echo "${newhostname}" > /etc/hostname
    hostnamectl set-hostname ${newhostname} || true
    echo -e "${newhostname}\nnopass" > /opt/vpn-name
    if [ -n "${oldhostname}" ]; then
      sed -i "s/$oldhostname/$newhostname/g" /etc/hosts || true
      sed -i "s/$oldhostname/$newhostname/g" /tmp/oldhostname.json || true
      if [ 1 -eq ${first_first} ]; then
        sed -i "s/$oldhostname/$newhostname/g" ${UCPE_CONFIG} || true
      fi
      /usr/bin/sysrepocfg --import=/tmp/oldhostname.json --datastore=startup --format=json -m ucpe || true
    fi
    # if new serial this is new device - new keys
    /usr/bin/ssh-keygen -A -vvv || true
  fi
  if [ ! -f ${ztp_authfile} ]; then
    touch ${ztp_authfile} || true
    chmod 777 ${ztp_authfile} || true
  fi
  # set some recent time before ntp kicks on
  date +%Y%m%d -s "20230101" || true
  #
  if [ ! /usr/bin/systemd-resolve ]; then
    ln -s /usr/bin/resolvectl /usr/bin/systemd-resolve || true
  fi
  # if /etc/resolv.conf is not a symlink, try to make one
  if [ -f /run/systemd/resolve/stub-resolv.conf ] && [ -f /etc/resolv.conf ] && [ ! -L /etc/resolv.conf ]; then
    rm -f /etc/resolv.conf || true
    ln -s /run/systemd/resolve/stub-resolv.conf /etc/resolv.conf || true
  fi

else

  # for non-NanoPI device, handle S/N check
  # kernel params:
  # kernel_serial_number=ABCDEF... -> "assume provided S/N"
  # sn_maxreboots=N
  sn_maxreboots=$(cat /proc/cmdline | egrep "sn_maxreboots=[0-9]+" | cut -f2 -d'=') && true
  if [ ! -f "/config/central-box" ] && [ -f "/config/_force_sn_check" ]; then
    rm -f /config/_force_sn_check || true
    /scripts/get_serial_number.sh
    NEWSN=$(cat /run/serial || tr -d "\n ")
    if [ -n "${NEWSN}" ] && [ "${NEWSN}" != "$(cat /config/.serial-number)" ]; then
      # mismatch of S/N
      if [ -z "$sn_maxreboots" ]; then
        sn_maxreboots=${SN_REBOOTMAX}
      fi
      if [ ! -f ${SN_REBOOTCNDFILE} ]; then
        echo 0 > ${SN_REBOOTCNDFILE}
      fi

      rebootcnt=$(cat ${SN_REBOOTCNDFILE} | tr -d "\n ") && true
      if [ ${rebootcnt} -le $sn_maxreboots ]; then
        # we have to handle S/N update

        echo "${NEWSN}" > /config/.serial-number || true
        newhostname="$(cat /config/.hwtype | tr -d ' \n' | tr '[:upper:]' '[:lower:]' )$(cat /config/.serial-number | tr -d ' \n')"
        /usr/bin/sysrepocfg --verbosity=4 --datastore=startup --format=json --export=/tmp/oldhostname.json || true

        oldhostname="$(grep hostname /tmp/oldhostname.json | cut -d ':' -f2 | tr -d '" ,\n')"
        echo "${newhostname}" > /etc/hostname
        hostnamectl set-hostname ${newhostname} || true
        echo -e "${newhostname}\nnopass" > /opt/vpn-name
        if [ -n "${oldhostname}" ]; then
          sed -i "s/$oldhostname/$newhostname/g" /etc/hosts || true
          sed -i "s/$oldhostname/$newhostname/g" /tmp/oldhostname.json || true

          /usr/bin/sysrepocfg --import=/tmp/oldhostname.json --datastore=startup --format=json -m ucpe || true
        fi
        # if new serial this is new device - new keys
        /usr/bin/ssh-keygen -A -vvv || true

        #update nics config
        /scripts/detect_nics.sh

        let rebootcnt+=1
        echo ${rebootcnt} > ${SN_REBOOTCNDFILE}
        sync
        sleep 1
        reboot
      else
        echo "MAX REBOOT COUNT REACHED."
      fi
    fi

  fi
fi
# remove IPv6 disabling in IPv4 only systems
sysctl -w net.ipv6.conf.all.disable_ipv6=0 || true
sysctl -w net.ipv6.conf.default.disable_ipv6=0 || true
sysctl -w net.ipv6.conf.lo.disable_ipv6=0 || true
# enable IPv6 routing
sysctl -w net.ipv6.conf.all.forwarding=1 || true
sysctl -w net.ipv6.conf.default.forwarding=1 || true
# IPv6 gateway delay at startup fix
# Number of RS to be sent (default 3)
sysctl -w net.ipv6.conf.all.router_solicitations=10 || true
sysctl -w net.ipv6.conf.default.router_solicitations=10 || true
# Number of seconds between RS (default 4)
sysctl -w net.ipv6.conf.all.router_solicitation_interval=6 || true
sysctl -w net.ipv6.conf.default.router_solicitation_interval=6 || true
# Number of seconds to wait on ifup before RS are sent (default 1)
sysctl -w net.ipv6.conf.all.router_solicitation_delay=3 || true
sysctl -w net.ipv6.conf.default.router_solicitation_delay=3 || true
#
sysctl -w net.ipv6.conf.all.router_solicitation_max_interval=60 || true
sysctl -w net.ipv6.conf.default.router_solicitation_max_interval=60 || true
# accept_ra must be 2, so that ipv6_forwarding is overwritten, so that RS/RA are accepted
sysctl -w net.ipv6.conf.all.accept_ra=2 || true
sysctl -w net.ipv6.conf.default.accept_ra=2 || true
# disable temp IPv6 addresses that stack up rfc 4941
sysctl -w net.ipv6.conf.all.use_tempaddr=0 || true
sysctl -w net.ipv6.conf.default.use_tempaddr=0 || true
sysctl -w net.ipv6.conf.all.autoconf=0 || true
sysctl -w net.ipv6.conf.default.autoconf=0 || true
sysctl -p || true
#
sysctl -w net.core.rmem_max=8738000 || true
sysctl -w net.core.wmem_max=6553600 || true
#
sysctl -w net.ipv4.conf.default.rp_filter=2 || true
sysctl -w net.ipv4.conf.all.rp_filter=0 || true
# restore tuned localhost for IPv6 and restore it to its original version
mv -f /etc/hosts /etc/hosts.tmpfb
sed -i "/^::1\s/d" /etc/hosts.tmpfb || true
echo "::1     ip6-localhost ip6-loopback" >> /etc/hosts.tmpfb
mv -f /etc/hosts.tmpfb /etc/hosts
#
# workaround for xeon problem
#echo "disable" > /sys/firmware/acpi/interrupts/gpe6F || true
echo "SRPARAM=" > /etc/sr-systemresources.env || true
#
echo "SVC_LIST=" > /config/svcs_to_restart || true
#
echo "No error." > /config/_hb/last_known_err
#
if [ -n "${isHypervisor}" ]; then
  if [ -f "/config/central-box" ]; then
    if [ -f /sys/devices/virtual/dmi/id/sys_vendor ] && [ "1" = "$(grep Microsoft /sys/devices/virtual/dmi/id/sys_vendor | wc -l)" ]; then
      # vm_nic_update.py is not working in Azure
      if [ -n "$(cat /sys/class/net/eth0/address)" ]; then
        echo "SUBSYSTEM==\"net\", ACTION==\"add\", DRIVERS==\"hv_netvsc\", ATTR{address}==\"$(cat /sys/class/net/eth0/address)\", NAME=\"data0\"" > /etc/udev/rules.d/70-persistent-net.rules
      elif [ -n "$(cat /sys/class/net/data0/address)" ]; then
        echo "SUBSYSTEM==\"net\", ACTION==\"add\", DRIVERS==\"hv_netvsc\", ATTR{address}==\"$(cat /sys/class/net/data0/address)\", NAME=\"data0\"" > /etc/udev/rules.d/70-persistent-net.rules
      fi
      mount -o ro -t udf,iso9660 /dev/sr0 /mnt/ 2>/dev/null || true
      mkdir -p /var/lib/waagent || true
      cp /mnt/* /var/lib/waagent/ 2>/dev/null || true
      umount /mnt 2>/dev/null || true
      sed -i 's/ResourceDisk.Format=y/ResourceDisk.Format=n/g' /etc/waagent.conf || true
      sed -i 's/ResourceDisk.EnableSwap=y/ResourceDisk.EnableSwap=n/g' /etc/waagent.conf || true
      if [ -f "/etc/cloud/cloud.cfg.d/10_net.cfg" ]; then
        if [ "1" == "$(grep eth0 /etc/cloud/cloud.cfg.d/10_net.cfg | wc -l)" ]; then
          cat > /etc/cloud/cloud.cfg.d/10_net.cfg <<EOF
network:
  config: disabled
EOF
        fi
      fi
      ifconfig eth0 down || true
      ip link set name data0 dev eth0 || true
      ifconfig data0 up || true
      systemctl enable cloud-init || true
      systemctl enable walinuxagent || true
      systemctl --no-block start walinuxagent || true
      dhclient data0 || true
      if [ ! -f /etc/collectd/collectd.conf.d/azuredf.conf ]; then
        cp -f /opt/etc/azuredf.conf /etc/collectd/collectd.conf.d/
      fi
    elif [ -f /sys/devices/virtual/dmi/id/sys_vendor ] && [ "1" = "$(grep -i openstack /sys/devices/virtual/dmi/id/sys_vendor | wc -l)" ]; then
      # vm_nic_update.py is not working in Amazon
      if [ -n "$(cat /sys/class/net/eth0/address)" ]; then
        echo "SUBSYSTEM==\"net\", ACTION==\"add\", DRIVERS==\"?*\", ATTR{address}==\"$(cat /sys/class/net/eth0/address)\", NAME=\"data0\"" > /etc/udev/rules.d/70-persistent-net.rules
      elif [ -n "$(cat /sys/class/net/data0/address)" ]; then
        echo "SUBSYSTEM==\"net\", ACTION==\"add\", DRIVERS==\"?*\", ATTR{address}==\"$(cat /sys/class/net/data0/address)\", NAME=\"data0\"" > /etc/udev/rules.d/70-persistent-net.rules
      fi
      if [ -f "/etc/cloud/cloud.cfg.d/10_net.cfg" ]; then
        if [ "1" == "$(grep eth0 /etc/cloud/cloud.cfg.d/10_net.cfg | wc -l)" ]; then
          cat > /etc/cloud/cloud.cfg.d/10_net.cfg <<EOF
network:
  config: disabled
EOF
        fi
      fi
      ifconfig eth0 down || true
      ip link set name data0 dev eth0 || true
      ifconfig data0 up || true
      systemctl enable cloud-init || true
      dhclient data0 || true
    elif ([ -f /sys/hypervisor/uuid ] && [ "1" = "$(grep -i ^ec2 /sys/hypervisor/uuid | wc -l)" ]) || \
         ([ -f /sys/devices/virtual/dmi/id/product_uuid ] && [ "1" = "$(grep -i ^ec2 /sys/devices/virtual/dmi/id/product_uuid | wc -l)" ]); then
      # vm_nic_update.py is not working in Amazon
      if [ -n "$(cat /sys/class/net/eth0/address)" ]; then
        echo "SUBSYSTEM==\"net\", ACTION==\"add\", DRIVERS==\"?*\", ATTR{address}==\"$(cat /sys/class/net/eth0/address)\", NAME=\"data0\"" > /etc/udev/rules.d/70-persistent-net.rules
      elif [ -n "$(cat /sys/class/net/data0/address)" ]; then
        echo "SUBSYSTEM==\"net\", ACTION==\"add\", DRIVERS==\"?*\", ATTR{address}==\"$(cat /sys/class/net/data0/address)\", NAME=\"data0\"" > /etc/udev/rules.d/70-persistent-net.rules
      fi
      if [ -f "/etc/cloud/cloud.cfg.d/10_net.cfg" ]; then
        if [ "1" == "$(grep eth0 /etc/cloud/cloud.cfg.d/10_net.cfg | wc -l)" ]; then
          cat > /etc/cloud/cloud.cfg.d/10_net.cfg <<EOF
network:
  config: disabled
EOF
        fi
      fi
      if [ -n "$(cat /sys/class/net/eth0/address)" ]; then
        ifconfig eth0 down || true
        ip link set name data0 dev eth0 || true
      fi
      ifconfig data0 up || true
      systemctl enable cloud-init || true
      systemctl enable amazon-ssm-agent.service || true
      systemctl --no-block start amazon-ssm-agent.service || true
      dhclient data0 || true
    else
      # if we are central cbox, and under hypervisor, adjust data0 pci id
      /scripts/vm_nic_update.py
    fi

    if [ -f /config/.hamode ]; then
      #if we are in HA mode, OpenVPN client depends on keepalived
      systemctl --no-block disable openvpn@client.service
    fi
  fi
fi
#
isserial=$(cat /proc/cmdline | grep ttyS0) && true
isinactive=$(systemctl is-active serial-getty@ttyS0.service | grep inactive) && true
if [ -n "${isserial}" ]; then
  if [ -n "${isinactive}" ]; then
    systemctl start --no-block serial-getty@ttyS0.service || true
  fi
fi
#
ismasked=$(systemctl is-enabled sr-currentconfig.service | grep masked) && true
if [ -n "$ismasked" ]; then
  # durng upgrade we mask this service
  systemctl unmask --no-block sr-currentconfig.service || true
  systemctl enable --no-block sr-currentconfig.service || true
fi

#
rmdir /config/_reboot/tve/now 2>/dev/null || true
rm -rf /config/_lte_dongle_detect/* 2>/dev/null || true
chmod a+w /run || true
#
echo "{\"ucpe:status\": {\"status\": \"unknown\", \"hb-status\": \"unknown\", \"nsStatus\": \"unknown\", \"ztp-status\": \"disabled\", \"certs\": \"default\", \"license-id\": \"unknown\", \"apps\": \"false\", \"cluster\": {}, \"appslist\":{}}}" > ${UCPE_STATUS_FILE} || true
chmod 777 ${UCPE_STATUS_FILE} || true
#
if [ -n "$(lspci -nn | grep -i 8086:37c8 | head -n 1)" ]; then
  for PCIID in $((lspci -d :37c8 ) | cut -d ' ' -f 1); do
    echo 16 > /sys/bus/pci/drivers/c6xx/0000:$PCIID/sriov_numvfs || true
  done
fi
#
if [ "aarch64" == "$(uname -p)" ] && [ "nanopir2s" != "$(cat /config/.hwtype)" -a "nanopir4s" != "$(cat /config/.hwtype)" ]; then
  libguestfs-test-tool || true &
fi
#
if [ -f /sys/hypervisor/uuid ] && [ "1" = "$(grep -i ^ec2 /sys/hypervisor/uuid | wc -l)" ]; then
  if [ -f /etc/amazon/ssm/seelog.xml.template ]; then
    mv /etc/amazon/ssm/seelog.xml.template /etc/amazon/ssm/seelog.xml
  fi
  if [ -f /etc/amazon/ssm/amazon-ssm-agent.json.template ]; then
    mv /etc/amazon/ssm/amazon-ssm-agent.json.template /etc/amazon/ssm/amazon-ssm-agent.json
  fi
  systemctl enable amazon-ssm-agent || true
elif [ -f /sys/devices/virtual/dmi/id/product_uuid ] && [ "1" = "$(grep -i ^ec2 /sys/devices/virtual/dmi/id/product_uuid | wc -l)" ]; then
  if [ -f /etc/amazon/ssm/seelog.xml.template ]; then
    mv /etc/amazon/ssm/seelog.xml.template /etc/amazon/ssm/seelog.xml
  fi
  if [ -f /etc/amazon/ssm/amazon-ssm-agent.json.template ]; then
    mv /etc/amazon/ssm/amazon-ssm-agent.json.template /etc/amazon/ssm/amazon-ssm-agent.json
  fi
  systemctl enable amazon-ssm-agent || true
else
  systemctl stop --no-block amazon-ssm-agent || true
fi
#
if [ "nanopir2s" = "$(cat /config/.hwtype)" -o "nanopir4s" = "$(cat /config/.hwtype)" ]; then
  if [ -f /config/nanopi_customization ]; then
    rm -f /config/nanopi_customization
    /scripts/apply_customization.sh || true
  fi
fi

if [ -f /config/central-box ] && [ -f /config/central-box-customization ]; then
  rm -f /config/central-box-customization || true
  /scripts/apply_customization.sh || true
fi
#

# NFV17321
if grep "tcpdumpuser" /scripts/cli/sshd_config; then
  true
else
    echo "Match Address 127.0.0.1,::1 User tcpdumpuser
       PubkeyAuthentication yes"  >> /scripts/cli/sshd_config
fi

if grep "tcpdumpuser" /etc/passwd; then
  true
else
  useradd -m --home /home/tcpdumpuser -G admin,tveapi --shell /scripts/cli/bin/clish tcpdumpuser
  sudo -u tcpdumpuser ssh-keygen -N "" -f /home/tcpdumpuser/.ssh/id_rsa <<<y
  cp /home/tcpdumpuser/.ssh/id_rsa.pub /home/tcpdumpuser/.ssh/authorized_keys
  chown tcpdumpuser:tveapi /home/tcpdumpuser
  chown tcpdumpuser:tveapi /home/tcpdumpuser/.ssh
  chown tcpdumpuser:tveapi /home/tcpdumpuser/.ssh/*
  chmod 755 /home/tcpdumpuser
  chmod 755 /home/tcpdumpuser/.ssh
  chmod 755 /home/tcpdumpuser/.ssh/*
fi


if [ ! -f "/config/central-box" ]; then
  # enforce licensing
  CPU_PRESENT=/sys/devices/system/cpu/present
  CPU_CAPACITY_FILE=/config/license/cpu
  cpu_present=$(cat ${CPU_PRESENT} | cut -d'-' -f2) || true
  if [ -f ${CPU_CAPACITY_FILE} ]; then
    cpu=$(cat ${CPU_CAPACITY_FILE} | xargs || true)
    if [[ "$cpu" == *"unlimited"* ]]; then
      cpu=999
    fi
  else
    cpu=3
  fi
  for (( cpu_i=$cpu; cpu_i<=$cpu_present; cpu_i++ )); do
    echo "set cpu $cpu_i offline" || true
    sudo echo "0">/sys/devices/system/cpu/cpu${cpu_i}/online || true
  done
fi

#
if [ -z "$(grep set_ciphers /usr/lib/python3/dist-packages/redis/connection.py)" ]; then
  sed -i "s/.*ssl\.create_default_context.*/&\n            context.set_ciphers('DEFAULT@SECLEVEL=1')/" /usr/lib/python3/dist-packages/redis/connection.py || true
fi
#

modprobe br_netfilter || true
truncate -s 0 /kubernetes/calico-ready || true

#
if [ ! -f "/scripts/cli/tvesh/jq-linux64" ]; then
    ln -s /usr/bin/jq /scripts/cli/tvesh/jq-linux64 || true
fi

[ "0" == "${firstboot}" ] && exit 0
#

# workaround to install HAILO NVIDIA Drivers
if [ -z "$(dkms status | grep hailo || true)" ]; then
  if [ -d /share/opt/hailo/linux/pcie/ ]; then
    pushd /share/opt/hailo/linux/pcie/
    make install_dkms || true
    modprobe -rq hailo_pci || true
    modprobe hailo_pci || true
    popd || true
    sleep 3s || true
  fi

  if [ -n "$(lspci -nn | grep -i 1e60:2864 || true)" ]; then
    # if there is hailo hardware, run update nics config, as PCI IDs may have changed
    /scripts/detect_nics.sh || true
  fi
fi



# workaround to spinup disk performance on armortec
if [ -n "$(cat /proc/cpuinfo | grep model | grep J4125 || true)" ]; then
  timeout --preserve-status 4m dd if=/dev/sda of=/dev/null || true &
fi

echo "fs.file-max = 10000000" >> /etc/sysctl.conf || true
echo "net.core.rmem_max = 8738000" >> /etc/sysctl.conf || true
echo "net.core.wmem_max = 6553600" >> /etc/sysctl.conf || true
echo "net.ipv4.tcp_tw_reuse = 1" >> /etc/sysctl.conf || true
echo "net.ipv4.tcp_max_tw_buckets = 360000" >> /etc/sysctl.conf || true
echo "net.core.netdev_max_backlog = 2500" >> /etc/sysctl.conf || true
echo "net.core.somaxconn = 65536" >> /etc/sysctl.conf || true
echo "kernel.printk = 1 2 2 2" >> /etc/sysctl.conf || true
echo "kernel.core_pattern=|/scripts/gen_compress_core.sh %e" >> /etc/sysctl.conf || true
echo "fs.suid_dumpable = 2" >> /etc/sysctl.conf || true
# somehow last line in sysctl.conf is lost ?
echo "fs.inotify.max_user_instances = 8192" >> /etc/sysctl.conf || true
echo "fs.inotify.max_user_instances = 8192" >> /etc/sysctl.conf || true

# limit corefile to 2GB=2097152KB
# all (*) not working
echo "root soft core unlimited" >> /etc/security/limits.conf || true
echo "admin soft core unlimited" >> /etc/security/limits.conf || true
echo "tveapi soft core unlimited" >> /etc/security/limits.conf || true
echo "root hard core unlimited" >> /etc/security/limits.conf || true
echo "admin hard core unlimited" >> /etc/security/limits.conf || true
echo "tveapi hard core unlimited" >> /etc/security/limits.conf || true
#
echo "root soft nofile 300000" >> /etc/security/limits.conf || true
echo "admin soft nofile 300000" >> /etc/security/limits.conf || true
echo "tveapi soft nofile 300000" >> /etc/security/limits.conf || true
echo "root hard nofile 300000" >> /etc/security/limits.conf || true
echo "admin hard nofile 300000" >> /etc/security/limits.conf || true
echo "tveapi hard nofile 300000" >> /etc/security/limits.conf || true
# /etc/security/limits.conf ignored ?@!, workaround
sed -i 's/#DefaultLimitNOFILE=.*/DefaultLimitNOFILE=300000/g' /etc/systemd/system.conf || true
sed -i 's/#DefaultLimitNOFILE=.*/DefaultLimitNOFILE=300000/g' /etc/systemd/user.conf || true
sed -i 's/#DefaultLimitCORE=.*/DefaultLimitCORE=infinity/g' /etc/systemd/system.conf || true
sed -i 's/#DefaultLimitCORE=.*/DefaultLimitCORE=infinity/g' /etc/systemd/user.conf || true
sed -i 's/#DumpCore=yes/DumpCore=yes/g' /etc/systemd/system.conf || true
#
touch /var/log/sysrepo_api.log || true
chmod 777 /var/log/sysrepo_api.log || true
#
cur_hostname=$(cat /etc/hostname)
SR_STARTUP='/etc/sysrepo/data/ucpe.startup'
TVE_CONFIG='/config/tve_config.json'
#
depmod -a
#
[ ! -f ${config_dir}/.track001 ] && mysql --user=root << _EOF_ && touch ${config_dir}/.track001
FLUSH PRIVILEGES;
ALTER USER 'root'@'localhost' IDENTIFIED BY 'root';
FLUSH PRIVILEGES;

_EOF_

# generate controller default config file
cat >/tmp/controller_defaults.py <<EOL
#!/usr/bin/env python3
import json
import re
import string
import subprocess
import syslog

ucpe_cfg_file = '/config/ucpe_config.json'
controller_cfg_file = '/config/controller_config.json'

cfg_f = open(ucpe_cfg_file, 'r')
cfg = json.load(cfg_f)
cfg_f.close()

cfg['ucpe:config']['dhcpserver:dhcp-server']['enabled'] = False
cfg['ucpe:config']['system:system']['hostname'] = 'controller'
cfg['ucpe:config']['system:system']['homecalling']['enabled'] = False
cfg['ucpe:config']['kubernetes:kubernetes']['cbox']['enabled'] = True
cfg['ucpe:config']['system-resources:system-resources']['huge-page-count'] = 512
for iface in cfg["ucpe:config"]["interfaces:interface"]:
 if iface['name'] == 'data0':
  iface['mode'] = 'virtio'
  continue
 if iface['name'] == 'wifi0':
  iface['management'] = False
  continue
 if 'lte' in iface['name']:
  iface['enable'] = False
  iface['management'] = False
  continue

cfg_f = open(controller_cfg_file, 'w+')
cfg_f.write(json.dumps(cfg, sort_keys=True, indent=4, separators=(',', ':')))
cfg_f.close()
EOL
chmod +x /tmp/controller_defaults.py
/tmp/controller_defaults.py


#
# Splash screen setup
/scripts/splash_setup.py || true
# END Splash screen setup
# Setup default HP size/count
/scripts/host-resources-setup.py || true
# END HP setup
#
[ ! -f ${config_dir}/.track006 ] && \
{
  cp -f ${UCPE_CONFIG} /var/log/installer || true
  tryExecute 'sysrepocfg --edit=${UCPE_CONFIG} --datastore=startup --format=json -m ucpe' \
             exitOnLine ${LINENO};
  # COPY startup TO running
  tryExecute 'sysrepocfg -C startup -m ucpe -v2' \
             exitOnLine ${LINENO};

} && touch ${config_dir}/.track006
#
[ ! -f ${config_dir}/.track007 ] && \
{ tryExecute 'hostname=$(get_json_value //hostname ${UCPE_CONFIG})' \
             exitOnLine ${LINENO};
  tryExecute 'echo -e "$hostname\nnopass" > /opt/vpn-name' \
             exitOnLine ${LINENO};

} && touch ${config_dir}/.track007
#
[ ! -f ${config_dir}/.track008 ] && \
{
  cp -f ${UCPE_CONFIG} /var/log/installer || true
  tryExecute 'sysrepocfg --edit=/config/modules/ietf-keystore.xml -d startup -f xml -m ietf-keystore -v2' \
             exitOnLine ${LINENO};
  # COPY startup TO running
  tryExecute 'sysrepocfg -C startup -m ietf-keystore -v2' \
             exitOnLine ${LINENO};

} && touch ${config_dir}/.track008
#
[ ! -f ${config_dir}/.track009 ] && \
{
  cp -f ${UCPE_CONFIG} /var/log/installer || true
  tryExecute 'sysrepocfg --edit=/config/modules/ietf-netconf-server.xml -d startup -f xml -m ietf-netconf-server -v2' \
             exitOnLine ${LINENO};
  # COPY startup TO running
  tryExecute 'sysrepocfg -C startup -m ietf-netconf-server -v2' \
             exitOnLine ${LINENO};

} && touch ${config_dir}/.track009
#
systemctl daemon-reload
#
# Read what sensors are available and load appropriate kernel modules
# modules=$(sensors-detect --auto | grep -A 30 '#----cut here----' | grep -B 30 '#----cut here----' | grep -v '^#')
# for module in $modules; do
# grep -q -F $module /etc/modules || echo $module >> /etc/modules
# done;

# workaround for failed proc-sys-fs-binfmt_misc.mount after SNMP polling
systemctl disable systemd-binfmt.service proc-sys-fs-binfmt_misc.mount proc-sys-fs-binfmt_misc.automount || true 2>/dev/null
systemctl mask systemd-binfmt.service proc-sys-fs-binfmt_misc.mount proc-sys-fs-binfmt_misc.automount || true 2>/dev/null

#
# END stop firstboot
# !!!!!! mariadb MUST be enable only after firstboot is over
#systemctl enable mariadb
# !!!!!!
mariadb -uroot -proot -e "CREATE DATABASE IF NOT EXISTS wireguard;"
#
echo ${firstboot_config_file}
echo ${config_dir}
rm -f ${firstboot_config_file} ${config_dir}/.track*
rm -f /var/log/firstboot.log
#
/scripts/apply_customization.sh
truncate -s 0 /kubernetes/calico-ready || true

if [ ! -f ${K8S_DISABLED_FILE} ]; then
  systemctl enable nfvtime-kubernetes-init.service > /dev/null || true
fi
#
exit 0
