#!/usr/bin/env python3
import os
import sys

import ha
import set_haproxy
import set_keepalived
import set_kubernetes


def main(_argv):
    log = ha.set_logs('hainit')
    log.debug('start hainit')
    svc_to_be_stopped = ['collectd.service', 'pingManager.service',
                         'containerd.service',
                         'sr-currentconfig.service',
                         'habeat.service', 'kubelet.service',
                         'openvpn@client.service']
    cmd = ['/bin/systemctl', 'stop']
    cmd.extend(svc_to_be_stopped)
    out = ha.exec_get_status(cmd, log)
    if not out:
        log.debug('Fail stopping services, ignore')
    else:
        log.debug('Stopping services OK')

    cmd = ['/bin/systemctl', 'reset-failed']
    cmd.extend(svc_to_be_stopped)
    out = ha.exec_get_status(cmd, log)
    if not out:
        log.debug('Fail reset-failed services, ignore')
    else:
        log.debug('Reset-failed OK')

    # must be before keepalived
    set_kubernetes.config_kubernetes(log)
    set_keepalived.config_keepalived(log)

    out = ha.exec_get_status(['/bin/systemctl', 'enable',
                              'keepalivedstate.service'], log)
    if not out:
        log.error('Fail enable keepalivedstate.service')
    else:
        log.debug('keepalivedstate.service enable OK')

    out = ha.exec_get_status(['/bin/systemctl', 'enable',
                              'keepalived.service'], log)
    if not out:
        log.error('Fail enable keepalived.service')
    else:
        log.debug('keepalived.service enable OK')

    set_haproxy.config_haproxy(log)
    out = ha.exec_get_status(['/bin/systemctl', 'enable',
                              'haproxy.service'], log)
    if not out:
        log.error('Fail enable haproxy.service')
    else:
        log.debug('haproxy.service enable OK')

    out = ha.exec_get_status(['/bin/systemctl', 'disable',
                              'collectd.service'], log)
    if not out:
        log.error('Fail disable collectd.service')
    else:
        log.debug('collectd.service disable OK')

    out = ha.exec_get_status(['/bin/systemctl', 'enable',
                              'nfvtime-kubernetes-init.service'], log)
    if not out:
        log.error('Fail enable nfvtime-kubernetes-init.service')
    else:
        log.debug('nfvtime-kubernetes-init.service enable OK')

    if os.path.exists('/config/_reboot/tve/now'):
        ha.exec_get_status(['/usr/bin/sudo', 'rmdir',
                            '/config/_reboot/tve/now'], log)

    out = ha.exec_get_status(['/usr/bin/sudo', 'mkdir',
                              '/config/_reboot/tve/now'], log)
    if not out:
        log.error('Can not create /config/_reboot/tve/now')
    else:
        log.debug('Created /config/_reboot/tve/now')

    log.debug('hainit done')


if __name__ == '__main__':
    main(sys.argv[1:])
