#!/bin/bash

interface=$1
hostname=`cat /etc/hostname`
LOG () { echo "$1" | systemd-cat -t k8s-ifup-handler -p "${2-info}" ; }

# check if controller mode
if [[ ! -f /config/central-box ]]; then
  LOG "not cbox - exit"
  exit 0
fi

# check if cluster exists
if [[ ! -f /etc/kubernetes/admin.conf ]]; then
  LOG "not cluster - exit"
  exit 0
fi

# check if bind interface
bind_interface=$(sysrepocfg --datastore=running --format=xml --xpath="/ucpe:config/kubernetes:kubernetes/bind-interface" --export | grep bind-interface | cut -d '>' -f 2 | cut -d '<' -f1)
if [[ $interface != "br-"$bind_interface ]]; then
  LOG "not bind interface '$interface' bind-if '$bind_interface'"
fi

if systemctl is-system-running | grep -qE "running|degraded"; then
  LOG "Setting ifup handler for controller interface '$interface'"

  sudo cp /etc/kubernetes/admin.conf /root/.kube/config
  sudo chown $(id -u):$(id -g) /root/.kube/config

  calico_pod=$(sudo kubectl get pods -A -o wide | grep calico-vpp-node | grep $hostname | awk '{ print $2 }')
  sudo kubectl delete pod $calico_pod -n calico-vpp-dataplane >/dev/null 2>&1
  LOG "calico pod '$calico_pod' deleted"

  coredns_pods=$(sudo kubectl get pods -A -o wide | grep coredns | awk '{ print $2 }')
  for i in $coredns_pods; do
    sudo kubectl delete pod $i -n kube-system >/dev/null 2>&1
    LOG "coredns pod '$i' deleted"
  done
fi
exit 0
