#!/bin/bash
# -*- coding: utf-8 -*-
# vim: tabstop=2 expandtab shiftwidth=2 softtabstop=2
# vim: set fileencoding=utf-8
### INIT VARS
source /scripts/basic_vars

### SOURCE SAME BASIC FUNCTIONS NEEDED
source /scripts/basic_functions
exec 1> >(systemd-cat -p info -t `basename $0 .sh`) 2>&1
HOMECALLING_STATUS=0
if [ $# -ne 2 ]; then
  echo "Usage: homecalling-api <path to: current-full-json-config-file> <path to: partial-json-config-file>"
  echo "Example: homecalling-api /config/tve-config.json /tmp/new-data.json"
  exit 1
fi

function compare_config_sections {
  ### CHECK IF WE ARE UP TO DATE
  echo "COMPARING DOWNLOADED AND CURRENT CONFIG"

  ### EVERY MATCH WILL RETURN expr 1 + 3 if empty match and 0 + > 10 if non-empty match
  # matching_scripts_server=$(compare_json_sections //boot/scripts-server $current_config_file $2)
  matching_boot_homecalling=$(compare_json_sections //boot/homecalling $current_config_file $2)
  # matching_boot_image=$(compare_json_sections //boot/image-server $current_config_file $2)
  matching_boot_server=$(compare_json_sections //boot/config-server $current_config_file $2)
  matching_hostname=$(compare_json_sections //config/hostname $current_config_file $2)
  matching_network_interfaces=$(compare_json_sections //config/networking/interfaces $current_config_file $2)
  matching_network_quagga=$(compare_json_sections //config/networking/quagga $current_config_file $2)
  matching_ssh_config=$(compare_json_sections //config/ssh $current_config_file $2)
  matching_ntp_config=$(compare_json_sections //config/ntp $current_config_file $2)
  matching_security_auth_config=$(compare_json_sections //config/security/authentication $current_config_file $2)
  matching_security_whitelist_config=$(compare_json_sections //config/security/cli-whitelist $current_config_file $2)
  matching_syslog_config=$(compare_json_sections //config/syslog/remote-servers $current_config_file $2)
  matching_snmp_server_config=$(compare_json_sections //config/snmp-server $current_config_file $2)
  matching_hardware_type=$(compare_json_sections //hardware/type $current_config_file $2)
  matching_dhcp_relay_config=$(compare_json_sections //config/dhcp-relay $current_config_file $2)
  matching_dhcp_server_config=$(compare_json_sections //config/dhcp-server $current_config_file $2)
  matching_kubernetes_config=$(compare_json_sections //config/kubernetes $current_config_file $2)
  if [ -f /config/act-scr-wiz.json ]; then
    matching_activation_screen_wizard_config=$(compare_json_sections //config/activation-screen-wizard /config/act-scr-wiz.json $2)
  else
    matching_activation_screen_wizard_config=0
  fi
}

function check_sysres_mismatch {
  # match_sysres_lightdpdk=$(compare_json_sections //boot/system-resources/light-dpdk $current_config_file $2)
  match_sysres_dpdkcpu=$(compare_json_sections //boot/system-resources/dpdk-reserved-cpus $current_config_file $2)
  match_sysres_hostcpu=$(compare_json_sections //boot/system-resources/host-reserved-cpus $current_config_file $2)
  match_sysres_hostmem=$(compare_json_sections //boot/system-resources/host-reserved-memory $current_config_file $2)
  match_sysres_hpsize=$(compare_json_sections //boot/system-resources/huge-page-size $current_config_file $2)
  # match_sysres_hpcount=$(compare_json_sections //boot/system-resources/huge-page-count $current_config_file $2)
  match_sysres_oversub=$(compare_json_sections //boot/system-resources/host-oversubscribe-enabled $current_config_file $2)
  is_sysres_hpcount=0
  is_sysres_hpcount=$(cat $2 | grep -c "huge-page-count") #Check is there huge_page parameter in given config
  if [ "$match_sysres_dpdkcpu" -eq 1 ] || [ "$match_sysres_hostmem" -eq 1 ] || [ "$match_sysres_hostcpu" -eq 1 ] ||
     [ "$match_sysres_hpsize" -eq 1 ] || [ "$match_sysres_oversub" -eq 1 ] || [ $is_sysres_hpcount -eq 1 ] ; then
    python3 /scripts/apply_sysrepo_config.py $2 "system-resources"
  fi
}

function check_homecalling_mismatch {
  ####
  # //BOOT/HOMECALLING
  ####
  if [ "$matching_boot_homecalling" -eq 1 ]; then
    echo "//BOOT/HOMECALLING MISMATCH DETECTED"
    apply_boot_homecalling.sh $2
    ### SYNC ON SUCCESS ONLY
    if [ $? -ne 0 ]; then
      HOMECALLING_STATUS=1
    fi
  fi
}

function check_config_server_mismatch {
  ####
  # //BOOT/CONFIG-SERVER
  ####

  if [ "$matching_boot_server" -eq 1 ]; then
    echo "//BOOT/CONFIG-SERVER MISMATCH DETECTED"
    ### SYNC ON SUCCESS ONLY
    if [ $? -eq 0 ]; then
      new_orchestrator=$(get_json_value //boot/config-server/ipv4-address $2)
      python3 /scripts/apply_system_sysrepo.py -c $new_orchestrator
    else
      HOMECALLING_STATUS=1
    fi
  fi
}

function check_ntp_mismatch {
  ####
  # //CONFIG/NTP
  ####
  if [ "$matching_ntp_config" -eq 1 ]; then
    echo "//CONFIG/NTP MISMATCH DETECTED"
    apply_config_ntp.sh $2
    ### SYNC ON SUCCESS ONLY
    if [ $? -ne 0 ]; then
      HOMECALLING_STATUS=1
    fi
  fi
}

function check_security_mismatch {
  ####
  # //CONFIG/SECURITY
  ####
  if [ "$matching_security_auth_config" -eq 1 ]; then
    echo "//CONFIG/SECURITY/AUTHENTICATION MISMATCH DETECTED"
    apply_config_security.sh $2
    ### SYNC ON SUCCESS ONLY
    if [ $? -ne 0 ]; then
      HOMECALLING_STATUS=1
    fi
  fi

  if [ "$matching_security_whitelist_config" -eq 1 ]; then
    echo "//CONFIG/SECURITY/CLI-WHITELIST MISMATCH DETECTED"
    apply_config_security_cli_whitelist.sh $2 $current_config_file
    ### SYNC ON SUCCESS ONLY
    if [ $? -ne 0 ]; then
      HOMECALLING_STATUS=1
    fi
  fi
}

function check_hostname_mismatch {
  ####
  # //CONFIG/HOSTNAME
  ####
  if [ "$matching_hostname" -eq 1 ]; then
    echo "//CONFIG/HOSTNAME MISMATCH DETECTED"
    apply_config_hostname.sh $2
    ### SYNC ON SUCCESS ONLY
    if [ $? -ne 0 ]; then
      HOMECALLING_STATUS=1
    fi
  fi
}

function check_networking_mismatch {
  ####
  # //CONFIG/NETWORKING
  ####

  if [ "$matching_network_interfaces" -eq 1 ] || [ "$matching_network_quagga" -eq 1 ] ; then
    echo "//CONFIG/NETWORKING MISMATCH DETECTED"

    if [ "$matching_network_interfaces" -eq 1 ] ; then
      python3 /scripts/apply_sysrepo_config.py $2 "interfaces"
    fi

    if [ "$matching_network_quagga" -eq 1 ] ; then
      python3 /scripts/apply_sysrepo_config.py $2 "quagga"
    fi

  fi
}

function check_kubernetes_mismatch {
  ####
  # //CONFIG/KUBERNETES
  ####
  if [ "$matching_kubernetes_config" -eq 1 ]; then
    echo "//CONFIG/KUBERNETES MISMATCH DETECTED"
    old_cbox_mode=$(get_json_value //config/kubernetes/cbox/enabled $current_config_file)
    new_cbox_mode=$(get_json_value //config/kubernetes/cbox/enabled $2)

    if [ "$old_cbox_mode" != "$new_cbox_mode" ]; then
      change_cbox_mode_required=1
    else
      change_cbox_mode_required=0
    fi
    apply_config_kubernetes.sh $2
    ret=$?
    ### SYNC ON SUCCESS ONLY
    if [ $ret -eq 0 ] || [ $ret -eq 111 ] ; then
      if [ "$change_cbox_mode_required" -eq 1 ] && [ "$system_status" == "starting" ]; then
        REBOOT_REQUIRED=1
      fi

#      if [ "$SYSREPO_ENABLED" -eq 0 ] ; then
#        sync_json_sections //config/kubernetes $2 $current_config_file
#      fi
    else
      HOMECALLING_STATUS=1
    fi

  fi
}

function check_syslog_mismatch {
  ####
  # //config/syslog/remote-servers
  ####
  if [ "$matching_syslog_config" -eq 1 ]; then
    echo "//CONFIG/SYSLOG MISMATCH DETECTED"
    apply_config_syslog_server.sh $2
    ### SYNC ON SUCCESS ONLY
    if [ $? -ne 0 ]; then
      HOMECALLING_STATUS=1
    fi
  fi
}

function check_snmp_server_mismatch {
  ####
  # //config/snmp-server
  ####
  if [ "$matching_snmp_server_config" -eq 1 ]; then
    echo "//CONFIG/SNMP-SERVER MISMATCH DETECTED"
    apply_config_snmp.sh $2
    ### SYNC ON SUCCESS ONLY
    if [ $? -ne 0 ]; then
      HOMECALLING_STATUS=1
    fi
  fi
}

function check_dhcp_relay_mismatch {
  ####
  # //config/dhcp-relay
  ####
  if [ "$matching_dhcp_relay_config" -eq 1 ]; then
    echo "//CONFIG/DHCP-RELAY MISMATCH DETECTED"
    apply_config_dhcp_relay.sh $2
    ### SYNC ON SUCCESS ONLY
    if [ $? -ne 0 ]; then
      HOMECALLING_STATUS=1
    fi
  fi
}

function check_dhcp_server_mismatch {
  ####
  # //config/dhcp-server
  ####
  if [ "$matching_dhcp_server_config" -eq 1 ]; then
    echo "//CONFIG/DHCP-SERVER MISMATCH DETECTED"
    apply_config_dhcp_server.sh $2
    ### SYNC ON SUCCESS ONLY
    if [ $? -ne 0 ]; then
      HOMECALLING_STATUS=1
    fi
  fi
}

function check_activation_screen_wizard_mismatch {
  ####
  # //config/activation-screen-wizard
  ####
  if [ "$matching_activation_screen_wizard_config" -eq 1 ]; then
    echo "//CONFIG/ACTIVATION-WIZARD-SCREEN MISMATCH DETECTED"
    apply_config_activation_screen_wizard.sh $2
    ### SYNC ON SUCCESS ONLY
    #if [ $? -eq 0 ]; then
    # sync_json_sections //config/activation-screen-wizard $2 $current_config_file
    #else
    # HOMECALLING_STATUS=1
    #fi
  fi
}

function handle_reboots {
  # TVE REBOOT
  ####
  if [ "$REBOOT_REQUIRED" -eq 1 ]; then
    echo "REBOOT IS REQUIRED TO APPLY THE CONFIG CHANGES"
    ### REBOOT THE TVE IF NEEDED
    if [ $reboot_count -le 2 ]; then
      logger -p daemon.warn "Requesting system reload [homecalling API handler]" || true
      $tve_reboot_cmd # REBOOTS HAPPENS HERE!
    else
      echo "REBOOT LOOP DETECTED! ABORTING REBOOTS"
      ### CLEANUP REBOOTS
      echo -n > $reboot_file
      HOMECALLING_STATUS=1
    fi
  else
    echo "HOMECALLING COMPLETE"
  fi
}

function handle_restarts {
  ### REBOOT SVCS HERE!!!, if system is not starting
  FULL_SVC_LIST=$(cat $svc_restart_file | grep SVC_LIST | cut -f2 -d=)
  if [ ! -z "$FULL_SVC_LIST" ] ; then
    if [ ! -f "$lock_file" ]; then
      touch $lock_file
    fi
    svc_restart=""
    for service in ${FULL_SVC_LIST} ; do
      if [[ ! "$svc_restart" == *"$service"* ]] ; then
        svc_restart="$svc_restart $service"
      fi
    done
    sed -i "s/^SVC_LIST=.*$/SVC_LIST=$svc_restart/" $svc_restart_file || true
    if [ "$system_status" != "starting" ] ; then
      svc_restart_iter=$svc_restart
      if [ ! -z "$svc_restart_iter" ]; then
        for service in $svc_restart_iter ; do
          if [ $service != "apache2" ]; then
            echo "restarting $service"
            systemctl restart $service || true
            svc_restart=${svc_restart//"$service "/}
            sed -i "s/^SVC_LIST=.*$/SVC_LIST=$svc_restart/" $svc_restart_file || true
          fi
        done
      fi
    fi
  fi
  #sed -i "s/^SVC_LIST=.*$/SVC_LIST=/" $svc_restart_file
} #handle_restarts

### GO GO GO
if [ "$LOG_IN_FILE" -eq 1 ]; then
  enable_logging /var/log/homecalling-api
fi

check_lock_file
compare_config_sections $current_config_file $2
check_homecalling_mismatch $current_config_file $2
check_config_server_mismatch $current_config_file $2
check_ntp_mismatch $current_config_file $2
check_activation_screen_wizard_mismatch $current_config_file $2
check_security_mismatch $current_config_file $2
check_hostname_mismatch $current_config_file $2
check_networking_mismatch $current_config_file $2
check_syslog_mismatch $current_config_file $2
check_snmp_server_mismatch $current_config_file $2
check_sysres_mismatch $current_config_file $2
check_dhcp_relay_mismatch $current_config_file $2
check_dhcp_server_mismatch $current_config_file $2
handle_reboots
handle_restarts
### CALL POST-HOMECALLING ONLY IF WE ARE NOT UPGRADING OR REBOOTING
if [ "$force_config" -eq 0 ] && [ "$REBOOT_REQUIRED" -eq 0 ]; then
  $call_post_homecalling_cmd
fi

### NOTTIFY SYSTEMD ON HOMECALLING STATUS
exit $HOMECALLING_STATUS
