#!/bin/bash
exec 1> >(systemd-cat -p info -t `basename $0 .sh`) 2>&1
source /scripts/basic_vars
source /scripts/basic_functions
#
function update_ztpstatus() {
  status=$1
  outf="$(mktemp "${TMPDIR:-/tmp}/XXXXXX")" || true
  /usr/bin/jq ".\"ucpe:status\".\"ztp-status\"=\"${status}\"" ${UCPE_STATUS_FILE} > $outf || true
  mv $outf ${UCPE_STATUS_FILE} || true
  chmod 777 ${UCPE_STATUS_FILE} || true
} # function update_ztpstatus
#
function check_skip_homecalling {
  gogo=$(sysrepocfg --datastore=running --format=json --xpath="/ucpe:config/system:system/homecalling/enabled" --export | grep enabled | cut -d':' -f2 | cut -f2 -d' ')
  if [ "${gogo}" == "false" ]; then
    echo "SKIPPING NETWORK UPDATE"
    update_ztpstatus "disabled"
    touch /run/homecalling-succeed || true
    rm -f /run/homecalling-failed || true
    exit 0
  fi
} # function check_skip_homecalling

function set_homecalling_retry {
  #if we ever succedded, no retry
  if [ ! -f "/run/homecalling-succeed" ]; then
    #if first retry, start with 30s
    if [ ! -f "/run/homecalling-failed" ]; then
      echo 30 > /run/homecalling-failed  || true
    else
      #30sec-60sec-120sec-240sec-240sec....-240sec
      value=`cat /run/homecalling-failed`
      case "$value" in
        30) newval="60"
            ;;
        60) newval="120"
            ;;
        120) newval="240"
             ;;
        *)  newval="240"
            ;;
      esac
      echo $newval > /run/homecalling-failed  || true
    fi
  fi

} # function set_homecalling_retry

#
function homecalling_download_config {
  echo "DOWNLOAD CONFIG FROM CURRENT BOOT-CONFIG SERVER"
  apply_boot_config_server.sh ${current_config_file}
  result=$?
  if [ "${result}" -eq 0 ]; then
    echo "CONFIG DOWNLOADED SUCCESSFULLY"
    touch /run/homecalling-succeed || true
    touch /run/manager-contacted || true
    rm -f /run/homecalling-failed || true
  elif [ "${result}" -eq 1 ]; then
    echo "CONFIG DOWNLOAD FAILED!"
    set_homecalling_retry
    exit 1
  elif [ "${result}" -eq 2 ]; then
    echo "CONFIG DOWNLOAD FAILED!"
    set_homecalling_retry
    exit 0
  fi
} # function homecalling_download_config
#
function check_new_orchestrator {
  matching_boot_config=$(compare_json_sections //json/tve/ucpe:config/system:system/config-server $current_config_file $working_dir/$new_config_file)
  if [ "$(sysrepocfg --datastore=running --format=json --xpath=\"/ucpe:config/system:system/config-server\" --export | grep -v config | grep server | cut -d':' -f2 | cut -f2 -d' ')" == "$(get_json_value //json/tve/ucpe:config/system:system/config-server $working_dir/$new_config_file | grep server | cut -d'"' -f4)" ]; then
    echo "//BOOT/CONFIG-SERVER MISMATCH DETECTED. RE-TRIGGERING NEW BOOT"
    matching_network_config=$(compare_json_sections //config/networking $current_config_file $working_dir/$new_config_file)
    if [ $matching_network_config -ne 0 ]; then
      apply_config_networking.sh $working_dir/$new_config_file $current_config_file
      for i in `ls /etc/network/interfaces.d/ifcfg-*`; do ifup $(basename $i); done
    fi

    apply_boot_config_server.sh ${working_dir}/${new_config_file}
    if [ $? -ne 0 ]; then
      echo "CONFIG DOWNLOAD FAILED!"
      if [ -f "/run/post_hm_lock_file" ]; then
        /bin/rm -rf /run/post_hm_lock_file
      fi
      exit 1
    else
      echo "CONFIG DOWNLOADED SUCCESSFULLY"
    fi
  fi
} # function check_new_orchestrator
#
function sync_hw_type {
  if [ $HOMECALLING_STATUS -ne 1 ]; then
    hwtype=$(get_json_value //hwtype ${working_dir}/tve_config_manager.json)
    [ -n "${hwtype}" ] && echo ${hwtype} > /config/.hwtype
  fi
} # function sync_hw_type
#
function handle_reboots {
  if [ "$REBOOT_REQUIRED" -eq 1 ]; then
    echo "REBOOT IS REQUIRED TO APPLY THE CONFIG CHANGES"
    if [ ${reboot_count} -le 2 ]; then
      echo "1" >> $reboot_file

      logger -p daemon.warn "Requesting system reload [homecalling handler]" || true
      $tve_reboot_cmd # REBOOTS HAPPENS HERE!
    else
      echo "REBOOT LOOP DETECTED! ABORTING REBOOTS"
      ### CLEANUP REBOOTS
      echo -n > $reboot_file
      HOMECALLING_STATUS=1
      ${downloading_config_stop}
    fi
  else
    echo "HOMECALLING COMPLETE"
  fi

} # function handle_reboots
#
function netconf_config {
  if [ "nanopir2s" = "$(cat /config/.hwtype)" -o "nanopir4s" = "$(cat /config/.hwtype)" ]; then
    return
  fi
  cat >> /tmp/fix_import_netconf.py << EOF
#!/usr/bin/env python3
import json

cfg_file = open('/tmp/config_from_manager.json', 'r')
config = json.loads(cfg_file.read())
cfg_file.close()
if 'ietf-netconf-server:netconf-server' not in config:
    exit(0)
data = {'ietf-netconf-server:netconf-server':{}}
data['ietf-netconf-server:netconf-server']=config.pop('ietf-netconf-server:netconf-server')

config=json.dumps(config,indent=4,sort_keys=True)
cfg_file = open('/tmp/config_from_manager.json', 'w+')
cfg_file.write(config)
cfg_file.close()

if 'listen' in data['ietf-netconf-server:netconf-server'] and \
   'endpoint' in data['ietf-netconf-server:netconf-server']['listen']:
    for i in data['ietf-netconf-server:netconf-server']['listen']['endpoint']:
        if i['ssh']:
            i['ssh'].update({'ssh-server-parameters':{'server-identity':{'host-key':[{'name':'default-key','public-key':{'keystore-reference':'genkey'}}]},'client-authentication':{'supported-authentication-methods':{'publickey':[None],'passsword':[None]}}}})

data=json.dumps(data,indent=4,sort_keys=True)
cfg_file = open('/tmp/netconf.json', 'w+')
cfg_file.write(data)
cfg_file.close()
EOF
  chmod +x /tmp/fix_import_netconf.py
  /tmp/fix_import_netconf.py
  sed -i 's/\: null/\: \[null\]/g' /tmp/netconf.json
  rm -rf /tmp/fix_import_netconf.py
} # function netconf_config
#
function main_loop {
  work_file=${working_dir}/config_from_manager.json
  if [ -f /tmp/in_tve_config.json ]; then
    /usr/bin/jq .json.tve /tmp/in_tve_config.json > /config/last_received_from_manager.json
    VERS=`/usr/bin/jq .\"protocol-version\" /tmp/in_tve_config.json | tr -d '"'`  || true
    /scripts/save_nulls_from_json.sh
    /usr/bin/jq .json.tve /tmp/in_tve_config.json | /usr/bin/jq "del(.\"ucpe:status\")" | /usr/bin/jq 'walk( if type == "array" then map(select(. != null)) elif type == "object" then with_entries( select( .value != null and ( .value != "" or .key == "auth-password" or .key == "notify-view" or .key == "priv-password" or .key == "remote-engine-id" or .key == "write-view" or .key == "ipv4-address" or .key == "read-view" or .key == "apn" or .key == "dns" or .key == "gateway" or .key == "trace-host") and .value != [] and .value != {} ) ) else . end )' > ${work_file}
    sync_hw_type
    hwtype=$(get_json_value //hwtype /tmp/in_tve_config.json)
    cp ${work_file} ${working_dir}/config_from_manager2.json
    cat >> /tmp/fix_import_config.py << EOF
#!/usr/bin/env python3
import json

cfg_file = open('/config/ucpe_config.json', 'r')
config = json.loads(cfg_file.read())
cfg_file.close()
data=config['ucpe:config']

cfg_file2 = open('/tmp/config_from_manager2.json', 'r')
config2 = json.loads(cfg_file2.read())
cfg_file2.close()
if not 'target-version' in config2['ucpe:config']['upgrade:upgrade']: config2['ucpe:config']['upgrade:upgrade']['target-version']= ''
config2=json.dumps(config2,indent=4,sort_keys=True)
cfg_file2 = open('/tmp/config_from_manager.json', 'w+')
cfg_file2.write(config2)
cfg_file2.close()
EOF
    chmod +x /tmp/fix_import_config.py
    /tmp/fix_import_config.py
    rm -rf /tmp/fix_import_config.py /tmp/config_from_manager2.json

    netconf_config
    if [ "nanopir2s" = "$(cat /config/.hwtype)" -o "nanopir4s" = "$(cat /config/.hwtype)" ]; then
      work_file_nanopir2s=${working_dir}/config_from_manager_nanopir2s.json
      rm -f ${work_file_nanopir2s}
      cp -f ${work_file} ${work_file_nanopir2s}
      /usr/bin/jq "del(.\"ietf-netconf-server:netconf-server\",.\"ucpe:config\".\"syslog:syslog\",.\"ucpe:config\".\"dhcprelay:dhcp-relay\",.\"ucpe:config\".\"snmp-server:snmp-server\",.\"ucpe:config\".\"system-resources:system-resources\")" ${work_file_nanopir2s} > ${work_file}
      rm -f ${work_file_nanopir2s}
    fi

    #work_file2=${working_dir}/config_from_manager2.json
    #cp -f ${work_file} ${work_file2}
    #/usr/bin/jq "del(.\"ucpe:config\".\"system-resources:system-resources\".\"huge-page-size\")" ${work_file2} > ${work_file}
    #rm -f ${work_file2}

    theoutput=$(sysrepocfg --datastore=running --format=json --import=${work_file} -m ucpe 2>&1)
    echo $theoutput
    if [[ "$theoutput" == *"The new configuration was successfully applied"* ]]; then
        echo -n "No error." > ${ztp_last_known_err}
    else
        echo $theoutput > ${ztp_last_known_err}
    fi

    if [ -f /tmp/netconf.json ]; then
      sysrepocfg --datastore=running --format=xml --import=/config/modules/ietf-keystore.xml -m ietf-keystore
      sysrepocfg --datastore=running --format=json --import=/tmp/netconf.json -m ietf-netconf-server
    fi
    /scripts/homecalling_apply.sh ${work_file}
    rm -rf /config/_sendconfig/config/*
    mkdir -p /config/_sendconfig/config/config
    # Disable splash screen
    #if [ -f /etc/apache2/conf-enabled/splash.conf ]; then
    #  a2disconf splash
    #  systemctl reload apache2 --no-block
    #fi
  fi
} # function main_loop

### GO GO GO
### CREATE SVC RESTART FILE
#we should not clear the svc_restart_file due to working homecalling not finished. NFV-12394
#echo "SVC_LIST=" > ${svc_restart_file}

### CLEANUP POSSIBLE STALE ENTRY
/bin/rm -rf /config/_reboot/network/now

echo 0 > ${force_config_file}

check_skip_homecalling
check_lock_file

if [ "$system_status" != "starting" ]; then
  homecalling_download_config
  ${downloading_config_start}
  echo "Downloading_config_start "
else
  echo "System is starting...Exit!"
  exit 0
fi

# Report 'ZTP in Progress' to manager
if [ ! -f "/run/post_hm_lock_file" ]; then
  touch /run/post_hm_lock_file
fi
check_new_orchestrator
main_loop
handle_reboots
/bin/rm -f ${working_dir}/ucpe_config.json
${downloading_config_stop}
echo "Downloading_config_stop "
### CALL POST-HOMECALLING ONLY IF WE ARE NOT UPGRADING OR REBOOTING
echo "force_config is ${force_config} reboot_required is ${REBOOT_REQUIRED}"
if [ "${force_config}" -eq 0 ] && [ "${REBOOT_REQUIRED}" -eq 0 ]; then
  ### LET SUBSCRIBERS CALL POST-HOMECALLING
  #$call_post_homecalling_cmd
  :
fi

if [ -f "/run/post_hm_lock_file" ]; then
  /bin/rm -rf /run/post_hm_lock_file
fi
echo "Exit status ${HOMECALLING_STATUS}"
exit ${HOMECALLING_STATUS}
