#!/usr/bin/env python3
import sys, json, shutil

if len(sys.argv) < 2:
    raise Exception("Failed to apply config! Missing config file!")

f = open(sys.argv[1], 'r')
config=f.read()
f.close()
if not config:
    shutil.copyfile(sys.argv[1], "/var/tmp/failed_to_apply_from_manager.json")
    raise Exception("Failed to apply config file {}. Invalid config!".format(sys.argv[1]))

module_name = sys.argv[2]

try:
    config = json.loads(config)
    if "interfaces:interface" in module_name:
        from sysrepo_interfaces import sysrepo_interfaces3 as api
        client = api.SysrepoInterfaces(app_name="homecalling apply {} config".format(module_name))
    elif "ntp:ntp" in module_name:
        from sysrepo_ntp import sysrepo_ntp as api
        client = api.SysrepoNTP(app_name="homecalling apply {} config".format(module_name))
    elif "kubernetes:kubernetes" in module_name:
        from sysrepo_kubernetes import sysrepo_kubernetes as api
        client = api.SysrepoKubernetes(app_name="homecalling apply {} config".format(module_name))        
    elif "quagga:quagga" in module_name:
        from sysrepo_quagga import sysrepo_quagga as api
        client = api.SysrepoQuagga(app_name="homecalling apply {} config".format(module_name))
    elif "security:security" in module_name:
        from sysrepo_security import sysrepo_security as api
        client = api.SysrepoSecurity(app_name="homecalling apply {} config".format(module_name))
    elif "snmp-server:snmp-server" in module_name:
        from sysrepo_snmp import sysrepo_snmp as api
        client = api.SysrepoSNMP(app_name="homecalling apply {} config".format(module_name))
    elif "syslog:syslog" in module_name:
        from sysrepo_syslog import sysrepo_syslog as api
        client = api.SysrepoSyslog(app_name="homecalling apply {} config".format(module_name))
    elif "system:system" in module_name:
        from sysrepo_system import sysrepo_system as api
        client = api.SysrepoSystem(app_name="homecalling apply {} config".format(module_name))
    elif "system-resources:system-resources" in module_name:
        from sysrepo_systemresources import sysrepo_systemresources as api
        client = api.SysrepoSystemResources(app_name="homecalling apply {} config".format(module_name))
    elif "dhcpserver:dhcp-server" in module_name:
        from sysrepo_dhcpserver import sysrepo_dhcpserver as api
        client = api.SysrepoDhcpServer(app_name="homecalling apply {} config".format(module_name))
    elif "dhcprelay:dhcp-relay" in module_name:
        from sysrepo_dhcprelay import sysrepo_dhcprelay as api
        client = api.SysrepoDhcpRelay(app_name="homecalling apply {} config".format(module_name))
    else:
        raise Exception("Unknown module name '{}'".format(module_name))

    intermediate = {"ucpe:config": {}}
    intermediate["ucpe:config"][module_name] = config["ucpe:config"].pop(module_name)
    client.apply_json_config(intermediate)
    client.commit()
    client.logger.info("Configuration applied successfully in sysrepo.")
    client.session_stop()


except Exception as ex:
    client.logger.info("Error applying config in sysrepo!")
    client.logger.error(ex)
    client.discard_changes()
    client.session_stop()
    sys.exit(1)

sys.exit(0)
