#!/usr/bin/env python3
import sys, re, json
from systemd import journal

CFG_FILE = "/config/ucpe_config.json"
host_resources_setup_stream=journal.stream(identifier='host-resources-setup')

if __name__ == "__main__":
    meminfo = None
    with open('/proc/cmdline') as f:
        meminfo = f.read()

    with open(CFG_FILE, "r") as cfg_f:
        cfg = json.load(cfg_f)

    try:
        hpsize_matched = re.search(r'default_hugepagesz=(\d+)', meminfo, re.MULTILINE)
        if hpsize_matched:
            hp_size = int(hpsize_matched.groups()[0])
            cfg['ucpe:config']['system-resources:system-resources']['huge-page-size'] = int(hp_size)

        hpcount_matched = re.search(r'hugepages=(\d+)', meminfo, re.MULTILINE)
        if hpcount_matched:
            hp_count = int(hpcount_matched.groups()[0])
            cfg['ucpe:config']['system-resources:system-resources']['huge-page-count'] = int(hp_count)

        print("Detected %d HugePages, %d MBytes each" % (int(hp_count), int(hp_size)),file=host_resources_setup_stream)

    except Exception as e:
        print("Failed to apply system resources on startup.{}".format(e),file=host_resources_setup_stream)

    with open(CFG_FILE, 'w') as cfg_f:
        new_cfg = json.dumps( cfg, indent=4, sort_keys=True, separators=(',', ': ') )
        cfg_f.write(new_cfg)
        cfg_f.close()

    sys.exit( 0 )
