#! /usr/bin/env python3
import os
from kubernetes import client, config
from kubernetes.client.models.v1_container_image import V1ContainerImage

log_unready_pods = ""
system_pods = ['kube-system', 'calico-vpp-dataplane', 'cdi', 'kubevirt', 'local-registry']

def names(self, names):
    self._names = names


def get_is_node_ready(hostname, detailed = False):
    global log_unready_pods
    # Configs can be set in Configuration class directly or using helper utility
    config.load_kube_config()
    v1 = client.CoreV1Api()

    V1ContainerImage.names = V1ContainerImage.names.setter(names)
    ret = v1.list_node(watch=False)
    result = False
    for i in ret.items:
        if (
            len(i.metadata.name) == len(hostname)
            and i.metadata.name in hostname
            and str(i.status.conditions[-1].status) in "True"
            and not detailed
        ):
            result = True
        elif (
            len(i.metadata.name) == len(hostname)
            and i.metadata.name in hostname
            and detailed
        ):
            # False if the node is not healthy and is not accepting pods, 
            # and Unknown if the node controller has not heard from the node 
            # in the last node-monitor-grace-period (default is 40 seconds)
            if str(i.status.conditions[-1].status) in ["False", "Unknown"]:
                return "NotReady"
            for condition in i.status.conditions:
                if eval(condition.status):
                    return condition.type


    if not result:
        log_unready_pods += "There is no node with such name in the cluster.\n"

    return result


def get_are_pods_ready(hostname):
    global log_unready_pods
    count_unready_pods_ctrl = 0
    count_unready_pods_node = 0
    import socket
    controller_hostname = socket.gethostname()

    # Configs can be set in Configuration class directly or using helper utility
    config.load_kube_config()
    v1 = client.CoreV1Api()
    ret = v1.list_pod_for_all_namespaces(watch=False)

    for i in ret.items:
        if i.spec.node_name is not None and i.spec.node_name in controller_hostname:
            # https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1ContainerStatus.md
            if i.status.phase not in "Running" or not i.status.container_statuses[0].ready:
                count_unready_pods_ctrl += 1
                log_unready_pods += to_string_pod(i)

        if i.spec.node_name is not None and i.spec.node_name in hostname:
            if i.status.phase not in "Running" or not i.status.container_statuses[0].ready:
                count_unready_pods_node += 1
                log_unready_pods += to_string_pod(i)

    if count_unready_pods_ctrl == 0 and count_unready_pods_node == 0:
        return True
    else:
        log_unready_pods = "There are {} unready pods from the controller \nand {} unready pods from the worker node:\n{}".format(
            count_unready_pods_ctrl, count_unready_pods_node, log_unready_pods
        ).strip()
        return False
    

def get_system_pods_ready():
    count_unready_system_pods = 0

    # Configs can be set in Configuration class directly or using helper utility
    config.load_kube_config()
    v1 = client.CoreV1Api()
    ret = v1.list_pod_for_all_namespaces(watch=False)

    for i in ret.items:
        if i.spec.node_name is not None and i.metadata.namespace in system_pods:
            # https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1ContainerStatus.md
            if i.status.phase not in "Running" or not i.status.container_statuses[0].ready:
                count_unready_system_pods += 1

    if count_unready_system_pods == 0:
        return True
    else:
        return False


def to_string_pod(pod):
    return "Pod {}, with ip {}, hosted on {}, is in status {} and is not running properly\n".format(
        pod.metadata.name, pod.status.pod_ip, pod.spec.node_name, pod.status.phase
    )


if __name__ == "__main__":
    global log_unready_pod

    try:
        import sys

        hostname = sys.argv[1]
    except:
        print("Error, you didn't pass in the node name")
        exit()

    detailed = ""
    try:
        import sys
        detailed = sys.argv[2]
    except:
        pass

    old = os.environ.get('HOME', 'Not Set')
    os.environ["HOME"] = "/root"
    resultPods = False
    if detailed == "all-pods":
        resultPods = get_are_pods_ready(hostname=hostname)
        if resultPods and get_is_node_ready(hostname=hostname):
            print("All pods are running properly.")
        else:
            print(log_unready_pods)
    elif detailed == "system-pods":
        print(get_system_pods_ready(), end="")
    elif detailed == "node-condition":
        print(get_is_node_ready(hostname=hostname, detailed=True), end="")
    else:
        print(get_is_node_ready(hostname=hostname))
    os.environ["HOME"] = old
