#!/bin/bash
# -*- coding: utf-8 -*-
# coding: utf-8
# vim: tabstop=2 expandtab shiftwidth=2 softtabstop=2
# vim: set fileencoding=utf-8
exec 1> >(systemd-cat -p info -t "keepalived_azure") 2>&1

function store_floating_ip {
    cat << EOF > /tmp/pyscript.py
#!/usr/bin/env python3
import json
import sys
import os

if os.path.exists('/config/azure.json'):
    meme = '/config/azure.json'
    try:
        with open(meme, 'r') as f:
            data = f.read()
    except FileNotFoundError:
        print('FileNotFound {}'.format(meme))
        sys.exit(2)

    try:
        cfg = json.loads(data)
    except json.JSONDecodeError:
        print('Corrupted JSON file')
        sys.exit(2)

    cfg["azure"]["private_address"] = "{}".format('${prvtipaddr}')
    cfg["azure"]["private_address_version"] = "{}".format('${prvtipver}')
    cfg["azure"]["public_ip"] = "{}".format('${pubip}')
    cfg["azure"]["subnet"] = "{}".format('${subnet}')
    cfg["azure"]["app_security_grps"] = "{}".format('${app_sec_grp}')
    with open(meme, 'w') as f:
        print(json.dumps(cfg, indent=4, sort_keys=False), file=f)

EOF
    chmod 755 /tmp/pyscript.py
    /tmp/pyscript.py
    rm /tmp/pyscript.py
} # function store_floating_ip

function get_ip_config_tool {
    cat << EOF > /tmp/pyscript.py
#!/usr/bin/env python3
import json
import sys

a=json.load(sys.stdin)
for x in a:
    if x["name"] == "${floatip}":
        if "applicationSecurityGroups" in x and x["applicationSecurityGroups"] is None:
            print("{}\n{}\n{}\n{}\n{}".format(x["privateIPAddress"],x["privateIPAddressVersion"],x["publicIPAddress"]["id"],x["subnet"]["id"],""))
        elif "applicationSecurityGroups" not in x:
            print("{}\n{}\n{}\n{}\n{}".format(x["privateIPAddress"],x["privateIPAddressVersion"],x["publicIPAddress"]["id"],x["subnet"]["id"],""))
        else:
            print("{}\n{}\n{}\n{}\n{}".format(x["privateIPAddress"],x["privateIPAddressVersion"],x["publicIPAddress"]["id"],x["subnet"]["id"]," ".join(y["id"] for y in x["applicationSecurityGroups"])))

        break
    else:
        continue

EOF
    chmod 755 /tmp/pyscript.py
} # function get_ip_config_tool

function azure_set_floating_ip {
    if [ -f "/config/azure.json" ]; then
        # This will return a lot of info in JSON format about the local VM instance
        arr1=( $(curl -H Metadata:true --noproxy "*" http://169.254.169.254/metadata/instance/compute?api-version=2020-09-01 | python3 -c 'import json,sys;x=json.load(sys.stdin);print("{}\n{}\n{}\n{}".format(x["name"],x["location"],x["resourceGroupName"],x["subscriptionId"]))' 2>/dev/null) )
        vm=`echo ${arr1[0]}` 
        location=`echo ${arr1[1]}` 
        resource=`echo ${arr1[2]}` 
        subscription=`echo ${arr1[3]}`
        floatip=`cat /config/azure.json | python3 -c 'import json,sys;print( json.load(sys.stdin)["azure"]["floating_ip"] )' 2>/dev/null`

        # login
        n=0
        until [ "$n" -ge 300 ]
        do
            az login --identity
            if [ $? != 0 ]; then
                echo "Unsuccessful authentication atempt to Azure AD!"
                echo "Verify the Managed Identity, connection and DNS configuration!"
                sleep 10
                n=$((n+10))
                if [ "$n" -ge 300 ]; then
                    touch /tmp/azure_timeout
                fi
            else
                # local nic
                nic=`az vm nic list -g $resource --vm-name $vm --subscription "$subscription" | python3 -c 'import json,sys;print(json.load(sys.stdin)[0]["id"])' 2>/dev/null | xargs basename 2>/dev/null`
                cluster=( $(cat /config/azure.json | python3 -c 'import json,sys;[print("{}".format(x["vm_name"])) for x in json.load(sys.stdin)["azure"]["ha_cluster"] ]' 2>/dev/null) )
                # Move the floating ip
                for i in "${cluster[@]}"; do
                    # get NIC name for VM $i
                    nici=`az vm nic list -g $resource --vm-name $i --subscription "$subscription" | python3 -c 'import json,sys;print(json.load(sys.stdin)[0]["id"])' 2>/dev/null | xargs basename 2>/dev/null`
                    get_ip_config_tool
                    arr2=( $(az network nic ip-config list -g $resource --nic-name $nici --subscription "$subscription" | /tmp/pyscript.py 2>/dev/null) )
                    rm /tmp/pyscript.py
                    prvtipaddr=`echo ${arr2[0]}`
                    prvtipver=`echo ${arr2[1]}`
                    pubip=`echo ${arr2[2]}`
                    subnet=`echo ${arr2[3]}`
                    app_sec_grp=`echo ${arr2[4]}`
                    if [ -n "${prvtipaddr}" ] && [ -n "${prvtipver}" ] && [ -n "${pubip}" ]; then
                        #save
                        store_floating_ip
                    else
                        #read
                        prvtipaddr=( $(cat /config/azure.json | python3 -c 'import json,sys;print( json.load(sys.stdin)["azure"]["private_address"] )' 2>/dev/null) )
                        prvtipver=( $(cat /config/azure.json | python3 -c 'import json,sys;print( json.load(sys.stdin)["azure"]["private_address_version"] )' 2>/dev/null) )
                        pubip=( $(cat /config/azure.json | python3 -c 'import json,sys;print( json.load(sys.stdin)["azure"]["public_ip"] )' 2>/dev/null) )
                        subnet=( $(cat /config/azure.json | python3 -c 'import json,sys;print( json.load(sys.stdin)["azure"]["subnet"] )' 2>/dev/null) )
                        app_sec_grp=( $(cat /config/azure.json | python3 -c 'import json,sys;print( json.load(sys.stdin)["azure"]["app_security_grps"] )' 2>/dev/null) )
                    fi
                    if [ -z "${prvtipaddr}" ] && [ -z "${prvtipver}" ] && [ -z "${pubip}" ]; then
                        # check for the floating address is on local nic
                        get_ip_config_tool
                        arr3=( $(az network nic ip-config list -g $resource --nic-name $nic --subscription "$subscription" | /tmp/pyscript.py 2>/dev/null) )
                        rm /tmp/pyscript.py
                        prvtipaddr=`echo ${arr3[0]}`
                        prvtipver=`echo ${arr3[1]}`
                        pubip=`echo ${arr3[2]}`
                        subnet=`echo ${arr3[3]}`
                        app_sec_grp=`echo ${arr3[4]}`
                        if [ -n "${prvtipaddr}" ] && [ -n "${prvtipver}" ] && [ -n "${pubip}" ]; then
                            #save
                            store_floating_ip
                            prvtipaddr=
                            prvtipver=
                            pubip=
                            subnet=
                            app_sec_grp=
                        fi
                    fi
                    if [ -n "${prvtipaddr}" ] && [ -n "${prvtipver}" ] && [ -n "${pubip}" ]; then
                        # Delete the floating IP from the former master
                        az network nic ip-config delete -n $floatip --nic-name $nici -g $resource --subscription "$subscription"
                        # Create the floating IP on the local NIC
                        if [ -n "${subnet}" ]; then
                            add_subnet="--subnet $subnet"
                        fi
                        if [ -n "${app_sec_grp// }" ]; then
                            add_app_sec_grp="--application-security-groups $app_sec_grp"
                        fi
                        # Retry up to 4 times in 15 seconds intervals
                        retry=0
                        while true; do
                            az network nic ip-config create -n $floatip --nic-name $nic -g $resource --subscription "$subscription" \
                                --private-ip-address $prvtipaddr --private-ip-address-version $prvtipver \
                                --public-ip-address $pubip $add_subnet $add_app_sec_grp
                            if [ $? != 0 ]; then
                                retry=$((retry+1))
                                if [ "$retry" -gt 12 ]; then
                                    touch /tmp/azure_timeout
                                    break
                                fi
                                sleep 15
                            else
                                break
                            fi
                        done
                        break
                    fi
                done

                # logout
                az logout
                break
            fi
        done
    fi
} # function azure_set_floating_ip

rm -rf /config/_azure/*
azure_set_floating_ip

exit 0
