#!/bin/bash

source /scripts/basic_vars
source /root/admin-openrc.sh
source /scripts/basic_functions

if [[ $# -eq 0 ]]; then
  echo " Missing arguments for port_update.sh"
  exit 1
fi

operation=$1
mac_arg=$2
ip_arg=$3
masked=''
declare -a ip_addr

function update_port_addr() {

    arr=("$1")

    for i in "${arr[@]}"; do
        if [[ "$i" == "${3}" ]]; then
            return
        fi
    done
    n=0
    until [ $n -ge 5 ]
    do
        neutron --insecure --http-timeout 60 port-update --fixed-ip subnet_id=$2,ip_address=$3 $4 && break
        n=$[$n+1]
        [ $n -eq 5 ] && exit 1
        sleep 30
    done
}

bind_interface=$(sysrepocfg --datastore=running --format=xml --xpath="/ucpe:config/kubernetes:kubernetes/bind-interface" --export | grep bind-interface | cut -d '>' -f 2 | cut -d '<' -f1)
FILES="/run/*.vnic"
for f in $FILES; do
    details=$(cat $f | grep "port_details=")
    if [[ $details == *"mgmt0"* || $details == *"tap300"* ]]; then
        mac=$(cat $f | grep "mac=" | cut -d \" -f2 | tr -d '[:space:]')
        vm_name=$(cat $f | grep vmname | cut -d'"' -f2)        
        port_id=${f%.*}
        port_id=${port_id##*/}
        if [[ -z "$port_id" || -z "$mac" ]]; then
            continue
        fi
        if [[ "$mac" != "$mac_arg" ]]; then
            continue
        fi
        port_info=$(openstack port show --insecure $port_id | awk 'NR>3')
        if [[ -z "$port_info"  ]]; then
            exit 1
        fi
        #ip_addr=$(echo ${port_info##*"ip_address="} | cut -d',' -f1 | tr -d \' | tr -d '[:space:]')
        ip_addr=($(echo ${port_info} | grep -Po "ip_address='\K.*?(?=')"))
        sub_id=$(echo ${port_info##*"subnet_id="} | cut -d'|' -f1 | tr -d \' | tr -d '[:space:]')
        #cur_status=$(echo ${port_info##*"status"} | cut -d'|' -f2 | tr -d '[:space:]')
        if [[ -z "$sub_id" ]]; then
            exit 1
        fi

        if [[ $operation == "remove" ]]; then
            while [ -n "${ip_addr[${i}]}" ]; do
                    if [[ "${ip_addr[$i]}" == "$ip_arg" ]]; then
                        openstack port set --insecure $port_id --no-fixed-ip
                        openstack port set --insecure $port_id --fixed-ip subnet=$sub_id
                    fi
                    i=$((i+1))
            done
            exit 0
        fi

        # in case of vpp and ipsec, send update for VM management interface        
        masked=$(systemctl is-enabled vpp-config.service | grep masked) || true 
        if [[ "$masked" == "" && $details == *"tap300"* && "$bind_interface" == "vti0" ]]; then
            if [[ ! -z "$ip_addr" ]]; then
                update_port_addr  ${ip_addr[@]} $sub_id $ip_arg $port_id
            fi
            /usr/bin/dbus-send --system --print-reply --dest=net.telco.NetIfMonitor /net/telco/NetIfMonitor net.telco.NetIfMonitor.ipsec_handle string:"add" string:"${vm_name#*:}" string:"${ip_arg}" 2>1 >/dev/null || true
            exit 0
        fi

        if [[ ! -z "$ip_addr" ]]; then
            update_port_addr ${ip_addr[@]} $sub_id $ip_arg $port_id
        fi
        exit 0
    fi
done

exit 0
