#!/bin/bash
exec 1> >(systemd-cat -p info -t `basename $0 .sh`) 2>&1
set -e # EXIT IMMEDIATELY UPON ERROR!

### SOURCE SAME BASIC VARS NEEDED
source /scripts/basic_vars

### SOURCE SAME BASIC FUNCTIONS NEEDED
source /scripts/basic_functions

# do not execute post_homecalling.sh, if homecalling.sh is running
# it will be executed after homecalling.sh
homecalling_sh_running=$(ps -C homecalling.sh --no-headers | wc -l)
if [ "$homecalling_sh_running" -eq 1 ]; then
  echo "homecalling.sh is running"
  exit 0
else
  echo "homecalling_sh_running ${homecalling_sh_running}"
fi
# do not execute post_homecalling.sh, if it is alredy running
self_is_running=$(ps -C post_homecalling.sh --no-headers | wc -l)
#when only 1 instance is running, self_is_running is 3
if [ "$self_is_running" -gt 3 ]; then
  echo "another post_homecalling.sh is running"
  if [ -d /config/_reboot/network/now ]; then
    /bin/rm -rf /config/_reboot/network/now
  fi
  exit 0
else
  while true; do
    last_modification_of_file_in_seconds=$(date +%s -r $svc_restart_file)
    current_time_in_seconds=$(date +%s)
    seconds_since_last_modification=$((current_time_in_seconds-last_modification_of_file_in_seconds))
    #echo "Last write to $svc_restart_file was before $seconds_since_last_modification seconds"
    if [[ "$seconds_since_last_modification" -gt 30 ]]; then
      echo "self_is_running ${self_is_running}"
      break
    fi
    touch /config/_hb/.hb_send
    sleep 1
  done
fi

echo "Start!"

if [ "$LOG_IN_FILE" -eq 1 ]; then
  enable_logging /var/log/homecalling
fi

if [ -d /config/_reboot/network/now ]; then
  /bin/rm -rf /config/_reboot/network/now
fi

echo 0 > $force_config_file

### MAIN
if [ -n "$1" ]; then
  if [ ! -f "/run/post_hm_lock_file" ]; then
    touch /run/post_hm_lock_file
  fi
  sleep $1
  downloading_config_seconds_counter=1
  downloading_config_seconds_max=300
  while [ -f ${downloading_config} ]; do
    echo "Downloading config sleep ... $downloading_config_seconds_counter"
    sleep 2
    downloading_config_seconds_counter=$(expr ${downloading_config_seconds_counter} + 1)
    if [ ${downloading_config_seconds_counter} -gt ${downloading_config_seconds_max} ]; then
      # failsafe
      ${downloading_config_stop}
    fi
  done

  NEW_OPENSTACK_BIND_IF=$(cat $svc_restart_file | grep NEW_OPENSTACK_BIND_IF) || true
  if [ -n "$NEW_OPENSTACK_BIND_IF" ]; then
    sed -i "/NEW_OPENSTACK_BIND_IF/d" $svc_restart_file
  fi

  if [ -f $interface_up_file ]; then
    while read p; do
      if [ "all" == "$p" ] ; then
        systemctl start nfvtime-network.service || true
        break
      fi
      ifup $p || true
    done < $interface_up_file
    rm -f $interface_up_file
  fi

  ### REBOOT SVCS HERE!!!, if system is not starting
  FULL_SVC_LIST=$(cat $svc_restart_file | grep SVC_LIST | cut -f2 -d= | awk '{for (i=1;i<=NF;i++) if (!a[$i]++) printf("%s%s",$i,FS)}{printf("\n")}')
  if [ -n "$FULL_SVC_LIST" ] ; then
    if [ ! -f "$lock_file" ]; then
      touch $lock_file
    fi
    # Re-read the list of services after each service restart as many
    # subscribers can add to the list at any time and spawn many
    # instances of this script. That causes for example 2 instances of
    # the script to restart a the same service at the same time or in
    # short time frame. At the top of the script there is a logic that
    # prevents concurrent instances of the script to run.
    while [ -n "$FULL_SVC_LIST" ]; do
      echo "Services to be restarted: $FULL_SVC_LIST"
      svc_restart=""
      for service in ${FULL_SVC_LIST} ; do
        if [[ ! "$svc_restart" == *"$service"* ]] ; then
          svc_restart="$svc_restart $service"
        fi
      done
      # Why?
      #sed -i "s/^SVC_LIST=.*$/SVC_LIST=$svc_restart/" $svc_restart_file || true
      OPEN_VPN=$(cat $svc_restart_file | sed -n -r 's/.*(openvpn@[a-z]*\.service).*/\1/p' )
      if [ "$system_status" != "starting" ] ; then
        svc_restart_iter=$svc_restart
        if [ -n "$svc_restart_iter" ]; then
          for service in $svc_restart_iter ; do
            echo "restarting $service"
            systemctl restart $service || true
            svc_restart=${svc_restart//" $service"/}
            sed -i "s/$service//g" $svc_restart_file || true
            touch $hb_force_file
            break
          done
        fi
      else
        #list of services which should be restarted even if system is starting
        restart_services_list="openvpn@client.service isc-dhcp-relay.service"
        svc_restart_iter=$svc_restart
        if [ -n "$svc_restart_iter" ]; then
          for service in $svc_restart_iter ; do
            service_in_list=$(echo $restart_services_list | grep "\b$service\b") || true
            if [ -n "$service_in_list" ]; then
              echo "restarting $service"
              systemctl restart $service || true
              svc_restart=${svc_restart//" $service"/}
              sed -i "s/$service//g" $svc_restart_file || true
              touch $hb_force_file
              break
            else
              #not suitable service to restart, just remove it from the list
              svc_restart=${svc_restart//" $service"/}
              sed -i "s/$service//g" $svc_restart_file || true
            fi
          done
        fi
      fi
      sed -i "s/= /=/" $svc_restart_file || true
      FULL_SVC_LIST=$(cat $svc_restart_file | grep SVC_LIST | cut -f2 -d= | awk '{for (i=1;i<=NF;i++) if (!a[$i]++) printf("%s%s",$i,FS)}{printf("\n")}')
    done

  fi

  sed -i "s/^SVC_LIST[ ]*=.*$/SVC_LIST=/" $svc_restart_file
  if [ -f "/run/post_hm_lock_file" ]; then
    /bin/rm -rf /run/post_hm_lock_file
  fi
else
  echo "Script to restart the services in /config/svcs_to_restart after short delay"
  echo "Usage: post_homecalling.sh <delay in seconds>"
  exit 1
fi
#
echo "Ready!"
exit 0
