#!/usr/bin/env python3

import json
import subprocess
import sys

ucpe_cfg_file = "/config/ucpe_config.json"


# NFV-14104 Gives access the pods to images in Manager
def create_registry_master(manager):
    # Check is K8S cluster in running state
    try:
        is_master = subprocess.call("export KUBECONFIG=/etc/kubernetes/admin.conf; kubectl version | grep Server", shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
    except Exception as ex:
        is_master = ''

    if is_master != '':
        with open(ucpe_cfg_file, 'r') as f:
            cfg = json.load(f)

        controller = cfg['ucpe:config']['kubernetes:kubernetes']['controller']

        with open('/etc/docker/daemon.json', 'w') as f:
            f.write('{ "exec-opts": ["native.cgroupdriver=systemd"], "insecure-registries":["%s:5001, "%s:5001"], "log-level":"error", "log-driver":"journald", "max-concurrent-uploads": 2, "max-concurrent-downloads": 4 }' % (str(manager), str(controller)))

        # Create sеcret with docker registry credentials
        try:
            subprocess.call("export KUBECONFIG=/etc/kubernetes/admin.conf; kubectl create secret docker-registry cfcr --docker-server=http://{}:5001 --docker-username=53g84u6a22vrl7xr --docker-password=rxpf7xoglzd8gshw --docker-email=telco@telco.com".format(manager), shell=True)
        except Exception as ex:
            print('Failed to create secret docker registry: {}'.format(ex))

        try:
            subprocess.call("export KUBECONFIG=/etc/kubernetes/admin.conf; kubectl create clusterrolebinding permissive-binding --clusterrole=cluster-admin --user=admin --user=kubelet --user=default --group=system:serviceaccounts", shell=True)
        except Exception as ex:
            print('Failed to create clusterrolebinding: {}'.format(ex))


if __name__ == "__main__":

    argc = len(sys.argv)

    if argc < 2:
        print("No enough arguments. Exit!")
        sys.exit(1)
    elif argc == 3 and sys.argv[1] == "master":
        create_registry_master(sys.argv[2])
    else:
        print("Unknown argument \"%s\"" % sys.argv[1])
        sys.exit(1)

    sys.exit(0)
