#!  /usr/bin/env python3
import hashlib
import sys
import syslog
from threading import Timer

from sysrepo_quagga import sysrepo_quagga as sr

QUAGGA_CONF_DIR = '/etc/frr'


def conf_name2module_key(config):
    if 'zebra.conf' == config:
        return 'zebrad-config'
    elif 'ospfd.conf' == config:
        return 'ospfd-config'
    elif 'ospf6d.conf' == config:
        return 'ospf6d-config'
    elif 'bgpd.conf' == config:
        return 'bgpd-config'

    return 'unknown'


def sr_config_file_changed(f_path, f_name):
    module_name = conf_name2module_key(f_name)

    quagga = sr.SysrepoQuagga(app_name='quagga_change')

    m = hashlib.sha256()
    m.update(open('%s/%s' % (QUAGGA_CONF_DIR, f_name), "rb").read())

    filename=None
    if 'zebra' in module_name:
        filename=quagga.get_zebrad_config_param('filename')
        if filename is not None and filename != '':
            quagga.set_zebrad_config_param('hash-sum', m.hexdigest())
    if 'ospfd' in module_name:
        filename=quagga.get_ospfdd_config_param('filename')
        if filename is not None and filename != '':
            quagga.set_ospfd_config_param('hash-sum', m.hexdigest())
    if 'ospf6d' in module_name:
        filename=quagga.get_ospf6d_config_param('filename')
        if filename is not None and filename != '':
            quagga.set_ospf6d_config_param('hash-sum', m.hexdigest())
    if 'bgp' in module_name:
        filename=quagga.get_bgpd_config_param('filename')
        if filename is not None and filename != '':
            quagga.set_bgpd_config_param('hash-sum', m.hexdigest())

    quagga.commit()

    syslog.syslog(syslog.LOG_DEBUG, "Hash for section \"%s\" is updated" % module_name)


if __name__ == '__main__':
    monitored = ['ospfd.conf', 'ospf6d.conf', 'bgpd.conf', 'zebra.conf']
    syslog.openlog("quagga-change", syslog.LOG_CONS | syslog.LOG_NDELAY, syslog.LOG_USER)
    syslog.setlogmask(syslog.LOG_UPTO(syslog.LOG_DEBUG))

    if 3 > len(sys.argv):
        syslog.syslog(syslog.LOG_DEBUG, "No enough arguments!")

    if sys.argv[2] in monitored:
        syslog.syslog(syslog.LOG_DEBUG, "%s/%s was changed !!!" % (sys.argv[1], sys.argv[2]))

        if sys.argv[2] == 'ospfd.conf':
            t = Timer(0.0, sr_config_file_changed, args=(sys.argv[1], sys.argv[2]))
        elif sys.argv[2] == 'bgpd.conf':
            t = Timer(4.0, sr_config_file_changed, args=(sys.argv[1], sys.argv[2]))
        elif sys.argv[2] == 'zebra.conf':
            t = Timer(8.0, sr_config_file_changed, args=(sys.argv[1], sys.argv[2]))
        elif sys.argv[2] == 'ospf6d.conf':
            t = Timer(12.0, sr_config_file_changed, args=(sys.argv[1], sys.argv[2]))
        else:
            sys.exit(0)

        t.start()
