#!/usr/bin/env python3
import sys
import json
import syslog
import quagga
from tveapi.json_tools import json_tools as jt
import subprocess

if __name__ == '__main__':
    if len( sys.argv ) < 3:
        print("No enough parameters")
        sys.exit(1)

    syslog.openlog( "quagga-hc", syslog.LOG_CONS|syslog.LOG_NDELAY, syslog.LOG_USER )
    syslog.setlogmask(syslog.LOG_UPTO(syslog.LOG_DEBUG))
        
    src_file_name = sys.argv[1]
    dst_file_name = sys.argv[2]

    src_f = open( src_file_name, "r" )
    src_cfg = json.load( src_f )
    src_f.close()

    dst_f = open( dst_file_name, "r+" )
    dst_cfg = json.load( dst_f )

    src_quagga_cfg = jt.get_nested_node( src_cfg, ['json', 'tve', 'config', 'networking', 'quagga'] )
    dst_quagga_cfg = jt.get_nested_node( dst_cfg, ['json', 'tve', 'config', 'networking', 'quagga'] )

    res = False

    if 'enabled' in src_quagga_cfg:
        quagga_enabled = src_quagga_cfg.get( 'enabled', False )
    else:
        quagga_enabled = dst_quagga_cfg.get( 'enabled', False )


    if 'enabled' in src_quagga_cfg and src_quagga_cfg.get( 'enabled', False ) != dst_quagga_cfg.get( 'enabled', False ):
        dst_quagga_cfg[ 'enabled' ] = src_quagga_cfg[ 'enabled' ]
        res = True


    if 'zebrad-config' in src_quagga_cfg:
        rst_zeb = quagga.update_module( src_cfg, dst_cfg, 'zebrad-config' )
        res = True if rst_zeb  else res
    else:
        rst_zeb = False


    if 'ospfd-config' in src_quagga_cfg:
        rst_ospf = quagga.update_module( src_cfg, dst_cfg, 'ospfd-config' )
        res = True if rst_ospf  else res
    else:
        rst_ospf = False

    if 'ospf6d-config' in src_quagga_cfg:
        rst_ospf6 = quagga.update_module( src_cfg, dst_cfg, 'ospf6d-config' )
        res = True if rst_ospf6  else res
    else:
        rst_ospf6 = False


    if 'bgpd-config' in src_quagga_cfg:
        rst_bgp = quagga.update_module( src_cfg, dst_cfg, 'bgpd-config' )
        res = True if rst_bgp  else res
    else:
        rst_bgp = False


    if True == res:
        dst_f.seek(0)
        dst_f.truncate()
        dst_f.seek(0)
        json.dump( dst_cfg, dst_f, sort_keys=True, indent=4, separators=(',', ': ') )
        dst_f.flush()
        dst_f.close()
    else:
        syslog.syslog( syslog.LOG_ERR, "Synchronization of %s FAILED" % src_file_name )


    start_stop_str = 'start' if quagga_enabled else 'stop'
    enable_string = 'enable' if quagga_enabled else 'disable'

    try:
        subprocess.check_call(['systemctl', start_stop_str, 'zebra.service'])
        subprocess.check_call(['systemctl', start_stop_str, 'ospfd.service'])
        subprocess.check_call(['systemctl', start_stop_str, 'ospf6d.service'])
        subprocess.check_call(['systemctl', start_stop_str, 'bgpd.service'])
        subprocess.check_call(['systemctl', enable_string, 'zebra.service'])
        subprocess.check_call(['systemctl', enable_string, 'ospfd.service'])
        subprocess.check_call(['systemctl', enable_string, 'ospf6d.service'])
        subprocess.check_call(['systemctl', enable_string, 'bgpd.service'])


        if quagga_enabled:
            #restart service in order to activate new configuration
            if rst_zeb is True:
                subprocess.check_call(['systemctl', 'restart', 'zebra.service'])
            if rst_ospf is True:    
                subprocess.check_call(['systemctl', 'restart', 'ospfd.service'])
            if rst_ospf6 is True:    
                subprocess.check_call(['systemctl', 'restart', 'ospf6d.service'])
            if rst_bgp is True:
                subprocess.check_call(['systemctl', 'restart', 'bgpd.service'])

    except subprocess.CalledProcessError as e:
        syslog.syslog( syslog.LOG_ERR, 'Command \"%s\" returns %d' %( e.cmd, e.returncode ) )
    except OSError as os_err:
        syslog.syslog( syslog.LOG_ERR, 'OS error %s' % os_err.strerror )

    syslog.closelog()
    sys.exit(0)

