#!/bin/bash
#
# This script provide upgrade capabilities
#
# -l    local image
#
# -r    remote downalod image with wget
#

source /scripts/basic_vars
source /scripts/basic_functions

SETNET=$scripts_dir/setnet.sh

if [ -z "$1" ] || [ "$1" = "-h" ] || [ "$1" == "--help" ]; then 
    echo "remotely_upgrade usage:"
    echo "-h         help"
    echo "-l         select local image from /upgrade/images in format img_ver.xz"
    echo "-r         download an select remote image in format ftp://x.x.x.x/img_ver.xz"
    echo "--no-post  user settings are not restored after upgrade"
    exit 1
fi

base_img=`echo $2 | sed -e s/\.xz//`
current_active=$(basename $(ls -lah $images_dir | grep current_active_version |awk '{print $11}'))

if [ "$current_active" == "$2" ]; then
	echo "This is already the active version! Nothing to do."
    exit 0
fi

### KEEP DATA IF "--no-post" is not passed
if [ -z "$1" ] && [ -z "$2" ] && [ ! "$3" == "--no-post" ]; then
    cp  /etc/network/interfaces.d/ifcfg-inband.4091 $image_dir/ifcfg-inband.4091
    cp  /root/tve_version.txt $images_dir/tve_version.txt
fi

### HANDLE REMOTE UPGRADE
if [ "$1" == "-r" ] && [ ! -z "$2" ]; then
    IMAGE=$images_dir/`basename $2`
    wget -P $images_dir $2 
fi

### HANDLE LOCAL UPGRADE
if [ "$1" == "-l" ] && [ ! -z "$2" ]; then
    IMAGE=$images_dir/$2
fi 

### CLEAN SOME OLD IMAGE (why?)
if [ -f $images_dir/TVE.img ]; then
    rm -rf $images_dir/TVE.img 
fi

### DECOMPRESS AND ACTIVATE THE IMAGE
xz -d -k $IMAGE
if [ "$?" != "0" ]; then    
    rm -rf $base_img
    echo "Extracting image failed, check if image is correct!"
    exit 1
fi 

mv $images_dir/$base_img $images_dir/TVE.img

ln -sf $images_dir/$2 $images_dir/current_active_version



if [ $is_uefi -eq 1 ]; then 
	if [ "$3" = "--no-post" ];then
		echo "no_post" > /boot/efi/.upgrade
	else
		touch /boot/efi/.upgrade
	fi
else
	if [ "$3" = "--no-post" ];then
		echo "no_post" > /boot/.upgrade
	else
		touch /boot/.upgrade
	fi
fi
### save network setiing for upgrade process

cat>$SETNET<<EOF
#!/bin/bash


INBAND_net=$(ifconfig | grep "inband:" -A 5 | grep "inet " | awk '{print $2}')
INBAND_mask=$(ifconfig | grep "inband:" -A 5 | grep "inet " | awk '{print $4}')
INBAND_mac=$(ifconfig | grep inband: -A 5 | grep ether | awk '{print $2}')
INBAND_real=\$(ip addr | grep "\$INBAND_mac" -B 1 | grep -o "^[0-9]*: [a-z]*[0-9]*:"|cut -d ':' -f 2 | cut -d ' ' -f 2)

OUTBAND_net=$(ifconfig | grep "outband:" -A 5 | grep "inet " | awk '{print $2}')
OUTBAND_mask=$(ifconfig | grep "outband:" -A 5 | grep "inet " | awk '{print $4}')
OUTBAND_mac=$(ifconfig | grep outband: -A 5 | grep ether | awk '{print $2}')
OUTBAND_real=\$(ip addr | grep "\$OUTBAND_mac" -B 1 | grep -o "^[0-9]*: [a-z]*[0-9]*:"|cut -d ':' -f 2 | cut -d ' ' -f 2)

if [ ! -z \$INBAND_net ] ; then
	ifconfig \$INBAND_real \$INBAND_net netmask \$INBAND_mask up
fi

if [ ! -z \$OUTBAND_net ] ;then
	ifconfig \$OUTBAND_real \$OUTBAND_net netmask \$OUTBAND_mask up
fi

echo "[INBAND_real][\$INBAND_mac]=\$INBAND_real"
echo "[OUTBAND_real][\$OUTBAND_mac]=\$OUTBAND_real"

EOF
chmod 777 $SETNET

if [ ! -z "$(ifconfig | grep ^[a-z]\*.[0-9][0-9][0-9][0-9]:)" ] ; then   
m=0
   	for i in $(ifconfig | grep ^[a-z]\*.[0-9][0-9][0-9][0-9]: | cut -d ':' -f 1)
	do 
		iface[$m]="$(echo $i | cut -d '.' -f1)"
		vlan[$m]="$(echo $i | cut -d '.' -f2)"
		mac[$m]="$(ifconfig | grep "${iface[$m]}:" -A 5 | grep ether | awk '{print $2}')"
		inet[$m]="$(ifconfig | grep "${iface[$m]}.${vlan[$m]}" -A 5 | grep "inet " | awk '{print $2}')"
		mask[$m]="$(ifconfig | grep "${iface[$m]}.${vlan[$m]}" -A 5 | grep "inet " | awk '{print $4}')"
		echo "iface$m=\$(ip addr | grep \"${mac[$m]}\" -B 1 | grep -o \"^[0-9]*: [a-z]*[0-9]*:\"|cut -d ':' -f 2 | cut -d ' ' -f 2)" >> $SETNET
		echo "vconfig add \$iface$m ${vlan[$m]}" >> $SETNET
		echo "ifconfig \$iface$m.${vlan[$m]} ${inet[$m]} netmask ${mask[$m]} up" >> $SETNET
		((m++))
	done
fi
#echo ".. extract done! System is rebooting to play new version!"
echo "OK"
