#!/usr/bin/env python3
import json
import os
import requests
import shutil
import subprocess
import sys
import syslog
from requests.packages.urllib3.exceptions import InsecureRequestWarning
from sysrepo_api import sysrepo_api3 as api
from time import sleep
import tempfile

CONFIG_NAMESPACE = "/ucpe:config"
STATUS_NAMESPACE = "/ucpe:status"
CONFIG_DATA = {'ucpe:config': {}}
STATUS_DATA = {'ucpe:status': {'system:system': {'hwtype': ''}}}
request_data = {"config": {}, "schema": {}, "nfvTimeOsVersion": ''}
sysrepo_client = None
manager_server_headers = {'Content-Type': 'application/json', }
manager_server_url = 'https://{}:{}/config'
ztp_port = 30636
cookie = None

yang_map = {'config': 'ucpe:config',
            'status': 'ucpe:status',
            'netconf-server': 'ietf-netconf-server:netconf-server',
            'ntp': 'ntp:ntp',
            'quagga': 'quagga:quagga',
            'syslog': 'syslog:syslog',
            'snmp-server': 'snmp-server:snmp-server',
            'security': 'security:security',
            'system-resources': 'system-resources:system-resources',
            'interface': 'interfaces:interface',
            'dhcp-relay': 'dhcprelay:dhcp-relay',
            'dhcp-server': 'dhcpserver:dhcp-server',
            'kubernetes': 'kubernetes:kubernetes',
            'system': 'system:system',
            'upgrade': 'upgrade:upgrade',
            'vrrp': 'vrrp:vrrp'
           }

# To comply sync in case of cbox mode, modify the data at uCPE side
# Latter (if that is handled by Manager) remove from - Here 2
def do_the_manager_job(config_data):
    # search parameter in yang config by given name iface
    def _find_in_ifaces(name, param):
        ret_val = ""
        ifaces = config_data["ucpe:config"]["interfaces:interface"]
        for iface in ifaces:
            if iface["name"] == name:
                try:
                    ret_val = iface[param]
                except Exception as exc:
                    ret_val = ""
                    syslog.syslog(syslog.LOG_ERR, " Exception 1 %s" % exc)
                    pass

        return ret_val

    # Merge empty fields for manager
    def _merge_config(config=None, inputfile='/config/last_received_from_manager.json'):
        configfile = '/tmp/source_for_merge.json'
        cfg_file = open(configfile, 'w+')
        config = json.dumps(config, indent=4, sort_keys=True)
        cfg_file.write(config)
        cfg_file.close()

        try:
            subprocess.check_call(['/scripts/merge_empty.sh', inputfile, configfile])
            with open('/tmp/send.json', 'r') as f:
                config = json.loads(f.read())
        except Exception as e:
            syslog.syslog(syslog.LOG_ERR, "Error during merging JSON file to send back at manager!")
            syslog.syslog(syslog.LOG_ERR, "{}".format(e))
            return None

        if os.path.isfile('/boot/armbianEnv.txt') \
           and os.path.isfile('/opt/etc/nano.json'):

            if os.path.isfile('/tmp/send.json'):
                if os.path.isfile('/tmp/send_with_nulls.json'):
                    os.remove('/tmp/send_with_nulls.json')

                shutil.copyfile('/tmp/send.json', '/tmp/send_with_nulls.json')
            else:
                syslog.syslog(syslog.LOG_ERR, 'Missing /tmp/send.json')

            if os.path.isfile('/tmp/send_with_nulls.json'):
                try:
                    subprocess.check_call(['/scripts/merge_empty.sh', '/tmp/send_with_nulls.json', '/opt/etc/nano.json'])
                    config = None
                    with open('/tmp/send.json', 'r') as f:
                        config = json.loads(f.read())
                except:
                    syslog.syslog(syslog.LOG_ERR, "Error during merging nano JSON file to send back at manager!")
                    return None

            else:
                syslog.syslog(syslog.LOG_ERR, 'Missing /tmp/send_with_nulls.json')

        managerfile = '/config/last_received_from_manager.json'
        manager_file = open(managerfile, 'r')
        managercfg = json.loads(manager_file.read())
        manager_file.close()

        # search parameter in yang config by given name iface
        if "ucpe:config" in managercfg:
            # search parameter in yang config by given name iface
            ifaces2 = managercfg["ucpe:config"]["interfaces:interface"]
            for iface1 in config["ucpe:config"]["interfaces:interface"]:
                for iface2 in ifaces2:
                    if iface1["name"] == iface2["name"]:
                        if not "ipv4-prefix" in iface1 and "ipv4-prefix" in iface2:
                            iface1["ipv4-prefix"] = iface2["ipv4-prefix"]
                        if not "ipv6-prefix" in iface1 and "ipv6-prefix" in iface2:
                            iface1["ipv6-prefix"] = iface2["ipv6-prefix"]
                        if not "wan-interface" in iface1 and "wan-interface" in iface2:
                            iface1["wan-interface"] = iface2["wan-interface"]
                        if not "dns" in iface1 and "dns" in iface2:
                            iface1["dns"] = iface2["dns"]
                        if not "gateway" in iface1 and "gateway" in iface2:
                            iface1["gateway"] = iface2["gateway"]
                        if not "dns6" in iface1 and "dns6" in iface2:
                            iface1["dns6"] = iface2["dns6"]
                        if not "gateway6" in iface1 and "gateway6" in iface2:
                            iface1["gateway6"] = iface2["gateway6"]
                        if not "trace-host" in iface1 and "trace-host" in iface2:
                            iface1["trace-host"] = iface2["trace-host"]
                        if not "tracehost" in iface1 and "tracehost" in iface2:
                            iface1["tracehost"] = iface2["tracehost"]
                        if "tracehost" in iface1 and "tracehost" in iface2 and "probes" in iface2["tracehost"] and not "probes" in iface1["tracehost"]:
                            iface1["tracehost"]["probes"] = iface2["tracehost"]["probes"]
                        if "tracehost" in iface1 and "probes" in iface1["tracehost"]:
                            for probe in iface1["tracehost"]["probes"]:
                                if "min-download" in probe:
                                    val = probe["min-download"]
                                    probe["min-download"] = str(val)
                                elif "tracehost" in iface2 and "probes" in iface2["tracehost"] and iface2["tracehost"]["probes"]:
                                    for pr in iface2["tracehost"]["probes"]:
                                        if pr["id"] == probe["id"] and "min-download" in pr:
                                            probe["min-download"] = str()
                                if "min-upload" in probe:
                                    val = probe["min-upload"]
                                    probe["min-upload"] = str(val)
                                elif "tracehost" in iface2 and "probes" in iface2["tracehost"] and iface2["tracehost"]["probes"]:
                                    for pr in iface2["tracehost"]["probes"]:
                                        if pr["id"] == probe["id"] and "min-upload" in pr:
                                            probe["min-upload"] = str()
                                if not "description" in probe:
                                    for pr in iface2["tracehost"]["probes"]:
                                        if pr["id"] == probe["id"] and "description" in pr:
                                            probe["description"] = pr["description"]
                                if not "endpoint" in probe:
                                    for pr in iface2["tracehost"]["probes"]:
                                        if pr["id"] == probe["id"] and "endpoint" in pr:
                                            probe["endpoint"] = pr["endpoint"]
                        if not "apn" in iface1 and "apn" in iface2:
                            iface1["apn"] = iface2["apn"]
                        if not "pass" in iface1 and "pass" in iface2:
                            iface1["pass"] = iface2["pass"]
                        if not "user" in iface1 and "user" in iface2:
                            iface1["user"] = iface2["user"]
                        if not "country" in iface1 and "country" in iface2:
                            iface1["country"] = iface2["country"]
                        if not "ssid" in iface1 and "ssid" in iface2:
                            iface1["ssid"] = iface2["ssid"]
                        if "ike-phase-1" in iface1 and "ike-phase-1" in iface2 and not "dh-groups" in iface1[
                            "ike-phase-1"] and "dh-groups" in iface2["ike-phase-1"]:
                            iface1["ike-phase-1"]["dh-groups"] = iface2["ike-phase-1"]["dh-groups"]
                        if "ike-phase-1" in iface1 and "ike-phase-1" in iface2 and not "local" in iface1[
                            "ike-phase-1"] and "local" in iface2["ike-phase-1"]:
                            iface1["ike-phase-1"]["local"] = iface2["ike-phase-1"]["local"]
                        if "ike-phase-1" in iface1 and "ike-phase-1" in iface2 and "local" in iface1[
                            "ike-phase-1"] and "local" in iface2["ike-phase-1"] and \
                                "rounds" not in iface1["ike-phase-1"]["local"] and "rounds" in iface2["ike-phase-1"][
                            "local"]:
                            iface1["ike-phase-1"]["local"]["rounds"] = iface2["ike-phase-1"]["local"]["rounds"]
                        if "ike-phase-1" in iface1 and "ike-phase-1" in iface2 and not "remote" in iface1[
                            "ike-phase-1"] and "remote" in iface2["ike-phase-1"]:
                            iface1["ike-phase-1"]["remote"] = iface2["ike-phase-1"]["remote"]
                        if "ike-phase-1" in iface1 and "ike-phase-1" in iface2 and "remote" in iface1[
                            "ike-phase-1"] and "remote" in iface2["ike-phase-1"] and \
                                "rounds" not in iface1["ike-phase-1"]["remote"] and "rounds" in iface2["ike-phase-1"][
                            "remote"]:
                            iface1["ike-phase-1"]["remote"]["rounds"] = iface2["ike-phase-1"]["remote"]["rounds"]
                        if "ike-phase-2" in iface1 and "ike-phase-2" in iface2 and not "dh-groups" in iface1[
                            "ike-phase-2"] and "dh-groups" in iface2["ike-phase-2"]:
                            iface1["ike-phase-2"]["dh-groups"] = iface2["ike-phase-2"]["dh-groups"]
                        if not "local-id" in iface1 and "local-id" in iface2:
                            iface1["local-id"] = iface2["local-id"]
                        if not "remote-id" in iface1 and "remote-id" in iface2:
                            iface1["remote-id"] = iface2["remote-id"]
                        if "security" in iface1 and not "mac-allow" in iface1["security"] and "security" in iface2 and "mac-allow" in iface2["security"]:
                            iface1["security"]["mac-allow"] = iface2["security"]["mac-allow"]
                        if "security" in iface1 and not "mac-deny" in iface1["security"] and "security" in iface2 and "mac-deny" in iface2["security"]:
                            iface1["security"]["mac-deny"] = iface2["security"]["mac-deny"]
                        if "security" in iface1 and not "passphrase" in iface1["security"] and "security" in iface2 and "passphrase" in iface2["security"]:
                            iface1["security"]["passphrase"] = iface2["security"]["passphrase"]
                        if not "wifi-wan-interface" in iface1 and "wifi-wan-interface" in iface2:
                            iface1["wifi-wan-interface"] = iface2["wifi-wan-interface"]
                        if "remote" in iface1 and "remote" in iface2 and not "mcast-vtep-addr" in iface1["remote"] and "mcast-vtep-addr" in iface2["remote"]:
                            iface1["remote"]["mcast-vtep-addr"] = iface2["remote"]["mcast-vtep-addr"]
                        if iface1["type"] == 'pppoe':
                            if not "ac-name" in iface1 and "ac-name" in iface2:
                                iface1["ac-name"] = iface2["ac-name"]
                            if not "username" in iface1 and "username" in iface2:
                                iface1["username"] = iface2["username"]
                            if not "password" in iface1 and "password" in iface2:
                                iface1["password"] = iface2["password"]
                            if not "ethernet-interface" in iface1 and "ethernet-interface" in iface2:
                                iface1["ethernet-interface"] = iface2["ethernet-interface"]
                        if "ovpn-cfg" in iface1 and "ovpn-cfg" in iface2 and not "concentrator-endpoint" in iface1["ovpn-cfg"] and "concentrator-endpoint" in iface2["ovpn-cfg"]:
                            iface1["ovpn-cfg"]["concentrator-endpoint"] = iface2["ovpn-cfg"]["concentrator-endpoint"]
                        break

            snmp2 = managercfg["ucpe:config"]["snmp-server:snmp-server"]["users"]
            for users1 in config["ucpe:config"]["snmp-server:snmp-server"]["users"]:
                for users2 in snmp2:
                    if users1["id"] == users2["id"]:
                        if not "auth-password" in users1 and "auth-password" in users2:
                            users1["auth-password"] = users2["auth-password"]
                        if not "notify-view" in users1 and "notify-view" in users2:
                            users1["notify-view"] = users2["notify-view"]
                        if not "priv-password" in users1 and "priv-password" in users2:
                            users1["priv-password"] = users2["priv-password"]
                        if not "remote-engine-id" in users1 and "remote-engine-id" in users2:
                            users1["remote-engine-id"] = users2["remote-engine-id"]
                        if not "write-view" in users1 and "write-view" in users2:
                            users1["write-view"] = users2["write-view"]
                        if not "read-view" in users1 and "read-view" in users2:
                            users1["read-view"] = users2["read-view"]
                        break

            syslog2 = managercfg["ucpe:config"]["syslog:syslog"]["remote-servers"]
            for server1 in config["ucpe:config"]["syslog:syslog"]["remote-servers"]:
                for server2 in syslog2:
                    if server1["id"] == server2["id"]:
                        if not "ip-address" in server1 and "ip-address" in server2:
                            server1["ip-address"] = server2["ip-address"]
                        break

        return config

    try:
        # make manager happy
        try:
            ifaces = config_data["ucpe:config"]["interfaces:interface"]
            for iface in ifaces:
                if iface["name"] == "wifi0":
                    if "mac-allow" not in iface["security"]:
                        iface["security"]["mac-allow"] = []
                    if "mac-deny" not in iface["security"]:
                        iface["security"]["mac-deny"] = []
        except Exception as e:
            pass

        if os.path.exists('/config/last_received_from_manager_nulls.json'):
            config_data = _merge_config(config_data)
        elif os.path.isfile('/boot/armbianEnv.txt'):
            nano_trimmed_config_file = '/tmp/nano_source_for_merge.json'
            nanoconfig = json.dumps(config_data, indent=4, sort_keys=True)
            with open(nano_trimmed_config_file, 'w') as f:
                f.write(nanoconfig)

            try:
                subprocess.check_call(['/scripts/merge_empty.sh',
                                       nano_trimmed_config_file,
                                       '/opt/etc/nano.json'])
                with open('/tmp/send.json', 'r') as f:
                    config_data = json.loads(f.read())
            except:
                syslog.syslog(syslog.LOG_ERR, 'Error during send untrimmed nano JSON file to manager!')
                return None

    except Exception as e:
        syslog.syslog(syslog.LOG_ERR, "Exception 5 %s" % e)
        pass

    return config_data


def sendData(manager_server, config_data, send_flag, send_schema, version_str):
    global request_data
    global ztp_port
    global cookie
    # Get machine serial number
    serial_number = None
    try:
        serial_file = open("/config/.serial-number", 'r')
        serial_number = serial_file.read().strip()
        serial_file.close()
        if serial_number:
            process_loop = True

    except (OSError, IOError):
        syslog.syslog(syslog.LOG_ERR, "Error opening file contains serial number")
        syslog.syslog(syslog.LOG_DEBUG, "Can not start working process")

    if config_data:
        config_data = do_the_manager_job(config_data)

    syslog.syslog(syslog.LOG_DEBUG, 'full {}'.format(config_data))
    # Send data to server if allowed
    if config_data and serial_number and version_str:
        if send_flag:
            # Create retry loop
            retry_loop = True
            retry_seconds = 5
            while retry_loop:
                manager_server_params = (('serial-number', serial_number),)
                try:
                    syslog.syslog(syslog.LOG_DEBUG, 'Try upload configuration to {}:{}'.format(
                        manager_server, ztp_port))

                    request_data["config"] = config_data
                    request_data["nfvTimeOsVersion"] = version_str
                    if send_schema:
                        schemaf = open("/etc/ucpe-schema.json", 'r')
                        schema = schemaf.read().strip()
                        schemaf.close()
                        request_data["schema"] = json.loads(schema)

                    response = requests.put(manager_server_url.format(manager_server, ztp_port),
                                            headers=manager_server_headers,
                                            params=manager_server_params, data=json.dumps(request_data),
                                            verify=False, timeout=(5, 5))

                    if response.status_code == requests.codes.ok:
                        syslog.syslog(syslog.LOG_INFO,
                                      'Upload configuration to {}:{} SUCCEEDED!'.format(
                                          manager_server, ztp_port))
                        retry_loop = False  # Exit from retry loop
                        try:
                            with open("/config/last_sent_to_manager.json", "w") as text_file:
                                text_file.write("{}" .format(json.dumps(request_data)))
                        except:
                            pass

                    else:
                        syslog.syslog(syslog.LOG_INFO,
                                      "Upload configuration to {}:{} FAILED with {}!".format(
                                          manager_server, ztp_port, response.status_code))
                        if response.status_code == 500:
                            syslog.syslog(syslog.LOG_DEBUG, "Please, check serial number!")
                            send_schema = True

                        if response.status_code == 409:
                            send_schema = True

                        syslog.syslog(syslog.LOG_DEBUG, "Retry in {} seconds...".format(retry_seconds))
                        sleep(retry_seconds)
                except Exception as e:
                    syslog.syslog(syslog.LOG_INFO,
                                  'Upload configuration to {}:{} exception {}'.format(
                                      manager_server, ztp_port, e))
                    syslog.syslog(syslog.LOG_DEBUG, "Please, check resolving/connection to that address/IP")
                    syslog.syslog(syslog.LOG_DEBUG, "Retry in {} seconds...".format(retry_seconds))
                    sleep(retry_seconds)

                if retry_seconds >= 30:
                    retry_seconds = 30
                else:
                    retry_seconds += 1

                if not os.path.exists(cookie):
                    retry_loop = False # exit retry loop because more recent instance took over
                    syslog.syslog(syslog.LOG_DEBUG, "More recent instance took over")

            else:
                syslog.syslog(syslog.LOG_ERR, "Homecalling is disabled. No data will be send to Manager!")
        else:
            syslog.syslog(syslog.LOG_ERR, "Invalid data sent to Working process serial {} version_str {} ztp_port {}".format(serial_number, version_str, ztp_port))

    operation = 'schema' if send_schema else 'config'

    return


def workingProcess(schema=False):
    global sysrepo_client
    if sysrepo_client:
        del sysrepo_client
        sysrepo_client = None
    try:
        if sysrepo_client is None:
            sysrepo_client = api.SysrepoAPI(app_name="currentconfig_app_worker")

    except:
        syslog.syslog(syslog.LOG_ERR, "Unable to create currentconfig_app_worker")
        return

    manager_server = None
    config_data = None
    hc_enabled = False
    try:
        syslog.syslog(syslog.LOG_DEBUG, "Get Running Config")
        sysrepo_client.save_config(src_ds="running", dest_ds="startup")
        sysrepo_client.commit()
        init_data = sysrepo_client.get_running_config()
        config_data = CONFIG_DATA

        for i in init_data['config']:
            config_data['ucpe:config'].update({'{}'.format(yang_map[i]):{}})
            config_data['ucpe:config'][yang_map[i]] = init_data['config'][i]

        init2_data = sysrepo_client.session.get_data("/ietf-netconf-server:netconf-server")
        netconf_data = {'ietf-netconf-server:netconf-server': init2_data['netconf-server']}
        if 'ietf-netconf-server:netconf-server' in netconf_data and \
           'listen' in netconf_data['ietf-netconf-server:netconf-server'] and \
           'endpoint' in netconf_data['ietf-netconf-server:netconf-server']['listen']:
            for i in netconf_data['ietf-netconf-server:netconf-server']['listen']['endpoint']:
                i['ssh'].pop('ssh-server-parameters')

        config_data.update(netconf_data)

        # Get status fields
        kernel = ""
        hwtype = ""
        version = ""
        host_available_cpus = ""
        host_available_memory = ""
        try:
            # Get status data - host-available-cpus
            # sysrepo_req = "{}/system-resources:system-resources/host-available-cpus".format(STATUS_NAMESPACE)
            # host_available_cpus = sysrepo_client.get_item(sysrepo_req)

            # Get status data - host-available-memory
            # sysrepo_req = "{}/system-resources:system-resources/host-available-memory".format(STATUS_NAMESPACE)
            # host_available_memory = sysrepo_client.get_item(sysrepo_req)

            # Get status data - Kernel
            # sysrepo_req = "{}/system-resources:system-resources/kernel".format(STATUS_NAMESPACE)
            # kernel = sysrepo_client.get_item(sysrepo_req)

            # Get status data - Hardware type
            hw_type_file = "/config/.hwtype"
            if not os.path.exists(hw_type_file):
                hwtype = ""
            else:
                f_hwtype = open(hw_type_file, "r")
                hwtype = f_hwtype.readline().strip()
                f_hwtype.close()

            # Get status data - version string
            version_f = subprocess.check_output(['dpkg-query', "--showformat='${Version}'", '--show', 'nfvtimeos'])
            version_f = version_f.decode().replace('\'', '')
            version = (version_f.strip())
        except Exception as exc:
            syslog.syslog(syslog.LOG_ERR, " Exception 7 %s" % exc)
            pass  # Silently ignore not presents of the data in sysrepo

        if hwtype:
            STATUS_DATA["ucpe:status"]["system:system"]["hwtype"] = hwtype

        config_data.update(STATUS_DATA)

        # Get server address
        sysrepo_req = "{}/system:system/config-server/server".format(CONFIG_NAMESPACE)
        manager_server = sysrepo_client.get_item(sysrepo_req)

        # Get status of homecalling
        hc_status_req = "{}/system:system/homecalling/enabled".format(CONFIG_NAMESPACE)
        hc_enabled = bool(sysrepo_client.get_item(hc_status_req))

    except:
        syslog.syslog(syslog.LOG_ERR, "Unable to get data from sysrepo!")
        # Force avoid next operation
        manager_server = None
        config_data = None
        hc_enabled = False

    # Pass data WP, but sending of data to Manager will be only when homecalling is enabled
    if manager_server and config_data and version:
        # To avoid blocking of callback here a working process will be bring to do processing
        sendData(manager_server, config_data, hc_enabled, schema, version)


if __name__ == "__main__":
    # The code below is not in separate function and it is not visible when the file is imported as lib
    debugmode = False
    schema = False

    for f in os.listdir('/tmp'):
        if f.startswith('sendconfig_') and f.endswith('.cookie'):
            os.unlink(os.path.join('/tmp', f))

    cookie=tempfile.NamedTemporaryFile(suffix='.cookie', prefix='sendconfig_', dir='/tmp/', delete=False).name

    for arg in sys.argv[1:]:
        if arg == "--debug":
            debugmode = True
            break
        if arg == "config":
            schema = False
            break
        if arg == "schema":
            schema = True
            break

    for i in ["config", "schema"]:
        if os.path.exists('/config/_sendconfig/{}/{}'.format(i, i)):
            os.rmdir('/config/_sendconfig/{}/{}'.format(i, i))

    syslog.openlog(ident='currentconfig cronjobs')
    if debugmode:
        syslog.setlogmask(syslog.LOG_UPTO(syslog.LOG_DEBUG))
    else:
        syslog.setlogmask(syslog.LOG_UPTO(syslog.LOG_INFO))
        requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

    sleep(5)
    workingProcess(schema)
