#!/usr/bin/env python3
import ha

HAPROXY_CONFIG = '/etc/haproxy/haproxy.cfg'


def config_haproxy(log):
    cluster_cfg = ha.get_cluster_cfg(log)
    data0, _ = ha.get_data0(log)

    haproxy_lines = '\n'.join((
        'global',
        '    log /dev/log local0 err',
        '    log /dev/log local1 err',
        '    daemon',
        '',
        'defaults',
        '    mode                    http',
        '    log                     global',
        '    option                  httplog',
        '    option                  dontlognull',
        '    option http-server-close',
        '    option forwardfor       except 127.0.0.0/8',
        '    option                  redispatch',
        '    retries                 1',
        '    timeout http-request    10s',
        '    timeout queue           20s',
        '    timeout connect         5s',
        '    timeout client          20s',
        '    timeout server          20s',
        '    timeout http-keep-alive 10s',
        '    timeout check           10s',
        '',
        'frontend apiserver',
        '    bind *:50443',
        '    mode tcp',
        '    option tcplog',
        '    default_backend apiserver',
        '',
        'backend apiserver',
        '    option httpchk GET /healthz',
        '    http-check expect status 200',
        '    mode tcp',
        '    option ssl-hello-chk',
        '    balance roundrobin',
        '    server {} {}:6443 check'.format(data0, data0),
        '',))

    with open(HAPROXY_CONFIG, 'w') as f:
        for line in haproxy_lines:
            print(line, end='', file=f)

    log.debug('config haproxy done')
