#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import os
import json

CFG_FILE = sys.argv[1]
NET_SNMP_FILE = '/var/lib/snmp/snmpd.conf'

def general_setup():
  if not os.path.isfile( CFG_FILE ):
    print("Cannot open config file")
    sys.exit( 1 )

  #cfg_f = open( CFG_FILE, "r" )
  with open(CFG_FILE, 'r') as cfg_f:
    cfg = json.load( cfg_f )
  cfg_f.close()

  with open('/etc/snmp/snmpd.conf', 'w') as file:
    file.write('### AgentX Sub-Agents\n')
    file.write('master agentx\n')
    #
    file.write('\n### System information\n')
    if cfg['json']['tve']['config']['snmp-server']['system-name'] != u'':
      file.write('sysname ' + cfg['json']['tve']['config']['snmp-server']['system-name'] + '\n')
    if cfg['json']['tve']['config']['snmp-server']['system-location'] != u'':
      file.write('syslocation ' + cfg['json']['tve']['config']['snmp-server']['system-location'] + '\n')
    if cfg['json']['tve']['config']['snmp-server']['system-contact'] != u'':
      file.write('syscontact ' + cfg['json']['tve']['config']['snmp-server']['system-contact'] + '\n')
    if cfg['json']['tve']['config']['snmp-server']['system-description'] != u'':
      file.write('sysdescr ' + cfg['json']['tve']['config']['snmp-server']['system-description'] + '\n')
    #
    file.write('sysServices 11\n')
    #
    file.write('\n### Agent Behaviour\n')
    file.write('agentaddress udp:' + cfg['json']['tve']['config']['snmp-server']['bind-address'] + ':' + str(cfg['json']['tve']['config']['snmp-server']['port'])  + '\n')
    #
    file.write('\n### VACM Configuration\n')
    file.write('\n# View\n')
    for views in cfg['json']['tve']['config']['snmp-server']['views']:
      file.write('view ' + views['name'] + ' ' + views['type'] + ' ' + views['oid'] + '\n')
    #
    file.write('\n# Access Control\n')
    for users in cfg['json']['tve']['config']['snmp-server']['users']:
      if users['message-model'] == 'v1' or users['message-model'] == 'v2c':
        file.write('com2sec ' + users['user-name'] + ' default ' + users['user-name'] + '\n')
        file.write('group ' + users['user-name'] + "Group " + users['message-model'] + ' ' + users['user-name'] +'\n')
        access = 'access ' + users['user-name'] + "Group \"\" " + users['message-model'] + ' ' +users['security-level'] + ' exact '
        if users['read-view'] == "":
          access = access + "none "
        else:
          access = access + users['read-view'] + ' '
        if users['write-view'] == "":
          access = access + "none "
        else:
          access = access + users['write-view'] + ' '
        if users['notify-view'] == "":
          access = access + "none\n"
        else:
          access = access + users['notify-view'] + '\n'
        #
        file.write(access)
    #
    file.write('rouser internal\n')
    #
    file.write('\n### Active Monitoring\n')
    file.write('\n# Notification Handling\n')
    file.write('trapcommunity monitoring\n')
    #
    for targets in cfg['json']['tve']['config']['snmp-server']['target-address']:
      if targets['enable'] is False:
        continue
      trapsess = 'trapsess -' + targets['message-model']
      if targets['message-model'] == 'v3':
        for targetuser in cfg['json']['tve']['config']['snmp-server']['users']:
          if targets['security-name'] == targetuser['user-name']:
            trapsess = trapsess + ' -u ' + targets['security-name'] + ' -l ' + targets['security-level'] + ' -a ' + targetuser['auth-type'] + ' -A ' + targetuser['auth-password'] + ' -x ' + targetuser['priv-type'] + ' -X ' + targetuser['priv-password']
      else:
        trapsess = trapsess + ' -c ' + targets['security-name']
      if targets['type'] == 'inform':
        trapsess = trapsess + ' -Ci ' + targets['ipv4-address'] + ':' + str(targets['dst-port']) + ' -r '+ str(targets['retry-count']) + ' -t ' + str(targets['timeout']) +'\n'
      else:
        trapsess = trapsess + ' ' + targets['ipv4-address'] + ':' + str(targets['dst-port']) + ' -r '+ str(targets['retry-count']) + ' -t ' + str(targets['timeout']) +'\n'
      #
      file.write(trapsess)
      #
    #
    file.write('# DisMan Event MIB\n')
    file.write('agentSecName internal\n')
    file.write('notificationEvent  linkUpTrap    linkUp   ifIndex ifAdminStatus ifOperStatus\n')
    file.write('notificationEvent  linkDownTrap  linkDown ifIndex ifAdminStatus ifOperStatus\n')
    file.write('monitor -r 1 -e linkUpTrap "Generate linkUp" ifOperStatus != 2\n')
    file.write('monitor -r 1 -e linkDownTrap "Generate linkDown" ifOperStatus == 2\n')
    file.write('sysObjectID .1.3.6.1.4.1.738.20.50001\n')
    #
    file.write('# Send traps on authentication failures\n')
    if cfg['json']['tve']['config']['snmp-server']['authentication-failure-trap-enabled'] is True:
      file.write('authtrapenable 1\n')
    else:
      file.write('authtrapenable 2\n')
    #
    file.write('maxGetbulkResponses 20\n')
    #
  file.close

def set_v3():
  pos = 20
  if not os.path.isfile( CFG_FILE ):
    print("Cannot open config file")
    sys.exit( 1 )

  cfg_f = open( CFG_FILE, "r" )
  cfg = json.load( cfg_f )

  #If we need backup file
  #if not os.path.isfile( NET_SNMP_FILE ):
  # if not os.path.isfile( (NET_SNMP_FILE + '.bak') ):
  #  print "Cannot open net-snmp snmdp.conf file"
  #  sys.exit( 1 )
  # else:
  #  copyfile( (NET_SNMP_FILE + '.bak'), NET_SNMP_FILE )

  with open(NET_SNMP_FILE, 'r') as file:
    flines = file.readlines()

  if len(flines) < (pos + 1):
    print("Malformed snmp/snmpd.conf file")
    sys.exit( 1 )

  #copyfile(NET_SNMP_FILE, (NET_SNMP_FILE + '.bak'))
  flines[pos] = '\n'
  flines[pos+1] = '\n'
  for users in cfg['json']['tve']['config']['snmp-server']['users']:
    if users['message-model'] == 'v3':
      flines[pos] = 'createUser ' + users['user-name'] + ' ' + users['auth-type'] + ' ' + users['auth-password'] + ' ' + users['priv-type'] + ' ' + users['priv-password'] + '\n'
      pos += 1
      authtype=True
      if '' == users['auth-type'] or "none" == users['auth-type']:
        authtype=False

      privtype=True
      if '' == users['priv-type'] or "none" == users['priv-type']:
        privtype=False

      #TODO read views !!!! to be done in sysrepo
      if True == privtype and True == authtype:
        flines [pos] = 'Rwuser ' + users['user-name']+' priv '+'1.3.6.1'+ '\n'
      elif False == privtype and True == authtype:
        flines [pos] = 'Rwuser ' + users['user-name']+' auth '+'1.3.6.1'+ '\n'
      elif False == privtype and False == authtype:
        flines [pos] = 'Rwuser ' + users['user-name']+' noauth '+'1.3.6.1'+ '\n'

      pos += 1

  with open('/var/lib/snmp/snmpd.conf', 'w') as file:
    for l in flines:
      file.write( l )
  file.close

if __name__ == "__main__":
  general_setup()
  set_v3()
  sys.exit( 0 )

