#!/usr/bin/env python3
import ha

KEEPALIVED_CONFIG = '/etc/keepalived/keepalived.conf'
VPN_NAME_FILE = '/opt/vpn-name'


def config_keepalived(log):
    cluster_cfg = ha.get_cluster_cfg(log)
    data0, data0_network = ha.get_data0(log)
    peer1 = ha.get_peer1_ip(log, cluster_cfg, data0)
    peer2 = ha.get_peer2_ip(log, cluster_cfg, data0)
    priority = int(data0.split('.')[3]) % 100

    keepalived_lines = '\n'.join((
        'vrrp_script chk_service {',
        ' script "/scripts/keepalived_vrrp_check.sh"',
        ' interval 2',
        ' fall 2',
        ' rise 2',
        '# weight 2',
        '',
        '}',
        '',
        'vrrp_instance VI_br_data0 {',
        'interface br-data0',
        'state BACKUP',
        'priority {}'.format(priority),
        'virtual_router_id 10',
        'advert_int 1',
        'nopreempt',
        'authentication {',
        'auth_type PASS',
        'auth_pass Nfv@admin21',
        '}',
        'virtual_ipaddress {',
        '{}/{} #VIRTUALIP'.format(cluster_cfg['virtual_address'], data0_network),
        '}',
        'unicast_src_ip {}'.format(data0),
        'unicast_peer {',
        '{} #UNICASTPEER'.format(peer1),
        '{} #UNICASTPEER'.format(peer2),
        '}',
        'track_script {',
        'chk_service',
        '}',
        'notify /scripts/keepalived.sh',
        '}',
        '',
        'vrrp_sync_group openvpn {',
        'group {',
        '       VI_br_data0',
        '      }',
        '}',
        '',))

    with open(KEEPALIVED_CONFIG, 'w') as f:
        for line in keepalived_lines:
            print(line, end='', file=f)

    vpnpass = '\n'.join((
        '{}'.format(cluster_cfg['virtual_hostname']),
        'nopass'
        '',))

    with open(VPN_NAME_FILE, 'w') as f:
        for line in vpnpass:
            print(line, end='', file=f)

    log.debug('Changed {} to {}'.format(VPN_NAME_FILE, cluster_cfg['virtual_hostname']))

    log.debug('config keepalived done')
