#!/usr/bin/env python3
import json
import os
import re
import sys
import time

import requests

import ha

K8S_MASTER_JOIN_FILE = '/config/.k8s_master_join.json'
URL_AUTH = 'https://{}:7374/v3/auth/tokens'
URL_TOKEN = 'https://{}:7373/v1/kubernetes/token'
URL_CERT = 'https://{}:7373/v1/kubernetes/cert'
# TODO what will happen when change password ?
DATA_AUTH = {'user': 'admin', 'password': 'Nfv@admin21'}


def request_retry(url, log,
                  num_retries=30, success_list=[200],
                  **kwargs):
    for _ in range(num_retries):
        try:
            log.debug('kwa {}'.format(kwargs))
            response = requests.get(url, verify=False, **kwargs)
            if response.status_code in success_list:
                # Return response if successful
                return response
            else:
                log.debug('Failed {} to get {}'.format(
                    response.status_code, url))
        except:
            log.error('ConnectionError to get {}. Retry in 5sec !'.format(url))

        time.sleep(5)

    return None


def config_etc_hosts(log):
    cluster_cfg = ha.get_cluster_cfg(log)

    log.debug('Add virtual entry in /etc/hosts')
    with open('/etc/hosts', 'r') as f:
        lines = f.readlines()

    new_lines = []
    for line in lines:
        if 'virtual_hostname' in cluster_cfg \
           and cluster_cfg['virtual_hostname'] \
           and re.search('\s*{}\s*'.format(cluster_cfg['virtual_hostname']), line):
            log.debug('old virtutal hostname {} found in /etc/hosts, remove it'.format(cluster_cfg['virtual_hostname']))
            continue

        if 'virtual_address' in cluster_cfg \
           and cluster_cfg['virtual_address'] \
           and re.search('^{}\s*'.format(cluster_cfg['virtual_address']), line):
            log.debug('old virtual_address {} found in /etc/hosts, remove it'.format(
                cluster_cfg['virtual_address']))
            continue

        new_lines.append(line)

    new_lines.append('{} {}\n'.format(
        cluster_cfg['virtual_address'],
        cluster_cfg['virtual_hostname']))

    with open('/etc/hosts', 'w') as f:
        for line in new_lines:
            print(line, end='', file=f)

    log.debug('done adding clean {} {} in /etc/hosts'.format(
        cluster_cfg['virtual_address'],
        cluster_cfg['virtual_hostname']))


def config_kubernetes(log):
    cluster_cfg = ha.get_cluster_cfg(log)

    config_etc_hosts(log)
    if cluster_cfg['initial_master']:
        log.debug('We are initial_master')
        return

    if os.path.exists(K8S_MASTER_JOIN_FILE):
        log.error('{} already exists. Exit !'.format(K8S_MASTER_JOIN_FILE))
        sys.exit(2)

    auth_r = request_retry(URL_AUTH.format(cluster_cfg['virtual_address']),
                           log=log,
                           data=json.dumps(DATA_AUTH),
                           headers={'Content-type': 'application/json'})
    if not auth_r:
        log.error('Can not get OS_TOKEN from {}. Exit !'.format(cluster_cfg['virtual_address']))
        sys.exit(2)

    token_r = request_retry(URL_TOKEN.format(cluster_cfg['virtual_address']),
                            log=log,
                            data=json.dumps(DATA_AUTH),
                            headers={
                                'X-Auth-Token': auth_r.content.decode('utf-8'),
                                'Content-type': 'application/json'})
    if not token_r:
        log.error('Can not get k8s token from {}. Exit !'.format(cluster_cfg['virtual_address']))
        sys.exit(2)

    cert_r = request_retry(URL_CERT.format(cluster_cfg['virtual_address']),
                           log=log,
                           data=json.dumps(DATA_AUTH),
                           headers={
                               'X-Auth-Token': auth_r.content.decode('utf-8'),
                               'Content-type': 'application/json'})
    if not cert_r:
        log.error('Can not get k8s token from {}. Exit !'.format(cluster_cfg['virtual_address']))
        sys.exit(2)

    cfg = {'token': token_r.content.split()[0].decode('utf-8'),
           'hash': token_r.content.split()[1].decode('utf-8'),
           'cert': cert_r.content.decode('utf-8')}
    log.debug('cfg: {}'.format(cfg))
    with open(K8S_MASTER_JOIN_FILE, 'w') as f:
        print(json.dumps(cfg, indent=4, sort_keys=False), file=f)

    log.debug('config kubernetes done')
