#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import os
import json
import ipaddress

import ucpeif.wg_concentratorif as wgc
import edgility_db.edgility_db as db

STATIC_POOL_PROPERTIES = (
            "hostname VARCHAR(255) UNIQUE, "
            "tunnel_ip VARCHAR(255)"
        )
STATIC_POOL_COLUMNS = [
            "hostname",
            "tunnel_ip"
    ]
STATIC_POOL_NAME = 'static_clients_'

def db_table_name(db_pool, intf_name):
        return db_pool + intf_name

def set_wireguard_concentrator(hostname, password, public_key, conc_port, endpoint):
    client = wgc.WireGuardConcentratorClient(hostname, password, public_key, int(conc_port), endpoint)
    print(client)
   
def add_del_static_client_in_db(intf_name, hostname, tunnel_ip=None, delete=False):
    db_int = db.EdgilityDatabase()
    
    print(intf_name, hostname)
    
    if delete:
        print(hostname)
        db_int.delete_value_from_table(db_table_name(STATIC_POOL_NAME, intf_name), 'hostname', hostname)
    else:
        tunnel_ip_int = int(ipaddress.IPv4Address(tunnel_ip))
        values = (hostname, tunnel_ip_int)
        db_int.create_table(db_table_name(STATIC_POOL_NAME, intf_name), STATIC_POOL_PROPERTIES)
        db_int.insert_values_in_table(db_table_name(STATIC_POOL_NAME, intf_name), STATIC_POOL_COLUMNS, values)

if __name__ == "__main__":
    argc = len(sys.argv)
    if argc < 3:
        sys.exit(1)

    if sys.argv[1] == "dynamic":
        set_wireguard_concentrator(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5], sys.argv[6])
    elif sys.argv[1] == "static":
        if sys.argv[4] == "delete":
            add_del_static_client_in_db(sys.argv[2], sys.argv[3], delete=True)
        else:
            add_del_static_client_in_db(sys.argv[2], sys.argv[3], sys.argv[4])

    sys.exit(0)

