#!/usr/bin/env python3
import sys
import os
import subprocess
import json
import iptc
import re

CFG_FILE = "/config/ucpe_config.json"
SPICE_CHAIN = "spice-filter"
INPUT_CHAIN = "INPUT"
INCRONFILE = '/config/_spice_handle/vm_changed'

def walk_vms_apply_rules (chain, bind_interfaces):
    try:
        output = subprocess.check_output( [ 'virsh',  'list', '--name' ] ).decode()
    except subprocess.CalledProcessError:
        output = ''
        pass
    vmlist = (output.strip()).split('\n')
    for vmname in vmlist:
      if vmname:
          domdisplay = subprocess.check_output( [ 'virsh',  'domdisplay', '%s' % (vmname) ]).decode()
          match = re.match ( r'^(\w+)://([\d\w\.]+):(\d+)$',domdisplay)
          if match.group(3):
              proto = match.group(1)
              port = match.group(3)
              for ifname in bind_interfaces:
                  apply_permit_rule(chain, port, ifname)

              apply_permit_rule(chain, port, "lo")
              apply_reject_rule(chain, port)


def apply_permit_rule(chain, dport = "5900", ininterface = "tun0"):
    rule = iptc.Rule()
    rule.protocol = "tcp"
    match = iptc.Match(rule, "tcp")
    match.dport = dport
    rule.in_interface = ininterface
    rule.target = iptc.Target(rule, "ACCEPT")
    rule.create_match('tcp')
    rule.add_match(match)
    chain.append_rule(rule)


def apply_reject_rule(chain, dport = "5900"):
    rule = iptc.Rule()
    rule.protocol = "tcp"
    match = iptc.Match(rule, "tcp")
    match.dport = dport
    rule.create_match('tcp')
    rule.add_match(match)
    rule.target = iptc.Target(rule, "DROP")
    chain.append_rule(rule)

def add_chain_rule(chain):
    rule = iptc.Rule()
    rule.target = rule.create_target(SPICE_CHAIN)
    chain.insert_rule(rule)

def remove_chain_rule(chain):
    for rule in chain.rules:
        if rule.target and rule.target.name == SPICE_CHAIN:
            chain.delete_rule(rule)
            return


def get_bind_interfaces(  ):
    cfg_f = open(CFG_FILE, "r")
    cfg = json.load(cfg_f)
    if_names = []
    for interface in cfg['ucpe:config']['interfaces:interface']:
        if 'management' in interface and interface['management']:
            ifman = interface['name']
            if 'data' in  ifman:
                ifman = "br-%s"%ifman

            if_names.append( ifman )

    if cfg['ucpe:config']['kubernetes:kubernetes']['bind-interface']:
        ifbind = cfg['ucpe:config']['kubernetes:kubernetes']['bind-interface']
        if 'data' in ifbind:
            ifbind = "br-%s"%ifbind
        if  not ifbind in  if_names:
            if_names.append ( ifbind )

    return if_names




if __name__ == "__main__":
    #set logging

    if iptc.is_table_available(iptc.Table.FILTER):
        table = iptc.Table(iptc.Table.FILTER)
        if table.is_chain(SPICE_CHAIN):
            chain = iptc.Chain(table, SPICE_CHAIN)
        else:
            chain = table.create_chain(SPICE_CHAIN)

        input_chain = iptc.Chain(table, "INPUT")

        remove_chain_rule(input_chain)
        chain.flush()

        management_ifs = get_bind_interfaces()
        walk_vms_apply_rules(chain, management_ifs)

        add_chain_rule(input_chain)



    os.remove( INCRONFILE ) if os.path.exists ( INCRONFILE ) else None
    sys.exit( 0 )
