#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import sys
import subprocess
import re
import time
import signal
import logging
import default_gw_set as defgw
from systemd import journal
from oslo_utils import netutils
from platform import machine

def is_arm_arch():
   if 'aarch64' in machine():
       return True
   return False

OVPN_DEF_GW = '/tmp/ovpn_def_gw'
IFF_UP = 0x1
IFF_RUNNING = 0x40
DEF_METRIC = 150



WIRELESS_PING_TIMEOUT = 3
WIRELESS_PING_COUNT = 6
PING_TIMEOUT = 3

#workaround for NFV-12516 ping tests fail when large transfer happens
# if is_arm_arch():
#     PING_TIMEOUT = 3
# else:
#     PING_TIMEOUT = 1

PING_COUNT = 5
trace_hosts = 0
monitor_loop = True
run = True
ipv6 = False

traceStream = journal.stream(identifier='trace_host')

class RttException(Exception):
    pass

#
def _isIPv6(ipaddr):
    """Check the IP address version and return True in case of IPv6"""
    if ipaddr is not None and ipaddr != "":
        return netutils.is_valid_ipv6(ipaddr)

    return False


#
def check_default_metric(metric, dev_name):
    output = subprocess.check_output(['ip', 'route', 'list']).decode()
    for ln in output.splitlines():
        if 'default' in ln:
            if 'dev ' + dev_name in ln:
                continue
            count = 0
            fields = ln.split(' ')
            for f in fields:
                if 'metric' in f:
                    if int(fields[count + 1]) == int(metric):
                        return 0
                    else:
                        break

                count += 1

    return 1


#
def get_def_gw():
    min_metric = None
    ifname = None
    iproutes = subprocess.check_output(['ip', 'route', 'list', 'exact', '0.0.0.0/0']).decode()

    for route in iproutes.splitlines():
        route = route.replace(" proto unspec", "")
        rt_line = route.split(' ')
        rt_line = [x for x in rt_line if x != '']
        # default via 192.168.8.1 dev lte0 metric 100
        if not min_metric:
            min_metric = int(rt_line[6])
            ifname = str(rt_line[4])
        elif int(rt_line[6]) < min_metric:
            min_metric = int(rt_line[6])
            ifname = str(rt_line[4])

    return ifname


#
def ovpn_client_restart():
    try:
        print('Restarting OpenVPN client...', file=traceStream)
        subprocess.check_call(['systemctl', 'restart', 'openvpn@client.service'])
    except subprocess.CalledProcessError as e:
        print('Command \'%s\' failed with code%d' % (e.cmd, e.returncode), file=traceStream)
    except OSError as e:
        print('System error - \'%s\'' % e.strerror, file=traceStream)


#
def ovpn_check_gw():
    if not os.path.exists(OVPN_DEF_GW):
        return

    ovpn_file = open(OVPN_DEF_GW, 'r')
    ovpn_gw = ovpn_file.read()
    ovpn_file.close()
    if not ovpn_gw:
        print('Failed to get ovpn config !', file=traceStream)
        return

    ifname = get_def_gw()
    if not ifname:
        print('Failed to get ip routes !', file=traceStream)
        return

    if ovpn_gw != ifname:
        ovpn_client_restart()


#
def trace_modify_route(ifname, metric, ipv6, is_vpp, is_nano, prop_vnics, worst=0):
    if is_vpp == True:
        UPDOWNTOOL = '/scripts/eth_up_down_vpplauncher.sh'
    else:
        UPDOWNTOOL = '/scripts/eth_up_down_launcher.sh'

    if 255 == metric or 1024 == metric:
        while (metric > 0):
            if 1 == check_default_metric(metric, ifname):
                break
            metric -= 1
    if ipv6:
        output = subprocess.check_output(['ip', '-6', 'route', 'list']).decode()
        worst = 1024
    else:
        output = subprocess.check_output(['ip', 'route', 'list']).decode()
        worst = 255
    for ln in output.splitlines():
        if ifname in str(ln) and 'default' in str(ln):
            fields = str(ln).split(' ')
            count = 0
            current_metric = 0
            for f in fields:
                if 'via' in f:
                    nexthop = fields[count + 1]
                if 'metric' in f:
                    current_metric = int(fields[count + 1])

                count += 1

            if metric != current_metric and current_metric <= worst:
                if ('default via %s dev %s metric %s' % (nexthop, ifname, metric)) not in output:
                    print('Going to change metric of %s from %s to %s,nexthop %s' % (
                        ifname, current_metric, metric, nexthop), file=traceStream)
                    try:
                        if ipv6:
                            subprocess.check_call(['ip', '-6', 'route', 'del', 'default', 'dev', str(ifname)])
                            defgw.del_def_gw(str(ifname), str(nexthop))
                            subprocess.check_call(
                                ['ip', '-6', 'route', 'add', 'default', 'via', str(nexthop), 'dev', str(ifname),
                                 'metric', str(metric)])
                            defgw.set_def_gw(str(ifname), str(nexthop), metric)
                        else:
                            subprocess.check_call(['ip', 'route', 'del', 'default', 'dev', str(ifname)])
                            defgw.del_def_gw(str(ifname), str(nexthop))
                            subprocess.check_call(
                                ['ip', 'route', 'add', 'default', 'via', str(nexthop), 'dev', str(ifname), 'metric',
                                 str(metric)])
                            defgw.set_def_gw(str(ifname), str(nexthop), metric)
                        if prop_vnics == True or prop_vnics == 'true' or prop_vnics == 'True':
                            if current_metric <= metric:
                                if is_nano:
                                    nano_state_propagation(str(ifname).strip('br-'), 'down')
                                else:
                                    subprocess.run([UPDOWNTOOL, str(ifname).strip('br-'), 'DOWN'])
                            else:
                                if is_nano:
                                    nano_state_propagation(str(ifname).strip('br-'), 'up')
                                else:
                                    subprocess.run([UPDOWNTOOL, str(ifname).strip('br-'), 'UP'])
                    except subprocess.CalledProcessError as e:
                        print('Command \'%s\' failed with code%d' % (e.cmd, e.returncode), file=traceStream)
                    except OSError as e:
                        print('System error - \'%s\'' % e.strerror, file=traceStream)

    ovpn_check_gw()


#
def disable_rp_filter(ifname, disable=True):
    # current scheme for rp_filter: 
    # config/all/rp_filter=0
    # config/dataX/rp_filter=2
    # config/br-dataX/rp_filter=0

    if not disable:
        return
        
    rp_value = 2 if disable else 1
    if "br-" in ifname and disable:
        rp_value = 0
    FNULL = open(os.devnull, 'w')
    try:
        sysctl_param = 'net.ipv4.conf.%s.rp_filter=%d' % (str(ifname).replace('.', '/'), rp_value)
        subprocess.check_call(['sysctl', sysctl_param], stdout=FNULL, stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as e:
        print('disable_rp_filter - FAIL', file=traceStream)
        print('%s returns %d' % (e.cmd, e.returncode), file=traceStream)

    FNULL.close()


#
def trace_host(ifname, host, ipv6, max_rtt):
    count = 0
    upcount = 0
    ret = 0
    disable_rp_filter(ifname, True)
    FNULL = open(os.devnull, 'w')
    ping_timeout = PING_TIMEOUT
    ping_count = PING_COUNT
    if "lte" in ifname or "wifi" in ifname:
        ping_timeout = WIRELESS_PING_TIMEOUT
        ping_count = WIRELESS_PING_COUNT

    while (count < ping_count):
        try:
            if ipv6:
                if int(max_rtt) > 0:
                    output = subprocess.check_output(['nice', '-n-10', 'ping6', '-I', str(ifname), str(host), '-c', '1', '-W', str(ping_timeout)]).decode()
                    rtt = float((re.search('time=(.*) ms', output)).group(1))
                    if (int(rtt) >= int(max_rtt)):
                        raise RttException('PING to %s via %s rtt %d - exeeded max RTT %d' % (str(host), str(ifname), int(rtt), int(max_rtt)))
                else:
                    subprocess.check_call(['nice', '-n-10', 'ping6', '-I', str(ifname), str(host), '-c', '1', '-W', str(ping_timeout)],
                                        stdout=FNULL, stderr=subprocess.STDOUT)
            else:
                if int(max_rtt) > 0:
                    output = subprocess.check_output(['nice', '-n-10', 'ping', '-I', str(ifname), str(host), '-c', '1', '-W', str(ping_timeout)]).decode()
                    rtt = float((re.search('time=(.*) ms', output)).group(1))
                    if (int(rtt) >= int(max_rtt)):
                        raise RttException('PING to %s via %s rtt %d - exeeded max RTT %d' % (str(host), str(ifname), int(rtt), int(max_rtt)))
                else:
                    subprocess.check_call(['nice', '-n-10', 'ping', '-I', str(ifname), str(host), '-c', '1', '-W', str(ping_timeout)],
                                        stdout=FNULL, stderr=subprocess.STDOUT)

            upcount += 1
            if upcount >= ping_count:
                ret = 1
                break
            # ret = 1
            # break
        except RttException as rtt_ex:
            print(rtt_ex.args[0], file=traceStream)
            count += 1
        except:
            print('PING to %s via %s - FAIL' % (str(host), str(ifname)), file=traceStream)
            count += 1

    disable_rp_filter(ifname, False)
    FNULL.close()
    return ret


#
def check_if_state(ifname):
    try:
        flags = open('/sys/class/net/' + ifname + '/flags').readline()
        if 1 == (int(flags, 16) & (IFF_UP | IFF_RUNNING)):
            return True
    except:
        pass

    return False


#
def get_interface_vtag(interface):
    vlan = i.get_interface_param(interface, 'vlan')

    if 'int' == type(vlan).__name__:
        return vlan

    if vlan is None or vlan == '': return 0

    vtag = int(vlan)

    return vtag


#
def get_system_if_name(interface):
    vlan = get_interface_vtag(interface)
    if 0 == vlan or 'inband.' in interface:
        ifname = interface
    else:
        ifname = "%s.%d" % (interface, vlan)

    if 'data' in ifname or 'lte' in ifname or 'wifi' in ifname:
        ifname = "br-%s" % ifname

    return ifname


#
def sig_handler(signal, fname):
    global monitor_loop
    global run
    print('Signal is received. Terminating ...', file=traceStream)
    monitor_loop = False
    run = False


#
def sig_reinit(signal, frame):
    global monitor_loop
    print('Signal sigusr1 is received...', file=traceStream)
    monitor_loop = False

#
def nano_state_propagation(ifname, state):
    clav_if = ''
    clav_path=None
    
    if os.path.exists('/root/apps/clavister/clav_ifs_setup'):
        clav_path = '/root/apps/clavister/clav_ifs_setup'
    elif os.path.exists('/root/apps/clavistervnf/clav_ifs_setup'):
        clav_path = '/root/apps/clavistervnf/clav_ifs_setup'
    
    if clav_path:
        with open('/root/apps/clavister/clav_ifs_setup') as fl:
            for line in fl:
                if str(ifname) in line:
                    clav_if = str(line.split()[-1])
            fl.close()

    if clav_if:
        FNULL = open(os.devnull, 'w')
        subprocess.check_call(['ip', 'link', 'set', 'dev', str(clav_if), str(state)],
                                        stdout=FNULL, stderr=subprocess.STDOUT)
        FNULL.close()
        print('Change interface {} state to {}' .format(clav_if, state), file=traceStream)

def main_sr():
    global monitor_loop
    global run
    signal.signal(signal.SIGINT, sig_handler)
    signal.signal(signal.SIGTERM, sig_handler)
    signal.signal(signal.SIGUSR1, sig_reinit)
    is_vpp = True if i.get_network_mode() == 'vpp' else False
    is_nano = True if os.path.isfile("/boot/armbianEnv.txt") else False

    while run:
        trace_hosts = 0
        i.session.refresh()
        interfaces = i.get_interfaces()

        for interface in interfaces:
            if interfaces[interface]['type'] != 'ethernet' and interfaces[interface]['type'] != 'lte' and \
               interfaces[interface]['type'] != 'wifi' and interfaces[interface]['type'] != 'pppoe':
                continue
            if 'trace-host' not in interfaces[interface]:
                continue
            if i.get_interface_param(interface, 'trace-host'):
                print('Going to trace %s via %s' % (i.get_interface_param(interface, 'trace-host'), interface),
                      file=traceStream)
                trace_hosts += 1

        if not trace_hosts:
            time.sleep(5)
            continue

        #os.system("for i in /proc/sys/net/ipv4/conf/*/rp_filter ; do echo 1 > $i ; done")
        monitor_loop = True
        while monitor_loop:
            for interface in interfaces:
                if interfaces[interface]['type'] != 'ethernet' and interfaces[interface]['type'] != 'lte' and \
                   interfaces[interface]['type'] != 'wifi' and interfaces[interface]['type'] != 'pppoe':
                    continue
                if not monitor_loop:
                    break
                if 'trace-host' not in interfaces[interface]:
                    continue
                t_host = i.get_interface_param(interface, 'trace-host')

                if 'tracehost' in interfaces[interface]:
                    if 'max-rtt' in interfaces[interface].get('tracehost', ''):
                        max_rtt = i.get_container_param(interface, 'tracehost', 'max-rtt')
                        if max_rtt == None or max_rtt == '':
                            max_rtt = 0
                    else:
                        max_rtt = 0

                    if 'trace-state-propagation' in interfaces[interface].get('tracehost', ''):
                        prop_vnics = i.get_container_param(interface, 'tracehost', 'trace-state-propagation', False)
                    else:
                        prop_vnics = False

                if t_host:
                    if interfaces[interface].get('vlan', 0):
                        vlan = get_interface_vtag(interface)
                    else:
                        vlan = 0
                    if 0 == vlan or 'inband.' in i.get_interface_param(interface, 'name'):
                        ifname = i.get_interface_param(interface, 'name')
                    else:
                        ifname = "%s.%d" % (i.get_interface_param(interface, 'name'), vlan)

                    if 'data' in ifname or 'lte' in ifname or 'wifi' in ifname:
                        ifname = "br-%s" % ifname

                    if _isIPv6(t_host):
                        ipv6 = True
                    else:
                        ipv6 = False

                    ifmode = i.get_interface_param(interface, 'mode') if 'mode' in interfaces[interface] else None
                    if ifmode and 'dpdk' in ifmode:
                        state = True
                    else:
                        state = check_if_state(i.get_interface_param(interface, 'name'))

                    if state:
                        if 1 == trace_host(ifname, t_host, ipv6, max_rtt):
                            ifmetric = i.get_interface_param(interface, 'metric')
                            if ifmetric:
                                metric = int(ifmetric)
                            else:
                                metric = 0

                            trace_modify_route(ifname, metric, ipv6, is_vpp, is_nano,prop_vnics)

                        else:
                            trace_modify_route(ifname, 255, ipv6, is_vpp, is_nano, prop_vnics, 255 - trace_hosts)

            time.sleep(1)

    # Cleanup
    print('Cleanup', file=traceStream)

    i.session.refresh()
    interfaces = i.get_interfaces()

    for interface in interfaces:
        if interfaces[interface]['type'] != 'ethernet' and interfaces[interface]['type'] != 'lte' and \
           interfaces[interface]['type'] != 'wifi' and interfaces[interface]['type'] != 'pppoe':
            continue

        cfgmetric = i.get_interface_param(interface, 'metric')
        t_host = i.get_interface_param(interface, 'trace-host')
        if 'trace-state-propagation' in interfaces[interface]:
            prop_vnics = i.get_container_param(interface, 'tracehost', 'trace-state-propagation', False)
        else:
            prop_vnics = False

        if cfgmetric is None:
            ifmetric = 0
        else:
            ifmetric = int(cfgmetric)

        if _isIPv6(t_host):
            ipv6 = True
        else:
            ipv6 = False

        ifname = get_system_if_name(i.get_interface_param(interface, 'name'))
        trace_modify_route(ifname, ifmetric, ipv6, is_vpp, is_nano, prop_vnics)

    sys.exit(0)


if __name__ == '__main__':
    from sysrepo_interfaces import sysrepo_interfaces3 as interfaces_api

    i = interfaces_api.SysrepoInterfaces(app_name="tracehost interfaces", log_level=logging.ERROR)
    main_sr()
