#!/usr/bin/env python3

import json
import string


def verify_serial_number(number):
    from binascii import unhexlify

    timestamp = number[0:8]
    hostname = number[8:40]
    random = number[40:48]
    batch_num = number[48:52]

    print(int(timestamp, 16))
    print(unhexlify(hostname).decode('utf-8'))
    print(int(random, 16))
    print(int(batch_num, 16))


# verify_serial_number('0x0c109a004445534b544f502d4c5535304e39360020e6260009')

def get_public_key():
    from OpenSSL import crypto
    with open("/etc/license/public_key.pem", "r") as myfile:
        return crypto.load_publickey(crypto.FILETYPE_PEM, myfile.read())


def new_verify_licenses(serial_num=None, license_pack="/config/license/license_pack.lic"):
    try:
        with open(license_pack, "r") as f:
            data = f.read()

        json_licenses = json.loads(data)
    except:
        return "Could not load license pack. Please make sure you have downloaded a correct license."

    # print('1 -- {}'.format(json_licenses['license_pack:pack']))
    # print('2 -- {}'.format(len(json_licenses['license_pack:pack'])))
    # print('3 -- {}'.format(json_licenses['license_pack:pack'][0]))
    # print('4 -- {}'.format(len(json_licenses['license_pack:pack'][0]['ucpe:license']['features'])))
    # print('5 -- {}'.format(json_licenses['license_pack:pack'][0]['ucpe:license']['features'][0]))
    # print('6 -- {}'.format(str(json_licenses['license_pack:pack'][0]['ucpe:license']['serial'])))
    # print('7 -- {}'.format(type(json_licenses['license_pack:pack'][0]['ucpe:license']['serial'])))
    # print('8 -- {}'.format(len(json_licenses['license_pack:pack'][0]['ucpe:license']['serial'])))
    # print('9 -- {}'.format(all(c in string.hexdigits for c in json_licenses['license_pack:pack'][0]['ucpe:license']['serial'][2:])))
    # for c in json_licenses['license_pack:pack'][0]['ucpe:license']['serial']:
    #     if c not in string.hexdigits:
    #         print('10 --{}'.format(c))

    if not 'license_pack:pack' in json_licenses \
       or not 1 == len(json_licenses['license_pack:pack']) \
       or not 'ucpe:license' in json_licenses['license_pack:pack'][0] \
       or not 'serial' in json_licenses['license_pack:pack'][0]['ucpe:license'] \
       or not 'features' in json_licenses['license_pack:pack'][0]['ucpe:license'] \
       or not 'expiration' in json_licenses['license_pack:pack'][0]['ucpe:license'] \
       or not 'bandwidth' in json_licenses['license_pack:pack'][0]['ucpe:license'] \
       or not 1 == len(json_licenses['license_pack:pack'][0]['ucpe:license']['features']) \
       or not 'vnf' == json_licenses['license_pack:pack'][0]['ucpe:license']['features'][0] \
       or not 48 == len(json_licenses['license_pack:pack'][0]['ucpe:license']['serial']) \
       or not all(c in string.hexdigits for c in json_licenses['license_pack:pack'][0]['ucpe:license']['serial'][2:]):
        return "bad Could not validate license pack."

    if not serial_num:
        return "License pack is valid."
    else:
        license_list = list(json_licenses.values())
        for i in range(0, len(license_list[0])):
            if serial_num == license_list[0][i]['ucpe:license']['serial']:
                print("Valid serial number.")
                return True

        print("Invalid serial number. Please enter a correct license number.")


if __name__ == '__main__':
    import sys

    if 2 == len(sys.argv):
        license_serial_num = sys.argv[1]
        print(new_verify_licenses(serial_num=license_serial_num))
    elif 3 == len(sys.argv):
        license_pack = sys.argv[2]
        print(new_verify_licenses(license_pack=license_pack))
    else:
        print('Unknown arguments')
