#!/usr/bin/env python3

import subprocess, os

if_name = "data0"
if_badname = "eth0"
rule_file = "/etc/udev/rules.d/70-persistent-net.rules"

status, output = subprocess.getstatusoutput("ifconfig " + if_name)
if status == 0:
    print ("No problems detected with " + if_name + " interface.")
    exit(0)

if not os.path.isfile(rule_file):
    print ("File " + rule_file + " does not exists.")
    exit(0)


if not os.path.isfile("/sys/class/net/%s/device/uevent" % if_badname):
    print ("Can't find interface")
    exit (0)

status, output = subprocess.getstatusoutput("ethtool -i %s | grep bus | cut -f 2 -d' '" % if_badname)
if status != 0:
    exit(0)


pciid = output.strip().split(" ")

if not pciid:
    print ("ERROR: Can't read PCIID of nic!")
    exit(0)

status, output = subprocess.getstatusoutput("cat " + rule_file)
if status != 0:
    print ("ERROR: VM NIC update failed!")
    print ('{}'.format(output))
    exit(0)

is_file_updated = False
new_rules = []
for line in output.split("\n"):
    if line.find("NAME=\"" + if_name + "\"") < 0:
        new_rules.append(line)
        continue
    start_pos = line.find("KERNELS==\"")
    if start_pos < 0:
        print ("ERROR: Can't find the KERNELS parameter in " + rule_file +  " for interface " + if_name + "!")
        print (line)
        exit(0)

    end_pos = line.find("\"",start_pos + len("KERNELS==\""))
    str_to_replace = line[start_pos:end_pos+1]
    #Check if the update is required
    if str_to_replace == "KERNELS==\"" + pciid[0] + "\"":
        new_rules.append(line)
        continue
    is_file_updated = True
    new_rules.append(line.replace(str_to_replace, "KERNELS==\"" + pciid[0] + "\""))

if is_file_updated:
    try:
        with open(rule_file, "w") as text_file:
            text_file.write("\n".join(new_rules))

        text_file.close()
        status, output = subprocess.getstatusoutput("udevadm control --reload")
        status, output = subprocess.getstatusoutput("udevadm trigger --action=add")

    except Exception as e:
        print ("ERROR: Can't update " + rule_file + " file!")
        print (e)
        exit(0)
else:
    print ("Nothing to update in " + rule_file)

exit(0)
