import subprocess
import json
import pexpect
from shlex import quote
import traceback
import time
import os
import logging

IMPORT_SCRIPT="/root/import_config.py"
HOME_DIR = "/home/admin/user_files/"
DEFAULT_TIMEOUT=600

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s [%(levelname)s] - %(message)s',
    handlers=[
        logging.StreamHandler()
    ]
)

def execute_command(command):
    try:
        return subprocess.check_output(command, shell=True)
    except subprocess.CalledProcessError as e:
        logging.error(f"Command execution failed: {e}")
        return None

def get_k8s_metadata(namespace, vm_name):
    command = f"kubectl get vm {vm_name} -n {namespace} -o json"
    output = execute_command(command)
    if output is None:
        return None

    try:
        output_dict = json.loads(output)  # parse the JSON string into a dictionary
        annotations = output_dict.get('metadata', {}).get('annotations', {})
        return annotations
    except ValueError as e:
        logging.error(f"Error parsing JSON: {e}")
        return None

def download_config_pfsense(repo, vm_name, password=None, user=None, directory=None, down_file=None):
    logging.info("ZTP script started")
    time.sleep(240)

    try:
        child = pexpect.spawn(f'virtctl console {quote(vm_name)}', timeout=200, maxread=400, encoding='utf-8')

        child.expect("The escape sequence", timeout=10)

        child.sendline("8\r")
        child.expect("root")
        child.sendline("")

        time.sleep(3)

        if None not in (password, user, directory, down_file):
            child.sendline(f"/usr/local/bin/python3.8 {quote(IMPORT_SCRIPT)} {quote(repo)} {quote(password)} {quote(user)} {quote(directory)} {quote(down_file)}")
        else:
            child.sendline(f"/usr/local/bin/python3.8 {quote(IMPORT_SCRIPT)} {quote(repo)}")

        time.sleep(10)
        child.sendline("")
        child.sendcontrol("]")
        child.close()

        return 0
    except Exception as e:
        logging.error(f"Traceback error {e} {traceback.format_exc()}")
        return 1

def execute_script(vm_name, filename, serial_number):
    logging.info("ZTP script started")
    time.sleep(60) # To make sure the VM boots
    expect_cmd_str=f"export HW_SERIAL={serial_number} && /usr/bin/timeout {DEFAULT_TIMEOUT} /usr/bin/expect -c \"spawn virtctl console {vm_name} \" {HOME_DIR}{filename}  2>&1 >/var/tmp/{vm_name}.log"

    if not os.path.isfile(HOME_DIR + filename):
        logging.error(f"VNF Script error! Can't find script {filename}")
        return 1

    if ".exp" in filename:
        result = subprocess.run(expect_cmd_str, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if result.returncode != 0:
            logging.error(f"VNF Script error! Non-zero return code: {result.returncode}")
            return 1
        return 0
    else:
        logging.error(f"VNF Script error! Unknown format of script {filename}")
        return 1

def main(namespace, vm_name):
    meta = get_k8s_metadata(namespace, vm_name)
    serial_number = None

    try:
        with open("/config/.serial-number", 'r') as serial_file:
            serial_number = serial_file.read().strip()
    except Exception as e:
        serial_number="unknown"
        logging.error(f"Couldn't read serial number: {e}")

    if meta:
        if "pfsense-onboot-import-config-once" in meta:
            pfsense_repo=meta["pfsense-onboot-import-config-once"]
            config_url = meta.get('pfsense-onboot-import-config-once')

            if "%%" in config_url:
                meta_split = config_url.split("%%")
                if len(meta_split) == 5 and "IMPORTFROMSERIAL" in config_url:
                    meta_split[0] = meta_split[0].replace("IMPORTFROMSERIAL", serial_number)
                    file_config = meta_split[4].split('.')
                    file_config[0] += serial_number
                    meta_split[4] = ''.join(file_config)

                ret = download_config_pfsense(*meta_split[:5], vm_name)
            else:
                if "IMPORTFROMSERIAL" in config_url:
                    pfsense_repo = config_url.replace("IMPORTFROMSERIAL", serial_number).strip()
                ret = download_config_pfsense(pfsense_repo, vm_name)

            if ret != 0:
                logging.error(f"VNF Script Error, return code {ret}")
            else:
                logging.info("pfSense import config engaged. Please check pfSense log file for final results located in /var/log/import_config.log")

        for key in ['vnf-onboot-script-once', 'vnf-onboot-script']:
            if key in meta:
                filename = meta[key]
                ret = execute_script(vm_name, filename, serial_number)
                if ret != 0:
                    logging.error(f"VNF Script Error, return code {ret}")
                else:
                    logging.info("VNF Script executed successfully.")
