#!  /usr/bin/env python3
import os
import subprocess
import sys
from pydbus import SystemBus
from systemd import journal
import vppapi_if.vppapi_if as vppif

vnf_port_updown = journal.stream(identifier='vnf_port_updown')

def is_vpp_masked():
    try:
        state = subprocess.check_output(['systemctl', 'is-enabled', 'vpp-config.service' ])
        if state.decode().rstrip() == 'masked':
            return True
    except subprocess.CalledProcessError as e:
        if e.output.decode().rstrip() == 'masked':
            return True

    return False

def get_file(filename, mode, param, value=None):
    try:
        with open(filename, mode) as f:
            if mode is 'r':
                lines=f.readlines()
                for s in lines:
                    if param in s:
                        value = s.split('\'')[3]
                        return value
            elif (mode is 'a' or mode is 'w') and value is not None:
                f.write("\'{}\':\'{}\'\n".format(param, value))
    except Exception as e:
        print("Failed to operate with file : {}".format(e), file=vnf_port_updown)
        return

def add_telco_port_to_vm_mapping(dev_name, ifuuid):
    global filename_backup
    vif = vppif.VppApiInterface('qemu-hook')
    tap_int = vif.get_tap_interface(dev_name)
    telco_interface_name = tap_int['host_if_name']
    telco_ip = subprocess.check_output("/scripts/get_port_address.sh {} || true".format(ifuuid), shell=True).decode()

    if 'nat' in telco_interface_name and telco_ip != '' and 'No Port' not in telco_ip:
        try:
            telco_bus = SystemBus()
            telco_object = telco_bus.get("net.telco.NatIfChange")
            get_file(filename_backup, 'w', 'telco_interface_name', telco_interface_name)
            get_file(filename_backup, 'a', 'telco_ip', telco_ip)
            print("Added rule in iptables for {}".format(telco_interface_name), file=vnf_port_updown)
            telco_object.nat_add_addr(telco_interface_name, telco_ip)
        except Exception as e:
            print("fail {}".format(e), file=vnf_port_updown)
            return
    else:
        print("Unable to get VNF fixed_ips", file=vnf_port_updown)

def rm_telco_port_to_vm_mapping(ifuuid):
    global filename_backup

    try:
        telco_bus = SystemBus()
        telco_object = telco_bus.get("net.telco.NatIfChange")
        telco_interface_name = get_file(filename_backup, 'r', 'telco_interface_name')
        telco_ip = get_file(filename_backup, 'r', 'telco_ip')
        print("Removed rule in iptables for {}".format(telco_interface_name), file=vnf_port_updown)
        telco_object.nat_del_addr(telco_interface_name, telco_ip)
        os.remove(filename_backup)
    except Exception as e:
        print("fail {}".format(e), file=vnf_port_updown)
        return

def main():
    global filename_backup
    if 3 > len(sys.argv):
        print("No enough arguments!", file=vnf_port_updown)

    is_vpp = not is_vpp_masked()
    dev_name = ''

    if is_vpp and sys.argv[2].endswith('.vnic'):
        filename = sys.argv[1] + sys.argv[2]
        filename_backup = '/tmp/' + sys.argv[2]
        ifuuid = filename.split('.')[0].split('/')[2]
        if os.path.exists(filename):
            dev_name = get_file(filename, 'r', 'port_name')

        if 'IN_CREATE' in sys.argv[3] and dev_name != '':
            add_telco_port_to_vm_mapping(dev_name, ifuuid)
        elif 'IN_DELETE' in sys.argv[3]:
            rm_telco_port_to_vm_mapping(ifuuid)
        else:
            pass

if __name__ == '__main__':
    try:
        main()
    except Exception as e:
        print("Exception in main: {}".format(e), file=vnf_port_updown)
        pass