#!  /usr/bin/env python3
import os
import sys
from systemd import journal
from configobj import ConfigObj
from keystoneauth1 import session
from keystoneauth1.identity import v3
from novaclient import client as novaclient

vnf_state_log = journal.stream(identifier='vnf_state_handler')
home_dir = "/home/admin/user_files/"
default_timeout=600


def execute_script(uuid, filename, serial_number):
    expect_cmd_str="export HW_SERIAL=%s && /usr/bin/timeout %s /usr/bin/expect -c \"spawn virsh console %s --force\" %s%s  2>&1 >/var/tmp/%s.log" % (serial_number, default_timeout, uuid, home_dir, filename,uuid)
    if not os.path.isfile(home_dir+filename) : 
        print("VNF Script error! Can't find script %s" % (filename) , file=vnf_state_log)
        return 1
    if ".exp" in filename:
        #we are executing expect
        ret = os.system(expect_cmd_str)
        os.system("cat /var/tmp/%s.log | systemd-cat -p info -t vnf_state_handler; rm -f  /var/tmp/%s.log" % (uuid, uuid))
        return ret
    else:
        print("VNF Script error! Unknown format of script %s" % (filename) , file=vnf_state_log)
        return 1

def get_nova_metadata(uuid):
    try:
        adminrc = ConfigObj("/root/admin-openrc.env")
        v3_auth = v3.Password(auth_url=adminrc['OS_AUTH_URL'],
                                   username=adminrc['OS_USERNAME'],
                                   password=adminrc['OS_PASSWORD'],
                                   project_name=adminrc['OS_PROJECT_NAME'],
                                   project_domain_name=adminrc['OS_PROJECT_DOMAIN_NAME'],
                                   user_domain_name=adminrc['OS_USER_DOMAIN_NAME'])
        sess = session.Session(auth=v3_auth, verify=False)
        nova = novaclient.Client(version='2.0', session=sess)
        instance = nova.servers.get(uuid)
        jsondata = instance.metadata
        return jsondata
    except Exception as e:
        #print("Cant find instance %s metadata %s"% (uuid, e) , file=vnf_state_log)
        return None



def main():
    if 2 > len(sys.argv):
        print("No enough arguments!", file=vnf_state_log)
    vm_id = sys.argv[1]
    
    
    try:
        serial_file = open("/config/.serial-number", 'r')
        serial_number = serial_file.read().strip()
        serial_file.close()
    except:
        serial_number="unknown"


    try:
        with open('/config/vnf/event/' + vm_id) as f:
            event = f.readlines()
            event = " ".join(event)
        os.remove('/config/vnf/event/' + vm_id)
    except:
        event = "Unknown"
    
    if "started" in event:
        #VM was started
        meta = get_nova_metadata(vm_id)
        if meta and 'vnf-onboot-script-once' in meta:
            filename=meta['vnf-onboot-script-once']
            if not os.path.isfile("/config/vnf/run_"+vm_id) : 
                os.system("touch %s" % ("/config/vnf/run_"+vm_id))
                ret = execute_script(vm_id, filename, serial_number)
                if ret != 0:
                    print("VNF Script Error, return code %s."% (ret) , file=vnf_state_log)
                    os.remove("/config/vnf/run_"+vm_id)
                else:
                    print("VNF Script executed successfully."  , file=vnf_state_log)

        if meta and "vnf-onboot-script" in meta:
            filename=meta['vnf-onboot-script']
            ret = execute_script(vm_id, filename, serial_number)
            if ret != 0:
                print("VNF Script Error, return code %s."% (ret) , file=vnf_state_log)
            else:
                print("VNF Script executed successfully."  , file=vnf_state_log)
        
    elif "release" in event:
        #VM was deleted
        #if os.path.isfile("/config/vnf/run_"+vm_id):
        #    os.remove("/config/vnf/run_"+vm_id)
        pass

if __name__ == '__main__':
    try:
        main()
    except Exception as e:
        pass
