#! /usr/bin/env python3
import os
import sys
# import argparse
# import subprocess
import vppapi_if.vppapi_if as vpp_if
from tveif import if_lte
import sysrepo_interfaces.sysrepo_interfaces3 as sri

LTE_OFFSET=100

def get_if_mac(ifname):
    if not os.path.exists('/sys/class/net/' + ifname + '/address'):
        return None

    mac = ""
    with open('/sys/class/net/' + ifname + '/address', 'r') as f:
        mac = f.readline()[0:17]
    return mac

def del_vpp_lte(vppif, if_name):
    
    tapif = vppif.get_tap_interface_by_host_ifname('br-{}' .format(if_name))
    if not tapif:
        return
    
    bd_id = vppif.get_interface_bd_id(tapif['sw_if_index'])
    loop_idx = vppif.get_bd_bvi_by_bdidx(bd_id)
    
    # cleanup drctl
    vppif.disable_drctl(tapif['sw_if_index'], tapif['host_mac_addr'])
    
    # del loop / tap / host-lte / bd
    vppif.delete_loopback_interface(loop_idx)
    vppif.delete_tap_interface(tapif['sw_if_index'])
    vppif.delete_host_interface(if_name)
    vppif.delete_bridge_domain(bd_id)

def set_vpp_lte(vppif, if_name):
    if not os.path.exists('/sys/class/net/{}'.format(if_name)):
        print('Interface {}, do not exist. Skip LTE reinit!'.format(if_name))
        return

    if if_lte.lte_check_raw_if(if_name):
        if_mac = if_lte.lte_get_raw_mac(if_name)
    else:
        if_mac = get_if_mac(if_name)
    
    
    tap_id = int(if_name.replace('lte',''))

    (ret, bridx)=vppif.create_tap_interface('{}{}'.format('br-',if_name), id=tap_id+LTE_OFFSET, host_mac_addr=if_mac)
    if 0 == ret:
        vppif.ifup(bridx)

    ret = vppif.create_bridge_domain(bridx)
    
    
    vppif.add_to_bridge(bridx,bridx)

    (ret, ifid) = vppif.create_host_interface(if_name)
    if ret != 0:
        print('Failed to attach {} interface in VPP!' .format(if_name))
        return

    # set polling rx mode
    ret = vppif.set_interface_rx_mode(idx=ifid, mode='polling')

    vppif.add_to_bridge(bridx,ifid)
    vppif.ifup(ifid)

    (retval, loop_idx) = vppif.create_loopback_interface(mac_addr=if_mac)
    
    #raw ip 
    if if_lte.lte_check_raw_if(if_name):
        if_mac = if_lte.lte_get_raw_mac(if_name)
        vppif.enable_rawip( idx=ifid, dst_mac=if_mac )
    
    vppif.add_to_bridge(bridx,loop_idx, port_type=vpp_if.L2_API_PORT_TYPE_BVI)
    vppif.set_interface_learning(bridx, disable=1)
    vppif.enable_drctl(bridx, mac=if_mac)
    vppif.drctl_set_arp(bridx, loop_idx)
    vppif.drctl_set_dhcp(bridx, loop_idx)
    vppif.ifup(loop_idx)

def lte_reinit(if_name):
    # lte_id = int(if_name.replace('lte',''))
    
    # vppif.delete_host_interface(if_name)
    del_vpp_lte(vppif, if_name)

    # hostifname = 'tap{}'.format(LTE_OFFSET+lte_id)
    # bridx = vppif.get_ifidx_by_name(hostifname)

    # (ret, ifid) = vppif.create_host_interface(if_name)
    # ret = vppif.set_interface_rx_mode(idx=ifid, mode='polling')
    # vppif.add_to_bridge(bridx,ifid)
    # vppif.ifup(ifid)
    set_vpp_lte(vppif, if_name)

def lte0_reinit(if_name):
    arp_term = False
    lte_id = int(if_name.replace('lte',''))
    
    #this call may fail, as the interface may already be deleted.
    try:
        vppif.delete_host_interface(if_name)
    except:
        pass
    hostifname = 'tap{}'.format(LTE_OFFSET+lte_id)
    bridx = vppif.get_ifidx_by_name(hostifname)

    (ret, ifid) = vppif.create_host_interface(if_name)
    ret = vppif.set_interface_rx_mode(idx=ifid, mode='polling')

    if if_lte.lte_check_raw_if(if_name):
        print("raw ip mode")
        if_mac = if_lte.lte_get_raw_mac(if_name)
        vppif.enable_rawip( idx=ifid, dst_mac=if_mac )
        arp_term=True

    vppif.add_to_bridge(bridx,ifid)


    inft_cfg = sri.SysrepoInterfaces(app_name='vpp-lte-ippass-check')
    if inft_cfg.get_interface_param(if_name, 'ip-passthrough'):
        domains = vppif.get_bridge_domains(bd_id=bridx)
        for bd in domains:
            intf_idx = [index['sw_if_index'] for index in bd['sw_if_details']]
            if ifid in intf_idx:
                intf_idx.remove(ifid)
                host_id = intf_idx[1]
                loop_id = intf_idx[0]
                try:
                    vppif.disable_ippass(host_idx=host_id, wan_idx=ifid, rt_idx=loop_id)
                except:
                    pass
                vppif.enable_ippass(host_idx=host_id, wan_idx=ifid, rt_idx=loop_id)
                if arp_term == True:
                    vppif.enable_ippass_arp_termination(host_idx=host_id)
                vppif.add_del_ippass_permit_host_entry(host_idx=host_id, protocol='icmp')
                vppif.add_del_ippass_permit_host_entry(host_id, 'dhcp')
                vppif.add_del_ippass_permit_host_entry(host_idx=host_id, protocol='udp', dst_port=53)
                vppif.add_del_ippass_permit_host_entry(host_idx=host_id, protocol='udp', dst_port=123)

    vppif.ifup(ifid)

def check_vpp_mode():

    if os.path.isfile('/run/vpp.pid'):
        return True
    return False



if __name__ == "__main__":

    if not check_vpp_mode():
        sys.exit(0)
    
    if len(sys.argv)==2:
        if_name = sys.argv[1]
    else:
        if_name = "lte0"

    vppif = vpp_if.VppApiInterface('LTE handler')
    if "lte0" == if_name:
        lte0_reinit(if_name)
    else:
        lte_reinit(if_name)
    del(vppif)

    sys.exit(0)
