#! /usr/bin/env python3
import os
import sys
import vppapi_if.vppapi_if as vpp_if
import sysrepo_interfaces.sysrepo_interfaces3 as sri
WIFI_OFFSET=400


def wifi_reinit():
    ifname = 'wifi0'
    tapid=0
    try:
        vppif.delete_host_interface(ifname)
    except:
        #this call may fail, and its normal.
        pass

    hostifname = 'tap{}'.format(WIFI_OFFSET+tapid)
    bridx = vppif.get_ifidx_by_name(hostifname)

    (ret, ifid) = vppif.create_host_interface(ifname)
    ret = vppif.set_interface_rx_mode(idx=ifid, mode='polling')
    vppif.add_to_bridge(bridx,ifid)

    inft_cfg = sri.SysrepoInterfaces(app_name='vpp-lte-ippass-check')
    if inft_cfg.get_interface_param(ifname, 'ip-passthrough'):
        domains = vppif.get_bridge_domains(bd_id=bridx)
        for bd in domains:
            intf_idx = [index['sw_if_index'] for index in bd['sw_if_details']]
            if ifid in intf_idx:
                intf_idx.remove(ifid)
                host_id = intf_idx[1]
                loop_id = intf_idx[0]
                vppif.disable_ippass(host_idx=host_id, wan_idx=ifid, rt_idx=loop_id)
                vppif.enable_ippass(host_idx=host_id, wan_idx=ifid, rt_idx=loop_id)
                vppif.add_del_ippass_permit_host_entry(host_idx=host_id, protocol='icmp')
                vppif.add_del_ippass_permit_host_entry(host_id, 'dhcp')
                vppif.add_del_ippass_permit_host_entry(host_idx=host_id, protocol='udp', dst_port=53)
                vppif.add_del_ippass_permit_host_entry(host_idx=host_id, protocol='udp', dst_port=123)

    vppif.ifup(ifid)

def check_vpp_mode():

    if os.path.isfile('/run/vpp.pid'):
        return True
    return False



if __name__ == "__main__":

    if not check_vpp_mode():
        sys.exit(0)

    vppif = vpp_if.VppApiInterface('Wifi WPA handler')
    wifi_reinit()

    sys.exit(0)
