#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import subprocess
import time
import string
import os
from binascii import hexlify
from systemd import journal
import systemd.daemon
import filecmp
import shutil

wpa_tmpfile = '/tmp/wpa_supplicant.tmp'
waStream = journal.stream(identifier='wifi-auto')
with open('/etc/wpa_supplicant/wpa_supplicant.conf', 'w') as f:
    print('ctrl_interface=/var/run/wpa_supplicant', file=f)
    print('network={', file=f)
    print('ssid="dummydummy"', file=f)
    print('key_mgmt=NONE', file=f)
    print('}', file=f)

systemd.daemon.notify('READY=1')
if not os.path.isdir('/sys/class/net/wifi0'):
    print('No wireless card !', file=waStream)
    exit(0)

try:
    while True:
        try:
            # sysrepocfg works even if sysrepod is down
            sysrepo = subprocess.run(['/usr/bin/sysrepocfg', '--datastore=running',
                                      '--format=xml',
                                      '--xpath=/ucpe:config/interfaces:interface[name=\"wifi0\"]//*',
                                      '--export'],
                                     stdout=subprocess.PIPE, encoding='UTF-8')
        except subprocess.CalledProcessError as cpe:
            print('ERROR {}'.format(cpe), file=waStream)
            pass
        except OSError:
            print('Missing /usr/bin/sysrepocfg', file=waStream)
            pass

        if 0 != sysrepo.returncode:
            print('FAIL get config from sysrepo !', file=waStream)
            time.sleep(2)
            continue

        is_auto = True if 0 <= sysrepo.stdout.find('<key-mgmt>AUTO</key-mgmt>') else False
        if not is_auto:
            print('wifi key-mgmt not AUTO !', file=waStream)
            time.sleep(2)
            continue

        is_client = True if 0 <= sysrepo.stdout.find('<wifi-mode>client</wifi-mode>') else False
        if not is_client:
            print('wifi not client !', file=waStream)
            time.sleep(10)
            continue

        is_enabled = True if 0 <= sysrepo.stdout.find('<enable>true</enable>') else False
        if not is_enabled:
            time.sleep(10)
            continue

        ssid_start = sysrepo.stdout.find('<ssid>')
        ssid_end = sysrepo.stdout.find('</ssid>')
        if 0 > ssid_start or 0 > ssid_end:
            print('FAIL parse ssid !', file=waStream)
            continue

        ssid = '{}'.format(sysrepo.stdout[ssid_start + len('<ssid>'):ssid_end])
        # print('searching for {}'.format(ssid), file=waStream)
        try:
            subprocess.run(['/sbin/iw', 'dev', 'wifi0', 'scan', 'sched_stop'],
                           stderr=subprocess.DEVNULL)
            subprocess.run(['/sbin/iw', 'dev', 'wifi0', 'scan', 'abort'],
                           stderr=subprocess.DEVNULL)
            scan = subprocess.run(['/sbin/iwlist', 'wifi0', 'scan'],
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.DEVNULL,
                                  encoding='UTF-8')
        except subprocess.CalledProcessError as cpe:
            print('ERROR {}'.format(cpe), file=waStream)
            pass
        except OSError:
            print('Missing /sbin/iwlist', file=waStream)
            pass

        if 0 != scan.returncode:
            print('fail wifi scanning for {}, retry... !'.format(ssid), file=waStream)
            time.sleep(1)
            continue

        found = False
        for cell in scan.stdout.split('Cell '):
            if 0 <= cell.find('ESSID:"{}"'.format(ssid)):
                found = True
                break

        # print('{} found {}'.format(ssid, found), file=waStream)
        if not found:
            print('FAIL get {} in wifi scanning !'.format(ssid), file=waStream)
            time.sleep(3)
            continue

        keymgmt_detected = None
        if 0 <= cell.find('IE: IEEE 802.11i/WPA2 Version 1'):
            keymgmt_detected = 'WPA-PSK\n\tpairwise=CCMP'
        elif 0 <= cell.find('IE: WPA Version 1'):
            keymgmt_detected = 'WPA-PSK\n\tpairwise=TKIP'
        else:
            keymgmt_detected = 'WEP'

        # print("{} detected".format(keymgmt_detected), file=waStream)
        passphrase_start = sysrepo.stdout.find('<passphrase>')
        passphrase_end = sysrepo.stdout.find('</passphrase>')
        if 0 > passphrase_start or 0 > passphrase_end:
            print('FAIL parse passphrase !', file=waStream)
            time.sleep(1)
            continue

        passphrase = '{}'.format(sysrepo.stdout[passphrase_start + len('<passphrase>'):passphrase_end])
        country_start = sysrepo.stdout.find('<country>')
        country_end = sysrepo.stdout.find('</country>')
        if 0 > country_start or 0 > country_end:
            print('FAIL parse country !', file=waStream)
            time.sleep(1)
            continue

        country = '{}'.format(sysrepo.stdout[country_start + len('<country>'):country_end])
        # print('country {} found'.format(country), file=waStream)
        if 'WEP' != keymgmt_detected:
            # print('wpa cases', file=waStream)
            try:
                wpa_out = subprocess.run(['/usr/bin/wpa_passphrase', ssid, passphrase],
                                         stdout=subprocess.PIPE, encoding='UTF-8')
            except subprocess.CalledProcessError as cpe:
                print('ERROR {}'.format(cpe), file=waStream)
                pass
            except OSError:
                print('Missing /usr/bin/wpa_passphrase', file=waStream)
                pass

            if 0 != wpa_out.returncode:
                print('wpa_passphrase fail', file=waStream)
                continue

            # print('write file', file=waStream)
            with open(wpa_tmpfile, 'w') as f:
                print('ctrl_interface=/run/wpa_supplicant', file=f)
                print('update_config=1', file=f)
                print('country={}'.format(country), file=f)
                print('network={', file=f)
                if 'WPA2-PSK' in keymgmt_detected or 'CCMP' in keymgmt_detected:
                    print('\tkey_mgmt={}'.format('WPA-PSK\n\tpairwise=CCMP'), file=f)
                else:
                    print('\tkey_mgmt={}'.format('WPA-PSK\n\tpairwise=TKIP'), file=f)

                print('{}'.format(wpa_out.stdout.split('\n', 1)[-1]), file=f)

            # print('write file done', file=waStream)

        else:
            if passphrase:
                # print('passphrase {} found'.format(passphrase), file=waStream)
                passphrase_len = len(passphrase)
                is_passphrase_in_hex = all(c in string.hexdigits for c in passphrase)
                if True == is_passphrase_in_hex and passphrase_len not in (10, 26, 32):
                    print('ERROR wrong WEP passphrase, setting 1234567890 !', file=waStream)
                    passphrase = str(1234567890)
                elif False == is_passphrase_in_hex and passphrase_len not in (5, 13, 16):
                    print('ERROR wrong WEP passphrase, setting 1234567890 !', file=waStream)
                    passphrase = str(1234567890)

            else:
                print('ERROR empty WEP passphrase, setting 1234567890 !', file=waStream)
                passphrase = str(1234567890)

            # check again
            is_passphrase_in_hex = all(c in string.hexdigits for c in passphrase)
            with open(wpa_tmpfile, 'w') as f:
                print('ctrl_interface=/run/wpa_supplicant', file=f)
                print('update_config=1', file=f)
                print('country={}'.format(country), file=f)
                print('network={', file=f)
                print('\tkey_mgmt={}'.format('NONE'), file=f)
                print('\tssid="{}"'.format(ssid), file=f)
                if is_passphrase_in_hex:
                    print('\twep_key0={}'.format(passphrase), file=f)
                else:
                    print('\twep_key0={}'.format(int(hexlify(passphrase.encode()))), file=f)

                print('}', file=f)

        # print('restart wpa_supplicant', file=waStream)
        try:
            filecmp.clear_cache()
            if False == filecmp.cmp(wpa_tmpfile, '/etc/wpa_supplicant/wpa_supplicant.conf'):
                shutil.copy(wpa_tmpfile, '/etc/wpa_supplicant/wpa_supplicant.conf')
                #subprocess.run(['/usr/bin/killall', '--quiet', '-HUP', 'wpa_supplicant'])
                subprocess.run(['/bin/systemctl','restart','--no-block','wpa_supplicant'])
                print('New configuration !', file=waStream)
                print('Done setting ssid:{} key:{} !'.format(ssid, keymgmt_detected), file=waStream)
        except subprocess.CalledProcessError as cpe:
            print('ERROR {}'.format(cpe), file=waStream)
            pass
        except OSError:
            print('Missing /usr/bin/killall', file=waStream)
            pass

        time.sleep(60)

except KeyboardInterrupt:
    pass
